@@ -4,6 +4,7 @@
 // implementationwise, it is *stateless*: it keeps no internal gui structure, hit tests are instant, usage & implementation is greatly simplified
 
 #include "engine.h"
+
 #ifndef NEWGUI
 #include "textedit.h"
 
@@ -20,11 +21,8 @@ enum {FIELDCOMMIT, FIELDABORT, FIELDEDIT, FIELDSHOW, FIELDKEY};
 static int fieldmode = FIELDSHOW;
 static bool fieldsactive = false;
 
-SVARP(guioverlaytex, "hud/2.2/guioverlay");
-SVARP(guiskintex, "hud/2.2/guiskin");
-SVARP(guislidertex, "hud/2.2/guislider");
+extern char *guioverlaytex, *guiskintex, *guislidertex;
 SVARP(guiicondir, "hud/2.2/icons");
-VARP(guierrors, 0, 0, 1);
 
 static bool hascursor;
 static float cursorx = 0.5f, cursory = 0.5f;
@@ -43,7 +41,7 @@ struct gui : g3d_gui
 {
     struct list
     {
-        int parent, w, h;
+        int parent, w, h, align;
     };
 
     int nextlist;
@@ -55,7 +53,7 @@ struct gui : g3d_gui
 
     static void reset()
     {
-        lists.setsize(0);
+        lists.shrink(0);
     }
 
     static int ty, tx, tpos, *tcurrent, tcolor; //tracking tab size and position since uses different layout method...
@@ -134,10 +132,12 @@ struct gui : g3d_gui
                 y1 = cury - ((skiny[6]-skiny[1])-(skiny[3]-skiny[2]))*SKIN_SCALE-h,
                 y2 = cury;
             bool hit = tcurrent && windowhit==this && hitx>=x1 && hity>=y1 && hitx<x2 && hity<y2;
-            if(hit && (!guiclicktab || mousebuttons&G3D_DOWN))
+            if(hit)
             {
-                *tcurrent = tpos; //roll-over to switch tab
-                color = guirollovercolour;
+                if(!guiclicktab || mousebuttons&G3D_DOWN)
+                    *tcurrent = tpos; //roll-over to switch tab
+
+                tcolor = guirollovercolour; //change colour on rollover
             }
 
             drawskin(x1-skinx[visible()?2:6]*SKIN_SCALE, y1-skiny[1]*SKIN_SCALE, w, h, visible()?10:19, 9, gui2d ? 1 : 2, light, alpha);
@@ -149,7 +149,7 @@ struct gui : g3d_gui
     bool ishorizontal() const { return curdepth&1; }
     bool isvertical() const { return !ishorizontal(); }
 
-    void pushlist()
+    void pushlist(int align = -1)
     {
         if(layoutpass)
         {
@@ -160,14 +160,27 @@ struct gui : g3d_gui
             }
             list &l = lists.add();
             l.parent = curlist;
+            l.align = align;
             curlist = lists.length()-1;
             xsize = ysize = 0;
         }
         else
         {
+            int xpad = xsize, ypad = ysize;
             curlist = nextlist++;
             xsize = lists[curlist].w;
             ysize = lists[curlist].h;
+            switch(align)
+            {
+            case 0:
+                if(ishorizontal()) cury += max(ypad - ysize, 0)/2;
+                else curx += max(xpad - xsize, 0)/2;
+                break;
+            case 1:
+                if(ishorizontal()) cury += max(ypad - ysize, 0);
+                else curx += max(xpad - xsize, 0);
+                break;
+            }
         }
         curdepth++;
     }
@@ -189,6 +202,17 @@ struct gui : g3d_gui
             if(ishorizontal()) cury -= l.h;
             else curx -= l.w;
             layout(l.w, l.h);
+            if(!layoutpass) switch(l.align)
+            {
+            case 0:
+                if(ishorizontal()) cury -= max(ysize - l.h, 0)/2;
+                else curx -= max(xsize - l.w, 0)/2;
+                break;
+            case 1:
+                if(ishorizontal()) cury -= max(ysize - l.h, 0);
+                else curx -= max(xsize - l.h, 0);
+                break;
+            }
         }
     }
 
@@ -200,9 +224,9 @@ struct gui : g3d_gui
     void progress(float percent) { autotab(); line_(FONTH*2/5, percent); }
 
     //use to set min size (useful when you have progress bars)
-    void strut(int size) { layout(isvertical() ? size*FONTW : 0, isvertical() ? 0 : size*FONTH); }
+    void strut(float size) { layout(isvertical() ? int(size*FONTW) : 0, isvertical() ? 0 : int(size*FONTH)); }
     //add space between list items
-    void space(int size) { layout(isvertical() ? 0 : size*FONTW, isvertical() ? size*FONTH : 0); }
+    void space(float size) { layout(isvertical() ? 0 : int(size*FONTW), isvertical() ? int(size*FONTH) : 0); }
 
     int layout(int w, int h)
     {
@@ -244,16 +268,16 @@ struct gui : g3d_gui
         autotab();
         if(scale==0) scale = 1;
         int size = (int)(scale*2*FONTH)-SHADOW;
-        if(visible()) icon_(t, overlaid, false, curx, cury, size, ishit(size+SHADOW, size+SHADOW));
+        if(visible()) icon_(t, overlaid, curx, cury, size, ishit(size+SHADOW, size+SHADOW));
         return layout(size+SHADOW, size+SHADOW);
     }
 
-    int texture(Texture *t, float scale, int rotate, int xoff, int yoff, Texture *glowtex, const vec &glowcolor, Texture *layertex)
+    int texture(VSlot &vslot, float scale, bool overlaid)
     {
         autotab();
         if(scale==0) scale = 1;
         int size = (int)(scale*2*FONTH)-SHADOW;
-        if(t!=notexture && visible()) icon_(t, true, true, curx, cury, size, ishit(size+SHADOW, size+SHADOW), rotate, xoff, yoff, glowtex, glowcolor, layertex);
+        if(visible()) previewslot(vslot, overlaid, curx, cury, size, ishit(size+SHADOW, size+SHADOW));
         return layout(size+SHADOW, size+SHADOW);
     }
 
@@ -344,6 +368,8 @@ struct gui : g3d_gui
         char *result = NULL;
         if(visible() && !layoutpass)
         {
+            e->rendered = true;
+
             bool hit = ishit(w, h);
             if(hit)
             {
@@ -367,13 +393,11 @@ struct gui : g3d_gui
 
             e->draw(curx+FONTW/2, cury, color, hit && editing);
 
-            notextureshader->set();
+            lineshader->set();
             glDisable(GL_TEXTURE_2D);
-            if(editing) glColor3f(1, 0, 0);
+            if(editing) glColor3ub(guirollovercolour >> 16, (guirollovercolour >> 8)&0xFF, guirollovercolour&0xFF);
             else glColor3ub(color>>16, (color>>8)&0xFF, color&0xFF);
-            glBegin(GL_LINE_LOOP);
-            rect_(curx, cury, w, h);
-            glEnd();
+            rect_(curx, cury, w, h, -1, true);
             glEnable(GL_TEXTURE_2D);
             defaultshader->set();
         }
@@ -394,18 +418,29 @@ struct gui : g3d_gui
         return result;
     }
 
-    void rect_(float x, float y, float w, float h, int usetc = -1)
+    void rect_(float x, float y, float w, float h, int usetc = -1, bool lines = false)
     {
+        glBegin(lines ? GL_LINE_LOOP : GL_TRIANGLE_STRIP);
         static const GLfloat tc[4][2] = {{0, 0}, {1, 0}, {1, 1}, {0, 1}};
         if(usetc>=0) glTexCoord2fv(tc[usetc]);
         glVertex2f(x, y);
         if(usetc>=0) glTexCoord2fv(tc[(usetc+1)%4]);
         glVertex2f(x + w, y);
-        if(usetc>=0) glTexCoord2fv(tc[(usetc+2)%4]);
-        glVertex2f(x + w, y + h);
+        if(lines)
+        {
+            if(usetc>=0) glTexCoord2fv(tc[(usetc+2)%4]);
+            glVertex2f(x + w, y + h);
+        }
         if(usetc>=0) glTexCoord2fv(tc[(usetc+3)%4]);
         glVertex2f(x, y + h);
+        if(!lines)
+        {
+            if(usetc>=0) glTexCoord2fv(tc[(usetc+2)%4]);
+            glVertex2f(x + w, y + h);
+        }
+        glEnd();
         xtraverts += 4;
+
     }
 
     void text_(const char *text, int x, int y, int color, bool shadow)
@@ -441,103 +476,138 @@ struct gui : g3d_gui
             }
             h = lists[parenth].h;
         }
-        glBegin(GL_QUADS);
         rect_(curx, cury, w, h);
-        glEnd();
         glEnable(GL_TEXTURE_2D);
         defaultshader->set();
     }
 
-    void icon_(Texture *t, bool overlaid, bool tiled, int x, int y, int size, bool hit, int rotate = 0, int xoff = 0, int yoff = 0, Texture *glowtex = NULL, const vec &glowcolor = vec(1, 1, 1), Texture *layertex = NULL)
+    void icon_(Texture *t, bool overlaid, int x, int y, int size, bool hit)
     {
-        float xs, ys, xt, yt;
-        if(tiled)
+        float scale = float(size)/max(t->xs, t->ys); //scale and preserve aspect ratio
+        float xs = t->xs*scale, ys = t->ys*scale;
+        x += int((size-xs)/2);
+        y += int((size-ys)/2);
+        if(hit && actionon)
         {
-            xt = min(1.0f, t->xs/(float)t->ys),
-            yt = min(1.0f, t->ys/(float)t->xs);
-            xs = size;
-            ys = size;
+            glDisable(GL_TEXTURE_2D);
+            notextureshader->set();
+            glColor4f(0, 0, 0, 0.75f);
+            rect_(x+SHADOW, y+SHADOW, xs, ys);
+            glEnable(GL_TEXTURE_2D);
+            defaultshader->set();
         }
-        else
+        static const float tc[4][2] = { { 0, 0 }, { 1, 0 }, { 1, 1 }, { 0, 1 } };
+        const vec &color = hit ? vec((guirollovercolour >> 16) / 255.0f, ((guirollovercolour >> 8) & 0xFF) / 255.0f, (guirollovercolour & 0xFF) / 255.0f) : (overlaid ? vec(1, 1, 1) : light);
+        glBindTexture(GL_TEXTURE_2D, t->id);
+        glColor3fv(color.v);
+        glBegin(GL_TRIANGLE_STRIP);
+        glTexCoord2fv(tc[0]); glVertex2f(x,    y);
+        glTexCoord2fv(tc[1]); glVertex2f(x+xs, y);
+        glTexCoord2fv(tc[3]); glVertex2f(x,    y+ys);
+        glTexCoord2fv(tc[2]); glVertex2f(x+xs, y+ys);
+        glEnd();
+
+        if(overlaid)
         {
-            xt = 1.0f;
-            yt = 1.0f;
-            float scale = float(size)/max(t->xs, t->ys); //scale and preserve aspect ratio
-            xs = t->xs*scale;
-            ys = t->ys*scale;
-            x += int((size-xs)/2);
-            y += int((size-ys)/2);
+            if(!overlaytex || overlaytex == notexture)
+            {
+                defformatstring(tname)("data/%s", guioverlaytex);
+                overlaytex = textureload(tname, 3);
+            }
+            glBindTexture(GL_TEXTURE_2D, overlaytex->id);
+            glColor3fv(light.v);
+            rect_(x, y, xs, ys, 0);
         }
+    }
+
+    void previewslot(VSlot &vslot, bool overlaid, int x, int y, int size, bool hit)
+    {
+        Slot &slot = *vslot.slot;
+        if(slot.sts.empty()) return;
+        VSlot *layer = NULL;
+        Texture *t = NULL, *glowtex = NULL, *layertex = NULL;
+        if(slot.loaded)
+        {
+            t = slot.sts[0].t;
+            if(t == notexture) return;
+            Slot &slot = *vslot.slot;
+            if(slot.texmask&(1<<TEX_GLOW)) { loopvj(slot.sts) if(slot.sts[j].type==TEX_GLOW) { glowtex = slot.sts[j].t; break; } }
+            if(vslot.layer)
+            {
+                layer = &lookupvslot(vslot.layer);
+                if(!layer->slot->sts.empty()) layertex = layer->slot->sts[0].t;
+            }
+        }
+        else if(slot.thumbnail && slot.thumbnail != notexture) t = slot.thumbnail;
+        else return;
+        float xt = min(1.0f, t->xs/(float)t->ys), yt = min(1.0f, t->ys/(float)t->xs), xs = size, ys = size;
         if(hit && actionon)
         {
             glColor4f(0, 0, 0, 0.75f);
-			glBindTexture(GL_TEXTURE_2D, t->id);
-            glBegin(GL_QUADS);
+            glBindTexture(GL_TEXTURE_2D, t->id);
+            glBegin(GL_TRIANGLE_FAN);
             rect_(x+SHADOW, y+SHADOW, xs, ys, 0);
             glEnd();
         }
-        if(tiled)
-        {
-            static Shader *rgbonlyshader = NULL;
-            if(!rgbonlyshader) rgbonlyshader = lookupshaderbyname("rgbonly");
-            rgbonlyshader->set();
-        }
+        static Shader *rgbonlyshader = NULL;
+        if(!rgbonlyshader) rgbonlyshader = lookupshaderbyname("rgbonly");
+        rgbonlyshader->set();
+        const vec &color = hit ? vec((guirollovercolour >> 16) / 255.0f, ((guirollovercolour >> 8) & 0xFF) / 255.0f, (guirollovercolour & 0xFF) / 255.0f) : (overlaid ? vec(1, 1, 1) : light);
         float tc[4][2] = { { 0, 0 }, { 1, 0 }, { 1, 1 }, { 0, 1 } };
-        if(rotate)
+        int xoff = vslot.xoffset, yoff = vslot.yoffset;
+        if(vslot.rotation)
         {
-            if((rotate&5) == 1) { swap(xoff, yoff); loopk(4) swap(tc[k][0], tc[k][1]); }
-            if(rotate >= 2 && rotate <= 4) { xoff *= -1; loopk(4) tc[k][0] *= -1; }
-            if(rotate <= 2 || rotate == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
+            if((vslot.rotation&5) == 1) { swap(xoff, yoff); loopk(4) swap(tc[k][0], tc[k][1]); }
+            if(vslot.rotation >= 2 && vslot.rotation <= 4) { xoff *= -1; loopk(4) tc[k][0] *= -1; }
+            if(vslot.rotation <= 2 || vslot.rotation == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
         }
         loopk(4) { tc[k][0] = tc[k][0]/xt - float(xoff)/t->xs; tc[k][1] = tc[k][1]/yt - float(yoff)/t->ys; }
-		vec color = hit ? vec( (guirollovercolour >> 16) & 0xFF, (guirollovercolour >> 8) & 0xFF, guirollovercolour & 0xFF).div(255.0f) : (overlaid ? vec(1, 1, 1) : light);
+        if(slot.loaded) glColor3f(color.x*vslot.colorscale.x, color.y*vslot.colorscale.y, color.z*vslot.colorscale.z);
+        else glColor3fv(color.v);
         glBindTexture(GL_TEXTURE_2D, t->id);
-        glColor3fv(color.v);
-        glBegin(GL_QUADS);
+        glBegin(GL_TRIANGLE_STRIP);
         glTexCoord2fv(tc[0]); glVertex2f(x,    y);
         glTexCoord2fv(tc[1]); glVertex2f(x+xs, y);
-        glTexCoord2fv(tc[2]); glVertex2f(x+xs, y+ys);
         glTexCoord2fv(tc[3]); glVertex2f(x,    y+ys);
+        glTexCoord2fv(tc[2]); glVertex2f(x+xs, y+ys);
         glEnd();
         if(glowtex)
         {
             glBlendFunc(GL_SRC_ALPHA, GL_ONE);
             glBindTexture(GL_TEXTURE_2D, glowtex->id);
-            if(hit || overlaid) { loopk(3) color[k] *= glowcolor[k]; glColor3fv(color.v); }
-            else glColor3fv(glowcolor.v);
-            glBegin(GL_QUADS);
+            if(hit || overlaid) glColor3f(color.x*vslot.glowcolor.x, color.y*vslot.glowcolor.y, color.z*vslot.glowcolor.z);
+            else glColor3fv(vslot.glowcolor.v);
+            glBegin(GL_TRIANGLE_STRIP);
             glTexCoord2fv(tc[0]); glVertex2f(x,    y);
             glTexCoord2fv(tc[1]); glVertex2f(x+xs, y);
-            glTexCoord2fv(tc[2]); glVertex2f(x+xs, y+ys);
             glTexCoord2fv(tc[3]); glVertex2f(x,    y+ys);
+            glTexCoord2fv(tc[2]); glVertex2f(x+xs, y+ys);
             glEnd();
             glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
         }
         if(layertex)
         {
             glBindTexture(GL_TEXTURE_2D, layertex->id);
-            glColor3fv(color.v);
-            glBegin(GL_QUADS);
+            glColor3f(color.x*layer->colorscale.x, color.y*layer->colorscale.y, color.z*layer->colorscale.z);
+            glBegin(GL_TRIANGLE_STRIP);
             glTexCoord2fv(tc[0]); glVertex2f(x+xs/2, y+ys/2);
             glTexCoord2fv(tc[1]); glVertex2f(x+xs,   y+ys/2);
-            glTexCoord2fv(tc[2]); glVertex2f(x+xs,   y+ys);
             glTexCoord2fv(tc[3]); glVertex2f(x+xs/2, y+ys);
+            glTexCoord2fv(tc[2]); glVertex2f(x+xs,   y+ys);
             glEnd();
         }
 
-        if(tiled) defaultshader->set();
+        defaultshader->set();
         if(overlaid)
         {
-	    defformatstring(tname)("data/%s", guioverlaytex);
-	    if(guierrors)
-	    	overlaytex = textureload(tname, 3);
-	    else
-            	overlaytex = textureload(tname, 3, true, false);
+            if(!overlaytex || overlaytex == notexture)
+            {
+                defformatstring(tname)("data/%s", guioverlaytex);
+                overlaytex = textureload(tname, 3);
+            }
             glBindTexture(GL_TEXTURE_2D, overlaytex->id);
             glColor3fv(light.v);
-            glBegin(GL_QUADS);
             rect_(x, y, xs, ys, 0);
-            glEnd();
         }
     }
 
@@ -545,14 +615,13 @@ struct gui : g3d_gui
     {
         if(visible())
         {
-	    defformatstring(tname)("data/%s", guislidertex);
- 	    if(guierrors)
-	    	slidertex = textureload(tname, 3);
-	    else
-            	slidertex = textureload(tname, 3, true, false);
+            if(!slidertex || slidertex == notexture)
+            {
+                defformatstring(tname)("data/%s", guislidertex);
+                slidertex = textureload(tname, 3);
+            }
             glEnable(GL_TEXTURE_2D);
             glBindTexture(GL_TEXTURE_2D, slidertex->id);
-            glBegin(GL_QUADS);
             if(percent < 0.99f)
             {
                 glColor4f(light.x, light.y, light.z, 0.375f);
@@ -566,7 +635,6 @@ struct gui : g3d_gui
                 rect_(curx + FONTH/2 - size, cury + ysize*(1-percent), size*2, ysize*percent, 0);
             else
                 rect_(curx, cury + FONTH/2 - size, xsize*percent, size*2, 1);
-            glEnd();
         }
         layout(ishorizontal() ? FONTH : 0, ishorizontal() ? 0 : FONTH);
     }
@@ -599,11 +667,11 @@ struct gui : g3d_gui
 
             if(icon)
             {
-                defformatstring(tname)("data/%s/%s", guiicondir, icon);
-	    if(guierrors)
-	    	icon_(textureload(tname, 3), false, false, x, cury, ICON_SIZE, clickable && hit);
-	    else
-            	icon_(textureload(tname, 3, true, false), false, false, x, cury, ICON_SIZE, clickable && hit);
+                if(!isspace(icon[0]))
+                {
+                    defformatstring(tname)("data/%s/%s", guiicondir, icon);
+                    icon_(textureload(tname, 3), false, x, cury, ICON_SIZE, clickable && hit);
+                }
                 x += ICON_SIZE;
             }
             if(icon && text) x += padding;
@@ -618,11 +686,11 @@ struct gui : g3d_gui
 
     static void drawskin(int x, int y, int gapw, int gaph, int start, int n, int passes = 1, const vec &light = vec(1, 1, 1), float alpha = 0.80f)//int vleft, int vright, int vtop, int vbottom, int start, int n)
     {
-    	defformatstring(tname)("data/%s", guiskintex);
-	if(guierrors)
-		skintex = textureload(tname, 3, true, false);
-	else
-        	skintex = textureload(tname, 3);
+        if(!skintex || skintex == notexture)
+        {
+            defformatstring(tname)("data/%s", guiskintex);
+            skintex = textureload(tname, 3);
+        }
         glBindTexture(GL_TEXTURE_2D, skintex->id);
         int gapx1 = INT_MAX, gapy1 = INT_MAX, gapx2 = INT_MAX, gapy2 = INT_MAX;
         float wscale = 1.0f/(SKIN_W*SKIN_SCALE), hscale = 1.0f/(SKIN_H*SKIN_SCALE);
@@ -638,7 +706,7 @@ struct gui : g3d_gui
                 int left = skinx[p.left]*SKIN_SCALE, right = skinx[p.right]*SKIN_SCALE,
                     top = skiny[p.top]*SKIN_SCALE, bottom = skiny[p.bottom]*SKIN_SCALE;
                 float tleft = left*wscale, tright = right*wscale,
-                      ttop = top*hscale, tbottom = bottom*hscale;
+                    ttop = top*hscale, tbottom = bottom*hscale;
                 if(p.flags&0x1)
                 {
                     gapx1 = left;
@@ -765,7 +833,7 @@ struct gui : g3d_gui
                 glScalef(-scale.x, scale.y, scale.z);
 
                 vec dir;
-                lightreaching(origin, light, dir, 0, 0.5f);
+                lightreaching(origin, light, dir, false, 0, 0.5f);
                 float intensity = vec(yaw, 0.0f).dot(dir);
                 light.mul(1.0f + max(intensity, 0.0f));
             }
@@ -823,6 +891,10 @@ struct gui : g3d_gui
 
 Texture *gui::skintex = NULL, *gui::overlaytex = NULL, *gui::slidertex = NULL;
 
+SVARFP(guioverlaytex, "hud/2.2/guioverlay", defformatstring(tname)("data/%s", guioverlaytex); gui::overlaytex = textureload(tname, 3););
+SVARFP(guiskintex, "hud/2.2/guiskin",       defformatstring(tname)("data/%s", guiskintex);    gui::skintex    = textureload(tname, 3););
+SVARFP(guislidertex, "hud/2.2/guislider",   defformatstring(tname)("data/%s", guislidertex);  gui::slidertex  = textureload(tname, 3););
+
 //chop skin into a grid
 const int gui::skiny[] = {0, 7, 21, 34, 43, 48, 56, 104, 111, 117, 128},
           gui::skinx[] = {0, 11, 23, 37, 105, 119, 137, 151, 215, 229, 246, 256};
@@ -1008,7 +1080,7 @@ void g3d_addgui(g3d_callback *cb, vec &origin, int flags)
     g.cb = cb;
     g.origin = origin;
     g.savedorigin = &origin;
-    g.dist = camera1->o.dist(g.origin);
+    g.dist = flags&GUI_BOTTOM && gui2d ? 1e16f : camera1->o.dist(g.origin);
     g.gui2d = gui2d;
 }
 
@@ -1039,8 +1111,8 @@ void g3d_render()
     if(actionon) mousebuttons |= G3D_PRESSED;
 
     gui::reset();
-    guis2d.setsize(0);
-    guis3d.setsize(0);
+    guis2d.shrink(0);
+    guis3d.shrink(0);
 
     // call all places in the engine that may want to render a gui from here, they call g3d_addgui()
     extern void g3d_texturemenu();
@@ -1118,10 +1190,26 @@ void g3d_render()
 }
 #endif
 
+
 void consolebox(int x1, int y1, int x2, int y2)
 {
-    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glPushMatrix();
+    #ifdef NEWGUI
+    glScalef(.5, .5, 1);
+    glTranslatef(x1, y1, 0);
+    glColor4f(1, 0, 0, .8);
+    glBegin(GL_TRIANGLE_STRIP);
+
+    glVertex2i(x1, y1);
+    glVertex2i(x2, y1);
+    glVertex2i(x1, y2);
+    glVertex2i(x2, y2);
+
+    glEnd();
+
+    #else
+
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
     glTranslatef(x1, y1, 0);
     float bw = x2 - x1, bh = y2 - y1, aspect = bw/bh, sh = bh, sw = sh*aspect;
     bw *= float(4*FONTH)/(SKIN_H*SKIN_SCALE);
@@ -1131,6 +1219,8 @@ void consolebox(int x1, int y1, int x2, int y2)
     glScalef(sw, sh, 1);
     gui::drawskin(-gui::skinx[1]*SKIN_SCALE, -gui::skiny[4]*SKIN_SCALE, int(bw), int(bh), 0, 9, 1, vec(1, 1, 1), 0.60f);
     gui::drawskin((-gui::skinx[1] + gui::skinx[2] - gui::skinx[5])*SKIN_SCALE, -gui::skiny[4]*SKIN_SCALE, int(bw), 0, 9, 1, 1, vec(1, 1, 1), 0.60f);
+
+    #endif
     glPopMatrix();
 }
 
