@@ -33,6 +33,8 @@ struct BlendMapBranch
     {
         loopi(4) children[i].cleanup(type[i]);
     }
+
+    uchar shrink(BlendMapNode &child, int quadrant);
 };
 
 struct BlendMapSolid
@@ -81,6 +83,16 @@ void BlendMapNode::splitsolid(uchar &type, uchar val)
     }
 }
 
+uchar BlendMapBranch::shrink(BlendMapNode &child, int quadrant)
+{
+    loopi(4) if(i != quadrant) children[i].cleanup(type[i]);
+    uchar childtype = type[quadrant];
+    child = children[quadrant];
+    type[quadrant] = BM_SOLID;
+    children[quadrant].solid = &bmsolids[0];
+    return childtype;
+}
+
 struct BlendMapRoot : BlendMapNode
 {
     uchar type;
@@ -89,20 +101,39 @@ struct BlendMapRoot : BlendMapNode
     BlendMapRoot(uchar type, const BlendMapNode &node) : BlendMapNode(node), type(type) {}
 
     void cleanup() { BlendMapNode::cleanup(type); }
+
+    void shrink(int quadrant)
+    {
+        if(type == BM_BRANCH)
+        {
+            BlendMapRoot oldroot = *this;
+            type = branch->shrink(*this, quadrant);
+            oldroot.cleanup();
+        }
+    }
 };
 
-static BlendMapRoot blendmap, curbm;
-static int curbmscale;
-static ivec curbmo;
+static BlendMapRoot blendmap;
 
-bool setblendmaporigin(const ivec &o, int size)
+struct BlendMapCache
+{
+    BlendMapRoot node;
+    int scale;
+    ivec origin;
+};
+
+BlendMapCache *newblendmapcache() { return new BlendMapCache; }
+
+void freeblendmapcache(BlendMapCache *&cache) { delete cache; cache = NULL; }
+
+bool setblendmaporigin(BlendMapCache *cache, const ivec &o, int size)
 {
     if(blendmap.type!=BM_BRANCH)
     {
-        curbm = blendmap;
-        curbmscale = worldscale-BM_SCALE;
-        curbmo = ivec(0, 0, 0);
-        return curbm.solid!=&bmsolids[0xFF];
+        cache->node = blendmap;
+        cache->scale = worldscale-BM_SCALE;
+        cache->origin = ivec(0, 0, 0);
+        return cache->node.solid!=&bmsolids[0xFF];
     }
 
     BlendMapBranch *bm = blendmap.branch;
@@ -118,24 +149,24 @@ bool setblendmaporigin(const ivec &o, int size)
         int n = (((y1>>bmscale)&1)<<1) | ((x1>>bmscale)&1);
         if(bm->type[n]!=BM_BRANCH)
         {
-            curbm = BlendMapRoot(bm->type[n], bm->children[n]);
-            curbmscale = bmscale;
-            curbmo = ivec(x1&(~0U<<bmscale), y1&(~0U<<bmscale), 0);
-            return curbm.solid!=&bmsolids[0xFF];
+            cache->node = BlendMapRoot(bm->type[n], bm->children[n]);
+            cache->scale = bmscale;
+            cache->origin = ivec(x1&(~0U<<bmscale), y1&(~0U<<bmscale), 0);
+            return cache->node.solid!=&bmsolids[0xFF];
         }
         bm = bm->children[n].branch;
     }
 
-    curbm.type = BM_BRANCH;
-    curbm.branch = bm;
-    curbmscale = bmscale;
-    curbmo = ivec(x1&(~0U<<bmscale), y1&(~0U<<bmscale), 0);
+    cache->node.type = BM_BRANCH;
+    cache->node.branch = bm;
+    cache->scale = bmscale;
+    cache->origin = ivec(x1&(~0U<<bmscale), y1&(~0U<<bmscale), 0);
     return true;
 }
 
-bool hasblendmap()
+bool hasblendmap(BlendMapCache *cache)
 {
-    return curbm.solid!=&bmsolids[0xFF];
+    return cache->node.solid!=&bmsolids[0xFF];
 }
 
 static uchar lookupblendmap(int x, int y, BlendMapBranch *bm, int bmscale)
@@ -153,20 +184,20 @@ static uchar lookupblendmap(int x, int y, BlendMapBranch *bm, int bmscale)
     }
 }
     
-uchar lookupblendmap(const vec &pos)
+uchar lookupblendmap(BlendMapCache *cache, const vec &pos)
 {
-    if(curbm.type==BM_SOLID) return curbm.solid->val;
+    if(cache->node.type==BM_SOLID) return cache->node.solid->val;
     
     uchar vals[4], *val = vals;
     float bx = pos.x/(1<<BM_SCALE) - 0.5f, by = pos.y/(1<<BM_SCALE) - 0.5f;
     int ix = (int)floor(bx), iy = (int)floor(by),
-        rx = ix-curbmo.x, ry = iy-curbmo.y;
+        rx = ix-cache->origin.x, ry = iy-cache->origin.y;
     loop(vy, 2) loop(vx, 2)
     {
-        int cx = clamp(rx+vx, 0, (1<<curbmscale)-1), cy = clamp(ry+vy, 0, (1<<curbmscale)-1);
-        if(curbm.type==BM_IMAGE)
-            *val++ = curbm.image->data[cy*BM_IMAGE_SIZE + cx];
-        else *val++ = lookupblendmap(cx, cy, curbm.branch, curbmscale);
+        int cx = clamp(rx+vx, 0, (1<<cache->scale)-1), cy = clamp(ry+vy, 0, (1<<cache->scale)-1);
+        if(cache->node.type==BM_IMAGE)
+            *val++ = cache->node.image->data[cy*BM_IMAGE_SIZE + cx];
+        else *val++ = lookupblendmap(cx, cy, cache->node.branch, cache->scale);
     }
     float fx = bx - ix, fy = by - iy;
     return uchar((1-fy)*((1-fx)*vals[0] + fx*vals[1]) +
@@ -196,9 +227,9 @@ static void fillblendmap(uchar &type, BlendMapNode &node, int size, uchar val, i
         if(y2 > size)
         {
             if(x1 < size) fillblendmap(node.branch->type[2], node.branch->children[2], size, val,
-                                        x1, y1-size, min(x2, size), y2-size);
+                                        x1, max(y1-size, 0), min(x2, size), y2-size);
             if(x2 > size) fillblendmap(node.branch->type[3], node.branch->children[3], size, val,
-                                        max(x1-size, 0), y1-size, x2-size, y2-size);
+                                        max(x1-size, 0), max(y1-size, 0), x2-size, y2-size);
         }
         loopi(4) if(node.branch->type[i]!=BM_SOLID || node.branch->children[i].solid->val!=val) return;
         node.cleanup(type);
@@ -242,6 +273,53 @@ void fillblendmap(int x, int y, int w, int h, uchar val)
     fillblendmap(blendmap.type, blendmap, bmsize, val, x1, y1, x2, y2);
 }
 
+static void invertblendmap(uchar &type, BlendMapNode &node, int size, int x1, int y1, int x2, int y2)
+{
+    if(type==BM_BRANCH)
+    {
+        size /= 2;
+        if(y1 < size)
+        {
+            if(x1 < size) invertblendmap(node.branch->type[0], node.branch->children[0], size,
+                                        x1, y1, min(x2, size), min(y2, size));
+            if(x2 > size) invertblendmap(node.branch->type[1], node.branch->children[1], size,
+                                        max(x1-size, 0), y1, x2-size, min(y2, size));
+        }
+        if(y2 > size)
+        {
+            if(x1 < size) invertblendmap(node.branch->type[2], node.branch->children[2], size,
+                                        x1, max(y1-size, 0), min(x2, size), y2-size);
+            if(x2 > size) invertblendmap(node.branch->type[3], node.branch->children[3], size,
+                                        max(x1-size, 0), max(y1-size, 0), x2-size, y2-size);
+        }
+        return;
+    }
+    else if(type==BM_SOLID)
+    {
+        fillblendmap(type, node, size, 255-node.solid->val, x1, y1, x2, y2);
+    }
+    else if(type==BM_IMAGE)
+    {
+        uchar *dst = &node.image->data[y1*BM_IMAGE_SIZE + x1];
+        loopi(y2-y1)
+        {
+            loopj(x2-x1) dst[j] = 255-dst[j];
+            dst += BM_IMAGE_SIZE;
+        }
+    }
+}
+
+void invertblendmap(int x, int y, int w, int h)
+{
+    int bmsize = worldsize>>BM_SCALE,
+        x1 = clamp(x, 0, bmsize),
+        y1 = clamp(y, 0, bmsize),
+        x2 = clamp(x+w, 0, bmsize),
+        y2 = clamp(y+h, 0, bmsize);
+    if(max(x1, y1) >= bmsize || min(x2, y2) <= 0 || x1>=x2 || y1>=y2) return;
+    invertblendmap(blendmap.type, blendmap, bmsize, x1, y1, x2, y2);
+}
+
 static void optimizeblendmap(uchar &type, BlendMapNode &node)
 {
     switch(type)
@@ -376,6 +454,11 @@ void enlargeblendmap()
     blendmap.branch = branch;
 }
 
+void shrinkblendmap(int octant)
+{
+    blendmap.shrink(octant&3);
+}
+
 struct BlendBrush
 {
     char *name;
@@ -529,9 +612,11 @@ COMMAND(setblendbrush, "s");
 COMMAND(getblendbrushname, "i");
 COMMAND(curblendbrush, "");
 
+extern int nompedit;
+
 bool canpaintblendmap(bool brush = true, bool sel = false, bool msg = true)
 {
-    if(noedit(!sel, msg)) return false;
+    if(noedit(!sel, msg) || (nompedit && multiplayer())) return false;
     if(!blendpaintmode)
     {
         if(msg) conoutf(CON_ERROR, "operation only allowed in blend paint mode");
@@ -603,7 +688,7 @@ ICOMMAND(paintblendmap, "D", (int *isdown),
     
 void clearblendmapsel()
 {
-    if(noedit(false)) return;
+    if(noedit(false) || (nompedit && multiplayer())) return;
     extern selinfo sel;
     int x1 = sel.o.x>>BM_SCALE, y1 = sel.o.y>>BM_SCALE,
         x2 = (sel.o.x+sel.s.x*sel.grid+(1<<BM_SCALE)-1)>>BM_SCALE,
@@ -615,9 +700,32 @@ void clearblendmapsel()
 
 COMMAND(clearblendmapsel, "");
 
+void invertblendmapsel()
+{
+    if(noedit(false) || (nompedit && multiplayer())) return;
+    extern selinfo sel;
+    int x1 = sel.o.x>>BM_SCALE, y1 = sel.o.y>>BM_SCALE,
+        x2 = (sel.o.x+sel.s.x*sel.grid+(1<<BM_SCALE)-1)>>BM_SCALE,
+        y2 = (sel.o.y+sel.s.y*sel.grid+(1<<BM_SCALE)-1)>>BM_SCALE;
+    invertblendmap(x1, y1, x2-x1, y2-y1);
+    previewblends(ivec(x1<<BM_SCALE, y1<<BM_SCALE, 0),
+                  ivec((x2-x1)<<BM_SCALE, (y2-y1)<<BM_SCALE, worldsize));
+}
+
+COMMAND(invertblendmapsel, "");
+
+void invertblendmap()
+{
+    if(noedit(false) || (nompedit && multiplayer())) return;
+    invertblendmap(0, 0, worldsize>>BM_SCALE, worldsize>>BM_SCALE);
+    previewblends(ivec(0, 0, 0), ivec(worldsize, worldsize, worldsize));
+}
+
+COMMAND(invertblendmap, "");
+
 void showblendmap()
 {
-    if(noedit(true)) return;
+    if(noedit(true) || (nompedit && multiplayer())) return;
     previewblends(ivec(0, 0, 0), ivec(worldsize, worldsize, worldsize));
 }
 
@@ -625,7 +733,7 @@ COMMAND(showblendmap, "");
 COMMAND(optimizeblendmap, "");
 ICOMMAND(clearblendmap, "", (),
 {
-    if(noedit(true)) return;
+    if(noedit(true) || (nompedit && multiplayer())) return;
     resetblendmap();
     showblendmap();
 });
