@@ -38,7 +38,7 @@ void conoutf(const char *fmt, ...)
     va_list args;
     va_start(args, fmt);
     conoutfv(CON_INFO, fmt, args);
-    va_end(args); 
+    va_end(args);
 }
 
 void conoutf(int type, const char *fmt, ...)
@@ -49,9 +49,8 @@ void conoutf(int type, const char *fmt, ...)
     va_end(args);
 }
 
-bool fullconsole = false;
-void toggleconsole() { fullconsole = !fullconsole; }
-COMMAND(toggleconsole, "");
+VAR(fullconsole, 0, 0, 1);
+ICOMMAND(toggleconsole, "", (), { fullconsole ^= 1; });
 
 int rendercommand(int x, int y, int w)
 {
@@ -132,7 +131,7 @@ int drawconlines(int conskip, int confade, int conwidth, int conheight, int cono
         char *line = conlines[idx].line;
         int width, height;
         text_bounds(line, width, height, conwidth);
-        if(dir <= 0) y -= height; 
+        if(dir <= 0) y -= height;
         draw_text(line, conoff, y, 0xFF, 0xFF, 0xFF, 0xFF, -1, conwidth);
         if(dir > 0) y += height;
     }
@@ -145,10 +144,10 @@ int renderconsole(int w, int h, int abovehud)                   // render buffer
         conoff = fullconsole ? FONTH : FONTH/3,
         conheight = min(fullconsole ? ((h*fullconsize/100)/FONTH)*FONTH : FONTH*consize, h - 2*(conpad + conoff)),
         conwidth = w - 2*(conpad + conoff) - (fullconsole ? 0 : game::clipconsole(w, h));
-    
+
     extern void consolebox(int x1, int y1, int x2, int y2);
     if(fullconsole) consolebox(conpad, conpad, conwidth+conpad+2*conoff, conheight+conpad+2*conoff);
-    
+
     int y = drawconlines(conskip, fullconsole ? 0 : confade, conwidth, conheight, conpad+conoff, fullconsole ? fullconfilter : confilter);
     if(!fullconsole && (miniconsize && miniconwidth))
         drawconlines(miniconskip, miniconfade, (miniconwidth*(w - 2*(conpad + conoff)))/100, min(FONTH*miniconsize, abovehud - y), conpad+conoff, miniconfilter, abovehud, -1);
@@ -166,7 +165,7 @@ struct keym
         ACTION_EDITING,
         NUMACTIONS
     };
-    
+
     int code;
     char *name;
     char *actions[NUMACTIONS];
@@ -180,13 +179,13 @@ hashtable<int, keym> keyms(128);
 
 void keymap(int *code, char *key)
 {
-    if(overrideidents) { conoutf(CON_ERROR, "cannot override keymap %s", code); return; }
+    if(identflags&IDF_OVERRIDDEN) { conoutf(CON_ERROR, "cannot override keymap %s", code); return; }
     keym &km = keyms[*code];
     km.code = *code;
     DELETEA(km.name);
     km.name = newstring(key);
 }
-    
+
 COMMAND(keymap, "is");
 
 keym *keypressed = NULL;
@@ -220,17 +219,17 @@ keym *findbind(char *key)
         if(!strcasecmp(km.name, key)) return &km;
     });
     return NULL;
-}   
-    
+}
+
 void getbind(char *key, int type)
 {
     keym *km = findbind(key);
     result(km ? km->actions[type] : "");
-} 
+}
 
 void bindkey(char *key, char *action, int state, const char *cmd)
 {
-    if(overrideidents) { conoutf(CON_ERROR, "cannot override %s \"%s\"", cmd, key); return; }
+    if(identflags&IDF_OVERRIDDEN) { conoutf(CON_ERROR, "cannot override %s \"%s\"", cmd, key); return; }
     keym *km = findbind(key);
     if(!km) { conoutf(CON_ERROR, "unknown key \"%s\"", key); return; }
     char *&binding = km->actions[state];
@@ -276,7 +275,7 @@ COMMAND(inputcommand, "sss");
 void pasteconsole()
 {
     #ifdef WIN32
-    if(!IsClipboardFormatAvailable(CF_TEXT)) return; 
+    if(!IsClipboardFormatAvailable(CF_TEXT)) return;
     if(!OpenClipboard(NULL)) return;
     char *cb = (char *)GlobalLock(GetClipboardData(CF_TEXT));
     concatstring(commandbuf, cb);
@@ -288,7 +287,7 @@ void pasteconsole()
 	mac_pasteconsole(commandbuf);
 	#else
     SDL_SysWMinfo wminfo;
-    SDL_VERSION(&wminfo.version); 
+    SDL_VERSION(&wminfo.version);
     wminfo.subsystem = SDL_SYSWM_X11;
     if(!SDL_GetWMInfo(&wminfo)) return;
     int cbsize;
@@ -338,7 +337,7 @@ struct hline
                (commandaction ? !action || strcmp(commandaction, action) : action!=NULL) ||
                (commandprompt ? !prompt || strcmp(commandprompt, prompt) : prompt!=NULL);
     }
-    
+
     void save()
     {
         buf = newstring(commandbuf);
@@ -360,6 +359,8 @@ struct hline
 vector<hline *> history;
 int histpos = 0;
 
+VARP(maxhistory, 0, 1000, 10000);
+
 void history_(int *n)
 {
     static bool inhistory = false;
@@ -380,18 +381,18 @@ struct releaseaction
 };
 vector<releaseaction> releaseactions;
 
-const char *addreleaseaction(const char *s)
+const char *addreleaseaction(char *s)
 {
     if(!keypressed) return NULL;
     releaseaction &ra = releaseactions.add();
     ra.key = keypressed;
-    ra.action = newstring(s);
+    ra.action = s;
     return keypressed->name;
 }
 
-void onrelease(char *s)
+void onrelease(const char *s)
 {
-    addreleaseaction(s);
+    addreleaseaction(newstring(s));
 }
 
 COMMAND(onrelease, "s");
@@ -411,7 +412,11 @@ void execbind(keym &k, bool isdown)
     if(isdown)
     {
         int state = keym::ACTION_DEFAULT;
+		#ifndef NEWGUI
         if(!mainmenu)
+		#else
+		if(!UI::mainmenu)
+		#endif
         {
             if(editmode) state = keym::ACTION_EDITING;
             else if(player->state==CS_SPECTATOR) state = keym::ACTION_SPECTATOR;
@@ -429,7 +434,7 @@ void execbind(keym &k, bool isdown)
 void consolekey(int code, bool isdown, int cooked)
 {
     #ifdef __APPLE__
-        #define MOD_KEYS (KMOD_LMETA|KMOD_RMETA) 
+        #define MOD_KEYS (KMOD_LMETA|KMOD_RMETA)
     #else
         #define MOD_KEYS (KMOD_LCTRL|KMOD_RCTRL)
     #endif
@@ -481,11 +486,12 @@ void consolekey(int code, bool isdown, int cooked)
                 break;
 
             case SDLK_UP:
-                if(histpos>0) history[--histpos]->restore(); 
+                if(histpos > history.length()) histpos = history.length();
+                if(histpos > 0) history[--histpos]->restore();
                 break;
 
             case SDLK_DOWN:
-                if(histpos+1<history.length()) history[++histpos]->restore();
+                if(histpos + 1 < history.length()) history[++histpos]->restore();
                 break;
 
             case SDLK_TAB:
@@ -527,7 +533,14 @@ void consolekey(int code, bool isdown, int cooked)
             if(commandbuf[0])
             {
                 if(history.empty() || history.last()->shouldsave())
-                    history.add(h = new hline)->save(); // cap this?
+                {
+                    if(maxhistory && history.length() >= maxhistory)
+                    {
+                        loopi(history.length()-maxhistory+1) delete history[i];
+                        history.remove(0, history.length()-maxhistory+1);
+                    }
+                    history.add(h = new hline)->save();
+                }
                 else h = history.last();
             }
             histpos = history.length();
@@ -554,7 +567,7 @@ void keypress(int code, bool isdown, int cooked)
     else if(!UI::keypress(code, isdown, cooked)) // 3D GUI mouse button intercept
     #else
     else if(!menukey(code, isdown, cooked)) // 3D GUI mouse button intercept
-    #endif  
+    #endif
     {
         if(commandmillis >= 0) consolekey(code, isdown, cooked);
         else if(haskey) execbind(*haskey, isdown);
@@ -606,18 +619,18 @@ struct filesval
     char *dir, *ext;
     vector<char *> files;
     int millis;
-    
+
     filesval(int type, const char *dir, const char *ext) : type(type), dir(newstring(dir)), ext(ext && ext[0] ? newstring(ext) : NULL), millis(-1) {}
-    ~filesval() { DELETEA(dir); DELETEA(ext); files.deletecontentsa(); }
+    ~filesval() { DELETEA(dir); DELETEA(ext); files.deletearrays(); }
 
     static int comparefiles(char **x, char **y) { return strcmp(*x, *y); }
 
     void update()
     {
         if(type!=FILES_DIR || millis >= commandmillis) return;
-        files.deletecontentsa();        
+        files.deletearrays();
         listfiles(dir, ext, files);
-        files.sort(comparefiles); 
+        files.sort(comparefiles);
         loopv(files) if(i && !strcmp(files[i], files[i-1])) delete[] files.remove(i--);
         millis = totalmillis;
     }
@@ -643,7 +656,7 @@ void resetcomplete() { completesize = 0; }
 
 void addcomplete(char *command, int type, char *dir, char *ext)
 {
-    if(overrideidents)
+    if(identflags&IDF_OVERRIDDEN)
     {
         conoutf(CON_ERROR, "cannot override complete %s", command);
         return;
@@ -670,7 +683,7 @@ void addcomplete(char *command, int type, char *dir, char *ext)
     if(!val)
     {
         filesval *f = new filesval(type, dir, ext);
-        if(type==FILES_LIST) explodelist(dir, f->files); 
+        if(type==FILES_LIST) explodelist(dir, f->files);
         val = &completefiles[fileskey(type, f->dir, f->ext)];
         *val = f;
     }
@@ -734,7 +747,7 @@ void complete(char *s)
     }
     else // complete using command names
     {
-        enumerate(*idents, ident, id,
+        enumerate(idents, ident, id,
             if(strncmp(id.name, s+1, completesize)==0 &&
                strcmp(id.name, lastcomplete) > 0 && (!nextcomplete || strcmp(id.name, nextcomplete) < 0))
                 nextcomplete = id.name;
