@@ -479,8 +479,8 @@ struct decalrenderer
             else
             {
                 bool solid = cu[i].ext && isclipped(cu[i].ext->material&MATF_VOLUME);
-                uchar vismask = 0;
-                loopj(6) if(!(avoid&(1<<j)) && (solid ? visiblematerial(cu[i], j, co.x, co.y, co.z, size)==MATSURF_VISIBLE : cu[i].texture[j]!=DEFAULT_SKY && visibleface(cu[i], j, co.x, co.y, co.z, size))) vismask |= 1<<j;
+                uchar vismask = 0, nmat = cu[i].ext && cu[i].ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA;
+                loopj(6) if(!(avoid&(1<<j)) && (solid ? visiblematerial(cu[i], j, co.x, co.y, co.z, size)==MATSURF_VISIBLE : cu[i].texture[j]!=DEFAULT_SKY && visibleface(cu[i], j, co.x, co.y, co.z, size, MAT_AIR, nmat, MAT_ALPHA))) vismask |= 1<<j;
                 if(!vismask) continue;
                 uchar vertused = fvmasks[vismask];
                 vec v[8];
@@ -494,9 +494,10 @@ struct decalrenderer
 decalrenderer decals[] =
 {
     decalrenderer("data/items"),
-    decalrenderer("data/hirato/burn"), //improve collision detection first
-    decalrenderer("data/hirato/stain", 0, 10, 2000, 20),
-    decalrenderer("data/hirato/ripple", 0, 10, 750, 20)
+    decalrenderer("packages/particles/burn"), //improve collision detection first
+    decalrenderer("packages/particles/stain", 0, 10, 2000, 20),
+    decalrenderer("packages/particles/ripple", 0, 10, 750, 20),
+    decalrenderer("packages/particles/leaves", DF_RND4|DF_ROTATE)
 };
 
 void initdecals()
