@@ -21,7 +21,7 @@
 #endif
 #endif
 
-#define CONFVERSION 7
+#define CONFVERSION 8
 
 // GL_ARB_multitexture
 extern PFNGLACTIVETEXTUREARBPROC       glActiveTexture_;
@@ -102,6 +102,11 @@ extern PFNGLCOMPRESSEDTEXSUBIMAGE2DARBPROC glCompressedTexSubImage2D_;
 extern PFNGLCOMPRESSEDTEXSUBIMAGE1DARBPROC glCompressedTexSubImage1D_;
 extern PFNGLGETCOMPRESSEDTEXIMAGEARBPROC   glGetCompressedTexImage_;
 
+// GL_EXT_fog_coord
+extern PFNGLFOGCOORDPOINTEREXTPROC glFogCoordPointer_;
+
+#include "varray.h"
+
 extern dynent *player;
 extern physent *camera1;                // special ent that acts as camera, same object as player1 in FPS mode
 
@@ -110,9 +115,8 @@ extern int mapversion;
 extern char *maptitle;
 extern vector<ushort> texmru;
 extern int xtraverts, xtravertsva;
-extern int curtexnum;
 extern const ivec cubecoords[8];
-extern const ushort fv[6][4];
+extern const uchar fv[6][4];
 extern const uchar fvmasks[64];
 extern const uchar faceedgesidx[6][4];
 extern bool inbetweenframes, renderedframe;
@@ -133,8 +137,8 @@ struct font
     char *name;
     Texture *tex;
     vector<charinfo> chars;
-    short defaultw, defaulth;
-    short offsetx, offsety, offsetw, offseth;
+    int charoffset, defaultw, defaulth;
+    int offsetx, offsety, offsetw, offseth;
 };
 
 #define FONTH (curfont->defaulth)
@@ -153,10 +157,10 @@ extern void loadalphamask(Texture *t);
 extern void loadlayermasks();
 extern Texture *cubemapload(const char *name, bool mipit = true, bool msg = true, bool transient = false);
 extern void drawcubemap(int size, const vec &o, float yaw, float pitch, const cubemapside &side);
-extern Slot &lookupmaterialslot(int slot, bool load = true);
-extern Slot &lookuptexture(int slot, bool load = true);
 extern void loadshaders();
-extern void createtexture(int tnum, int w, int h, void *pixels, int clamp, int filter, GLenum component = GL_RGB, GLenum target = GL_TEXTURE_2D, int pw = 0, int ph = 0, int pitch = 0, bool resize = true);
+extern void setuptexparameters(int tnum, void *pixels, int clamp, int filter, GLenum format = GL_RGB, GLenum target = GL_TEXTURE_2D);
+extern void createtexture(int tnum, int w, int h, void *pixels, int clamp, int filter, GLenum component = GL_RGB, GLenum target = GL_TEXTURE_2D, int pw = 0, int ph = 0, int pitch = 0, bool resize = true, GLenum format = GL_FALSE);
+extern void blurtexture(int n, int bpp, int w, int h, uchar *dst, const uchar *src);
 extern void renderpostfx();
 extern void initenvmaps();
 extern void genenvmaps();
@@ -167,6 +171,12 @@ extern GLuint lookupenvmap(Slot &slot);
 extern bool reloadtexture(Texture &tex);
 extern bool reloadtexture(const char *name);
 extern void setuptexcompress();
+extern void clearslots();
+extern void compacteditvslots();
+extern void compactmruvslots();
+extern void compactvslots(cube *c, int n = 8);
+extern void compactvslot(int &index);
+extern int compactvslots();
 extern const char *textypename(int i);
 extern void cleartexanims(bool clear = true);
 extern void reconftexanims();
@@ -182,7 +192,6 @@ extern bool isshadowmapreceiver(vtxarray *va);
 extern void rendershadowmap();
 extern void pushshadowmap();
 extern void popshadowmap();
-extern void adjustshadowmatrix(const ivec &o, float scale);
 extern void rendershadowmapreceivers();
 extern void guessshadowdir();
 
@@ -201,11 +210,12 @@ static inline bool pvsoccluded(const ivec &bborigin, int size)
 }
 
 // rendergl
-extern bool hasVBO, hasDRE, hasOQ, hasTR, hasFBO, hasDS, hasTF, hasBE, hasBC, hasCM, hasNP2, hasTC, hasTE, hasMT, hasD3, hasAF, hasVP2, hasVP3, hasPP, hasMDA, hasTE3, hasTE4, hasVP, hasFP, hasGLSL, hasGM, hasNVFB, hasSGIDT, hasSGISH, hasDT, hasSH, hasNVPCF, hasRN, hasPBO, hasFBB;
+extern bool hasVBO, hasDRE, hasOQ, hasTR, hasFBO, hasDS, hasTF, hasBE, hasBC, hasCM, hasNP2, hasTC, hasTE, hasMT, hasD3, hasAF, hasVP2, hasVP3, hasPP, hasMDA, hasTE3, hasTE4, hasVP, hasFP, hasGLSL, hasGM, hasNVFB, hasSGIDT, hasSGISH, hasDT, hasSH, hasNVPCF, hasRN, hasPBO, hasFBB, hasUBO, hasBUE, hasFC, hasTEX;
 extern int hasstencil;
 
-extern bool envmapping, renderedgame;
-extern glmatrixf mvmatrix, projmatrix, mvpmatrix, invmvmatrix, invmvpmatrix;
+extern bool envmapping, minimapping, renderedgame;
+extern const glmatrixf viewmatrix;
+extern glmatrixf mvmatrix, projmatrix, mvpmatrix, invmvmatrix, invmvpmatrix, fogmatrix, invfogmatrix, envmatrix;
 extern bvec fogcolor;
 
 extern void gl_checkextensions();
@@ -215,13 +225,12 @@ extern void rendergame(bool mainpass = false);
 extern void invalidatepostfx();
 extern void gl_drawframe(int w, int h);
 extern void gl_drawmainmenu(int w, int h);
+extern void drawminimap();
 extern void enablepolygonoffset(GLenum type);
 extern void disablepolygonoffset(GLenum type);
 extern void calcspherescissor(const vec &center, float size, float &sx1, float &sy1, float &sx2, float &sy2);
 extern int pushscissor(float sx1, float sy1, float sx2, float sy2);
 extern void popscissor();
-extern void setfogplane(const plane &p, bool flush = false);
-extern void setfogplane(float scale = 0, float z = 0, bool flush = false, float fadescale = 0, float fadeoffset = 0);
 extern void recomputecamera();
 extern void findorientation();
 extern void writecrosshairs(stream *f);
@@ -233,25 +242,29 @@ extern void render3dbox(vec &o, float tofloor, float toceil, float xradius, floa
 extern cube *newcubes(uint face = F_EMPTY);
 extern cubeext *newcubeext(cube &c);
 extern void getcubevector(cube &c, int d, int x, int y, int z, ivec &p);
-extern void setcubevector(cube &c, int d, int x, int y, int z, ivec &p);
+extern void setcubevector(cube &c, int d, int x, int y, int z, const ivec &p);
 extern int familysize(cube &c);
 extern void freeocta(cube *c);
-extern void discardchildren(cube &c);
+extern void discardchildren(cube &c, bool fixtex = false, int depth = 0);
 extern void optiface(uchar *p, cube &c);
 extern void validatec(cube *c, int size);
 extern bool isvalidcube(cube &c);
-extern cube &lookupcube(int tx, int ty, int tz, int tsize = 0);
-extern cube &neighbourcube(int x, int y, int z, int size, int rsize, int orient);
+extern ivec lu;
+extern int lusize;
+extern cube &lookupcube(int tx, int ty, int tz, int tsize = 0, ivec &ro = lu, int &rsize = lusize);
+extern cube *neighbourstack[32];
+extern int neighbourdepth;
+extern cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro = lu, int &rsize = lusize);
 extern void newclipplanes(cube &c);
 extern void freeclipplanes(cube &c);
-extern void forcemip(cube &c);
+extern int getmippedtexture(cube &p, int orient);
+extern void forcemip(cube &c, bool fixtex = true);
 extern bool subdividecube(cube &c, bool fullcheck=true, bool brighten=true);
 extern void converttovectorworld();
 extern int faceverts(cube &c, int orient, int vert);
 extern int faceconvexity(cube &c, int orient);
-extern void calcvert(cube &c, int x, int y, int z, int size, vvec &vert, int i, bool solid = false);
+extern void calcvert(cube &c, int x, int y, int z, int size, ivec &vert, int i, bool solid = false);
 extern void calcvert(cube &c, int x, int y, int z, int size, vec &vert, int i, bool solid = false);
-extern int calcverts(cube &c, int x, int y, int z, int size, vvec *verts, bool *usefaces);
 extern uint faceedges(cube &c, int orient);
 extern bool collapsedface(uint cfe);
 extern bool touchingface(cube &c, int orient);
@@ -264,8 +277,8 @@ extern int visibleorient(cube &c, int orient);
 extern bool threeplaneintersect(plane &pl1, plane &pl2, plane &pl3, vec &dest);
 extern void genvectorvert(const ivec &p, cube &c, ivec &v);
 extern void freemergeinfo(cube &c);
-extern void genmergedverts(cube &cu, int orient, const ivec &co, int size, const mergeinfo &m, vvec *vv, plane *p = NULL);
-extern int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, const vvec *vv);
+extern void genmergedverts(cube &cu, int orient, const ivec &co, int size, const mergeinfo &m, vec *vv, plane *p = NULL);
+extern int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, const vec *vv);
 extern void invalidatemerges(cube &c, bool msg);
 extern void calcmerges();
 
@@ -275,7 +288,7 @@ struct cubeface : mergeinfo
 };
 
 extern int mergefaces(int orient, cubeface *m, int sz);
-extern void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo &orig, mergeinfo &cf);
+extern void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo &orig, mergeinfo &cf, uchar nmat = MAT_AIR, uchar matmask = MATF_VOLUME);
 
 static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &o, const ivec &s)
 {
@@ -291,16 +304,6 @@ static inline uchar octantrectangleoverlap(const ivec &c, int size, const ivec &
     return p;
 }
 
-static inline bool insideworld(const vec &o)
-{
-    return o.x>=0 && o.x<worldsize && o.y>=0 && o.y<worldsize && o.z>=0 && o.z<worldsize;
-}
-
-static inline bool insideworld(const ivec &o)
-{
-    return uint(o.x)<uint(worldsize) && uint(o.y)<uint(worldsize) && uint(o.z)<uint(worldsize);
-}
-
 // ents
 extern char *entname(entity &e);
 extern bool haveselent();
@@ -326,13 +329,12 @@ extern void updatevabb(vtxarray *va, bool force = false);
 extern void updatevabbs(bool force = false);
 
 // renderva
-extern GLuint fogtex;
-
-extern void visiblecubes(float fov, float fovy);
-extern void reflectvfcP(float z, float minyaw = -M_PI, float maxyaw = M_PI, float minpitch = -M_PI, float maxpitch = M_PI);
+extern void visiblecubes(bool cull = true);
+extern void setvfcP(float z = -1, const vec &bbmin = vec(-1, -1, -1), const vec &bbmax = vec(1, 1, 1));
+extern void savevfcP();
 extern void restorevfcP();
-extern void createfogtex();
 extern void rendergeom(float causticspass = 0, bool fogpass = false);
+extern void renderalphageom(bool fogpass = false);
 extern void rendermapmodels();
 extern void renderreflectedgeom(bool causticspass = false, bool fogpass = false);
 extern void renderreflectedmapmodels();
@@ -346,7 +348,7 @@ extern occludequery *newquery(void *owner);
 extern bool checkquery(occludequery *query, bool nowait = false);
 extern void resetqueries();
 extern int getnumqueries();
-extern void drawbb(const ivec &bo, const ivec &br, const vec &camera = camera1->o, int scale = 0, const ivec &origin = ivec(0, 0, 0));
+extern void drawbb(const ivec &bo, const ivec &br, const vec &camera = camera1->o);
 
 #define startquery(query) { glBeginQuery_(GL_SAMPLES_PASSED_ARB, ((occludequery *)(query))->id); }
 #define endquery(query) \
@@ -361,7 +363,7 @@ extern void drawbb(const ivec &bo, const ivec &br, const vec &camera = camera1->
 extern void updatedynlights();
 extern int finddynlights();
 extern void calcdynlightmask(vtxarray *va);
-extern int setdynlights(vtxarray *va, const ivec &vaorigin);
+extern int setdynlights(vtxarray *va);
 extern bool getdynlight(int n, vec &o, float &radius, vec &color);
 
 // material
@@ -382,14 +384,14 @@ extern int visiblematerial(cube &c, int orient, int x, int y, int z, int size, u
 extern int refracting;
 extern bool reflecting, fading, fogging;
 extern float reflectz;
-extern int reflectdist, vertwater, refractfog, waterrefract, waterreflect, waterfade, caustics, waterfallrefract, waterfog, lavafog;
+extern int reflectdist, vertwater, waterrefract, waterreflect, waterfade, caustics, waterfallrefract, waterfog, lavafog;
 extern bvec watercolor, waterfallcolor, lavacolor;
 
 extern void cleanreflections();
 extern void queryreflections();
 extern void drawreflections();
 extern void renderwater();
-extern void renderlava(materialsurface &m, Texture *tex, float scale);
+extern void renderlava(const materialsurface &m, Texture *tex, float scale);
 extern void loadcaustics(bool force = false);
 extern void preloadwatershaders(bool force = false);
 
@@ -423,21 +425,20 @@ extern void addserver(const char *name, int port = 0, const char *password = NUL
 extern void writeservercfg();
 
 // client
-extern void localdisconnect(bool cleanup = true);
 extern void localservertoclient(int chan, ENetPacket *packet);
 extern void connectserv(const char *servername, int port, const char *serverpassword);
 extern void abortconnect();
 extern void clientkeepalive();
 
 // command
-extern hashtable<const char *, ident> *idents;
-extern bool overrideidents, persistidents;
+extern hashset<ident> idents;
+extern int identflags;
 
 extern void explodelist(const char *s, vector<char *> &elems);
 extern char *indexlist(const char *s, int pos);
 
 extern void clearoverrides();
-extern void writecfg();
+extern void writecfg(const char *name = NULL);
 
 extern void checksleep(int millis);
 extern void clearsleep(bool clearoverrides = true);
@@ -451,12 +452,14 @@ extern void conoutf(int type, const char *s, ...);
 extern void resetcomplete();
 extern void complete(char *s);
 const char *getkeyname(int code);
-extern const char *addreleaseaction(const char *s);
+extern const char *addreleaseaction(char *s);
 extern void writebinds(stream *f);
 extern void writecompletions(stream *f);
 
 // main
 
+extern string imagelogo;
+
 enum
 {
     NOT_INITING = 0,
@@ -496,15 +499,22 @@ extern bool pointincube(const clipplanes &p, const vec &v);
 extern bool overlapsdynent(const vec &o, float radius);
 extern void rotatebb(vec &center, vec &radius, int yaw);
 extern float shadowray(const vec &o, const vec &ray, float radius, int mode, extentity *t = NULL);
+struct ShadowRayCache;
+extern ShadowRayCache *newshadowraycache();
+extern void freeshadowraycache(ShadowRayCache *&cache);
+extern void resetshadowraycache(ShadowRayCache *cache);
+extern float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radius, int mode, extentity *t = NULL);
 
 // world
 
 extern vector<int> outsideents;
 
+extern void entcancel();
 extern void entitiesinoctanodes();
 extern void attachentities();
 extern void freeoctaentities(cube &c);
 extern bool pointinsel(selinfo &sel, vec &o);
+extern const int getattrnum(int type);
 
 extern void resetmap();
 extern void startmap(const char *name);
@@ -518,6 +528,7 @@ extern mapmodelinfo &getmminfo(int i);
 extern void startmodelquery(occludequery *query);
 extern void endmodelquery();
 extern void preloadmodelshaders();
+extern void preloadusedmapmodels(bool msg = false, bool bih = false);
 
 // renderparticles
 extern void particleinit();
@@ -557,15 +568,15 @@ extern bool limitsky();
 
 // 3dgui
 #ifndef NEWGUI
+extern int mainmenu;
+extern void clearmainmenu();
 extern void g3d_render();
 extern bool g3d_windowhit(bool on, bool act);
 
 // menus
-extern int mainmenu;
-
-extern void clearmainmenu();
 extern void g3d_mainmenu();
 #else
+
 //new UI by neal
 namespace UI
 {
@@ -577,6 +588,8 @@ namespace UI
     extern void setup();
     extern void update();
     extern void render();
+    extern void clearmainmenu();
+    extern int mainmenu;
 }
 #endif
 
@@ -597,11 +610,15 @@ extern void rendergrass();
 // blendmap
 extern int blendpaintmode;
 
-extern bool setblendmaporigin(const ivec &o, int size);
-extern bool hasblendmap();
-extern uchar lookupblendmap(const vec &pos);
+struct BlendMapCache;
+extern BlendMapCache *newblendmapcache();
+extern void freeblendmapcache(BlendMapCache *&cache);
+extern bool setblendmaporigin(BlendMapCache *cache, const ivec &o, int size);
+extern bool hasblendmap(BlendMapCache *cache);
+extern uchar lookupblendmap(BlendMapCache *cache, const vec &pos);
 extern void resetblendmap();
 extern void enlargeblendmap();
+extern void shrinkblendmap(int octant);
 extern void optimizeblendmap();
 extern void stoppaintblendmap();
 extern void trypaintblendmap();
