@@ -1,8 +1,9 @@
 // menus.cpp: ingame menu system (also used for scores and serverlist)
 
-#include "engine.h"
 #ifndef NEWGUI
 
+#include "engine.h"
+
 HVARP(guititlecolour, 0, 0xDDFF00, 0xFFFFFF);
 HVARP(guibuttoncolour, 0, 0xFFFF33, 0xFFFFFF);
 HVARP(guitextcolour, 0, 0x00FF64, 0xFFFFFF);
@@ -30,7 +31,8 @@ COMMANDN(guiresetcolours, resetcolours, "");
 
 struct menu : g3d_callback
 {
-    char *name, *header, *contents, *onclear;
+    char *name, *header;
+    uint *contents, *onclear;
 
     menu() : name(NULL), header(NULL), contents(NULL), onclear(NULL) {}
 
@@ -50,9 +52,79 @@ struct menu : g3d_callback
     }
 };
 
+struct delayedupdate
+{
+    enum
+    {
+        INT,
+        FLOAT,
+        STRING,
+        ACTION
+    } type;
+    ident *id;
+    union
+    {
+        int i;
+        float f;
+        char *s;
+    } val;
+    delayedupdate() : type(ACTION), id(NULL) { val.s = NULL; }
+    ~delayedupdate() { if(type == STRING || type == ACTION) DELETEA(val.s); }
+
+    void schedule(const char *s) { type = ACTION; val.s = newstring(s); }
+    void schedule(ident *var, int i) { type = INT; id = var; val.i = i; }
+    void schedule(ident *var, float f) { type = FLOAT; id = var; val.f = f; }
+    void schedule(ident *var, char *s) { type = STRING; id = var; val.s = newstring(s); }
+
+    int getint() const
+    {
+        switch(type)
+        {
+            case INT: return val.i;
+            case FLOAT: return int(val.f);
+            case STRING: return int(strtol(val.s, NULL, 0));
+            default: return 0;
+        }
+    }
+
+    float getfloat() const
+    {
+        switch(type)
+        {
+            case INT: return float(val.i);
+            case FLOAT: return val.f;
+            case STRING: return float(parsefloat(val.s));
+            default: return 0;
+        }
+    }
+
+    const char *getstring() const
+    {
+        switch(type)
+        {
+	    case INT: return intstr(val.i);
+            case FLOAT: return intstr(int(floor(val.f)));
+            case STRING: return val.s;
+            default: return "";
+        }
+    }
+
+    void run()
+    {
+        if(type == ACTION) { if(val.s) execute(val.s); }
+        else if(id) switch(id->type)
+        {
+            case ID_VAR: setvarchecked(id, getint()); break;
+            case ID_FVAR: setfvarchecked(id, getfloat()); break;
+            case ID_SVAR: setsvarchecked(id, getstring()); break;
+            case ID_ALIAS: alias(id->name, getstring()); break;
+        }
+    }
+};
+
 static hashtable<const char *, menu> guis;
 static vector<menu *> guistack;
-static vector<char *> executelater;
+static vector<delayedupdate> updatelater;
 static bool shouldclearmenu = true, clearlater = false;
 
 VARP(menudistance,  16, 40,  256);
@@ -116,10 +188,21 @@ void showgui(const char *name)
     else restoregui(pos);
 }
 
+bool activegui(const char *name)
+{
+	if(!guistack.length()) return false;
+	return !strcmp(guistack.last()->name, name);
+}
+
+int numgui()
+{
+	return guistack.length();
+}
+
 int cleargui(int n)
 {
     int clear = guistack.length();
-    if(mainmenu && !isconnected(true) && clear > 0 && guistack[0]->name && !game::getclientmap()[0] && !strcmp(guistack[0]->name, "main"))
+    if(mainmenu && !isconnected(true) && clear > 0 && guistack[0]->name && !strcmp(guistack[0]->name, "main"))
     {
         clear--;
         if(!clear) return 1;
@@ -135,10 +218,10 @@ void guionclear(char *action)
     if(guistack.empty()) return;
     menu *m = guistack.last();
     DELETEA(m->onclear);
-    if(action[0]) m->onclear = newstring(action);
+    if(action[0]) m->onclear = compilecode(action);
 }
 
-void guistayopen(char *contents)
+void guistayopen(uint *contents)
 {
     bool oldclearmenu = shouldclearmenu;
     shouldclearmenu = false;
@@ -146,7 +229,7 @@ void guistayopen(char *contents)
     shouldclearmenu = oldclearmenu;
 }
 
-void guinoautotab(char *contents)
+void guinoautotab(uint *contents)
 {
     if(!cgui) return;
     cgui->allowautotab(false);
@@ -158,10 +241,11 @@ void guinoautotab(char *contents)
 void guibutton(char *name, char *action, char *icon, int *colour)
 {
     if(!cgui) return;
-    int ret = cgui->button(name,  *colour ? *colour : guibuttoncolour, *icon ? icon : (strstr(action, "showgui") ? "menu" : "action"));
+    bool hideicon = !strcmp(icon, "0");
+    int ret = cgui->button(name, *colour ? *colour : guibuttoncolour, hideicon ? NULL : (icon[0] ? icon : (strstr(action, "showgui") ? "menu" : "action")));
     if(ret&G3D_UP)
     {
-        executelater.add(newstring(*action ? action : name));
+        updatelater.add().schedule(action[0] ? action : name);
         if(shouldclearmenu) clearlater = true;
     }
     else if(ret&G3D_ROLLOVER)
@@ -185,7 +269,7 @@ void guiimage(char *path, char *action, float *scale, int *overlaid, char *alt)
     {
         if(*action)
         {
-            executelater.add(newstring(action));
+            updatelater.add().schedule(action);
             if(shouldclearmenu) clearlater = true;
         }
     }
@@ -206,12 +290,13 @@ void guicolor(int *color)
 }
 void guitextbox(char *text, int *width, int *height, int *color)
 {
-    if(cgui && text[0]) cgui->textbox(text, *width ? *width : 12, *height ? *height : 1, *color ? *color : 0xFFFFFF);
+    if(cgui && text[0]) cgui->textbox(text, *width ? *width : 12, *height ? *height : 1, *color ? *color : guitextcolour);
 }
 
 void guitext(char *name, char *icon, int *colour)
 {
-    if(cgui) cgui->text(name, *colour ? *colour : (icon[0] ? guibuttoncolour : guitextcolour), icon[0] ? icon : "info");
+    bool hideicon = !strcmp(icon, "0");
+    if(cgui) cgui->text(name, *colour ? *colour : (!hideicon && icon[0] ? guibuttoncolour : guitextcolour), hideicon ? NULL : (icon[0] ? icon : "info"));
 }
 
 void guititle(char *name, int *colour)
@@ -229,50 +314,21 @@ void guibar()
     if(cgui) cgui->separator();
 }
 
-static void updateval(char *var, int val, char *onchange)
+void guistrut(float *strut, int *alt)
 {
-    ident *id = getident(var);
-    string assign;
-    if(!id) return;
-    switch(id->type)
-    {
-        case ID_VAR:
-        case ID_FVAR:
-        case ID_SVAR:
-            formatstring(assign)("%s %d", var, val);
-            break;
-        case ID_ALIAS:
-            formatstring(assign)("%s = %d", var, val);
-            break;
-        default:
-            return;
-    }
-    executelater.add(newstring(assign));
-    if(onchange[0]) executelater.add(newstring(onchange));
+	if(cgui)
+	{
+		if(!*alt) cgui->pushlist();
+		cgui->strut(*strut);
+		if(!*alt) cgui->poplist();
+	}
 }
 
-static void updatefval(char *var, float val, char *onchange)
+template<class T> static void updateval(char *var, T val, char *onchange)
 {
-    ident *id = getident(var);
-    string assign;
-    if(!id) return;
-    switch(id->type)
-    {
-        case ID_FVAR:
-            formatstring(assign)("%s %f", var, val);
-            break;
-        case ID_VAR:
-        case ID_SVAR:
-            formatstring(assign)("%s %d", var, int(val));
-            break;
-        case ID_ALIAS:
-            formatstring(assign)("%s = %d", var, int(val));
-            break;
-        default:
-            return;
-    }
-    executelater.add(newstring(assign));
-    if(onchange[0]) executelater.add(newstring(onchange));
+    ident *id = newident(var);
+    updatelater.add().schedule(id, val);
+    if(onchange[0]) updatelater.add().schedule(onchange);
 }
 
 static int getval(char *var)
@@ -283,24 +339,38 @@ static int getval(char *var)
     {
         case ID_VAR: return *id->storage.i;
         case ID_FVAR: return int(*id->storage.f);
-        case ID_SVAR: return atoi(*id->storage.s);
-        case ID_ALIAS: return atoi(id->action);
+        case ID_SVAR: return parseint(*id->storage.s);
+        case ID_ALIAS: return id->getint();
         default: return 0;
     }
 }
 
 static float getfval(char *var)
 {
-	ident *id = getident(var);
-	if(!id) return 0;
-	switch(id->type)
-	{
-		case ID_VAR: return *id->storage.i;
-		case ID_FVAR: return *id->storage.f;
-		case ID_SVAR: return atof(*id->storage.s);
-		case ID_ALIAS: return atof(id->action);
-		default: return 0;
-	}
+    ident *id = getident(var);
+    if(!id) return 0;
+    switch(id->type)
+    {
+        case ID_VAR: return *id->storage.i;
+        case ID_FVAR: return *id->storage.f;
+        case ID_SVAR: return parsefloat(*id->storage.s);
+        case ID_ALIAS: return id->getfloat();
+        default: return 0;
+    }
+}
+
+static const char *getsval(char *var)
+{
+    ident *id = getident(var);
+    if(!id) return "";
+    switch(id->type)
+    {
+        case ID_VAR: return intstr(*id->storage.i);
+        case ID_FVAR: return floatstr(*id->storage.f);
+        case ID_SVAR: return *id->storage.s;
+        case ID_ALIAS: return id->getstr();
+        default: return "";
+    }
 }
 
 void guislider(char *var, int *min, int *max, char *onchange, int *colour, int *reverse)
@@ -318,7 +388,7 @@ void guilistslider(char *var, char *list, char *onchange, int *colour, int *reve
     list += strspn(list, "\n\t ");
     while(*list)
     {
-        vals.add(atoi(list));
+        vals.add(parseint(list));
         list += strcspn(list, "\n\t \0");
         list += strspn(list, "\n\t ");
     }
@@ -337,7 +407,7 @@ void guinameslider(char *var, char *names, char *list, char *onchange, int *colo
     list += strspn(list, "\n\t ");
     while(*list)
     {
-        vals.add(atoi(list));
+        vals.add(parseint(list));
         list += strcspn(list, "\n\t \0");
         list += strspn(list, "\n\t ");
     }
@@ -355,7 +425,7 @@ void guicheckbox(char *name, char *var, float *on, float *off, char *onchange, i
     bool enabled = getfval(var)!=*off;
     if(cgui && cgui->button(name, *colour ? *colour : guicheckboxcolour, enabled ? "checkbox_on" : "checkbox_off")&G3D_UP)
     {
-        updatefval(var, enabled ? *off : (*on || *off ? *on : 1), onchange);
+        updateval(var, enabled ? *off : (*on || *off ? *on : 1.0f), onchange);
     }
 }
 
@@ -364,7 +434,7 @@ void guiradio(char *name, char *var, float *n, char *onchange, int *colour)
     bool enabled = getfval(var)==*n;
     if(cgui && cgui->button(name, *colour ? *colour : guicheckboxcolour, enabled ? "radio_on" : "radio_off")&G3D_UP)
     {
-        if(!enabled) updatefval(var, *n, onchange);
+        if(!enabled) updateval(var, *n, onchange);
     }
 }
 
@@ -382,15 +452,9 @@ void guibitfield(char *name, char *var, int *mask, char *onchange, int *colour)
 void guifield(char *var, int *maxlength, char *onchange, int *colour)
 {
     if(!cgui) return;
-    const char *initval = "";
-    ident *id = getident(var);
-    if(id && id->type==ID_ALIAS) initval = id->action;
-	char *result = cgui->field(var, *colour ? *colour : guibuttoncolour, *maxlength ? *maxlength : 12, 0, initval);
-    if(result)
-    {
-        alias(var, result);
-        if(onchange[0]) executelater.add(newstring(onchange));
-    }
+    const char *initval = getsval(var);
+    char *result = cgui->field(var, *colour ? *colour : guibuttoncolour, *maxlength ? *maxlength : 12, 0, initval);
+    if(result) updateval(var, result, onchange);
 }
 
 //-ve maxlength indicates a wrapped text field of any (approx 260 chars) length, |maxlength| is the field width
@@ -405,19 +469,13 @@ void guieditor(char *name, int *maxlength, int *height, int *mode, int *colour)
 void guikeyfield(char *var, int *maxlength, char *onchange)
 {
     if(!cgui) return;
-    const char *initval = "";
-    ident *id = getident(var);
-    if(id && id->type==ID_ALIAS) initval = id->action;
+    const char *initval = getsval(var);
     char *result = cgui->keyfield(var, guibuttoncolour, *maxlength ? *maxlength : -8, 0, initval);
-    if(result)
-    {
-        alias(var, result);
-        if(onchange[0]) execute(onchange);
-    }
+    if(result) updateval(var, result, onchange);
 }
 
 
-void guilist(char *contents)
+void guilist(uint *contents)
 {
     if(!cgui) return;
     cgui->pushlist();
@@ -425,6 +483,14 @@ void guilist(char *contents)
     cgui->poplist();
 }
 
+void guialign(int *align, uint *contents)
+{
+    if(!cgui) return;
+    cgui->pushlist(clamp(*align, -1, 1));
+    execute(contents);
+    cgui->poplist();
+}
+
 void newgui(char *name, char *contents, char *header)
 {
     menu *m = guis.access(name);
@@ -440,7 +506,7 @@ void newgui(char *name, char *contents, char *header)
         DELETEA(m->contents);
     }
     m->header = header && header[0] ? newstring(header) : NULL;
-    m->contents = newstring(contents);
+    m->contents = compilecode(contents);
 }
 
 void guiservers()
@@ -451,7 +517,7 @@ void guiservers()
         char *command = showservers(cgui);
         if(command)
         {
-            executelater.add(command);
+            updatelater.add().schedule(command);
             if(shouldclearmenu) clearlater = true;
         }
     }
@@ -464,12 +530,14 @@ COMMAND(guiservers, "s");
 ICOMMAND(cleargui, "i", (int *n), intret(cleargui(*n)));
 COMMAND(showgui, "s");
 COMMAND(guionclear, "s");
-COMMAND(guistayopen, "s");
-COMMAND(guinoautotab, "s");
+COMMAND(guistayopen, "e");
+COMMAND(guinoautotab, "e");
 
-COMMAND(guilist, "s");
+COMMAND(guilist, "e");
+COMMAND(guialign, "ie");
 COMMAND(guititle, "si");
 COMMAND(guibar,"");
+COMMAND(guistrut,"fi");
 COMMAND(guiimage,"ssfis");
 COMMAND(guislider,"siisii");
 COMMAND(guilistslider, "sssii");
@@ -494,11 +562,6 @@ struct change
 };
 static vector<change> needsapply;
 
-SVAR(MENU1, "");
-SVAR(MENU2, "");
-SVAR(MENU3, "");
-SVAR(MENU4, "");
-
 static struct applymenu : menu
 {
     void gui(g3d_gui &g, bool firstpass)
@@ -513,8 +576,8 @@ static struct applymenu : menu
         {
             int changetypes = 0;
             loopv(needsapply) changetypes |= needsapply[i].type;
-            if(changetypes&CHANGE_GFX) executelater.add(newstring("resetgl"));
-            if(changetypes&CHANGE_SOUND) executelater.add(newstring("resetsound"));
+            if(changetypes&CHANGE_GFX) updatelater.add().schedule("resetgl");
+            if(changetypes&CHANGE_SOUND) updatelater.add().schedule("resetsound");
             clearlater = true;
         }
         if(g.button("no", guibuttoncolour, "action")&G3D_UP)
@@ -525,7 +588,7 @@ static struct applymenu : menu
     void clear()
     {
         menu::clear();
-        needsapply.setsize(0);
+        needsapply.shrink(0);
     }
 } applymenu;
 
@@ -559,8 +622,8 @@ void menuprocess()
 {
     processingmenu = true;
     int wasmain = mainmenu, level = guistack.length();
-    loopv(executelater) execute(executelater[i]);
-    executelater.deletecontentsa();
+    loopv(updatelater) updatelater[i].run();
+    updatelater.shrink(0);
     if(wasmain > mainmenu || clearlater)
     {
         if(wasmain > mainmenu || level==guistack.length())
@@ -570,7 +633,7 @@ void menuprocess()
                 menu *m = guistack[i];
                 if(m->onclear)
                 {
-                    char *action = m->onclear;
+                    uint *action = m->onclear;
                     m->onclear = NULL;
                     execute(action);
                     delete[] action;
@@ -580,7 +643,7 @@ void menuprocess()
         }
         clearlater = false;
     }
-    if(mainmenu && !isconnected(true) && !game::getclientmap()[0] && guistack.empty()) showgui("main");
+    if(mainmenu && !isconnected(true) && guistack.empty()) showgui("main");
     processingmenu = false;
 }
 
@@ -588,7 +651,7 @@ VAR(mainmenu, 1, 1, 0);
 
 void clearmainmenu()
 {
-    if(mainmenu && (game::getclientmap()[0] || isconnected() || haslocalclients()))
+    if(mainmenu && (isconnected() || haslocalclients()))
     {
         mainmenu = 0;
         if(!processingmenu) cleargui();
