@@ -197,7 +197,7 @@ struct aviwriter
         f = openfile(seqfilename, "wb");
         if(!f) return false;
 
-        index.setsize(0);
+        index.shrink(0);
         chunkdepth = -1;
 
         listchunk("RIFF", "AVI ");
@@ -601,6 +601,7 @@ namespace recorder
     static enum { REC_OK = 0, REC_USERHALT, REC_TOOSLOW, REC_FILERROR } state = REC_OK;
 
     static aviwriter *file = NULL;
+    ICOMMAND(movierecording, "", (), intret(file ? 1 : 0));
     static int starttime = 0;
 
     static int stats[1000];
@@ -826,11 +827,11 @@ namespace recorder
 
     void drawquad(float tw, float th, float x, float y, float w, float h)
     {
-        glBegin(GL_QUADS);
+        glBegin(GL_TRIANGLE_STRIP);
         glTexCoord2f(0,  0);  glVertex2f(x,   y);
         glTexCoord2f(tw, 0);  glVertex2f(x+w, y);
-        glTexCoord2f(tw, th); glVertex2f(x+w, y+h);
         glTexCoord2f(0,  th); glVertex2f(x,   y+h);
+        glTexCoord2f(tw, th); glVertex2f(x+w, y+h);
         glEnd();
     }
 
