@@ -33,7 +33,7 @@ cube *newcubes(uint face)
         setfaces(*c, face);
         loopl(6)
         {
-            c->texture[l] = 2+l;
+            c->texture[l] = DEFAULT_GEOM;
         }
         c++;
     }
@@ -61,7 +61,7 @@ void freecubeext(cube &c)
     DELETEP(c.ext);
 }
 
-void discardchildren(cube &c)
+void discardchildren(cube &c, bool fixtex, int depth)
 {
     if(c.ext)
     {
@@ -78,8 +78,19 @@ void discardchildren(cube &c)
     }
     if(c.children)
     {
-        freeocta(c.children);
-        c.children = NULL;
+        uint filled = F_EMPTY;
+        loopi(8) 
+        {
+            discardchildren(c.children[i], fixtex, depth+1);
+            filled |= c.children[i].faces[0];
+        }
+        if(fixtex) 
+        {
+            loopi(6) c.texture[i] = getmippedtexture(c, i);
+            if(depth > 0 && filled != F_EMPTY) c.faces[0] = F_SOLID;
+        }
+        DELETEA(c.children);
+        allocnodes--;
     }
 }
 
@@ -91,7 +102,7 @@ void getcubevector(cube &c, int d, int x, int y, int z, ivec &p)
         p[i] = edgeget(cubeedge(c, i, v[R[i]], v[C[i]]), v[D[i]]);
 }
 
-void setcubevector(cube &c, int d, int x, int y, int z, ivec &p)
+void setcubevector(cube &c, int d, int x, int y, int z, const ivec &p)
 {
     ivec v(d, x, y, z);
 
@@ -99,6 +110,20 @@ void setcubevector(cube &c, int d, int x, int y, int z, ivec &p)
         edgeset(cubeedge(c, i, v[R[i]], v[C[i]]), v[D[i]], p[i]);
 }
 
+static inline void getcubevector(cube &c, int i, ivec &p)
+{
+    p.x = edgeget(cubeedge(c, 0, (i>>R[0])&1, (i>>C[0])&1), (i>>D[0])&1);
+    p.y = edgeget(cubeedge(c, 1, (i>>R[1])&1, (i>>C[1])&1), (i>>D[1])&1);
+    p.z = edgeget(cubeedge(c, 2, (i>>R[2])&1, (i>>C[2])&1), (i>>D[2])&1);
+}
+
+static inline void setcubevector(cube &c, int i, const ivec &p)
+{
+    edgeset(cubeedge(c, 0, (i>>R[0])&1, (i>>C[0])&1), (i>>D[0])&1, p.x);
+    edgeset(cubeedge(c, 1, (i>>R[1])&1, (i>>C[1])&1), (i>>D[1])&1, p.y);
+    edgeset(cubeedge(c, 2, (i>>R[2])&1, (i>>C[2])&1), (i>>D[2])&1, p.z);
+}
+
 void optiface(uchar *p, cube &c)
 {
     loopi(4) if(edgeget(p[i], 0)!=edgeget(p[i], 1)) return;
@@ -107,7 +132,7 @@ void optiface(uchar *p, cube &c)
 
 void printcube()
 {
-    cube &c = lookupcube(lu.x, lu.y, lu.z, 0); // assume this is cube being pointed at
+    cube &c = lookupcube(lu.x, lu.y, lu.z); // assume this is cube being pointed at
     conoutf(CON_DEBUG, "= %p = (%d, %d, %d) @ %d", &c, lu.x, lu.y, lu.z, lusize);
     conoutf(CON_DEBUG, " x  %.8x", c.faces[0]);
     conoutf(CON_DEBUG, " y  %.8x", c.faces[1]);
@@ -122,9 +147,9 @@ bool isvalidcube(cube &c)
     genclipplanes(c, 0, 0, 0, 256, p);
     loopi(8) // test that cube is convex
     {
-        vvec v;
+        vec v;
         calcvert(c, 0, 0, 0, 256, v, i);
-        if(!pointincube(p, v.tovec()))
+        if(!pointincube(p, v))
             return false;
     }
     return true;
@@ -136,10 +161,10 @@ void validatec(cube *c, int size)
     {
         if(c[i].children)
         {
-            if(size<=(8>>VVEC_FRAC))
+            if(size<=1)
             {
                 solidfaces(c[i]);
-                discardchildren(c[i]);
+                discardchildren(c[i], true);
             }
             else validatec(c[i].children, size>>1);
         }
@@ -156,49 +181,33 @@ void validatec(cube *c, int size)
 
 ivec lu;
 int lusize;
-cube &lookupcube(int tx, int ty, int tz, int tsize)
+cube &lookupcube(int tx, int ty, int tz, int tsize, ivec &ro, int &rsize)
 {
-    int size = worldsize;
-    int x = 0, y = 0, z = 0;
-    cube *c = worldroot;
-    for(;;)
+    tx = clamp(tx, 0, worldsize-1);
+    ty = clamp(ty, 0, worldsize-1);
+    tz = clamp(tz, 0, worldsize-1);
+    int scale = worldscale-1, csize = abs(tsize);
+    cube *c = &worldroot[octastep(tx, ty, tz, scale)];
+    if(!(csize>>scale)) do
     {
-        size >>= 1;
-        ASSERT(size);
-        if(tz>=z+size) { z += size; c += 4; }
-        if(ty>=y+size) { y += size; c += 2; }
-        if(tx>=x+size) { x += size; c += 1; }
-        //if(tsize==size) break;
-        if(abs(tsize)>=size) break;
-        if(c->children==NULL)
+        if(!c->children)
         {
-            //if(!tsize) break;
-            if(tsize<=0) break;
-            subdividecube(*c);
+            if(tsize > 0) do
+            {
+                subdividecube(*c);
+                scale--;
+                c = &c->children[octastep(tx, ty, tz, scale)];
+            } while(!(csize>>scale));
+            break;
         }
-        c = c->children;
-    }
-    lu.x = x;
-    lu.y = y;
-    lu.z = z;
-    lusize = size;
+        scale--;
+        c = &c->children[octastep(tx, ty, tz, scale)];
+    } while(!(csize>>scale));
+    ro = ivec(tx, ty, tz).mask(~0<<scale);
+    rsize = 1<<scale;
     return *c;
 }
 
-cube &neighbourcube(int x, int y, int z, int size, int rsize, int orient)
-{
-    switch(orient)
-    {
-        case O_BOTTOM: z -= size; break;
-        case O_TOP:    z += size; break;
-        case O_BACK:   y -= size; break;
-        case O_FRONT:  y += size; break;
-        case O_LEFT:   x -= size; break;
-        case O_RIGHT:  x += size; break;
-    }
-    return lookupcube(x, y, z, rsize);
-}
-
 int lookupmaterial(const vec &v)
 {
     ivec o(v);
@@ -213,6 +222,38 @@ int lookupmaterial(const vec &v)
     return c->ext ? c->ext->material : MAT_AIR;
 }
 
+cube *neighbourstack[32];
+int neighbourdepth = -1;
+
+cube &neighbourcube(cube &c, int orient, int x, int y, int z, int size, ivec &ro, int &rsize)
+{
+    ivec n(x, y, z);
+    int dim = dimension(orient);
+    uint diff = n[dim];
+    if(dimcoord(orient)) n[dim] += size; else n[dim] -= size;
+    diff ^= n[dim];
+    if(diff >= uint(worldsize)) { ro = n; rsize = size; return c; }
+    int scale = worldscale;
+    cube *nc = worldroot;
+    if(neighbourdepth >= 0)
+    {
+        scale -= neighbourdepth + 1;
+        diff >>= scale;
+        do { scale++; diff >>= 1; } while(diff);
+        nc = neighbourstack[worldscale - scale];
+    }
+    scale--;
+    nc = &nc[octastep(n.x, n.y, n.z, scale)];
+    if(!(size>>scale) && nc->children) do
+    {
+        scale--;
+        nc = &nc->children[octastep(n.x, n.y, n.z, scale)];
+    } while(!(size>>scale) && nc->children);
+    ro = n.mask(~0<<scale);
+    rsize = 1<<scale;
+    return *nc;
+}
+
 ////////// (re)mip //////////
 
 int getmippedtexture(cube &p, int orient)
@@ -220,30 +261,24 @@ int getmippedtexture(cube &p, int orient)
     cube *c = p.children;
     int d = dimension(orient);
     int dc = dimcoord(orient);
-    int tex[4] = {-1,-1,-1,-1};
-    loop(x,2) loop(y,2)
+    int tex[4] = { -1, -1, -1, -1 };
+    loop(x, 2) loop(y, 2)
     {
         int n = octaindex(d, x, y, dc);
         if(isempty(c[n]))
+        {
             n = oppositeocta(d, n);
-        if(isempty(c[n]))
-            continue;
-
-        loopk(3)
-            if(tex[k] == c[n].texture[orient])
-                return tex[k];
-
-        if(c[n].texture[orient] > 0) // assume 0 is sky. favour non-sky tex
-            tex[x*2+y] = c[n].texture[orient];
+            if(isempty(c[n]))
+                continue;
+        }
+        loopk(3) if(tex[k] == c[n].texture[orient]) return tex[k];
+        if(c[n].texture[orient] > DEFAULT_SKY) tex[x*2+y] = c[n].texture[orient];
     }
-
-    loopk(4)
-        if(tex[k]>0) return tex[k];
-
-    return p.texture[orient];
+    loopk(4) if(tex[k] > DEFAULT_SKY) return tex[k];
+    return DEFAULT_GEOM;
 }
 
-void forcemip(cube &c)
+void forcemip(cube &c, bool fixtex)
 {
     cube *ch = c.children;
     emptyfaces(c);
@@ -253,22 +288,15 @@ void forcemip(cube &c)
         int n = i^(j==3 ? 4 : (j==4 ? 3 : j));
         if(!isempty(ch[n])) // breadth first search for cube near vert
         {
-            ivec v, p(i);
-            getcubevector(ch[n], 2, p.x, p.y, p.z, v);
-
-            loopk(3) // adjust vert to parent size
-            {
-                if(octacoord(k, n) == 1)
-                    v[k] += 8;
-                v[k] >>= 1;
-            }
-
-            setcubevector(c, 2, p.x, p.y, p.z, v);
+            ivec v;
+            getcubevector(ch[n], i, v);
+            // adjust vert to parent size
+            setcubevector(c, i, ivec(n, v.x, v.y, v.z, 8).shr(1));
             break;
         }
     }
 
-    loopj(6)
+    if(fixtex) loopj(6)
         c.texture[j] = getmippedtexture(c, j);
 }
 
@@ -295,6 +323,7 @@ int midedge(const ivec &a, const ivec &b, int xd, int yd, bool &perfect)
 bool subdividecube(cube &c, bool fullcheck, bool brighten)
 {
     if(c.children) return true;
+    if(c.ext && c.ext->surfaces) freesurfaces(c);
 	if(isempty(c) || isentirelysolid(c))
     {
         int mat = c.ext ? c.ext->material : MAT_AIR;
@@ -313,8 +342,7 @@ bool subdividecube(cube &c, bool fullcheck, bool brighten)
     ivec v[8];
     loopi(8)
     {
-        ivec p(i);
-        getcubevector(c, 2, p.x, p.y, p.z, v[i]);
+        getcubevector(c, i, v[i]);
         v[i].mul(2);
         if(mat!=MAT_AIR) ext(ch[i]).material = mat;
     }
@@ -352,11 +380,8 @@ bool subdividecube(cube &c, bool fullcheck, bool brighten)
             ch[i].texture[j] = c.texture[j];
             loop(y, 2) loop(x, 2) // assign child edges
             {
-                int ce = e[x+o[R[d]]][y+o[C[d]]];
-                if(o[D[d]]) ce -= 8;
-                ce = min(max(ce, 0), 8);
-                uchar &f = cubeedge(ch[i], d, x, y);
-                edgeset(f, z, ce);
+                int ce = clamp(e[x+((i>>R[d])&1)][y+((i>>C[d])&1)] - ((i>>D[d])&1)*8, 0, 8);
+                edgeset(cubeedge(ch[i], d, x, y), z, ce);
             }
         }
     }
@@ -401,7 +426,7 @@ bool remip(cube &c, int x, int y, int z, int size)
     cube *ch = c.children;
     if(!ch)
     {
-        if(size<<1 <= VVEC_INT_MASK+1) return true;
+        if(size<<1 <= 0x1000) return true;
         subdividecube(c);
         ch = c.children;
     }
@@ -419,13 +444,13 @@ bool remip(cube &c, int x, int y, int z, int size)
         c.texture[j] = getmippedtexture(c, j); // parents get child texs regardless
 
     if(!perfect) return false;
-    if(size<<1 > VVEC_INT_MASK+1) return false;
+    if(size<<1 > 0x1000) return false;
 
     uchar mat = MAT_AIR;
     loopi(8)
     {
         mat = ch[i].ext ? ch[i].ext->material : MAT_AIR;
-        if((mat&MATF_CLIP) == MAT_NOCLIP)
+        if((mat&MATF_CLIP) == MAT_NOCLIP || mat&MAT_ALPHA)
         {
             if(i > 0) return false;
             while(++i < 8) if(!ch[i].ext || ch[i].ext->material != mat) return false;
@@ -436,7 +461,7 @@ bool remip(cube &c, int x, int y, int z, int size)
             while(++i < 8)
             {
                 uchar omat = ch[i].ext ? ch[i].ext->material : MAT_AIR;
-                if(isentirelysolid(ch[i]) ? (omat&MATF_CLIP) == MAT_NOCLIP : mat != omat) return false;
+                if(isentirelysolid(ch[i]) ? (omat&MATF_CLIP) == MAT_NOCLIP || omat&MAT_ALPHA : mat != omat) return false;
             }
             break;
         }
@@ -461,7 +486,7 @@ bool remip(cube &c, int x, int y, int z, int size)
 
         ivec o(i, x, y, z, size);
         loop(orient, 6)
-            if(visibleface(ch[i], orient, o.x, o.y, o.z, size))
+            if(visibleface(ch[i], orient, o.x, o.y, o.z, size, MAT_AIR, (mat&MAT_ALPHA)^MAT_ALPHA, MAT_ALPHA))
             {
                 if(ch[i].texture[orient] != n.texture[orient]) { freeocta(nh); return false; }
                 vis[orient] |= 1<<i;
@@ -505,14 +530,9 @@ void remip_()
 
 COMMANDN(remip, remip_, "");
 
-uchar &edgelookup(cube &c, const ivec &p, int dim)
+static inline int edgeval(cube &c, const ivec &p, int dim, int coord)
 {
-   return c.edges[dim*4 +(p[C[dim]]>>3)*2 +(p[R[dim]]>>3)];
-}
-
-int edgeval(cube &c, const ivec &p, int dim, int coord)
-{
-    return edgeget(edgelookup(c,p,dim), coord);
+    return edgeget(cubeedge(c, dim, p[R[dim]]>>3, p[C[dim]]>>3), coord);
 }
 
 void genvertp(cube &c, ivec &p1, ivec &p2, ivec &p3, plane &pl)
@@ -620,7 +640,7 @@ const ivec cubecoords[8] = // verts of bounding cube
     ivec(8, 0, 0),
 };
 
-const ushort fv[6][4] = // indexes for cubecoords, per each vert of a face orientation
+const uchar fv[6][4] = // indexes for cubecoords, per each vert of a face orientation
 {
     { 2, 1, 6, 5 },
     { 3, 4, 7, 0 },
@@ -743,7 +763,7 @@ struct facevec
 static inline void genfacevecs(cube &c, int orient, const ivec &pos, int size, bool solid, facevec *fvecs)
 {
     int dim = dimension(orient), coord = dimcoord(orient);
-    const ushort *fvo = fv[orient];
+    const uchar *fvo = fv[orient];
     loopi(4)
     {
         const ivec &cc = cubecoords[fvo[i]];
@@ -759,15 +779,15 @@ static inline void genfacevecs(cube &c, int orient, const ivec &pos, int size, b
             x = edgeval(c, cc, C[dim], cc[C[dim]]);
             y = edgeval(c, cc, R[dim], cc[R[dim]]);
         }
-        f.x = x*size/(8>>VVEC_FRAC)+(pos[C[dim]]<<VVEC_FRAC);
-        f.y = y*size/(8>>VVEC_FRAC)+(pos[R[dim]]<<VVEC_FRAC);
+        f.x = x*size+(pos[C[dim]]<<3);
+        f.y = y*size+(pos[R[dim]]<<3);
     }
 }
 
 static inline int genoppositefacevecs(cube &c, int orient, const ivec &pos, int size, facevec *fvecs)
 {
     int dim = dimension(orient), coord = dimcoord(orient), touching = 0;
-    const ushort *fvo = fv[orient];
+    const uchar *fvo = fv[orient];
     loopi(4)
     {
         const ivec &cc = cubecoords[fvo[coord ? i : 3 - i]];
@@ -776,8 +796,8 @@ static inline int genoppositefacevecs(cube &c, int orient, const ivec &pos, int
             int x = edgeval(c, cc, C[dim], cc[C[dim]]),
                 y = edgeval(c, cc, R[dim], cc[R[dim]]);
             facevec &f = fvecs[touching++];
-            f.x = x*size/(8>>VVEC_FRAC)+(pos[C[dim]]<<VVEC_FRAC);
-            f.y = y*size/(8>>VVEC_FRAC)+(pos[R[dim]]<<VVEC_FRAC);
+            f.x = x*size+(pos[C[dim]]<<3);
+            f.y = y*size+(pos[R[dim]]<<3);
         }
     }
     return touching;
@@ -854,9 +874,9 @@ static inline bool insideface(const facevec *p, int nump, const facevec *o, int
 
 static inline int clipfacevecs(const facevec *o, int cx, int cy, int size, facevec *rvecs)
 {
-    cx <<= VVEC_FRAC;
-    cy <<= VVEC_FRAC;
-    size <<= VVEC_FRAC;
+    cx <<= 3;
+    cy <<= 3;
+    size <<= 3;
 
     int r = 0;
     loopi(4)
@@ -919,14 +939,16 @@ bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat,
     }
     else
     {
+        if(collapsedface(cfe) && flataxisface(c, orient)) return false;
         if(!touchingface(c, orient)) return true;
-        if(collapsedface(cfe)) return false;
     }
 
-    cube &o = neighbourcube(x, y, z, size, -size, orient);
+    ivec no;
+    int nsize;
+    cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
     if(&o==&c) return false;
 
-    if(lusize > size || (lusize == size && !o.children))
+    if(nsize > size || (nsize == size && !o.children))
     {
         if(nmat != MAT_AIR && o.ext && (o.ext->material&matmask) == nmat) return true;
         if(isentirelysolid(o)) return false;
@@ -935,25 +957,27 @@ bool visibleface(cube &c, int orient, int x, int y, int z, int size, uchar mat,
         if(touchingface(o, opposite(orient)) && faceedges(o, opposite(orient)) == F_SOLID) return false;
 
         ivec vo(x, y, z);
-        vo.mask(VVEC_INT_MASK);
-        lu.mask(VVEC_INT_MASK);
+        vo.mask(0xFFF);
+        no.mask(0xFFF);
         facevec cf[4], of[4];
         genfacevecs(c, orient, vo, size, mat != MAT_AIR, cf);
-        int numo = genoppositefacevecs(o, opposite(orient), lu, lusize, of);
+        int numo = genoppositefacevecs(o, opposite(orient), no, nsize, of);
         return numo < 3 || !insideface(cf, 4, of, numo);
     }
 
     ivec vo(x, y, z);
-    vo.mask(VVEC_INT_MASK);
-    lu.mask(VVEC_INT_MASK);
+    vo.mask(0xFFF);
+    no.mask(0xFFF);
     facevec cf[4];
     genfacevecs(c, orient, vo, size, mat != MAT_AIR, cf);
-    return !occludesface(o, opposite(orient), lu, lusize, vo, size, mat, nmat, matmask, cf);
+    return !occludesface(o, opposite(orient), no, nsize, vo, size, mat, nmat, matmask, cf);
 }
 
 // more expensive version that checks both triangles of a face independently
 int visibletris(cube &c, int orient, int x, int y, int z, int size)
 {
+    if(collapsedface(faceedges(c, orient)) && flataxisface(c, orient)) return 0;
+
     int dim = dimension(orient), coord = dimcoord(orient);
     uint face = c.faces[dim];
     if(coord) face = (face&0xF0F0F0F0)^0x80808080;
@@ -966,30 +990,33 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size)
     if(face&0xFF000000) notouch++;
     if(notouch>=2) return 3;
 
-    if(collapsedface(faceedges(c, orient))) return 0;
-
-    cube &o = neighbourcube(x, y, z, size, -size, orient);
+    ivec no;
+    int nsize;
+    cube &o = neighbourcube(c, orient, x, y, z, size, no, nsize);
     if(&o==&c) return 0;
 
+    uchar nmat = c.ext && c.ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, matmask = MAT_ALPHA;
+
     ivec vo(x, y, z);
-    vo.mask(VVEC_INT_MASK);
-    lu.mask(VVEC_INT_MASK);
+    vo.mask(0xFFF);
+    no.mask(0xFFF);
     facevec cf[4], of[4];
     int opp = opposite(orient), numo = 0;
-    if(lusize > size || (lusize == size && !o.children))
+    if(nsize > size || (nsize == size && !o.children))
     {
         if(isempty(o)) return 3;
+        if(nmat != MAT_AIR && o.ext && (o.ext->material&matmask) == nmat) return 3;
         if(!notouch && (isentirelysolid(o) || (touchingface(o, opp) && faceedges(o, opp) == F_SOLID))) return 0;
 
         genfacevecs(c, orient, vo, size, false, cf);
-        numo = genoppositefacevecs(o, opp, lu, lusize, of);
+        numo = genoppositefacevecs(o, opp, no, nsize, of);
         if(numo < 3) return 3;
         if(!notouch && insideface(cf, 4, of, numo)) return 0;
     }
     else
     {
         genfacevecs(c, orient, vo, size, false, cf);
-        if(!notouch && occludesface(o, opp, lu, lusize, vo, size, MAT_AIR, MAT_AIR, MATF_VOLUME, cf)) return 0;
+        if(!notouch && occludesface(o, opp, no, nsize, vo, size, MAT_AIR, nmat, matmask, cf)) return 0;
     }
 
     static const int trimasks[2][2] = { { 0x7, 0xD }, { 0xE, 0xB } };
@@ -1027,7 +1054,7 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size)
             if(!numo)
             {
                 tf[3] = cf[v3];
-                if(!occludesface(o, opp, lu, lusize, vo, size, MAT_AIR, MAT_AIR, MATF_VOLUME, tf)) continue;
+                if(!occludesface(o, opp, no, nsize, vo, size, MAT_AIR, nmat, matmask, tf)) continue;
             }
             else if(!insideface(tf, 3, of, numo)) continue;
             return vis & ~(1<<i);
@@ -1045,61 +1072,42 @@ int visibletris(cube &c, int orient, int x, int y, int z, int size)
     return 3;
 }
 
-void calcvert(cube &c, int x, int y, int z, int size, vvec &v, int i, bool solid)
+void calcvert(cube &c, int x, int y, int z, int size, ivec &v, int i, bool solid)
 {
-    ivec vv;
-    if(solid || isentirelysolid(c)) vv = cubecoords[i];
-    else genvectorvert(cubecoords[i], c, vv);
-    v = vvec(vv.v);
+    if(solid || isentirelysolid(c)) v = cubecoords[i];
+    else genvectorvert(cubecoords[i], c, v);
     // avoid overflow
     if(size>=8) v.mul(size/8);
     else v.div(8/size);
-    v.add(vvec(x, y, z));
+    v.add(ivec(x, y, z).shl(3));
 }
 
 void calcvert(cube &c, int x, int y, int z, int size, vec &v, int i, bool solid)
 {
-    ivec vv;
-    if(solid || isentirelysolid(c)) vv = cubecoords[i];
-    else genvectorvert(cubecoords[i], c, vv);
-    v = vv.tovec().mul(size/8.0f).add(vec(x, y, z));
-}
-
-int calcverts(cube &c, int x, int y, int z, int size, vvec *verts, bool *usefaces)
-{
-    int vertused = 0;
-    loopi(6) if((usefaces[i] = visibleface(c, i, x, y, z, size))) vertused |= fvmasks[1<<i];
-    //loopk(4) vertused |= 1<<faceverts(c,i,k);
-    loopi(8) if(vertused&(1<<i)) calcvert(c, x, y, z, size, verts[i], i);
-    return vertused;
+    ivec iv;
+    if(solid || isentirelysolid(c)) iv = cubecoords[i];
+    else genvectorvert(cubecoords[i], c, iv);
+    v = iv.tovec().mul(size/8.0f).add(vec(x, y, z));
 }
 
 int genclipplane(cube &c, int orient, vec *v, plane *clip)
 {
-    int planes = 0;
-    vec p[4];
-    loopk(4) p[k] = v[faceverts(c,orient,k)];
-
-    if(p[0]==p[2]) return 0;
-    if(p[0]!=p[1] && p[1]!=p[2]) clip[planes++].toplane(p[0], p[1], p[2]);
-    if(p[0]!=p[3] && p[2]!=p[3] && (!planes || faceconvexity(c, orient))) clip[planes++].toplane(p[0], p[2], p[3]);
+    int planes = 0, convex = faceconvexity(c, orient), order = convex < 0 ? 1 : 0;
+    const vec &v0 = v[fv[orient][order]], &v1 = v[fv[orient][order+1]], &v2 = v[fv[orient][order+2]], &v3 = v[fv[orient][(order+3)&3]];
+    if(v0==v2) return 0;
+    if(v0!=v1 && v1!=v2) clip[planes++].toplane(v0, v1, v2);
+    if(v0!=v3 && v2!=v3 && (!planes || convex)) clip[planes++].toplane(v0, v2, v3);
     return planes;
 }
 
 void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
 {
-    bool usefaces[6];
-    vvec sv[8];
+    int usefaces[6];
     vec mx(x, y, z), mn(x+size, y+size, z+size);
-    int vertused = calcverts(c, x, y, z, size, sv, usefaces);
-
+    loopi(6) usefaces[i] = visibletris(c, i, x, y, z, size);
     loopi(8)
     {
-        if(!(vertused&(1<<i))) // need all verts for proper box
-            calcvert(c, x, y, z, size, sv[i], i);
-
-        p.v[i] = sv[i].tovec(x, y, z);
-
+        calcvert(c, x, y, z, size, p.v[i], i);
         loopj(3) // generate tight bounding box
         {
             mn[j] = min(mn[j], p.v[i].v[j]);
@@ -1118,7 +1126,18 @@ void genclipplanes(cube &c, int x, int y, int z, int size, clipplanes &p)
     loopi(6) if(usefaces[i] && !touchingface(c, i)) // generate actual clipping planes
     {
         if(flataxisface(c, i)) p.visible |= 1<<i;
-        else p.size += genclipplane(c, i, p.v, &p.p[p.size]);
+        else
+        {
+            int convex = faceconvexity(c, i), order = convex < 0 ? 1 : 0;
+            const vec &v0 = p.v[fv[i][order]], &v1 = p.v[fv[i][order+1]], &v2 = p.v[fv[i][order+2]], &v3 = p.v[fv[i][(order+3)&3]];
+            if(v0==v2) continue;
+            if(usefaces[i]&1 && v0!=v1 && v1!=v2)
+            {
+                p.side[p.size] = i; p.p[p.size++].toplane(v0, v1, v2);
+                if(usefaces[i]&2 && v0!=v3 && v2!=v3 && convex) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v2, v3); }
+            }
+            else if(usefaces[i]&2 && v0!=v3 && v2!=v3) { p.side[p.size] = i; p.p[p.size++].toplane(v0, v2, v3); }
+        }
     }
 }
 
@@ -1187,7 +1206,7 @@ int mergefaces(int orient, cubeface *m, int sz)
 
 struct cfkey
 {
-    uchar orient;
+    uchar orient, material;
     ushort tex;
     ivec n;
     int offset;
@@ -1195,12 +1214,12 @@ struct cfkey
 
 static inline bool htcmp(const cfkey &x, const cfkey &y)
 {
-    return x.orient == y.orient && x.tex == y.tex && x.n == y.n && x.offset == y.offset;
+    return x.orient == y.orient && x.tex == y.tex && x.n == y.n && x.offset == y.offset && x.material==y.material;
 }
 
 static inline uint hthash(const cfkey &k)
 {
-    return hthash(k.n)^k.offset^k.tex^k.orient;
+    return hthash(k.n)^k.offset^k.tex^k.orient^k.material;
 }
 
 struct cfval
@@ -1210,7 +1229,7 @@ struct cfval
 
 static hashtable<cfkey, cfval> cfaces;
 
-void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo &orig, mergeinfo &cf)
+void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo &orig, mergeinfo &cf, uchar nmat, uchar matmask)
 {
     int dim = dimension(orient);
     if(cu.children)
@@ -1218,23 +1237,22 @@ void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo
         size >>= 1;
         int coord = dimcoord(orient);
         loopi(8) if(octacoord(dim, i) == coord)
-            mincubeface(cu.children[i], orient, ivec(i, o.x, o.y, o.z, size), size, orig, cf);
+            mincubeface(cu.children[i], orient, ivec(i, o.x, o.y, o.z, size), size, orig, cf, nmat, matmask);
         return;
     }
     int c = C[dim], r = R[dim];
-    ushort uco = ushort((o[c]&VVEC_INT_MASK)<<VVEC_FRAC), vco = ushort((o[r]&VVEC_INT_MASK)<<VVEC_FRAC);
-    ushort uc1 = uco, vc1 = vco, uc2 = ushort(size<<VVEC_FRAC)+uco, vc2 = ushort(size<<VVEC_FRAC)+vco;
+    ushort uco = (o[c]&0xFFF)<<3, vco = (o[r]&0xFFF)<<3;
+    ushort uc1 = uco, vc1 = vco, uc2 = ushort(size<<3)+uco, vc2 = ushort(size<<3)+vco;
     uc1 = max(uc1, orig.u1);
     uc2 = min(uc2, orig.u2);
     vc1 = max(vc1, orig.v1);
     vc2 = min(vc2, orig.v2);
-    if(!isempty(cu) && touchingface(cu, orient))
+    if(!isempty(cu) && touchingface(cu, orient) && !(nmat!=MAT_AIR && cu.ext && (cu.ext->material&matmask)==nmat))
     {
         uchar r1 = cu.edges[faceedgesrcidx[orient][0]], r2 = cu.edges[faceedgesrcidx[orient][1]],
               c1 = cu.edges[faceedgesrcidx[orient][2]], c2 = cu.edges[faceedgesrcidx[orient][3]];
-        ushort scale = ushort(size/(8>>VVEC_FRAC)),
-              u1 = max(c1&0xF, c2&0xF)*scale+uco, u2 = min(c1>>4, c2>>4)*scale+uco,
-              v1 = max(r1&0xF, r2&0xF)*scale+vco, v2 = min(r1>>4, r2>>4)*scale+vco;
+        ushort u1 = max(c1&0xF, c2&0xF)*size+uco, u2 = min(c1>>4, c2>>4)*size+uco,
+               v1 = max(r1&0xF, r2&0xF)*size+vco, v2 = min(r1>>4, r2>>4)*size+vco;
         u1 = max(u1, orig.u1);
         u2 = min(u2, orig.u2);
         v1 = max(v1, orig.v1);
@@ -1260,13 +1278,15 @@ void mincubeface(cube &cu, int orient, const ivec &o, int size, const mergeinfo
 
 bool mincubeface(cube &cu, int orient, const ivec &co, int size, mergeinfo &orig)
 {
-    cube &nc = neighbourcube(co.x, co.y, co.z, size, -size, orient);
+    ivec no;
+    int nsize;
+    cube &nc = neighbourcube(cu, orient, co.x, co.y, co.z, size, no, nsize);
     mergeinfo mincf;
     mincf.u1 = orig.u2;
     mincf.u2 = orig.u1;
     mincf.v1 = orig.v2;
     mincf.v2 = orig.v1;
-    mincubeface(nc, opposite(orient), lu, lusize, orig, mincf);
+    mincubeface(nc, opposite(orient), no, nsize, orig, mincf, cu.ext && cu.ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, MAT_ALPHA);
     bool smaller = false;
     if(mincf.u1 > orig.u1) { orig.u1 = mincf.u1; smaller = true; }
     if(mincf.u2 < orig.u2) { orig.u2 = mincf.u2; smaller = true; }
@@ -1289,15 +1309,14 @@ bool gencubeface(cube &cu, int orient, const ivec &co, int size, ivec &n, int &o
     ivec v[4];
     loopi(4) genvectorvert(cubecoords[fv[orient][i]], cu, v[i]);
 
-    int scale = size/(8>>VVEC_FRAC);
-    v[3].mul(scale);
+    v[3].mul(size);
     int dim = dimension(orient), c = C[dim], r = R[dim];
     cf.u1 = cf.u2 = ushort(v[3][c]);
     cf.v1 = cf.v2 = ushort(v[3][r]);
 
     loopi(3)
     {
-        ushort uc = ushort(v[i][c]*scale), vc = ushort(v[i][r]*scale);
+        ushort uc = ushort(v[i][c]*size), vc = ushort(v[i][r]*size);
         cf.u1 = min(cf.u1, uc);
         cf.u2 = max(cf.u2, uc);
         cf.v1 = min(cf.v1, vc);
@@ -1305,10 +1324,9 @@ bool gencubeface(cube &cu, int orient, const ivec &co, int size, ivec &n, int &o
     }
 
     ivec vo(co);
-    vo.mask(VVEC_INT_MASK);
-    vo.mul(1<<VVEC_FRAC);
+    vo.mask(0xFFF).shl(3);
 
-    ushort uco = ushort(vo[c]), vco = ushort(vo[r]);
+    ushort uco = vo[c], vco = vo[r];
     cf.u1 += uco;
     cf.u2 += uco;
     cf.v1 += vco;
@@ -1380,13 +1398,14 @@ void freemergeinfo(cube &c)
     DELETEA(c.ext->merges);
 }
 
-VARR(maxmerge, 0, 6, VVEC_INT-1);
+VARR(maxmerge, 0, 6, 12);
 
 static int genmergeprogress = 0;
 
 void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size = worldsize>>1)
 {
     if((genmergeprogress++&0xFFF)==0) renderprogress(float(genmergeprogress)/allocnodes, "merging surfaces...");
+    neighbourstack[++neighbourdepth] = c;
     loopi(8)
     {
         ivec co(i, o.x, o.y, o.z, size);
@@ -1396,7 +1415,7 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
             c[i].ext->merged = 0;
         }
         if(c[i].children) genmergeinfo(c[i].children, co, size>>1);
-        else if(!isempty(c[i])) loopj(6) if(visibleface(c[i], j, co.x, co.y, co.z, size))
+        else if(!isempty(c[i])) loopj(6) if(visibleface(c[i], j, co.x, co.y, co.z, size, MAT_AIR, c[i].ext && c[i].ext->material&MAT_ALPHA ? MAT_AIR : MAT_ALPHA, MAT_ALPHA))
         {
             cfkey k;
             cubeface cf;
@@ -1409,6 +1428,7 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
                 }
                 k.orient = j;
                 k.tex = c[i].texture[j];
+                k.material = c[i].ext ? c[i].ext->material&MAT_ALPHA : 0;
                 cfaces[k].faces.add(cf);
             }
         }
@@ -1416,7 +1436,7 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
         {
             ASSERT(size <= 1<<maxmerge);
             enumeratekt(cfaces, cfkey, key, cfval, val,
-                val.faces.setsize(mergefaces(key.orient, val.faces.getbuf(), val.faces.length()));
+                val.faces.shrink(mergefaces(key.orient, val.faces.getbuf(), val.faces.length()));
                 loopvj(val.faces) if(val.faces[j].c->ext && val.faces[j].c->ext->merged&(1<<key.orient))
                 {
                     addmergeinfo(*val.faces[j].c, key.orient, val.faces[j]);
@@ -1426,64 +1446,55 @@ void genmergeinfo(cube *c = worldroot, const ivec &o = ivec(0, 0, 0), int size =
         }
 
     }
+    --neighbourdepth;
 }
 
-void genmergedverts(cube &cu, int orient, const ivec &co, int size, const mergeinfo &m, vvec *vv, plane *p)
+void genmergedverts(cube &cu, int orient, const ivec &co, int size, const mergeinfo &m, vec *vv, plane *p)
 {
-    ivec vo(co);
-    vo.mask(VVEC_INT_MASK);
-    vo.mul(1<<VVEC_FRAC);
-
-    ivec v[3], n;
-    loopi(3) genvectorvert(cubecoords[faceverts(cu, orient, i)], cu, v[i]);
-    v[1].sub(v[0]);
-    v[2].sub(v[0]);
-    n.cross(v[1], v[2]);
-
-    ASSERT(size >= 8>>VVEC_FRAC);
-    v[0].mul(size/(8>>VVEC_FRAC));
-    v[0].add(vo);
-    int offset = -n.dot(v[0]);
+    ivec e[3], n;
+    loopi(3) genvectorvert(cubecoords[faceverts(cu, orient, i)], cu, e[i]);
+    n.cross(e[1].sub(e[0]), e[2].sub(e[0]));
+    int offset = -n.dot(e[0].mul(size).add(ivec(co).mask(0xFFF).shl(3)));
 
     int dim = dimension(orient), c = C[dim], r = R[dim];
+    vec vo = ivec(co).mask(~0xFFF).tovec();
     loopi(4)
     {
         const ivec &coords = cubecoords[fv[orient][i]];
         int cc = coords[c] ? m.u2 : m.u1,
             rc = coords[r] ? m.v2 : m.v1,
             dc = -(offset + n[c]*cc + n[r]*rc)/n[dim];
-        vv[i][c] = ushort(cc);
-        vv[i][r] = ushort(rc);
-        vv[i][dim] = ushort(dc);
+        vec &v = vv[i];
+        v = vo;
+        v[c] += cc/8.0f;
+        v[r] += rc/8.0f;
+        v[dim] += dc/8.0f;
     }
 
     if(p)
     {
-        ivec po(co);
-        po.mask(~VVEC_INT_MASK);
-        vec pn(n.tovec());
-        float mag = pn.magnitude();
-        pn.div(mag);
-        *p = plane(pn, (offset-(n.dot(po)<<VVEC_FRAC))/(mag*(1<<VVEC_FRAC)));
+        vec pn = n.tovec();
+        float scale = 1.0f/pn.magnitude();
+        *p = plane(pn.mul(scale), (offset/8.0f-n.dot(ivec(co).mask(~0xFFF)))*scale);
     }
 }
 
-int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, const vvec *vv)
+int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, const vec *vv)
 {
     int dim = dimension(orient), c = C[dim], r = R[dim];
-    ushort d1 = vv[3][dim], d2 = d1;
+    int origin = (co[dim]&~0xFFF)<<3, d1 = int(vv[3][dim]*8) - origin, d2 = d1;
     loopi(3)
     {
-        d1 = min(d1, vv[i][dim]);
-        d2 = max(d2, vv[i][dim]);
+        d1 = min(d1, int(vv[i][dim]*8) - origin);
+        d2 = max(d2, int(vv[i][dim]*8) - origin);
     }
     int bits = 0;
     while(1<<bits < size) ++bits;
-    bits += VVEC_FRAC;
+    bits += 3;
     ivec mo(co);
-    mo.mask(VVEC_INT_MASK);
-    mo.mul(1<<VVEC_FRAC);
-    while(bits<VVEC_INT+VVEC_FRAC-1)
+    mo.mask(0xFFF);
+    mo.shl(3);
+    while(bits<15)
     {
         mo.mask(~((1<<bits)-1));
         if(mo[dim] <= d1 && mo[dim] + (1<<bits) >= d2 &&
@@ -1492,7 +1503,7 @@ int calcmergedsize(int orient, const ivec &co, int size, const mergeinfo &m, con
             break;
         bits++;
     }
-    return bits-VVEC_FRAC;
+    return bits-3;
 }
 
 static void invalidatemerges(cube &c)
