@@ -10,7 +10,7 @@ void boxs(int orient, vec o, const vec &s)
     float f = !outline ? 0 : (dc>0 ? 0.2f : -0.2f);
     o[D[d]] += float(dc) * s[D[d]] + f,
 
-    glBegin(GL_POLYGON);
+    glBegin(GL_LINE_LOOP);
 
     glVertex3fv(o.v); o[R[d]] += s[R[d]];
     glVertex3fv(o.v); o[C[d]] += s[C[d]];
@@ -89,7 +89,7 @@ VARF(moving, 0, 0, 1,
     if(moving) havesel = false; // tell cursorupdate to create handle
 );
 
-VARF(gridpower, 3-VVEC_FRAC, 3, VVEC_INT-1,
+VARF(gridpower, 0, 3, 12,
 {
     if(dragging) return;
     gridsize = 1<<gridpower;
@@ -111,8 +111,6 @@ void cubecancel()
     forcenextundo();
 }
 
-extern void entcancel();
-
 void cancelsel()
 {
     cubecancel();
@@ -197,9 +195,11 @@ COMMAND(selextend, "");
 
 cube &blockcube(int x, int y, int z, const block3 &b, int rgrid) // looks up a world cube, based on coordinates mapped by the block
 {
-    ivec s(dimension(b.orient), x*b.grid, y*b.grid, dimcoord(b.orient)*(b.s[dimension(b.orient)]-1)*b.grid);
-
-    return neighbourcube(b.o.x+s.x, b.o.y+s.y, b.o.z+s.z, -z*b.grid, rgrid, b.orient);
+    int dim = dimension(b.orient), dc = dimcoord(b.orient);
+    ivec s(dim, x*b.grid, y*b.grid, dc*(b.s[dim]-1)*b.grid);
+    s.add(b.o);
+    if(dc) s[dim] -= z*b.grid; else s[dim] += z*b.grid;
+    return lookupcube(s.x, s.y, s.z, rgrid);
 }
 
 #define loopxy(b)        loop(y,(b).s[C[dimension((b).orient)]]) loop(x,(b).s[R[dimension((b).orient)]])
@@ -284,8 +284,6 @@ VARP(showselgrid, 0, 1, 1);
 
 void rendereditcursor()
 {
-    if(sel.grid == 0) sel.grid = gridsize;
-
     int d   = dimension(sel.orient),
         od  = dimension(orient),
         odc = dimcoord(orient);
@@ -428,6 +426,8 @@ void rendereditcursor()
 
     // cursors
 
+    lineshader->set();
+
     renderentselection(player->o, camdir, entmoving!=0);
 
     enablepolygonoffset(GL_POLYGON_OFFSET_LINE);
@@ -476,13 +476,22 @@ void rendereditcursor()
 
     disablepolygonoffset(GL_POLYGON_OFFSET_LINE);
 
+    notextureshader->set();
+
     glDisable(GL_BLEND);
 }
 
 void tryedit()
 {
     extern int hidehud;
-    if(!editmode || hidehud || mainmenu) return;
+    if(!editmode || hidehud) return;
+
+    #ifndef NEWGUI
+    if(mainmenu) return;
+    #else
+    if(UI::mainmenu) return;
+    #endif
+
     if(blendpaintmode) trypaintblendmap();
 }
 
@@ -509,10 +518,10 @@ void readychanges(block3 &b, cube *c, const ivec &cor, int size)
         }
         if(c[i].children)
         {
-            if(size<=(8>>VVEC_FRAC))
+            if(size<=1)
             {
                 solidfaces(c[i]);
-                discardchildren(c[i]);
+                discardchildren(c[i], true);
                 brightencube(c[i]);
             }
             else readychanges(b, c[i].children, o, size/2);
@@ -776,11 +785,135 @@ void swapundo(undolist &a, undolist &b, const char *s)
 void editundo() { swapundo(undos, redos, "undo"); }
 void editredo() { swapundo(redos, undos, "redo"); }
 
+vector<editinfo *> editinfos;
 editinfo *localedit = NULL;
 
+static void packcube(cube &c, vector<uchar> &buf)
+{
+    if(c.children)
+    {
+        buf.put(0xFF);
+        loopi(8) packcube(c.children[i], buf);
+    }
+    else
+    {
+        buf.put(c.ext ? c.ext->material : 0);
+        cube data = c;
+        lilswap(data.texture, 6);
+        buf.put(data.edges, sizeof(data.edges));
+        buf.put((uchar *)data.texture, sizeof(data.texture));
+    }
+}
+
+static bool packeditinfo(editinfo *e, vector<uchar> &buf)
+{
+    if(!e || !e->copy || e->copy->size() > (16<<20)) return false;
+    block3 &b = *e->copy;
+    block3 hdr = b;
+    lilswap(hdr.o.v, 3);
+    lilswap(hdr.s.v, 3);
+    lilswap(&hdr.grid, 1);
+    lilswap(&hdr.orient, 1);
+    buf.put((const uchar *)&hdr, sizeof(hdr));
+    cube *c = b.c();
+    loopi(b.size()) packcube(c[i], buf);
+    return true;
+}
+
+static void unpackcube(cube &c, ucharbuf &buf)
+{
+    int mat = buf.get();
+    if(mat == 0xFF)
+    {
+        c.children = newcubes(F_EMPTY);
+        loopi(8) unpackcube(c.children[i], buf);
+    }
+    else
+    {
+        if(mat != MAT_AIR) ext(c).material = mat;
+        buf.get(c.edges, sizeof(c.edges));
+        buf.get((uchar *)c.texture, sizeof(c.texture));
+        lilswap(c.texture, 6);
+    }
+}
+
+static bool unpackeditinfo(editinfo *&e, ucharbuf &buf)
+{
+    if(!e) e = editinfos.add(new editinfo);
+    if(e->copy) { freeblock(e->copy); e->copy = NULL; }
+    block3 hdr;
+    buf.get((uchar *)&hdr, sizeof(hdr));
+    lilswap(hdr.o.v, 3);
+    lilswap(hdr.s.v, 3);
+    lilswap(&hdr.grid, 1);
+    lilswap(&hdr.orient, 1);
+    if(hdr.size() > (16<<20)) return false;
+    e->copy = (block3 *)new uchar[sizeof(block3)+hdr.size()*sizeof(cube)];
+    block3 &b = *e->copy;
+    b = hdr;
+    cube *c = b.c();
+    memset(c, 0, b.size()*sizeof(cube));
+    loopi(b.size()) unpackcube(c[i], buf);
+    return true;
+}
+
+static bool compresseditinfo(const uchar *inbuf, int inlen, uchar *&outbuf, int &outlen)
+{
+    uLongf len = compressBound(inlen);
+    if(len > (1<<20)) return false;
+    outbuf = new uchar[len];
+    if(compress2((Bytef *)outbuf, &len, (const Bytef *)inbuf, inlen, Z_BEST_COMPRESSION) != Z_OK || len > (1<<16))
+    {
+        delete[] outbuf;
+        outbuf = NULL;
+        return false;
+    }
+    outlen = len;
+    return true;
+}
+
+static bool uncompresseditinfo(const uchar *inbuf, int inlen, uchar *&outbuf, int &outlen)
+{
+    if(compressBound(outlen) > (1<<20)) return false;
+    uLongf len = outlen;
+    outbuf = new uchar[len];
+    if(uncompress((Bytef *)outbuf, &len, (const Bytef *)inbuf, inlen) != Z_OK)
+    {
+        delete[] outbuf;
+        outbuf = NULL;
+        return false;
+    }
+    outlen = len;
+    return true;
+}
+
+bool packeditinfo(editinfo *e, int &inlen, uchar *&outbuf, int &outlen)
+{
+    vector<uchar> buf;
+    if(!packeditinfo(e, buf)) return false;
+    inlen = buf.length();
+    return compresseditinfo(buf.getbuf(), buf.length(), outbuf, outlen);
+}
+
+bool unpackeditinfo(editinfo *&e, const uchar *inbuf, int inlen, int outlen)
+{
+    if(e && e->copy) { freeblock(e->copy); e->copy = NULL; }
+    uchar *outbuf = NULL;
+    if(!uncompresseditinfo(inbuf, inlen, outbuf, outlen)) return false;
+    ucharbuf buf(outbuf, outlen);
+    if(!unpackeditinfo(e, buf))
+    {
+        delete[] outbuf;
+        return false;
+    }
+    delete[] outbuf;
+    return true;
+}
+
 void freeeditinfo(editinfo *&e)
 {
     if(!e) return;
+    editinfos.removeobj(e);
     if(e->copy) freeblock(e->copy);
     delete e;
     e = NULL;
@@ -792,7 +925,7 @@ void freeeditinfo(editinfo *&e)
 void mpcopy(editinfo *&e, selinfo &sel, bool local)
 {
     if(local) game::edittrigger(sel, EDIT_COPY);
-    if(e==NULL) e = new editinfo;
+    if(e==NULL) e = editinfos.add(new editinfo);
     if(e->copy) freeblock(e->copy);
     e->copy = NULL;
     protectsel(e->copy = blockcopy(block3(sel), sel.grid));
@@ -840,6 +973,24 @@ COMMAND(paste, "");
 COMMANDN(undo, editundo, "");
 COMMANDN(redo, editredo, "");
 
+static VSlot *editingvslot = NULL;
+
+void compacteditvslots()
+{
+    if(editingvslot && editingvslot->layer) compactvslot(editingvslot->layer);
+    loopv(editinfos)
+    {
+        editinfo *e = editinfos[i];
+        compactvslots(e->copy->c(), e->copy->size());
+    }
+    for(undoblock *u = undos.first; u; u = u->next)
+        if(!u->numents)
+            compactvslots(u->block()->c(), u->block()->size());
+    for(undoblock *u = redos.first; u; u = u->next)
+        if(!u->numents)
+            compactvslots(u->block()->c(), u->block()->size());
+}
+
 ///////////// height maps ////////////////
 
 #define MAXBRUSH    64
@@ -877,7 +1028,7 @@ vector<int> htextures;
 
 COMMAND(clearbrush, "");
 COMMAND(brushvert, "iii");
-ICOMMAND(hmapcancel, "", (), htextures.setsizenodelete(0); );
+ICOMMAND(hmapcancel, "", (), htextures.setsize(0); );
 ICOMMAND(hmapselect, "", (),
     int t = lookupcube(cur.x, cur.y, cur.z).texture[orient];
     int i = htextures.find(t);
@@ -926,8 +1077,8 @@ namespace hmap
         t[d] += dcr*f*gridsize;
         if(t[d] > nz || t[d] < mz) return NULL;
         cube *c = &lookupcube(t.x, t.y, t.z, gridsize);
-        if(c->children) forcemip(*c);
-        discardchildren(*c);
+        if(c->children) forcemip(*c, false);
+        discardchildren(*c, true);
         if(!isheightmap(sel.orient, d, true, c)) return NULL;
         if     (t.x < changes.o.x) changes.o.x = t.x;
         else if(t.x > changes.s.x) changes.s.x = t.x;
@@ -1264,7 +1415,7 @@ void mpeditface(int dir, int mode, selinfo &sel, bool local)
     loopselxyz(
         if(c.children) solidfaces(c);
         uchar mat = getmaterial(c);
-        discardchildren(c);
+        discardchildren(c, true);
         if(mat!=MAT_AIR) ext(c).material = mat;
         if(mode==1) // fill command
         {
@@ -1273,7 +1424,7 @@ void mpeditface(int dir, int mode, selinfo &sel, bool local)
                 solidfaces(c);
                 cube &o = blockcube(x, y, 1, sel, -sel.grid);
                 loopi(6)
-                    c.texture[i] = o.texture[i];
+                    c.texture[i] = o.children ? DEFAULT_GEOM : o.texture[i];
             }
             else
                 emptyfaces(c);
@@ -1337,13 +1488,17 @@ void pushsel(int *dir)
     int d = dimension(orient);
     int s = dimcoord(orient) ? -*dir : *dir;
     sel.o[d] += s*sel.grid;
-    if(selectionsurf==1) player->o[d] += s*sel.grid;
+    if(selectionsurf==1)
+    {
+        player->o[d] += s*sel.grid;
+        player->resetinterp();
+    }
 }
 
 void mpdelcube(selinfo &sel, bool local)
 {
     if(local) game::edittrigger(sel, EDIT_DELCUBE);
-    loopselxyz(discardchildren(c); emptyfaces(c));
+    loopselxyz(discardchildren(c, true); emptyfaces(c));
 }
 
 void delcube()
@@ -1375,6 +1530,70 @@ void tofronttex()                                       // maintain most recentl
 selinfo repsel;
 int reptex = -1;
 
+struct vslotmap
+{
+    int index;
+    VSlot *vslot;
+
+    vslotmap() {}
+    vslotmap(int index, VSlot *vslot) : index(index), vslot(vslot) {}
+};
+static vector<vslotmap> remappedvslots;
+
+VAR(usevdelta, 1, 0, 0);
+
+static VSlot *remapvslot(int index, const VSlot &ds)
+{
+    loopv(remappedvslots) if(remappedvslots[i].index == index) return remappedvslots[i].vslot;
+    VSlot &vs = lookupvslot(index, false);
+    if(vs.index < 0 || vs.index == DEFAULT_SKY) return NULL;
+    VSlot *edit = NULL;
+    if(usevdelta)
+    {
+        VSlot ms;
+        mergevslot(ms, vs, ds);
+        edit = ms.changed ? editvslot(vs, ms) : vs.slot->variants;
+    }
+    else edit = ds.changed ? editvslot(vs, ds) : vs.slot->variants;
+    if(!edit) edit = &vs;
+    remappedvslots.add(vslotmap(vs.index, edit));
+    return edit;
+}
+
+static void remapvslots(cube &c, const VSlot &ds, int orient, bool &findrep, VSlot *&findedit)
+{
+    if(c.children)
+    {
+        loopi(8) remapvslots(c.children[i], ds, orient, findrep, findedit);
+        return;
+    }
+    static VSlot ms;
+    if(orient<0) loopi(6)
+    {
+        VSlot *edit = remapvslot(c.texture[i], ds);
+        if(edit)
+        {
+            c.texture[i] = edit->index;
+            if(!findedit) findedit = edit;
+        }
+    }
+    else
+    {
+        int i = visibleorient(c, orient);
+        VSlot *edit = remapvslot(c.texture[i], ds);
+        if(edit)
+        {
+            if(findrep)
+            {
+                if(reptex < 0) reptex = c.texture[i];
+                else if(reptex != c.texture[i]) findrep = false;
+            }
+            c.texture[i] = edit->index;
+            if(!findedit) findedit = edit;
+        }
+    }
+}
+
 void edittexcube(cube &c, int tex, int orient, bool &findrep)
 {
     if(orient<0) loopi(6) c.texture[i] = tex;
@@ -1391,9 +1610,139 @@ void edittexcube(cube &c, int tex, int orient, bool &findrep)
     if(c.children) loopi(8) edittexcube(c.children[i], tex, orient, findrep);
 }
 
-extern int curtexnum;
 VAR(allfaces, 0, 0, 1);
 
+void mpeditvslot(VSlot &ds, int allfaces, selinfo &sel, bool local)
+{
+    if(local)
+    {
+        if(!(lastsel==sel)) tofronttex();
+        if(allfaces || !(repsel == sel)) reptex = -1;
+        repsel = sel;
+    }
+    bool findrep = local && !allfaces && reptex < 0;
+    VSlot *findedit = NULL;
+    editingvslot = &ds;
+    loopselxyz(remapvslots(c, ds, allfaces ? -1 : sel.orient, findrep, findedit));
+    editingvslot = NULL;
+    remappedvslots.setsize(0);
+    if(local && findedit)
+    {
+        lasttex = findedit->index;
+        lasttexmillis = totalmillis;
+        curtexindex = texmru.find(lasttex);
+        if(curtexindex < 0)
+        {
+            curtexindex = texmru.length();
+            texmru.add(lasttex);
+        }
+    }
+}
+
+void vdelta(char *body)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    usevdelta++;
+    execute(body);
+    usevdelta--;
+}
+COMMAND(vdelta, "s");
+
+void vrotate(int *n)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_ROTATION;
+    ds.rotation = usevdelta ? *n : clamp(*n, 0, 5);
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vrotate, "i");
+
+void voffset(int *x, int *y)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_OFFSET;
+    ds.xoffset = usevdelta ? *x : max(*x, 0);
+    ds.yoffset = usevdelta ? *y : max(*y, 0);
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(voffset, "ii");
+
+void vscroll(float *s, float *t)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_SCROLL;
+    ds.scrollS = *s/1000.0f;
+    ds.scrollT = *t/1000.0f;
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vscroll, "ff");
+
+void vscale(float *scale)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_SCALE;
+    ds.scale = *scale <= 0 ? 1 : (usevdelta ? *scale : clamp(*scale, 1/8.0f, 8.0f));
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vscale, "f");
+
+void vlayer(int *n)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_LAYER;
+    ds.layer = vslots.inrange(*n) ? *n : 0;
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vlayer, "i");
+
+void valpha(float *front, float *back)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_ALPHA;
+    ds.alphafront = clamp(*front, 0.0f, 1.0f);
+    ds.alphaback = clamp(*back, 0.0f, 1.0f);
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(valpha, "ff");
+
+void vcolor(float *r, float *g, float *b)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_COLOR;
+    ds.colorscale = vec(clamp(*r, 0.0f, 1.0f), clamp(*g, 0.0f, 1.0f), clamp(*b, 0.0f, 1.0f));
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vcolor, "fff");
+
+void vreset()
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vreset, "");
+
+void vshaderparam(const char *name, float *x, float *y, float *z, float *w)
+{
+    if(noedit() || (nompedit && multiplayer())) return;
+    VSlot ds;
+    ds.changed = 1<<VSLOT_SHPARAM;
+    if(name[0])
+    {
+        ShaderParam p = { getshaderparamname(name), SHPARAM_LOOKUP, -1, -1, {*x, *y, *z, *w} };
+        ds.params.add(p);
+    }
+    mpeditvslot(ds, allfaces, sel, true);
+}
+COMMAND(vshaderparam, "sffff");
+
 void mpedittex(int tex, int allfaces, selinfo &sel, bool local)
 {
     if(local)
@@ -1408,12 +1757,45 @@ void mpedittex(int tex, int allfaces, selinfo &sel, bool local)
 
 void filltexlist()
 {
-    if(texmru.length()!=curtexnum)
+    if(texmru.length()!=vslots.length())
     {
-        loopv(texmru) if(texmru[i]>=curtexnum) texmru.remove(i--);
-        loopi(curtexnum) if(texmru.find(i)<0) texmru.add(i);
+        loopvrev(texmru) if(texmru[i]>=vslots.length())
+        {
+            if(curtexindex > i) curtexindex--;
+            else if(curtexindex == i) curtexindex = -1;
+            texmru.remove(i);
+        }
+        loopv(vslots) if(texmru.find(i)<0) texmru.add(i);
     }
 }
+COMMAND(filltexlist, "");
+
+void compactmruvslots()
+{
+    remappedvslots.setsize(0);
+    loopvrev(texmru)
+    {
+        if(vslots.inrange(texmru[i]))
+        {
+            VSlot &vs = *vslots[texmru[i]];
+            if(vs.index >= 0)
+            {
+                texmru[i] = vs.index;
+                continue;
+            }
+        }
+        if(curtexindex > i) curtexindex--;
+        else if(curtexindex == i) curtexindex = -1;
+        texmru.remove(i);
+    }
+    if(vslots.inrange(lasttex))
+    {
+        VSlot &vs = *vslots[lasttex];
+        lasttex = vs.index >= 0 ? vs.index : 0;
+    }
+    else lasttex = 0;
+    reptex = vslots.inrange(reptex) ? vslots[reptex]->index : -1;
+}
 
 void edittex(int i, bool save = true)
 {
@@ -1432,16 +1814,17 @@ void edittex_(int *dir)
     filltexlist();
     texpaneltimer = 5000;
     if(!(lastsel==sel)) tofronttex();
-    curtexindex = clamp(curtexindex<0 ? 0 : curtexindex+*dir, 0, curtexnum-1);
+    curtexindex = clamp(curtexindex<0 ? 0 : curtexindex+*dir, 0, texmru.length()-1);
     edittex(texmru[curtexindex], false);
 }
 
 void gettex()
 {
-    if(noedit()) return;
+    if(noedit(true)) return;
     filltexlist();
-    loopxyz(sel, sel.grid, curtexindex = c.texture[sel.orient]);
-    loopi(curtexnum) if(texmru[i]==curtexindex)
+    int tex = -1;
+    loopxyz(sel, sel.grid, tex = c.texture[sel.orient]);
+    loopv(texmru) if(texmru[i]==tex)
     {
         curtexindex = i;
         tofronttex();
@@ -1460,7 +1843,7 @@ void getcurtex()
 
 void getseltex()
 {
-    if(!editmode) return;
+    if(noedit(true)) return;
     cube &c = lookupcube(sel.o.x, sel.o.y, sel.o.z, -sel.grid);
     if(c.children || isempty(c)) return;
     intret(c.texture[sel.orient]);
@@ -1469,7 +1852,8 @@ void getseltex()
 void gettexname(int *tex, int *subslot)
 {
     if(!editmode || *tex<0) return;
-    Slot &slot = lookuptexture(*tex);
+    VSlot &vslot = lookupvslot(*tex);
+    Slot &slot = *vslot.slot;
     if(!slot.sts.inrange(*subslot)) return;
     result(slot.sts[*subslot].name);
 }
@@ -1478,6 +1862,7 @@ COMMANDN(edittex, edittex_, "i");
 COMMAND(gettex, "");
 COMMAND(getcurtex, "");
 COMMAND(getseltex, "");
+ICOMMAND(getreptex, "", (), { if(!noedit()) intret(vslots.inrange(reptex) ? reptex : -1); });
 COMMAND(gettexname, "ii");
 
 void replacetexcube(cube &c, int oldtex, int newtex)
@@ -1486,21 +1871,29 @@ void replacetexcube(cube &c, int oldtex, int newtex)
     if(c.children) loopi(8) replacetexcube(c.children[i], oldtex, newtex);
 }
 
-void mpreplacetex(int oldtex, int newtex, selinfo &sel, bool local)
+void mpreplacetex(int oldtex, int newtex, bool insel, selinfo &sel, bool local)
 {
-    if(local) game::edittrigger(sel, EDIT_REPLACE, oldtex, newtex);
-    loopi(8) replacetexcube(worldroot[i], oldtex, newtex);
+    if(local) game::edittrigger(sel, EDIT_REPLACE, oldtex, newtex, insel ? 1 : 0);
+    if(insel)
+    {
+        loopselxyz(replacetexcube(c, oldtex, newtex));
+    }
+    else
+    {
+        loopi(8) replacetexcube(worldroot[i], oldtex, newtex);
+    }
     allchanged();
 }
 
-void replace()
+void replace(bool insel)
 {
     if(noedit()) return;
     if(reptex < 0) { conoutf(CON_ERROR, "can only replace after a texture edit"); return; }
-    mpreplacetex(reptex, lasttex, sel, true);
+    mpreplacetex(reptex, lasttex, insel, sel, true);
 }
 
-COMMAND(replace, "");
+ICOMMAND(replace, "", (), replace(false));
+ICOMMAND(replacesel, "", (), replace(true));
 
 ////////// Art Culling functions /////////
 
@@ -1518,7 +1911,7 @@ void cullmodels()
 
 			if(e.type == ET_MAPMODEL)
 			{
-				if(e.attr2 == i)
+				if(e.attr[1] == i)
 				{
 					found = true;
 					break;
@@ -1542,8 +1935,8 @@ void cullmodels()
 
 			if(e.type == ET_MAPMODEL)
 			{
-				if(e.attr2 >= i)
-					e.attr2 -= 1;
+				if(e.attr[1] >= i)
+					e.attr[1] -= 1;
 			}
 			else if(e.type >= ET_GAMESPECIFIC)
 			{
@@ -1561,7 +1954,7 @@ void cullmodels()
 
 COMMAND(cullmodels, "");
 
-extern vector<Slot> slots;
+extern vector<Slot *> slots;
 
 bool recursetextures(cube *parent, int tex)
 {
@@ -1578,16 +1971,24 @@ bool recursetextures(cube *parent, int tex)
 		{
 			loopj(6)
 			{
-				if(parent[i].texture[j] == tex ||
-					(curtexnum-1 >= parent[i].texture[j] && parent[i].texture[j] > 0 && slots[parent[i].texture[j]].layer == tex)
-				)
+				if(parent[i].texture[j] == tex)
 				{
 					return true;
 				}
+
+				if(slots.inrange(parent[i].texture[j]))
+				{
+					VSlot *slot = slots[parent[i].texture[j]]->variants;
+					while(slot)
+					{
+						if(slot->layer == tex) return true;
+
+						slot = slot->next;
+					}
+				}
 			}
 		}
 	}
-
 	return false;
 }
 
@@ -1595,9 +1996,9 @@ void culltextures()
 {
 	if(nompedit && multiplayer()) return;
 
-	loopirev(curtexnum)
+	loopvrev(slots)
 	{
-		if(curtexnum <= 8) return;
+		if(slots.length() <= 8) return;
 		if(!recursetextures(worldroot, i))
 		{
 			bool del = true;
@@ -1629,22 +2030,36 @@ void culltextures()
 				}
 			}
 
-			loopj(curtexnum-i)
+			loopj(slots.length()-i)
 			{
 				int oldtex = i+j;
 				loopk(8) replacetexcube(worldroot[k], oldtex, oldtex -1);
 			}
 			loopvj(slots)
 			{
-				if(slots[j].layer > i)
-					slots[j].layer--;
-				else if(slots[j].layer == i)
-					slots[j].layer = 0;
+				VSlot *vs = slots[i]->variants;
+				while(vs)
+				{
+					if(vs->layer > i)
+						vs->layer--;
+					else if(vs->layer == i)
+						vs->layer = 0;
+
+					vs = vs->next;
+				}
 			}
 
-			slots[i].reset();
+			extern Slot dummyslot;
+			VSlot *vs = slots[i]->variants;
+			while(vs)
+			{
+				vs->slot = &dummyslot;
+				vs = vs->next;
+			}
+
+			slots[i]->reset();
+			delete slots[i];
 			slots.remove(i);
-			curtexnum--;
 		}
 	}
 	allchanged();
@@ -1822,20 +2237,18 @@ void editmat(char *name, char *filtername)
 
 COMMAND(editmat, "ss");
 
-//originally 10, increased for "convenience"
-#define TEXGUI_WIDTH 12
-//originally 7, reduced to host improvement test
-#define TEXGUI_HEIGHT 5
+#ifndef NEWGUI
 extern int menudistance, menuautoclose;
 
-VARP(thumbtime, 0, 50, 1000);
+VARP(texguiwidth, 1, 15, 1000);
+VARP(texguiheight, 1, 6, 1000);
+VARP(texguitime, 0, 25, 1000);
 
 static int lastthumbnail = 0;
 static int texhoveridx = 0;
 
 VARP(texgui2d, 0, 1, 1);
 
-#ifndef NEWGUI
 struct texturegui : g3d_callback
 {
     bool menuon;
@@ -1846,94 +2259,83 @@ struct texturegui : g3d_callback
 
     void gui(g3d_gui &g, bool firstpass)
     {
-        int origtab = menutab, numtabs = max((curtexnum + TEXGUI_WIDTH*TEXGUI_HEIGHT - 1)/(TEXGUI_WIDTH*TEXGUI_HEIGHT), 1);
+        int origtab = menutab, numtabs = max((slots.length() + texguiwidth*texguiheight - 1)/(texguiwidth*texguiheight), 1);
+
         g.start(menustart, 0.04f, &menutab);
         loopi(numtabs)
         {
             g.tab(!i ? "Textures" : NULL, 0xAAFFAA);
             if(i+1 != origtab) continue; //don't load textures on non-visible tabs!
 
-		Texture *prevtex = notexture, *prevglowtex = NULL, *prevlayertex = NULL;
-            loop(h, TEXGUI_HEIGHT)
+            loop(h, texguiheight)
             {
                 g.pushlist();
-                loop(w, TEXGUI_WIDTH)
+                loop(w, texguiwidth)
                 {
-                    int ti = (i*TEXGUI_HEIGHT+h)*TEXGUI_WIDTH+w;
-                    if(ti<curtexnum)
+                    extern VSlot dummyvslot;
+                    int ti = (i*texguiheight+h)*texguiwidth+w;
+                    if(ti<slots.length())
                     {
-                        Texture *tex = notexture, *glowtex = NULL, *layertex = NULL;
-                        Slot &slot = lookuptexture(ti, false);
+                        Slot &slot = lookupslot(ti, false);
+                        VSlot &vslot = *slot.variants;
                         if(slot.sts.empty()) continue;
-                        else if(slot.loaded)
+                        else if(!slot.loaded && !slot.thumbnail)
                         {
-                            tex = slot.sts[0].t;
-                            if(slot.texmask&(1<<TEX_GLOW)) { loopvj(slot.sts) if(slot.sts[j].type==TEX_GLOW) { glowtex = slot.sts[j].t; break; } }
-                            if(slot.layer)
+                            if(totalmillis-lastthumbnail<texguitime)
                             {
-                                Slot &layer = lookuptexture(slot.layer);
-                                if(!layer.sts.empty()) layertex = layer.sts[0].t;
+                                g.texture(dummyvslot, 1.0, false); //create an empty space
+                                continue;
                             }
+                            loadthumbnail(slot);
+                            lastthumbnail = totalmillis;
                         }
-                        else if(slot.thumbnail) tex = slot.thumbnail;
-                        else if(totalmillis-lastthumbnail>=thumbtime) { tex = loadthumbnail(slot); lastthumbnail = totalmillis; }
-
-			int test = g.texture(tex, 1.0, slot.rotation, slot.xoffset, slot.yoffset, glowtex, slot.glowcolor, layertex);
-                        if(test&G3D_UP && (slot.loaded || tex!=notexture))
-                            edittex(ti);
-			if(test&G3D_ROLLOVER)
+                        int test = g.texture(vslot, 1.0f, true);
+			if(test & G3D_UP && (slot.loaded || slot.thumbnail!=notexture))
+                            edittex(vslot.index);
+			if(test & G3D_ROLLOVER)
 			{
+				if(!slot.loaded && !slot.thumbnail)
+					loadthumbnail(slot);
+
 				texhoveridx = ti;
 			}
                     }
                     else
-                        g.texture(notexture, 1.0); //create an empty space
+                    {
+                        g.texture(dummyvslot, 1.0, false); //create an empty space
+                    }
                 }
                 g.poplist();
             }
 
-		Slot &prev = lookuptexture(texhoveridx, false);
-		if(prev.loaded)
-		{
-			prevtex = prev.sts[0].t;
-			if(prev.texmask&(1<<TEX_GLOW)) { loopvj(prev.sts) if(prev.sts[i].type==TEX_GLOW) { prevglowtex = prev.sts[i].t; break; } }
-			if(prev.layer)
-			{
-				Slot &prevlayer = lookuptexture(prev.layer);
-				if(!prevlayer.sts.empty()) prevlayertex = prevlayer.sts[0].t;
-			}
-		}
-		else if(prev.thumbnail) prevtex = prev.thumbnail;
-		else prevtex = loadthumbnail(prev);
-		//keep preview present at all times, if valid
+		Slot &prev = lookupslot(texhoveridx, false);
+		VSlot &vprev = *prev.variants;
 
 		g.pushlist();
-		g.texture(prevtex, 2.0, prev.rotation, prev.xoffset, prev.yoffset, prevglowtex, prev.glowcolor, prevlayertex);
+		g.texture(vprev, 2.0f, true);
 
 		extern int guitextcolour;
 		g.pushlist();
 		defformatstring(ds)("Texture: %s", prev.sts[0].name);
-		if(ds[45]) //shorten strings, to avoid a jittery interface
+		if(ds[60]) //shorten strings, to avoid a jittery interface
 		{
-			ds[45] = '\0';
-			ds[44] = ds[43] = ds[42] = '.';
+			ds[60] = '\0';
+			ds[59] = ds[58] = ds[57] = '.';
 		}
 		g.text(ds, guitextcolour, "info");
 
-		formatstring(ds)("layer: %i\tShader: %s", prev.layer, prev.shader->name);
-		if(ds[45])
+		formatstring(ds)("layer: %i\tShader: %s", vprev.layer, prev.shader->name);
+		if(ds[60]) //shorten strings, to avoid a jittery interface
 		{
-			ds[45] = '\0';
-			ds[44] = ds[43] = ds[42] = '.';
+			ds[60] = '\0';
+			ds[59] = ds[58] = ds[57] = '.';
 		}
 		g.text(ds, guitextcolour, "info");
 
-		formatstring(ds)("Size: %ix%i\tScale: %.2f (%ix%i)",
-			prevtex->xs, prevtex->ys, prev.scale, (int) (prevtex->xs * prev.scale), (int) (prevtex->ys * prev.scale)); //w and h corresponds to the current scale
+		formatstring(ds)("Scale: %.2f\tAlpha %.2f %.2f", vprev.scale, vprev.alphafront, vprev.alphaback);
 		g.text(ds, guitextcolour, "info");
 
-		formatstring(ds)("Scroll: %.2f %.2f\toffset %i(%i%%) %i(%i%%)", prev.scrollS * 1000.0f, prev.scrollT * 1000.0f,
-			prev.xoffset, abs(100*prev.xoffset/prevtex->xs)%100, prev.yoffset, abs(100*prev.yoffset/prevtex->ys)%100);
+		formatstring(ds)("Scroll: %.2f %.2f\toffset %i %i", vprev.scrollS * 1000.0f, vprev.scrollT * 1000.0f, vprev.xoffset, vprev.yoffset);
 		g.text(ds, guitextcolour, "info");
 
 		g.poplist();
@@ -1946,7 +2348,8 @@ struct texturegui : g3d_callback
     {
         if(on != menuon && (menuon = on))
         {
-            if(menustart <= lasttexmillis) menutab = 1+clamp(lasttex, 0, curtexnum-1)/(TEXGUI_WIDTH*TEXGUI_HEIGHT);
+            if(menustart <= lasttexmillis)
+                menutab = 1+clamp(lookupvslot(lasttex, false).slot->index, 0, slots.length()-1)/(texguiwidth*texguiheight);
             menupos = menuinfrontofplayer();
             menustart = starttime();
         }
@@ -1995,55 +2398,57 @@ void rendertexturepanel(int w, int h)
         loopi(7)
         {
             int s = (i == 3 ? 285 : 220), ti = curtexindex+i-3;
-            if(ti>=0 && ti<curtexnum)
+            if(ti>=0 && ti<texmru.length())
             {
-                Slot &slot = lookuptexture(texmru[ti]);
-                Texture *tex = slot.sts[0].t, *glowtex = NULL, *layertex = NULL;
+                VSlot &vslot = lookupvslot(texmru[ti]), *layer = NULL;
+                Slot &slot = *vslot.slot;
+                Texture *tex = slot.sts.empty() ? notexture : slot.sts[0].t, *glowtex = NULL, *layertex = NULL;
                 if(slot.texmask&(1<<TEX_GLOW))
                 {
                     loopvj(slot.sts) if(slot.sts[j].type==TEX_GLOW) { glowtex = slot.sts[j].t; break; }
                 }
-                if(slot.layer)
+                if(vslot.layer)
                 {
-                    Slot &layer = lookuptexture(slot.layer);
-                    layertex = layer.sts[0].t;
+                    layer = &lookupvslot(vslot.layer);
+                    layertex = layer->slot->sts.empty() ? notexture : layer->slot->sts[0].t;
                 }
                 float sx = min(1.0f, tex->xs/(float)tex->ys), sy = min(1.0f, tex->ys/(float)tex->xs);
                 int x = w*1800/h-s-50, r = s;
                 float tc[4][2] = { { 0, 0 }, { 1, 0 }, { 1, 1 }, { 0, 1 } };
-                float xoff = slot.xoffset, yoff = slot.yoffset;
-                if(slot.rotation)
+                float xoff = vslot.xoffset, yoff = vslot.yoffset;
+                if(vslot.rotation)
                 {
-                    if((slot.rotation&5) == 1) { swap(xoff, yoff); loopk(4) swap(tc[k][0], tc[k][1]); }
-                    if(slot.rotation >= 2 && slot.rotation <= 4) { xoff *= -1; loopk(4) tc[k][0] *= -1; }
-                    if(slot.rotation <= 2 || slot.rotation == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
+                    if((vslot.rotation&5) == 1) { swap(xoff, yoff); loopk(4) swap(tc[k][0], tc[k][1]); }
+                    if(vslot.rotation >= 2 && vslot.rotation <= 4) { xoff *= -1; loopk(4) tc[k][0] *= -1; }
+                    if(vslot.rotation <= 2 || vslot.rotation == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
                 }
                 loopk(4) { tc[k][0] = tc[k][0]/sx - xoff/tex->xs; tc[k][1] = tc[k][1]/sy - yoff/tex->ys; }
                 glBindTexture(GL_TEXTURE_2D, tex->id);
                 loopj(glowtex ? 3 : 2)
                 {
-                    if(j < 2) glColor4f(j, j, j, texpaneltimer/1000.0f);
+                    if(j < 2) glColor4f(j*vslot.colorscale.x, j*vslot.colorscale.y, j*vslot.colorscale.z, texpaneltimer/1000.0f);
                     else
                     {
                         glBindTexture(GL_TEXTURE_2D, glowtex->id);
                         glBlendFunc(GL_SRC_ALPHA, GL_ONE);
-                        glColor4f(slot.glowcolor.x, slot.glowcolor.y, slot.glowcolor.z, texpaneltimer/1000.0f);
+                        glColor4f(vslot.glowcolor.x, vslot.glowcolor.y, vslot.glowcolor.z, texpaneltimer/1000.0f);
                     }
-                    glBegin(GL_QUADS);
+                    glBegin(GL_TRIANGLE_STRIP);
                     glTexCoord2fv(tc[0]); glVertex2f(x,   y);
                     glTexCoord2fv(tc[1]); glVertex2f(x+r, y);
-                    glTexCoord2fv(tc[2]); glVertex2f(x+r, y+r);
                     glTexCoord2fv(tc[3]); glVertex2f(x,   y+r);
+                    glTexCoord2fv(tc[2]); glVertex2f(x+r, y+r);
                     glEnd();
                     xtraverts += 4;
                     if(j==1 && layertex)
                     {
+                        glColor4f(layer->colorscale.x, layer->colorscale.y, layer->colorscale.z, texpaneltimer/1000.0f);
                         glBindTexture(GL_TEXTURE_2D, layertex->id);
-                        glBegin(GL_QUADS);
+                        glBegin(GL_TRIANGLE_STRIP);
                         glTexCoord2fv(tc[0]); glVertex2f(x+r/2, y+r/2);
                         glTexCoord2fv(tc[1]); glVertex2f(x+r,   y+r/2);
-                        glTexCoord2fv(tc[2]); glVertex2f(x+r,   y+r);
                         glTexCoord2fv(tc[3]); glVertex2f(x+r/2, y+r);
+                        glTexCoord2fv(tc[2]); glVertex2f(x+r,   y+r);
                         glEnd();
                         xtraverts += 4;
                     }
