@@ -10,16 +10,18 @@ const int MAXCLIPPLANES = 1024;
 
 clipplanes clipcache[MAXCLIPPLANES], *nextclip = clipcache;
 
+void gencubeclip(cube &c, const ivec &o, int size)
+{
+    if(nextclip >= &clipcache[MAXCLIPPLANES]) nextclip = clipcache;
+    ext(c).clip = nextclip;
+    nextclip->owner = &c;
+    genclipplanes(c, o.x, o.y, o.z, size, *nextclip);
+    nextclip++;
+}
+
 static inline void setcubeclip(cube &c, const ivec &o, int size)
 {
-    if(!c.ext || !c.ext->clip || c.ext->clip->owner!=&c)
-    {
-        if(nextclip >= &clipcache[MAXCLIPPLANES]) nextclip = clipcache;
-        ext(c).clip = nextclip;
-        nextclip->owner = &c;
-        genclipplanes(c, o.x, o.y, o.z, size, *nextclip);
-        nextclip++;
-    }
+    if(!c.ext || !c.ext->clip || c.ext->clip->owner!=&c) gencubeclip(c, o, size);
 }
 
 void freeclipplanes(cube &c)
@@ -48,18 +50,17 @@ bool pointincube(const clipplanes &p, const vec &v)
     return true;
 }
 
-#define INTERSECTPLANES(setentry) \
-    clipplanes &p = *c.ext->clip; \
+#define INTERSECTPLANES(setentry, exit) \
     float enterdist = -1e16f, exitdist = 1e16f; \
     loopi(p.size) \
     { \
-        float pdist = p.p[i].dist(o), facing = ray.dot(p.p[i]); \
+        float pdist = p.p[i].dist(v), facing = ray.dot(p.p[i]); \
         if(facing < 0) \
         { \
             pdist /= -facing; \
             if(pdist > enterdist) \
             { \
-                if(pdist > exitdist) return false; \
+                if(pdist > exitdist) exit; \
                 enterdist = pdist; \
                 setentry; \
             } \
@@ -69,51 +70,42 @@ bool pointincube(const clipplanes &p, const vec &v)
             pdist /= -facing; \
             if(pdist < exitdist) \
             { \
-                if(pdist < enterdist) return false; \
+                if(pdist < enterdist) exit; \
                 exitdist = pdist; \
             } \
         } \
-        else if(pdist > 0) return false; \
+        else if(pdist > 0) exit; \
     }
 
-#define INTERSECTBOX(setentry) \
+#define INTERSECTBOX(setentry, exit) \
     loop(i, 3) \
     { \
         if(ray[i]) \
         { \
-            float prad = fabs(p.r[i] / ray[i]), pdist = (p.o[i] - o[i]) / ray[i], pmin = pdist - prad, pmax = pdist + prad; \
+            float prad = fabs(p.r[i] * invray[i]), pdist = (p.o[i] - v[i]) * invray[i], pmin = pdist - prad, pmax = pdist + prad; \
             if(pmin > enterdist) \
             { \
-                if(pmin > exitdist) return false; \
+                if(pmin > exitdist) exit; \
                 enterdist = pmin; \
                 setentry; \
             } \
             if(pmax < exitdist) \
             { \
-                if(pmax < enterdist) return false; \
+                if(pmax < enterdist) exit; \
                 exitdist = pmax; \
             } \
          } \
-         else if(o[i] < p.o[i]-p.r[i] || o[i] > p.o[i]+p.r[i]) return false; \
+         else if(v[i] < p.o[i]-p.r[i] || v[i] > p.o[i]+p.r[i]) exit; \
     }
 
-// optimized shadow version
-static inline bool shadowcubeintersect(const cube &c, const vec &o, const vec &ray, float &dist)
-{
-    INTERSECTPLANES({});
-    INTERSECTBOX({});
-    if(exitdist < 0) return false;
-    dist = max(enterdist+0.1f, 0.0f);
-    return true;
-}
-
 vec hitsurface;
 
-static inline bool raycubeintersect(const cube &c, const vec &o, const vec &ray, float &dist)
+static inline bool raycubeintersect(const cube &c, const vec &v, const vec &ray, const vec &invray, float &dist)
 {
     int entry = -1, bbentry = -1;
-    INTERSECTPLANES(entry = i);
-    INTERSECTBOX(bbentry = i);
+    clipplanes &p = *c.ext->clip;
+    INTERSECTPLANES(entry = i, return false);
+    INTERSECTBOX(bbentry = i, return false);
     if(exitdist < 0) return false;
     dist = max(enterdist+0.1f, 0.0f);
     if(bbentry>=0) { hitsurface = vec(0, 0, 0); hitsurface[bbentry] = ray[bbentry]>0 ? -1 : 1; }
@@ -154,7 +146,6 @@ static float disttoent(octaentities *oc, octaentities *last, const vec &o, const
     }
 
     entintersect(RAY_POLY, mapmodels,
-        if(e.attr3 && (e.triggerstate == TRIGGER_DISAPPEARED || !checktriggertype(e.attr3, TRIG_COLLIDE) || e.triggerstate == TRIGGERED) && (mode&RAY_ENTS)!=RAY_ENTS) continue;
         orient = 0; // FIXME, not set
         if(!mmintersect(e, o, ray, radius, mode, f)) continue;
     );
@@ -204,7 +195,6 @@ static float shadowent(octaentities *oc, octaentities *last, const vec &o, const
     {
         extentity &e = *ents[oc->mapmodels[i]];
         if(!e.inoctanode || &e==t) continue;
-        if(e.attr3 && (e.triggerstate == TRIGGER_DISAPPEARED || !checktriggertype(e.attr3, TRIG_COLLIDE) || e.triggerstate == TRIGGERED)) continue;
         if(!mmintersect(e, o, ray, radius, mode, f)) continue;
         if(f>0 && f<dist) dist = f;
     }
@@ -215,7 +205,7 @@ static float shadowent(octaentities *oc, octaentities *last, const vec &o, const
     octaentities *oclast = NULL; \
     float dist = 0, dent = mode&RAY_BB ? 1e16f : 1e14f; \
     vec v(o), invray(ray.x ? 1/ray.x : 1e16f, ray.y ? 1/ray.y : 1e16f, ray.z ? 1/ray.z : 1e16f); \
-    static cube *levels[32]; \
+    cube *levels[20]; \
     levels[worldscale] = worldroot; \
     int lshift = worldscale; \
     ivec lsizemask(invray.x>0 ? 1 : 0, invray.y>0 ? 1 : 0, invray.z>0 ? 1 : 0); \
@@ -316,7 +306,7 @@ float raycube(const vec &o, const vec &ray, float radius, int mode, int size, ex
         {
             setcubeclip(c, lo, lsize);
             float f = 0;
-            if(raycubeintersect(c, v, ray, f) && (dist+f>0 || !(mode&RAY_SKIPFIRST)))
+            if(raycubeintersect(c, v, ray, invray, f) && (dist+f>0 || !(mode&RAY_SKIPFIRST)))
                 return min(dent, dist+f);
         }
 
@@ -334,7 +324,7 @@ float shadowray(const vec &o, const vec &ray, float radius, int mode, extentity
     INITRAYCUBE;
     CHECKINSIDEWORLD;
 
-    int x = int(v.x), y = int(v.y), z = int(v.z);
+    int side = O_BOTTOM, x = int(v.x), y = int(v.y), z = int(v.z);
     for(;;)
     {
         DOWNOCTREE(shadowent, );
@@ -342,15 +332,69 @@ float shadowray(const vec &o, const vec &ray, float radius, int mode, extentity
         cube &c = *lc;
         ivec lo(x&(~0<<lshift), y&(~0<<lshift), z&(~0<<lshift));
 
-        if(!isempty(c))
+        if(!isempty(c) && !(c.ext && c.ext->material&MAT_ALPHA))
         {
-            if(isentirelysolid(c)) return dist;
+            if(isentirelysolid(c)) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist;
             setcubeclip(c, lo, 1<<lshift);
-            float f = 0;
-            if(shadowcubeintersect(c, v, ray, f)) return dist+f;
+            clipplanes &p = *c.ext->clip;
+            INTERSECTPLANES(side = p.side[i], goto nextcube);
+            INTERSECTBOX(side = (i<<1) + 1 - lsizemask[i], goto nextcube);
+            if(exitdist >= 0) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist+max(enterdist+0.1f, 0.0f);
         }
 
-        FINDCLOSEST( , , );
+    nextcube:
+        FINDCLOSEST(side = O_RIGHT - lsizemask.x, side = O_FRONT - lsizemask.y, side = O_TOP - lsizemask.z);
+
+        if(dist>=radius) return dist;
+
+        UPOCTREE(return radius);
+    }
+}
+
+// thread safe version
+
+struct ShadowRayCache
+{
+    clipplanes clipcache[MAXCLIPPLANES];
+    int version;
+
+    ShadowRayCache() : version(0) {}
+};
+
+ShadowRayCache *newshadowraycache() { return new ShadowRayCache; }
+
+void freeshadowraycache(ShadowRayCache *&cache) { delete cache; cache = NULL; }
+
+void resetshadowraycache(ShadowRayCache *cache)
+{
+    if(!cache->version++) memset(cache->clipcache, 0, sizeof(cache->clipcache));
+}
+
+float shadowray(ShadowRayCache *cache, const vec &o, const vec &ray, float radius, int mode, extentity *t)
+{
+    INITRAYCUBE;
+    CHECKINSIDEWORLD;
+
+    int side = O_BOTTOM, x = int(v.x), y = int(v.y), z = int(v.z);
+    for(;;)
+    {
+        DOWNOCTREE(shadowent, );
+
+        cube &c = *lc;
+        ivec lo(x&(~0<<lshift), y&(~0<<lshift), z&(~0<<lshift));
+
+        if(!isempty(c) && !(c.ext && c.ext->material&MAT_ALPHA))
+        {
+            if(isentirelysolid(c)) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist;
+            clipplanes &p = cache->clipcache[int(&c - worldroot)&(MAXCLIPPLANES-1)];
+            if(p.owner != &c || p.version != cache->version) { p.owner = &c; p.version = cache->version; genclipplanes(c, lo.x, lo.y, lo.z, 1<<lshift, p); }
+            INTERSECTPLANES(side = p.side[i], goto nextcube);
+            INTERSECTBOX(side = (i<<1) + 1 - lsizemask[i], goto nextcube);
+            if(exitdist >= 0) return c.texture[side]==DEFAULT_SKY && mode&RAY_SKIPSKY ? radius : dist+max(enterdist+0.1f, 0.0f);
+        }
+
+    nextcube:
+        FINDCLOSEST(side = O_RIGHT - lsizemask.x, side = O_FRONT - lsizemask.y, side = O_TOP - lsizemask.z);
 
         if(dist>=radius) return dist;
 
@@ -377,6 +421,7 @@ float raycubepos(const vec &o, const vec &ray, vec &hitpos, float radius, int mo
 {
     hitpos = ray;
     float dist = raycube(o, ray, radius, mode, size);
+    if(radius>0 && dist>=radius) dist = radius;
     hitpos.mul(dist).add(o);
     return dist;
 }
@@ -565,7 +610,7 @@ const vector<physent *> &checkdynentcache(int x, int y)
     dec.x = x;
     dec.y = y;
     dec.frame = dynentframe;
-    dec.dynents.setsize(0);
+    dec.dynents.shrink(0);
     int numdyns = game::numdynents(), dsize = 1<<dynentsize, dx = x<<dynentsize, dy = y<<dynentsize;
     loopi(numdyns)
     {
@@ -655,7 +700,7 @@ bool plcollide(physent *d, const vec &dir)    // collide with player or monster
                     break;
             }
             hitplayer = o;
-            if((d->type==ENT_AI || d->type==ENT_INANIMATE) && wall.z>0) d->onplayer = o;
+            game::dynentcollide(d, o, wall);
             return false;
         }
     }
@@ -693,7 +738,6 @@ void rotatebb(vec &center, vec &radius, int yaw)
         vec2(1.00000000, 0.00000000) // 360
     };
 
-    yaw += 180;
     if(yaw < 0) yaw = 360 + yaw%360;
     else if(yaw >= 360) yaw %= 360;
     const vec2 &rot = rots[(yaw + 7)/15];
@@ -726,12 +770,12 @@ bool mmcollide(physent *d, const vec &dir, octaentities &oc)               // co
     loopv(oc.mapmodels)
     {
         extentity &e = *ents[oc.mapmodels[i]];
-        if(e.attr3 && e.attr3!=15 && (e.triggerstate == TRIGGER_DISAPPEARED || !checktriggertype(e.attr3, TRIG_COLLIDE) || e.triggerstate == TRIGGERED || (e.triggerstate == TRIGGERING && lastmillis-e.lasttrigger >= 500))) continue;
-        model *m = loadmodel(NULL, e.attr2);
+        if(e.flags&extentity::F_NOCOLLIDE) continue;
+        model *m = loadmodel(NULL, e.attr[1]);
         if(!m || !m->collide) continue;
         vec center, radius;
         m->collisionbox(0, center, radius);
-        float yaw = 180 + float((e.attr1+7)-(e.attr1+7)%15);
+        float yaw = float((e.attr[0]+7)-(e.attr[0]+7)%15);
         switch(d->collidetype)
         {
             case COLLIDE_ELLIPSE:
@@ -752,7 +796,7 @@ bool mmcollide(physent *d, const vec &dir, octaentities &oc)               // co
                 break;
             case COLLIDE_AABB:
             default:
-                rotatebb(center, radius, e.attr1);
+                rotatebb(center, radius, e.attr[0]);
                 if(!rectcollide(d, dir, center.add(e.o), radius.x, radius.y, radius.z, radius.z)) return false;
                 break;
         }
@@ -1150,11 +1194,12 @@ bool trystepup(physent *d, vec &dir, const vec &obstacle, float maxstep, const v
 
     if(cansmooth)
     {
-        d->o = old;
         vec checkdir = stairdir;
         checkdir.z += 1;
         checkdir.mul(maxstep);
+        d->o = old;
         d->o.add(checkdir);
+        int scale = 2;
         if(!collide(d, checkdir))
         {
             if(collide(d, vec(0, 0, -1), SLOPEZ))
@@ -1162,18 +1207,25 @@ bool trystepup(physent *d, vec &dir, const vec &obstacle, float maxstep, const v
                 d->o = old;
                 return false;
             }
+            d->o.add(checkdir);
+            if(!collide(d, vec(0, 0, -1), SLOPEZ)) scale = 1;
+        }
+        if(scale != 1)
+        {
+            d->o = old;
+            d->o.sub(checkdir.mul(vec(2, 2, 1)));
+            if(collide(d, vec(0, 0, -1), SLOPEZ)) scale = 1;
         }
 
-        /* try stepping up half as much as forward */
         d->o = old;
         vec smoothdir(dir.x, dir.y, 0);
         float magxy = smoothdir.magnitude();
         if(magxy > 1e-9f)
         {
-            if(magxy > 2*dir.z)
+            if(magxy > scale*dir.z)
             {
                 smoothdir.mul(1/magxy);
-                smoothdir.z = 0.5f;
+                smoothdir.z = 1.0f/scale;
                 smoothdir.mul(dir.magnitude()/smoothdir.magnitude());
             }
             else smoothdir.z = dir.z;
@@ -1212,8 +1264,7 @@ bool trystepup(physent *d, vec &dir, const vec &obstacle, float maxstep, const v
     return false;
 }
 
-#if 0
-bool trystepdown(physent *d, const vec &dir, float step, float xy, float z)
+bool trystepdown(physent *d, vec &dir, float step, float xy, float z, bool init = false)
 {
     vec stepdir(dir.x, dir.y, 0);
     stepdir.z = -stepdir.magnitude2()*z/xy;
@@ -1222,40 +1273,76 @@ bool trystepdown(physent *d, const vec &dir, float step, float xy, float z)
 
     vec old(d->o);
     d->o.add(vec(stepdir).mul(STAIRHEIGHT/fabs(stepdir.z))).z -= STAIRHEIGHT;
+    d->zmargin = -STAIRHEIGHT;
     if(!collide(d, vec(0, 0, -1), SLOPEZ))
     {
         d->o = old;
         d->o.add(vec(stepdir).mul(step));
+        d->zmargin = 0;
         if(collide(d, vec(0, 0, -1)))
         {
-            stepdir.mul(-stepdir.z).z += 1;
-            stepdir.normalize();
-            switchfloor(d, dir, stepdir);
-            d->floor = stepdir;
+            vec stepfloor(stepdir);
+            stepfloor.mul(-stepfloor.z).z += 1;
+            stepfloor.normalize();
+            if(d->physstate >= PHYS_SLOPE && d->floor != stepfloor)
+            {
+                // prevent alternating step-down/step-up states if player would keep bumping into the same floor
+                vec stepped(d->o);
+                d->o.z -= 0.5f;
+                d->zmargin = -0.5f;
+                if(!collide(d, stepdir) && wall == d->floor)
+                {
+                    d->o = old;
+                    if(!init) { d->o.x += dir.x; d->o.y += dir.y; if(dir.z <= 0 || !collide(d, dir)) d->o.z += dir.z; }
+                    d->zmargin = 0;
+                    d->physstate = PHYS_STEP_DOWN;
+                    d->timeinair = 0;
+                    return true;
+                }
+                d->o = init ? old : stepped;
+                d->zmargin = 0;
+            }
+            else if(init) d->o = old;
+            switchfloor(d, dir, stepfloor);
+            d->floor = stepfloor;
+            d->physstate = PHYS_STEP_DOWN;
+            d->timeinair = 0;
             return true;
         }
     }
     d->o = old;
+    d->zmargin = 0;
     return false;
 }
+
+bool trystepdown(physent *d, vec &dir, bool init = false)
+{
+    if(!game::allowmove(d) || (!d->move && !d->strafe)) return false;
+    vec old(d->o);
+    d->o.z -= STAIRHEIGHT;
+    d->zmargin = -STAIRHEIGHT;
+    if(collide(d, vec(0, 0, -1), SLOPEZ))
+    {
+        d->o = old;
+        d->zmargin = 0;
+        return false;
+    }
+    d->o = old;
+    d->zmargin = 0;
+    float step = dir.magnitude();
+#if 1
+    // weaker check, just enough to avoid hopping up slopes
+    if(trystepdown(d, dir, step, 4, 1, init)) return true;
+#else
+    if(trystepdown(d, dir, step, 2, 1, init)) return true;
+    if(trystepdown(d, dir, step, 1, 1, init)) return true;
+    if(trystepdown(d, dir, step, 1, 2, init)) return true;
 #endif
+    return false;
+}
 
 void falling(physent *d, vec &dir, const vec &floor)
 {
-#if 0
-    if(d->physstate >= PHYS_FLOOR && (d->physstate != PHYS_STEP_DOWN || dir.z < -0.25f*dir.magnitude2()) && (floor.z == 0.0f || floor.z == 1.0f))
-    {
-        vec moved(d->o);
-        d->o.z -= STAIRHEIGHT + 0.1f;
-        if(!collide(d, vec(0, 0, -1), SLOPEZ))
-        {
-            d->o = moved;
-            d->physstate = PHYS_STEP_DOWN;
-            return;
-        }
-        else d->o = moved;
-    }
-#endif
     if(floor.z > 0.0f && floor.z < SLOPEZ)
     {
         if(floor.z >= WALLZ) switchfloor(d, dir, floor);
@@ -1263,7 +1350,8 @@ void falling(physent *d, vec &dir, const vec &floor)
         d->physstate = PHYS_SLIDE;
         d->floor = floor;
     }
-    else d->physstate = PHYS_FALL;
+    else if(d->physstate < PHYS_SLOPE || dir.dot(d->floor) > 0.01f*dir.magnitude() || (floor.z != 0.0f && floor.z != 1.0f) || !trystepdown(d, dir, true))
+        d->physstate = PHYS_FALL;
 }
 
 void landing(physent *d, vec &dir, const vec &floor, bool collided)
@@ -1277,7 +1365,7 @@ void landing(physent *d, vec &dir, const vec &floor, bool collided)
 #endif
     switchfloor(d, dir, floor);
     d->timeinair = 0;
-    if(d->physstate!=PHYS_STEP_UP || !collided)
+    if((d->physstate!=PHYS_STEP_UP && d->physstate!=PHYS_STEP_DOWN) || !collided)
         d->physstate = floor.z >= FLOORZ ? PHYS_FLOOR : PHYS_SLOPE;
     d->floor = floor;
 }
@@ -1287,7 +1375,7 @@ bool findfloor(physent *d, bool collided, const vec &obstacle, bool &slide, vec
     bool found = false;
     vec moved(d->o);
     d->o.z -= 0.1f;
-    if(!collide(d, vec(0, 0, -1), d->physstate == PHYS_SLOPE ? SLOPEZ : FLOORZ))
+    if(!collide(d, vec(0, 0, -1), d->physstate == PHYS_SLOPE || d->physstate == PHYS_STEP_DOWN ? SLOPEZ : FLOORZ))
     {
         floor = wall;
         found = true;
@@ -1326,18 +1414,6 @@ bool findfloor(physent *d, bool collided, const vec &obstacle, bool &slide, vec
 bool move(physent *d, vec &dir)
 {
     vec old(d->o);
-#if 0
-    if(d->physstate == PHYS_STEP_DOWN && dir.z <= 0.0f && game::allowmove(pl) && (d->move || d->strafe))
-    {
-        float step = dir.magnitude();
-        if(trystepdown(d, dir, step, 2, 1)) return true;
-        if(trystepdown(d, dir, step, 1, 1)) return true;
-        if(trystepdown(d, dir, step, 1, 2)) return true;
-        d->o.z -= step;
-        if(collide(d, vec(0, 0, -1))) return true;
-        d->o = old;
-    }
-#endif
     bool collided = false, slidecollide = false;
     vec obstacle;
     d->o.add(dir);
@@ -1353,7 +1429,7 @@ bool move(physent *d, vec &dir)
         d->o = old;
         d->o.z -= STAIRHEIGHT;
         d->zmargin = -STAIRHEIGHT;
-        if(d->physstate == PHYS_SLOPE || d->physstate == PHYS_FLOOR || (!collide(d, vec(0, 0, -1), SLOPEZ) && (d->physstate==PHYS_STEP_UP || wall.z>=FLOORZ)))
+        if(d->physstate == PHYS_SLOPE || d->physstate == PHYS_FLOOR || (!collide(d, vec(0, 0, -1), SLOPEZ) && (d->physstate==PHYS_STEP_UP || d->physstate==PHYS_STEP_DOWN || wall.z>=FLOORZ)))
         {
             d->o = old;
             d->zmargin = 0;
@@ -1376,6 +1452,13 @@ bool move(physent *d, vec &dir)
             d->o.add(dir);
         }
     }
+    else if(d->physstate == PHYS_STEP_DOWN && dir.dot(d->floor) <= 1e-6f)
+    {
+        vec moved(d->o);
+        d->o = old;
+        if(trystepdown(d, dir)) return true;
+        d->o = moved;
+    }
     vec floor(0, 0, 0);
     bool slide = collided,
          found = findfloor(d, collided, obstacle, slide, floor);
@@ -1419,6 +1502,7 @@ bool bounce(physent *d, float secs, float elasticity, float waterfric)
         }
         else if(hitplayer) break;
         d->o = old;
+        game::bounced(d, wall);
         float c = wall.dot(d->vel),
               k = 1.0f + (1.0f-elasticity)*c/d->vel.magnitude();
         d->vel.mul(k);
@@ -1480,15 +1564,25 @@ bool movecamera(physent *pl, const vec &dir, float dist, float stepdist)
 
 bool droptofloor(vec &o, float radius, float height)
 {
-    if(!insideworld(o)) return false;
+    static struct dropent : physent
+    {
+        dropent()
+        {
+            type = ENT_CAMERA;
+            collidetype = COLLIDE_AABB;
+            vel = vec(0, 0, -1);
+        }
+    } d;
+    d.o = o;
+    if(!insideworld(d.o))
+    {
+        if(d.o.z < worldsize) return false;
+        d.o.z = worldsize - 1e-3f;
+        if(!insideworld(d.o)) return false;
+    }
     vec v(0.0001f, 0.0001f, -1);
     v.normalize();
-    if(raycube(o, v, worldsize) >= worldsize) return false;
-    physent d;
-    d.type = ENT_CAMERA;
-    d.collidetype = COLLIDE_AABB;
-    d.o = o;
-    d.vel = vec(0, 0, -1);
+    if(raycube(d.o, v, worldsize) >= worldsize) return false;
     d.radius = d.xradius = d.yradius = radius;
     d.eyeheight = height;
     d.aboveeye = radius;
@@ -1530,8 +1624,8 @@ void vecfromyawpitch(float yaw, float pitch, int move, int strafe, vec &m)
 {
     if(move)
     {
-        m.x = move*sinf(RAD*yaw);
-        m.y = move*-cosf(RAD*yaw);
+        m.x = move*-sinf(RAD*yaw);
+        m.y = move*cosf(RAD*yaw);
     }
     else m.x = m.y = 0;
 
@@ -1545,14 +1639,14 @@ void vecfromyawpitch(float yaw, float pitch, int move, int strafe, vec &m)
 
     if(strafe)
     {
-        m.x += strafe*-cosf(RAD*yaw);
-        m.y += strafe*-sinf(RAD*yaw);
+        m.x += strafe*cosf(RAD*yaw);
+        m.y += strafe*sinf(RAD*yaw);
     }
 }
 
 void vectoyawpitch(const vec &v, float &yaw, float &pitch)
 {
-    yaw = -(float)atan2(v.x, v.y)/RAD + 180;
+    yaw = -atan2(v.x, v.y)/RAD;
     pitch = asin(v.z/v.magnitude())/RAD;
 }
 
@@ -1567,7 +1661,7 @@ void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curt
         if(pl->jumping)
         {
             pl->jumping = false;
-            pl->vel.z = pl->jumpvel * (jumpvel / 200.0f);
+            pl->vel.z = max(pl->vel.z, pl->jumpvel * (jumpvel / 200.0f));
             pl->lastjump = lastmillis;
         }
     }
@@ -1578,13 +1672,13 @@ void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curt
         {
             pl->jumping = false;
 
-		if(pl->physstate < PHYS_SLOPE && !water)
-		{
-			particle_splash(PART_STEAM, 80, 200, vec(pl->o.x, pl->o.y, pl->o.z-pl->eyeheight), 0xBFBFBF, 2, 150, -10); //double jump
-			pl->falling.div(8);
-		}
+            if(pl->physstate < PHYS_SLOPE && !water)
+            {
+                particle_splash(PART_STEAM, 80, 200, vec(pl->o.x, pl->o.y, pl->o.z-pl->eyeheight), 0xBFBFBF, 2, 150, -10); //double jump
+                pl->falling.div(8);
+            }
 
-            pl->vel.z = pl->jumpvel * (jumpvel / 200.0f); // physics impulse upwards
+            pl->vel.z = max(pl->vel.z, pl->jumpvel * (jumpvel / 200.0f)); // physics impulse upwards
             pl->lastjump = lastmillis;
             if(water) { pl->vel.x /= 8.0f; pl->vel.y /= 8.0f; } // dampen velocity change even harder, gives correct water feel
 
@@ -1594,30 +1688,7 @@ void modifyvelocity(physent *pl, bool local, bool water, bool floating, int curt
     if(!floating && pl->physstate == PHYS_FALL) pl->timeinair += curtime;
 
     vec m(0.0f, 0.0f, 0.0f);
-    if(pl->type==ENT_AI)
-    {
-        dynent *d = (dynent *)pl;
-        if(d->rotspeed && d->yaw!=d->targetyaw)
-        {
-            float oldyaw = d->yaw, diff = d->rotspeed*curtime/1000.0f, maxdiff = fabs(d->targetyaw-d->yaw);
-            if(diff >= maxdiff)
-            {
-                d->yaw = d->targetyaw;
-                d->rotspeed = 0;
-            }
-            else d->yaw += (d->targetyaw>d->yaw ? 1 : -1) * min(diff, maxdiff);
-            d->normalize_yaw(d->targetyaw);
-            if(!plcollide(d, vec(0, 0, 0)))
-            {
-                d->yaw = oldyaw;
-                m.x = d->o.x - hitplayer->o.x;
-                m.y = d->o.y - hitplayer->o.y;
-                if(!m.iszero()) m.normalize();
-            }
-        }
-    }
-
-    if(m.iszero() && game::allowmove(pl) && (pl->move || pl->strafe))
+    if(game::allowmove(pl) && (pl->move || pl->strafe))
     {
         vecfromyawpitch(pl->yaw, floating || water || pl->type==ENT_CAMERA ? pl->pitch : 0, pl->move, pl->strafe, m);
 
@@ -1709,8 +1780,6 @@ bool moveplayer(physent *pl, int moveres, bool local, int curtime)
     d.mul(secs);
 
     pl->blocked = false;
-    pl->moving = true;
-    pl->onplayer = NULL;
 
     if(floating)                // just apply velocity
     {
@@ -1738,11 +1807,6 @@ bool moveplayer(physent *pl, int moveres, bool local, int curtime)
 
     if(pl->state==CS_ALIVE) updatedynentcache(pl);
 
-    if(!pl->timeinair && pl->physstate >= PHYS_FLOOR && pl->vel.squaredlen() < 1e-4f) pl->moving = false;
-
-    pl->lastmoveattempt = lastmillis;
-    if(pl->o!=oldpos) pl->lastmove = lastmillis;
-
     // automatically apply smooth roll when strafing
 
     if(pl->strafe==0)
@@ -1859,8 +1923,9 @@ void updatephysstate(physent *d)
     {
         case PHYS_SLOPE:
         case PHYS_FLOOR:
+        case PHYS_STEP_DOWN:
             d->o.z -= 0.15f;
-            if(!collide(d, vec(0, 0, -1), d->physstate == PHYS_SLOPE ? SLOPEZ : FLOORZ))
+            if(!collide(d, vec(0, 0, -1), d->physstate == PHYS_SLOPE || d->physstate == PHYS_STEP_DOWN ? SLOPEZ : FLOORZ))
                 d->floor = wall;
             break;
 
@@ -1942,7 +2007,7 @@ bool moveplatform(physent *p, const vec &dir)
     p->o = oldpos;
 
     static vector<platforment> ents;
-    ents.setsizenodelete(0);
+    ents.setsize(0);
     for(int x = int(max(p->o.x-p->radius-PLATFORMBORDER, 0.0f))>>dynentsize, ex = int(min(p->o.x+p->radius+PLATFORMBORDER, worldsize-1.0f))>>dynentsize; x <= ex; x++)
     for(int y = int(max(p->o.y-p->radius-PLATFORMBORDER, 0.0f))>>dynentsize, ey = int(min(p->o.y+p->radius+PLATFORMBORDER, worldsize-1.0f))>>dynentsize; y <= ey; y++)
     {
@@ -1955,10 +2020,10 @@ bool moveplatform(physent *p, const vec &dir)
         }
     }
     static vector<platforment *> passengers, colliders;
-    passengers.setsizenodelete(0);
-    colliders.setsizenodelete(0);
+    passengers.setsize(0);
+    colliders.setsize(0);
     static vector<platformcollision> collisions;
-    collisions.setsizenodelete(0);
+    collisions.setsize(0);
     // build up collision DAG of colliders to be pushed off, and DAG of stacked passengers
     loopv(ents)
     {
@@ -2011,7 +2076,6 @@ bool moveplatform(physent *p, const vec &dir)
             physent *d = passengers[i]->d;
             d->o.add(dir);
             d->newpos.add(dir);
-            d->lastmove = lastmillis;
             if(dir.x || dir.y) updatedynentcache(d);
         }
     }
@@ -2023,7 +2087,6 @@ bool moveplatform(physent *p, const vec &dir)
         physent *d = ent->d;
         d->o.add(dir);
         d->newpos.add(dir);
-        d->lastmove = lastmillis;
         if(dir.x || dir.y) updatedynentcache(d);
 
         for(int n = ent->stacks; n>=0; n = collisions[n].next)
@@ -2035,7 +2098,6 @@ bool moveplatform(physent *p, const vec &dir)
 
     p->o.add(dir);
     p->newpos.add(dir);
-    p->lastmove = lastmillis;
     if(dir.x || dir.y) updatedynentcache(p);
 
     return true;
