@@ -29,11 +29,15 @@ static int __dummy__##modelclass = addmodeltype((modeltype), __loadmodel__##mode
 #include "md3.h"
 #include "md5.h"
 #include "obj.h"
+#include "smd.h"
+#include "iqm.h"
 
 MODELTYPE(MDL_MD2, md2);
 MODELTYPE(MDL_MD3, md3);
 MODELTYPE(MDL_MD5, md5);
 MODELTYPE(MDL_OBJ, obj);
+MODELTYPE(MDL_SMD, smd);
+MODELTYPE(MDL_IQM, iqm);
 
 #define checkmdl if(!loadingmodel) { conoutf(CON_ERROR, "not loading a model"); return; }
 
@@ -158,14 +162,15 @@ void mdlshader(char *shader)
 
 COMMAND(mdlshader, "s");
 
-void mdlspin(float *yaw, float *pitch)
+void mdlspin(float *yaw, float *pitch, float *roll)
 {
     checkmdl;
     loadingmodel->spinyaw = *yaw;
     loadingmodel->spinpitch = *pitch;
+    loadingmodel->spinroll = *roll;
 }
 
-COMMAND(mdlspin, "ff");
+COMMAND(mdlspin, "fff");
 
 void mdlscale(int *percent)
 {
@@ -202,6 +207,14 @@ void mdlpitch(float *angle)
 
 COMMAND(mdlpitch, "f");
 
+void mdlroll(float *angle)
+{
+	checkmdl;
+	loadingmodel->offsetroll = *angle;
+}
+
+COMMAND(mdlroll, "f");
+
 void mdlshadow(int *shadow)
 {
     checkmdl;
@@ -279,9 +292,9 @@ void rdjoint(int *n, int *t, char *v1, char *v2, char *v3)
     ragdollskel::joint &j = ragdoll->joints.add();
     j.bone = *n;
     j.tri = *t;
-    j.vert[0] = v1[0] ? atoi(v1) : -1;
-    j.vert[1] = v2[0] ? atoi(v2) : -1;
-    j.vert[2] = v3[0] ? atoi(v3) : -1;
+    j.vert[0] = v1[0] ? parseint(v1) : -1;
+    j.vert[1] = v2[0] ? parseint(v2) : -1;
+    j.vert[2] = v3[0] ? parseint(v3) : -1;
 }
 COMMAND(rdjoint, "iisss");
 
@@ -307,6 +320,13 @@ void rdlimitrot(int *t1, int *t2, float *maxangle, float *qx, float *qy, float *
 }
 COMMAND(rdlimitrot, "iifffff");
 
+void rdanimjoints(int *on)
+{
+    checkragdoll;
+    ragdoll->animjoints = *on!=0;
+}
+COMMAND(rdanimjoints, "i");
+
 // mapmodels
 
 vector<mapmodelinfo> mapmodels;
@@ -323,10 +343,10 @@ void mapmodelcompat(int *rad, int *h, int *tex, char *name, char *shadow)
     mmodel(name);
 }
 
-void mapmodelreset()
+void mapmodelreset(int *n)
 {
-    if(!overrideidents && !game::allowedittoggle()) return;
-    mapmodels.setsize(0);
+    if(!(identflags&IDF_OVERRIDDEN) && !game::allowedittoggle()) return;
+    mapmodels.shrink(clamp(*n, 0, mapmodels.length()));
 }
 
 mapmodelinfo &getmminfo(int i) { return mapmodels.inrange(i) ? mapmodels[i] : *(mapmodelinfo *)0; }
@@ -334,7 +354,7 @@ const char *mapmodelname(int i) { return mapmodels.inrange(i) ? mapmodels[i].nam
 
 COMMAND(mmodel, "s");
 COMMANDN(mapmodel, mapmodelcompat, "iiiss");
-COMMAND(mapmodelreset, "");
+COMMAND(mapmodelreset, "i");
 ICOMMAND(mapmodelname, "i", (int *index), { result(mapmodels.inrange(*index) ? mapmodels[*index].name : ""); });
 ICOMMAND(nummapmodels, "", (), { intret(mapmodels.length()); });
 
@@ -356,7 +376,29 @@ void flushpreloadedmodels()
         loadprogress = float(i+1)/preloadmodels.length();
         loadmodel(preloadmodels[i], -1, true);
     }
-    preloadmodels.deletecontentsa();
+    preloadmodels.deletearrays();
+    loadprogress = 0;
+}
+
+void preloadusedmapmodels(bool msg, bool bih)
+{
+    vector<extentity *> &ents = entities::getents();
+    vector<int> mapmodels;
+    loopv(ents)
+    {
+        extentity &e = *ents[i];
+        if(e.type==ET_MAPMODEL && e.attr[1] >= 0 && mapmodels.find(e.attr[1]) < 0) mapmodels.add(e.attr[1]);
+    }
+
+    loopv(mapmodels)
+    {
+        loadprogress = float(i+1)/mapmodels.length();
+        int mmindex = mapmodels[i];
+        mapmodelinfo &mmi = getmminfo(mmindex);
+        if(!&mmi) { if(msg) conoutf(CON_WARN, "could not find map model: %d", mmindex); }
+        else if(!loadmodel(NULL, mmindex, msg)) { if(msg) conoutf(CON_WARN, "could not load model: %s", mmi.name); }
+        else if(mmi.m && bih) mmi.m->preloadBIH();
+    }
     loadprogress = 0;
 }
 
@@ -374,6 +416,7 @@ model *loadmodel(const char *name, int i, bool msg)
     if(mm) m = *mm;
     else
     {
+        if(lightmapping > 1) return NULL;
         if(msg)
         {
             defformatstring(filename)("packages/models/%s", name);
@@ -450,7 +493,7 @@ void render3dbox(vec &o, float tofloor, float toceil, float xradius, float yradi
     c.sub(vec(xradius, yradius, tofloor));
     float xsz = xradius*2, ysz = yradius*2;
     float h = tofloor+toceil;
-    notextureshader->set();
+    lineshader->set();
     glDisable(GL_TEXTURE_2D);
     glColor3f(1, 1, 1);
     render2dbox(c, xsz, 0, h);
@@ -464,7 +507,7 @@ void render3dbox(vec &o, float tofloor, float toceil, float xradius, float yradi
 
 void renderellipse(vec &o, float xradius, float yradius, float yaw)
 {
-    notextureshader->set();
+    lineshader->set();
     glDisable(GL_TEXTURE_2D);
     glColor3f(0.5f, 0.5f, 0.5f);
     glBegin(GL_LINE_LOOP);
@@ -483,7 +526,7 @@ struct batchedmodel
 {
     vec pos, color, dir;
     int anim;
-    float yaw, pitch, transparent;
+    float yaw, pitch, roll, transparent;
     int basetime, basetime2, flags;
     dynent *d;
     int attached;
@@ -503,7 +546,7 @@ static occludequery *modelquery = NULL;
 void startmodelbatches()
 {
     numbatches = 0;
-    modelattached.setsizenodelete(0);
+    modelattached.setsize(0);
 }
 
 modelbatch &addbatchedmodel(model *m)
@@ -515,7 +558,7 @@ modelbatch &addbatchedmodel(model *m)
         if(numbatches<batches.length())
         {
             b = batches[numbatches];
-            b->batched.setsizenodelete(0);
+            b->batched.setsize(0);
         }
         else b = batches.add(new modelbatch);
         b->m = m;
@@ -539,9 +582,10 @@ void renderbatchedmodel(model *m, batchedmodel &b)
     else
     {
         if(b.flags&MDL_FULLBRIGHT) anim |= ANIM_FULLBRIGHT;
+        if(b.flags&MDL_GHOST) anim |= ANIM_GHOST;
     }
 
-    m->render(anim, b.basetime, b.basetime2, b.pos, b.yaw, b.pitch, b.d, a, b.color, b.dir, b.transparent);
+    m->render(anim, b.basetime, b.basetime2, b.pos, b.yaw, b.pitch, b.roll, b.d, a, b.color, b.dir, b.transparent);
 }
 
 struct transparentmodel
@@ -579,10 +623,25 @@ void endmodelbatches()
         }
         bool rendered = false;
         occludequery *query = NULL;
+        if(b.flags&MDL_GHOST)
+        {
+            loopvj(b.batched)
+            {
+                batchedmodel &bm = b.batched[j];
+                if((bm.flags&(MDL_CULL_VFC|MDL_GHOST))!=MDL_GHOST || bm.query) continue;
+                if(!rendered) { b.m->startrender(); rendered = true; }
+                renderbatchedmodel(b.m, bm);
+            }
+            if(rendered)
+            {
+                b.m->endrender();
+                rendered = false;
+            }
+        }
         loopvj(b.batched)
         {
             batchedmodel &bm = b.batched[j];
-            if(bm.flags&MDL_CULL_VFC) continue;
+            if(bm.flags&(MDL_CULL_VFC|MDL_GHOST)) continue;
             if(bm.query!=query)
             {
                 if(query) endquery(query);
@@ -668,7 +727,7 @@ void endmodelquery()
     }
     endquery(modelquery);
     modelquery = NULL;
-    modelattached.setsizenodelete(minattached);
+    modelattached.setsize(minattached);
 }
 
 VARP(maxmodelradiusdistance, 10, 200, 1000);
@@ -697,7 +756,7 @@ void rendermodelquery(model *m, dynent *d, const vec &center, float radius)
 
 extern int oqfrags;
 
-void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, float yaw, float pitch, int flags, dynent *d, modelattach *a, int basetime, int basetime2, float trans)
+void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, float yaw, float pitch, float roll, int flags, dynent *d, modelattach *a, int basetime, int basetime2, float trans, int colour)
 {
     if(shadowmapping && !(flags&(MDL_SHADOW|MDL_DYNSHADOW))) return;
     model *m = loadmodel(mdl);
@@ -717,7 +776,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         }
         else
         {
-            center.rotate_around_z((-180-yaw)*RAD);
+            center.rotate_around_z(-yaw*RAD);
             center.add(o);
         }
         if(flags&MDL_CULL_DIST && center.dist(camera1->o)/radius>maxmodelradiusdistance) return;
@@ -769,7 +828,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
     }
 
     if(flags&MDL_NORENDER) anim |= ANIM_NORENDER;
-    else if(showboundingbox && !shadowmapping && !reflecting && !refracting)
+    else if(showboundingbox && !shadowmapping && !reflecting && !refracting && editmode)
     {
         if(d && showboundingbox==1)
         {
@@ -803,8 +862,8 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
                     pos.z += radius/2;
                 }
                 else if(d->type < ENT_CAMERA) pos.z += 0.75f*(d->eyeheight + d->aboveeye);
-                lightreaching(pos, light->color, light->dir);
-                dynlightreaching(pos, light->color, light->dir);
+                lightreaching(pos, light->color, light->dir, (flags&MDL_LIGHT_FAST)!=0);
+                dynlightreaching(pos, light->color, light->dir, (flags&MDL_HUD)!=0);
                 game::lighteffects(d, light->color, light->dir);
                 light->millis = lastmillis;
             }
@@ -813,18 +872,18 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         {
             if(!light)
             {
-                lightreaching(pos, lightcolor, lightdir);
-                dynlightreaching(pos, lightcolor, lightdir);
+                lightreaching(pos, lightcolor, lightdir, (flags&MDL_LIGHT_FAST)!=0);
+                dynlightreaching(pos, lightcolor, lightdir, (flags&MDL_HUD)!=0);
             }
             else if(light->millis!=lastmillis)
             {
-                lightreaching(pos, light->color, light->dir);
-                dynlightreaching(pos, light->color, light->dir);
+                lightreaching(pos, light->color, light->dir, (flags&MDL_LIGHT_FAST)!=0);
+                dynlightreaching(pos, light->color, light->dir, (flags&MDL_HUD)!=0);
                 light->millis = lastmillis;
             }
         }
         if(light) { lightcolor = light->color; lightdir = light->dir; }
-        if(flags&MDL_DYNLIGHT) dynlightreaching(pos, lightcolor, lightdir);
+        if(flags&MDL_DYNLIGHT) dynlightreaching(pos, lightcolor, lightdir, (flags&MDL_HUD)!=0);
     }
 
     if(a) for(int i = 0; a[i].tag; i++)
@@ -835,10 +894,13 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
 
     if(!d || reflecting || refracting || shadowmapping) doOQ = false;
 
+    lightcolor.mul(vec((colour >> 16) & 0xFF, (colour >> 8) & 0xFF, colour & 0xFF).div(255.0f));
+
     if(numbatches>=0)
     {
         modelbatch &mb = addbatchedmodel(m);
         batchedmodel &b = mb.batched.add();
+
         b.query = modelquery;
         b.pos = o;
         b.color = lightcolor;
@@ -846,6 +908,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         b.anim = anim;
         b.yaw = yaw;
         b.pitch = pitch;
+	b.roll = roll;
         b.basetime = basetime;
         b.basetime2 = basetime2;
         b.transparent = trans;
@@ -880,6 +943,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
     else
     {
         if(flags&MDL_FULLBRIGHT) anim |= ANIM_FULLBRIGHT;
+        if(flags&MDL_GHOST) anim |= ANIM_GHOST;
     }
 
     if(doOQ)
@@ -888,7 +952,7 @@ void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, fl
         if(d->query) startquery(d->query);
     }
 
-    m->render(anim, basetime, basetime2, o, yaw, pitch, d, a, lightcolor, lightdir, trans);
+    m->render(anim, basetime, basetime2, o, yaw, pitch, roll, d, a, lightcolor, lightdir, trans);
 
     if(doOQ && d->query) endquery(d->query);
 
@@ -929,37 +993,24 @@ bool matchanim(const char *name, const char *pattern)
 
 void findanims(const char *pattern, vector<int> &anims)
 {
-    static const char *names[] =
-    {
-        "dead", "dying", "idle",
-        "forward", "backward", "left", "right",
-        "hold 1", "hold 2", "hold 3", "hold 4", "hold 5", "hold 6", "hold 7",
-        "attack 1", "attack 2", "attack 3", "attack 4", "attack 5", "attack 6", "attack 7",
-        "pain",
-        "jump", "sink", "swim",
-        "edit", "lag", "taunt", "win", "lose",
-        "gun idle", "gun shoot",
-        "vwep idle", "vwep shoot", "shield", "powerup",
-        "mapmodel", "trigger"
-    };
-    loopi(sizeof(names)/sizeof(names[0])) if(matchanim(names[i], pattern)) anims.add(i);
+    loopi(sizeof(animnames)/sizeof(animnames[0])) if(matchanim(animnames[i], pattern)) anims.add(i);
 }
 
 void loadskin(const char *dir, const char *altdir, Texture *&skin, Texture *&masks) // model skin sharing
 {
 #define ifnoload(tex, path) if((tex = textureload(path, 0, true, false))==notexture)
 
-#define tryload(tex, cmd, name) \
-    ifnoload(tex, makerelpath(mdir, name "", NULL, cmd)) \
+#define tryload(tex, prefix, cmd, name) \
+    ifnoload(tex, makerelpath(mdir, name "", prefix, cmd)) \
     { \
-        ifnoload(tex, makerelpath(maltdir, name "", NULL, cmd)) return; \
+        ifnoload(tex, makerelpath(maltdir, name "", prefix, cmd)) return; \
     }
 
     defformatstring(mdir)("packages/models/%s", dir);
     defformatstring(maltdir)("packages/models/%s", altdir);
     masks = notexture;
-    tryload(skin, NULL, "skin");
-    tryload(masks, "<ffmask:25>", "masks");
+    tryload(skin, NULL, NULL, "skin");
+    tryload(masks, "<stub>", NULL, "masks");
 }
 
 // convenient function that covers the usual anims for players/monsters/npcs
@@ -968,7 +1019,7 @@ VAR(animoverride, -1, 0, NUMANIMS-1);
 VAR(testanims, 0, 0, 1);
 VAR(testpitch, -90, 0, 90);
 
-void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int hold, int attack, int attackdelay, int lastaction, int lastpain, float fade, bool ragdoll)
+void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int hold, int attack, int attackdelay, int lastaction, int lastpain, float fade, bool ragdoll, int colour)
 {
     int anim = hold ? hold : ANIM_IDLE|ANIM_LOOP;
     float yaw = testanims && d==player ? 0 : d->yaw+90,
@@ -1024,7 +1075,7 @@ void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int
     else flags |= MDL_CULL_DIST;
     if(d->state==CS_LAGGED) fade = min(fade, 0.3f);
     else flags |= MDL_DYNSHADOW;
-    rendermodel(NULL, mdlname, anim, o, yaw, pitch, flags, d, attachments, basetime, 0, fade);
+    rendermodel(NULL, mdlname, anim, o, yaw, pitch, d->roll, flags, d, attachments, basetime, 0, fade, colour);
 }
 
 void setbbfrommodel(dynent *d, const char *mdl)
