@@ -23,9 +23,12 @@ Texture *loadskyoverlay(const char *basename)
 }
 
 SVARFR(skybox, "", { if(skybox[0]) loadsky(skybox, sky); });
+HVARR(skyboxcolour, 0, 0xFFFFFF, 0xFFFFFF);
 FVARR(spinsky, -720, 0, 720);
 VARR(yawsky, 0, 0, 360);
 SVARFR(cloudbox, "", { if(cloudbox[0]) loadsky(cloudbox, clouds); });
+HVARR(cloudboxcolour, 0, 0xFFFFFF, 0xFFFFFF);
+FVARR(cloudboxalpha, 0, 1, 1);
 FVARR(spinclouds, -720, 0, 720);
 VARR(yawclouds, 0, 0, 360);
 FVARR(cloudclip, 0, 0.5f, 1);
@@ -48,65 +51,67 @@ void draw_envbox_face(float s0, float t0, int x0, int y0, int z0,
                       GLuint texture)
 {
     glBindTexture(GL_TEXTURE_2D, texture);
-    glBegin(GL_QUADS);
+    glBegin(GL_TRIANGLE_STRIP);
     glTexCoord2f(s3, t3); glVertex3f(x3, y3, z3);
     glTexCoord2f(s2, t2); glVertex3f(x2, y2, z2);
-    glTexCoord2f(s1, t1); glVertex3f(x1, y1, z1);
     glTexCoord2f(s0, t0); glVertex3f(x0, y0, z0);
+    glTexCoord2f(s1, t1); glVertex3f(x1, y1, z1);
     glEnd();
     xtraverts += 4;
 }
 
-void draw_envbox(int w, float zclip = 0.0f, int faces = 0x3F, Texture **sky = NULL)
+void draw_envbox(int w, float z1clip = 0.0f, float z2clip = 1.0f, int faces = 0x3F, Texture **sky = NULL)
 {
-    float vclip = 1-zclip;
-    int z = int(ceil(2*w*(vclip-0.5f)));
+    if(z1clip >= z2clip) return;
+
+    float v1 = 1-z1clip, v2 = 1-z2clip;
+    int z1 = int(ceil(2*w*(z1clip-0.5f))), z2 = int(ceil(2*w*(z2clip-0.5f)));
 
     if(faces&0x01)
-        draw_envbox_face(0.0f, 0.0f, -w, -w, -w,
-                         1.0f, 0.0f, -w,  w, -w,
-                         1.0f, vclip, -w,  w,  z,
-                         0.0f, vclip, -w, -w,  z, sky[0] ? sky[0]->id : notexture->id);
+        draw_envbox_face(0.0f, v2,  -w, -w, z2,
+                         1.0f, v2,  -w,  w, z2,
+                         1.0f, v1,  -w,  w, z1,
+                         0.0f, v1,  -w, -w, z1, sky[0] ? sky[0]->id : notexture->id);
 
     if(faces&0x02)
-        draw_envbox_face(1.0f, vclip, +w, -w,  z,
-                         0.0f, vclip, +w,  w,  z,
-                         0.0f, 0.0f, +w,  w, -w,
-                         1.0f, 0.0f, +w, -w, -w, sky[1] ? sky[1]->id : notexture->id);
+        draw_envbox_face(1.0f, v1, w, -w, z1,
+                         0.0f, v1, w,  w, z1,
+                         0.0f, v2, w,  w, z2,
+                         1.0f, v2, w, -w, z2, sky[1] ? sky[1]->id : notexture->id);
 
     if(faces&0x04)
-        draw_envbox_face(1.0f, vclip, -w, -w,  z,
-                         0.0f, vclip,  w, -w,  z,
-                         0.0f, 0.0f,  w, -w, -w,
-                         1.0f, 0.0f, -w, -w, -w, sky[2] ? sky[2]->id : notexture->id);
+        draw_envbox_face(1.0f, v1, -w, -w, z1,
+                         0.0f, v1,  w, -w, z1,
+                         0.0f, v2,  w, -w, z2,
+                         1.0f, v2, -w, -w, z2, sky[2] ? sky[2]->id : notexture->id);
 
     if(faces&0x08)
-        draw_envbox_face(1.0f, vclip, +w,  w,  z,
-                         0.0f, vclip, -w,  w,  z,
-                         0.0f, 0.0f, -w,  w, -w,
-                         1.0f, 0.0f, +w,  w, -w, sky[3] ? sky[3]->id : notexture->id);
+        draw_envbox_face(1.0f, v1,  w,  w, z1,
+                         0.0f, v1, -w,  w, z1,
+                         0.0f, v2, -w,  w, z2,
+                         1.0f, v2,  w,  w, z2, sky[3] ? sky[3]->id : notexture->id);
 
-    if(!zclip && faces&0x10)
-        draw_envbox_face(0.0f, 1.0f, -w,  w,  w,
-                         0.0f, 0.0f, +w,  w,  w,
-                         1.0f, 0.0f, +w, -w,  w,
-                         1.0f, 1.0f, -w, -w,  w, sky[4] ? sky[4]->id : notexture->id);
+    if(z1clip <= 0 && faces&0x10)
+        draw_envbox_face(0.0f, 1.0f, -w,  w,  -w,
+                         0.0f, 0.0f,  w,  w,  -w,
+                         1.0f, 0.0f,  w, -w,  -w,
+                         1.0f, 1.0f, -w, -w,  -w, sky[4] ? sky[4]->id : notexture->id);
 
-    if(faces&0x20)
-        draw_envbox_face(0.0f, 1.0f, +w,  w, -w,
-                         0.0f, 0.0f, -w,  w, -w,
-                         1.0f, 0.0f, -w, -w, -w,
-                         1.0f, 1.0f, +w, -w, -w, sky[5] ? sky[5]->id : notexture->id);
+    if(z2clip >= 1 && faces&0x20)
+        draw_envbox_face(0.0f, 1.0f,  w,  w, w,
+                         0.0f, 0.0f, -w,  w, w,
+                         1.0f, 0.0f, -w, -w, w,
+                         1.0f, 1.0f,  w, -w, w, sky[5] ? sky[5]->id : notexture->id);
 }
 
 void draw_env_overlay(int w, Texture *overlay = NULL, float tx = 0, float ty = 0)
 {
-    float z = -w*cloudheight, tsz = 0.5f*(1-cloudfade)/cloudscale, psz = w*(1-cloudfade);
+    float z = w*cloudheight, tsz = 0.5f*(1-cloudfade)/cloudscale, psz = w*(1-cloudfade);
     glBindTexture(GL_TEXTURE_2D, overlay ? overlay->id : notexture->id);
     float r = (cloudcolour>>16)/255.0f, g = ((cloudcolour>>8)&255)/255.0f, b = (cloudcolour&255)/255.0f;
     glColor4f(r, g, b, cloudalpha);
-    glBegin(GL_POLYGON);
-    loopi(cloudsubdiv)
+    glBegin(GL_TRIANGLE_FAN);
+    loopi(cloudsubdiv+1)
     {
         vec p(1, 1, 0);
         p.rotate_around_z((-2.0f*M_PI*i)/cloudsubdiv);
@@ -114,7 +119,7 @@ void draw_env_overlay(int w, Texture *overlay = NULL, float tx = 0, float ty = 0
     }
     glEnd();
     float tsz2 = 0.5f/cloudscale;
-    glBegin(GL_QUAD_STRIP);
+    glBegin(GL_TRIANGLE_STRIP);
     loopi(cloudsubdiv+1)
     {
         vec p(1, 1, 0);
@@ -133,9 +138,9 @@ static struct domevert
     uchar color[4];
 
 	domevert() {}
-	domevert(const vec &pos, float alpha) : pos(pos)
+	domevert(const vec &pos, const bvec &fcolor, float alpha) : pos(pos)
 	{
-		memcpy(color, fogcolor.v, 3);
+		memcpy(color, fcolor.v, 3);
 		color[3] = uchar(alpha*255);
 	}
 	domevert(const domevert &v0, const domevert &v1) : pos(vec(v0.pos).add(v1.pos).normalize())
@@ -148,7 +153,7 @@ static GLushort *domeindices = NULL;
 static int domenumverts = 0, domenumindices = 0, domecapindices = 0;
 static GLuint domevbuf = 0, domeebuf = 0;
 static bvec domecolor(0, 0, 0);
-static float domeminalpha = 0, domemaxalpha = 0, domecapsize = -1;
+static float domeminalpha = 0, domemaxalpha = 0, domecapsize = -1, domeclipz = 1;
 
 static void subdivide(int depth, int face);
 
@@ -190,7 +195,7 @@ static int sortdomecap(const GLushort *x, const GLushort *y)
     return 0;
 }
 
-static void initdome(float minalpha = 0.0f, float maxalpha = 1.0f, float capsize = -1, int hres = 16, int depth = 2)
+static void initdome(const bvec &color, float minalpha = 0.0f, float maxalpha = 1.0f, float capsize = -1, float clipz = 1, int hres = 16, int depth = 2)
 {
     const int tris = hres << (2*depth);
     domenumverts = domenumindices = domecapindices = 0;
@@ -198,21 +203,49 @@ static void initdome(float minalpha = 0.0f, float maxalpha = 1.0f, float capsize
     DELETEA(domeindices);
     domeverts = new domevert[tris+1 + (capsize >= 0 ? 1 : 0)];
     domeindices = new GLushort[(tris + (capsize >= 0 ? hres<<depth : 0))*3];
-	domeverts[domenumverts++] = domevert(vec(0.0f, 0.0f, 1.0f), minalpha); //build initial 'hres' sided pyramid
-    loopi(hres)
+    if(clipz >= 1)
     {
-        float angle = 2*M_PI*float(i)/hres;
-        domeverts[domenumverts++] = domevert(vec(cosf(angle), sinf(angle), 0.0f), maxalpha);
+        domeverts[domenumverts++] = domevert(vec(0.0f, 0.0f, 1.0f), color, minalpha); //build initial 'hres' sided pyramid
+        loopi(hres)
+        {
+            float angle = 2*M_PI*float(i)/hres;
+            domeverts[domenumverts++] = domevert(vec(cosf(angle), sinf(angle), 0.0f), color, maxalpha);
+        }
+        loopi(hres) genface(depth, 0, i+1, 1+(i+1)%hres);
+    }
+    else if(clipz <= 0)
+    {
+        loopi(hres<<depth)
+        {
+            float angle = 2*M_PI*float(i)/(hres<<depth), x = cosf(angle), y = sinf(angle);
+            domeverts[domenumverts++] = domevert(vec(x, y, 0.0f), color, maxalpha);
+        }
+    }
+    else
+    {
+        float clipxy = sqrtf(1 - clipz*clipz), xm = cosf(M_PI/hres), ym = sinf(M_PI/hres);
+        loopi(hres)
+        {
+            float angle = 2*M_PI*float(i)/hres, x = cosf(angle), y = sinf(angle);
+            domeverts[domenumverts++] = domevert(vec(x*clipxy, y*clipxy, clipz), color, minalpha);
+            domeverts[domenumverts++] = domevert(vec(x, y, 0.0f), color, maxalpha);
+            domeverts[domenumverts++] = domevert(vec(x*xm - y*ym, y*xm + x*ym, 0.0f), color, maxalpha);
+        }
+        loopi(hres)
+        {
+            genface(depth-1, 3*i, 3*i+1, 3*i+2);
+            genface(depth-1, 3*i, 3*i+2, 3*((i+1)%hres));
+            genface(depth-1, 3*i+2, 3*((i+1)%hres)+1, 3*((i+1)%hres));
+        }
     }
-    loopi(hres) genface(depth, 0, i+1, 1+(i+1)%hres);
 
     if(capsize >= 0)
     {
-        GLushort *domecap = &domeindices[domenumindices];
-        int domecapverts = 0;
-        loopi(domenumverts) if(!domeverts[i].pos.z) domecap[domecapverts++] = i;
-        domeverts[domenumverts++] = domevert(vec(0.0f, 0.0f, -capsize), maxalpha);
-        quicksort(domecap, domecapverts, sortdomecap);
+		GLushort *domecap = &domeindices[domenumindices];
+		int domecapverts = 0;
+		loopi(domenumverts) if(!domeverts[i].pos.z) domecap[domecapverts++] = i;
+        domeverts[domenumverts++] = domevert(vec(0.0f, 0.0f, -capsize), color, maxalpha);
+		quicksort(domecap, domecapverts, sortdomecap);
         loopi(domecapverts)
         {
             int n = domecapverts-1-i;
@@ -249,19 +282,27 @@ static void deletedome()
 FVARR(fogdomeheight, -1, -0.5f, 1);
 FVARR(fogdomemin, 0, 0, 1);
 FVARR(fogdomemax, 0, 0, 1);
-VARR(fogdomecap, 0, 0, 1);
+VARR(fogdomecap, 0, 1, 1);
+FVARR(fogdomeclip, 0, 1, 1);
+bvec fogdomecolor(0, 0, 0);
+HVARFR(fogdomecolour, 0, 0, 0xFFFFFF,
+{
+    fogdomecolor = bvec((fogdomecolour>>16)&0xFF, (fogdomecolour>>8)&0xFF, fogdomecolour&0xFF);
+});
 
 static void drawdome()
 {
     float capsize = fogdomecap && fogdomeheight < 1 ? (1 + fogdomeheight) / (1 - fogdomeheight) : -1;
-	if(!domenumverts || domecolor != fogcolor || domeminalpha != fogdomemin || domemaxalpha != fogdomemax || domecapsize != capsize)
-	{
-		initdome(min(fogdomemin, fogdomemax), fogdomemax, capsize);
-		domecolor = fogcolor;
-		domeminalpha = fogdomemin;
-		domemaxalpha = fogdomemax;
-		domecapsize = capsize;
-	}
+    bvec color = fogdomecolour ? fogdomecolor : fogcolor;
+    if(!domenumverts || domecolor != color || domeminalpha != fogdomemin || domemaxalpha != fogdomemax || domecapsize != capsize || domeclipz != fogdomeclip)
+    {
+        initdome(color, min(fogdomemin, fogdomemax), fogdomemax, capsize, fogdomeclip);
+        domecolor = color;
+        domeminalpha = fogdomemin;
+        domemaxalpha = fogdomemax;
+        domecapsize = capsize;
+        domeclipz = fogdomeclip;
+    }
 
     if(hasVBO)
     {
@@ -313,7 +354,7 @@ bool drawskylimits(bool explicitonly)
 
 void drawskyoutline()
 {
-    notextureshader->set();
+    lineshader->set();
 
     glDisable(GL_TEXTURE_2D);
     glDepthMask(GL_FALSE);
@@ -338,6 +379,31 @@ void drawskyoutline()
 
 VAR(clampsky, 0, 1, 1);
 
+VARR(fogdomeclouds, 0, 1, 1);
+
+static void drawfogdome(int farplane)
+{
+    notextureshader->set();
+    glDisable(GL_TEXTURE_2D);
+
+    glEnable(GL_BLEND);
+    glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+    glPushMatrix();
+    glLoadMatrixf(viewmatrix.v);
+    glRotatef(camera1->roll, 0, 1, 0);
+    glRotatef(camera1->pitch, -1, 0, 0);
+    glRotatef(camera1->yaw, 0, 0, -1);
+    if(reflecting) glScalef(1, 1, -1);
+    glTranslatef(0, 0, farplane*fogdomeheight*0.5f);
+    glScalef(farplane/2, farplane/2, farplane*(0.5f - fogdomeheight*0.5f));
+    drawdome();
+    glPopMatrix();
+
+    glDisable(GL_BLEND);
+    glEnable(GL_TEXTURE_2D);
+}
+
 static int yawskyfaces(int faces, int yaw, float spin = 0)
 {
     if(spin || yaw%90) return faces&0x0F ? faces | 0x0F : faces;
@@ -386,8 +452,13 @@ void drawskybox(int farplane, bool limited)
         renderedskyclip = 0;
     }
 
-    float skyclip = clipsky ? max(renderedskyclip-1, 0) : 0;
-    if(reflectz<worldsize && reflectz>skyclip) skyclip = reflectz;
+    float skyclip = clipsky ? max(renderedskyclip-1, 0) : 0, topclip = 1;
+    if(reflectz<worldsize)
+    {
+        if(refracting<0) topclip = 0.5f + 0.5f*(reflectz-camera1->o.z)/float(worldsize);
+        else if(reflectz>skyclip) skyclip = reflectz;
+    }
+    if(skyclip) skyclip = 0.5f + 0.5f*(skyclip-camera1->o.z)/float(worldsize); 
 
     if(glaring)
     {
@@ -397,7 +468,7 @@ void drawskybox(int farplane, bool limited)
     }
     else defaultshader->set();
 
-    if(renderpath!=R_FIXEDFUNCTION || !fogging) glDisable(GL_FOG);
+    glDisable(GL_FOG);
 
     if(limited)
     {
@@ -410,18 +481,24 @@ void drawskybox(int farplane, bool limited)
 
     if(clampsky) glDepthRange(1, 1);
 
-    glColor3f(1, 1, 1);
+    glColor3f((skyboxcolour>>16)/255.0f, ((skyboxcolour>>8)&255)/255.0f, (skyboxcolour&255)/255.0f);
 
     glPushMatrix();
-    glLoadIdentity();
-    glRotatef(camera1->roll, 0, 0, 1);
+    glLoadMatrixf(viewmatrix.v);
+    glRotatef(camera1->roll, 0, 1, 0);
     glRotatef(camera1->pitch, -1, 0, 0);
-    glRotatef(camera1->yaw+spinsky*lastmillis/1000.0f+yawsky, 0, 1, 0);
-    glRotatef(90, 1, 0, 0);
+    glRotatef(camera1->yaw+spinsky*lastmillis/1000.0f+yawsky, 0, 0, -1);
     if(reflecting) glScalef(1, 1, -1);
-    draw_envbox(farplane/2, skyclip ? 0.5f + 0.5f*(skyclip-camera1->o.z)/float(worldsize) : 0, yawskyfaces(renderedskyfaces, yawsky, spinsky), sky);
+    draw_envbox(farplane/2, skyclip, topclip, yawskyfaces(renderedskyfaces, yawsky, spinsky), sky);
     glPopMatrix();
 
+    if(!glaring && fogdomemax && !fogdomeclouds)
+    {
+        if(fading) glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
+        drawfogdome(farplane);
+        defaultshader->set();
+    }
+
     if(!glaring && cloudbox[0])
     {
         if(fading) glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
@@ -429,14 +506,15 @@ void drawskybox(int farplane, bool limited)
         glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
+        glColor4f((cloudboxcolour>>16)/255.0f, ((cloudboxcolour>>8)&255)/255.0f, (cloudboxcolour&255)/255.0f, cloudboxalpha);
+
         glPushMatrix();
-        glLoadIdentity();
-        glRotatef(camera1->roll, 0, 0, 1);
+        glLoadMatrixf(viewmatrix.v);
+        glRotatef(camera1->roll, 0, 1, 0);
         glRotatef(camera1->pitch, -1, 0, 0);
-        glRotatef(camera1->yaw+spinclouds*lastmillis/1000.0f+yawclouds, 0, 1, 0);
-        glRotatef(90, 1, 0, 0);
+        glRotatef(camera1->yaw+spinclouds*lastmillis/1000.0f+yawclouds, 0, 0, -1);
         if(reflecting) glScalef(1, 1, -1);
-        draw_envbox(farplane/2, skyclip ? 0.5f + 0.5f*(skyclip-camera1->o.z)/float(worldsize) : cloudclip, yawskyfaces(renderedskyfaces, yawclouds, spinclouds), clouds);
+        draw_envbox(farplane/2, skyclip ? skyclip : cloudclip, topclip, yawskyfaces(renderedskyfaces, yawclouds, spinclouds), clouds);
         glPopMatrix();
 
         glDisable(GL_BLEND);
@@ -452,11 +530,10 @@ void drawskybox(int farplane, bool limited)
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
         glPushMatrix();
-        glLoadIdentity();
-        glRotatef(camera1->roll, 0, 0, 1);
+        glLoadMatrixf(viewmatrix.v);
+        glRotatef(camera1->roll, 0, 1, 0);
         glRotatef(camera1->pitch, -1, 0, 0);
-        glRotatef(camera1->yaw+spincloudlayer*lastmillis/1000.0f+yawcloudlayer, 0, 1, 0);
-        glRotatef(90, 1, 0, 0);
+        glRotatef(camera1->yaw+spincloudlayer*lastmillis/1000.0f+yawcloudlayer, 0, 0, -1);
         if(reflecting) glScalef(1, 1, -1);
         draw_env_overlay(farplane/2, cloudoverlay, cloudscrollx * lastmillis/1000.0f, cloudscrolly * lastmillis/1000.0f);
         glPopMatrix();
@@ -466,30 +543,10 @@ void drawskybox(int farplane, bool limited)
         glEnable(GL_CULL_FACE);
     }
 
-	if(!glaring && fogdomemax)
+	if(!glaring && fogdomemax && fogdomeclouds)
 	{
         if(fading) glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_FALSE);
-
-        notextureshader->set();
-        glDisable(GL_TEXTURE_2D);
-
-        glEnable(GL_BLEND);
-        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
-
-        glPushMatrix();
-        glLoadIdentity();
-        glRotatef(camera1->roll, 0, 0, 1);
-        glRotatef(camera1->pitch, -1, 0, 0);
-        glRotatef(camera1->yaw, 0, 1, 0);
-        glRotatef(90, 1, 0, 0);
-        if(reflecting) glScalef(1, 1, -1);
-		glTranslatef(0, 0, -farplane*fogdomeheight*0.5f);
-		glScalef(farplane/2, farplane/2, -farplane*(0.5f - fogdomeheight*0.5f));
-		drawdome();
-        glPopMatrix();
-
-        glDisable(GL_BLEND);
-		glEnable(GL_TEXTURE_2D);
+        drawfogdome(farplane);
 	}
 
     if(clampsky) glDepthRange(0, 1);
@@ -504,7 +561,7 @@ void drawskybox(int farplane, bool limited)
     }
     else glDepthFunc(GL_LESS);
 
-    if(renderpath!=R_FIXEDFUNCTION || !fogging) glEnable(GL_FOG);
+    glEnable(GL_FOG);
 }
 
 VARNR(skytexture, useskytexture, 0, 1, 1);
