@@ -3,8 +3,6 @@
 
 #include "engine.h"
 
-int dedi = 0;
-
 #ifdef STANDALONE
 void fatal(const char *s, ...)
 {
@@ -96,7 +94,7 @@ int getuint(ucharbuf &p)
         n += (p.get() << 7) - 0x80;
         if(n & (1<<14)) n += (p.get() << 14) - (1<<14);
         if(n & (1<<21)) n += (p.get() << 21) - (1<<21);
-        if(n & (1<<28)) n |= 0xF0000000;
+        if(n & (1<<28)) n |= -1<<28;
     }
     return n;
 }
@@ -171,7 +169,6 @@ struct client                   // server side version of "dynent" type
 vector<client *> clients;
 
 ENetHost *serverhost = NULL;
-size_t bsend = 0, brec = 0;
 int laststatus = 0;
 ENetSocket pongsock = ENET_SOCKET_NULL, lansock = ENET_SOCKET_NULL;
 
@@ -186,9 +183,10 @@ void cleanupserver()
 }
 
 void process(ENetPacket *packet, int sender, int chan);
-void disconnect_client(int n, int reason);
+//void disconnect_client(int n, int reason);
 
 void *getclientinfo(int i) { return !clients.inrange(i) || clients[i]->type==ST_EMPTY ? NULL : clients[i]->info; }
+ENetPeer *getclientpeer(int i) { return clients.inrange(i) && clients[i]->type==ST_TCPIP ? clients[i]->peer : NULL; }
 int getnumclients()        { return clients.length(); }
 uint getclientip(int n)    { return clients.inrange(n) && clients[n]->type==ST_TCPIP ? clients[n]->peer->address.host : 0; }
 
@@ -205,7 +203,6 @@ void sendpacket(int n, int chan, ENetPacket *packet, int exclude)
         case ST_TCPIP:
         {
             enet_peer_send(clients[n]->peer, chan, packet);
-            bsend += packet->dataLength;
             break;
         }
 
@@ -217,13 +214,12 @@ void sendpacket(int n, int chan, ENetPacket *packet, int exclude)
     }
 }
 
-void sendf(int cn, int chan, const char *format, ...)
+ENetPacket *sendf(int cn, int chan, const char *format, ...)
 {
     int exclude = -1;
     bool reliable = false;
     if(*format=='r') { reliable = true; ++format; }
-    ENetPacket *packet = enet_packet_create(NULL, MAXTRANS, reliable ? ENET_PACKET_FLAG_RELIABLE : 0);
-    ucharbuf p(packet->data, packet->dataLength);
+    packetbuf p(MAXTRANS, reliable ? ENET_PACKET_FLAG_RELIABLE : 0);
     va_list args;
     va_start(args, format);
     while(*format) switch(*format++)
@@ -256,37 +252,30 @@ void sendf(int cn, int chan, const char *format, ...)
         case 'm':
         {
             int n = va_arg(args, int);
-            enet_packet_resize(packet, packet->dataLength+n);
-            p.buf = packet->data;
-            p.maxlen += n;
             p.put(va_arg(args, uchar *), n);
             break;
         }
     }
     va_end(args);
-    enet_packet_resize(packet, p.length());
+    ENetPacket *packet = p.finalize();
     sendpacket(cn, chan, packet, exclude);
-    if(packet->referenceCount==0) enet_packet_destroy(packet);
+    return packet->referenceCount > 0 ? packet : NULL;
 }
 
-void sendfile(int cn, int chan, stream *file, const char *format, ...)
+ENetPacket *sendfile(int cn, int chan, stream *file, const char *format, ...)
 {
     if(cn < 0)
     {
 #ifdef STANDALONE
-        return;
+        return NULL;
 #endif
     }
-    else if(!clients.inrange(cn)) return;
+    else if(!clients.inrange(cn)) return NULL;
 
     int len = file->size();
-    if(len <= 0) return;
+    if(len <= 0) return NULL;
 
-    bool reliable = false;
-    if(*format=='r') { reliable = true; ++format; }
-    ENetPacket *packet = enet_packet_create(NULL, MAXTRANS+len, ENET_PACKET_FLAG_RELIABLE);
-
-    ucharbuf p(packet->data, packet->dataLength);
+    packetbuf p(MAXTRANS+len, ENET_PACKET_FLAG_RELIABLE);
     va_list args;
     va_start(args, format);
     while(*format) switch(*format++)
@@ -301,24 +290,24 @@ void sendfile(int cn, int chan, stream *file, const char *format, ...)
         case 'l': putint(p, len); break;
     }
     va_end(args);
-    enet_packet_resize(packet, p.length()+len);
 
     file->seek(0, SEEK_SET);
-    file->read(&packet->data[p.length()], len);
-    enet_packet_resize(packet, p.length()+len);
+    file->read(p.subbuf(len).buf, len);
+
+    ENetPacket *packet = p.finalize();
 
     if(cn >= 0) sendpacket(cn, chan, packet, -1);
 #ifndef STANDALONE
     else sendclientpacket(packet, chan);
 #endif
-    if(!packet->referenceCount) enet_packet_destroy(packet);
+    return packet->referenceCount > 0 ? packet : NULL;
 }
 
 const char *disc_reasons[] = { "normal", "end of packet", "client num", "kicked/banned", "tag type", "ip is banned", "server is in private mode", "server FULL", "connection timed out", "overflow" };
 
 void disconnect_client(int n, int reason)
 {
-    if(clients[n]->type!=ST_TCPIP) return;
+    if(!clients.inrange(n) || clients[n]->type!=ST_TCPIP) return;
     enet_peer_disconnect(clients[n]->peer, reason);
     server::clientdisconnect(n);
     clients[n]->type = ST_EMPTY;
@@ -388,20 +377,28 @@ int lastupdatemaster = 0;
 vector<char> masterout, masterin;
 int masteroutpos = 0, masterinpos = 0;
 VARN(updatemaster, allowupdatemaster, 0, 1, 1);
-SVAR(mastername, server::defaultmaster());
 
 void disconnectmaster()
 {
-    if(mastersock == ENET_SOCKET_NULL) return;
+    if(mastersock != ENET_SOCKET_NULL) 
+    {
+        enet_socket_destroy(mastersock);
+        mastersock = ENET_SOCKET_NULL;
+    }
 
-    enet_socket_destroy(mastersock);
-    mastersock = ENET_SOCKET_NULL;
-
-    masterout.setsizenodelete(0);
-    masterin.setsizenodelete(0);
+    masterout.setsize(0);
+    masterin.setsize(0);
     masteroutpos = masterinpos = 0;
+
+    masteraddress.host = ENET_HOST_ANY;
+    masteraddress.port = ENET_PORT_ANY;
+
+    lastupdatemaster = 0;
 }
 
+SVARF(mastername, server::defaultmaster(), disconnectmaster());
+VARF(masterport, 1, server::masterport(), 0xFFFF, disconnectmaster());
+
 ENetSocket connectmaster()
 {
     if(!mastername[0]) return ENET_SOCKET_NULL;
@@ -411,7 +408,7 @@ ENetSocket connectmaster()
 #ifdef STANDALONE
         printf("looking up %s...\n", mastername);
 #endif
-        masteraddress.port = server::masterport();
+        masteraddress.port = masterport;
         if(!resolverwait(mastername, &masteraddress)) return ENET_SOCKET_NULL;
     }
     ENetSocket sock = enet_socket_create(ENET_SOCKET_TYPE_STREAM);
@@ -477,7 +474,7 @@ void processmasterinput()
 
     if(masterinpos >= masterin.length())
     {
-        masterin.setsizenodelete(0);
+        masterin.setsize(0);
         masterinpos = 0;
     }
 }
@@ -495,7 +492,7 @@ void flushmasteroutput()
         masteroutpos += sent;
         if(masteroutpos >= masterout.length())
         {
-            masterout.setsizenodelete(0);
+            masterout.setsize(0);
             masteroutpos = 0;
         }
     }
@@ -579,9 +576,8 @@ int curtime = 0, lastmillis = 0, totalmillis = 0;
 
 void updatemasterserver()
 {
-    if(!mastername[0] || !allowupdatemaster) return;
-
-    requestmasterf("regserv %d\n", serverport);
+    if(mastername[0] && allowupdatemaster) requestmasterf("regserv %d\n", serverport);
+    lastupdatemaster = totalmillis ? totalmillis : 1;
 }
 
 void serverslice(bool dedicated, uint timeout)   // main server update, called from main loop in sp, or from below in dedicated server
@@ -605,25 +601,23 @@ void serverslice(bool dedicated, uint timeout)   // main server update, called f
     if(dedicated)
     {
         int millis = (int)enet_time_get();
-        curtime = millis - totalmillis;
-        lastmillis = totalmillis = millis;
+        curtime = server::ispaused() ? 0 : millis - totalmillis;
+        totalmillis = millis;
+        lastmillis += curtime;
     }
     server::serverupdate();
 
     flushmasteroutput();
     checkserversockets();
 
-    if(totalmillis-lastupdatemaster>60*60*1000)       // send alive signal to masterserver every hour of uptime
-    {
+    if(!lastupdatemaster || totalmillis-lastupdatemaster>60*60*1000)       // send alive signal to masterserver every hour of uptime
         updatemasterserver();
-        lastupdatemaster = totalmillis;
-    }
 
     if(totalmillis-laststatus>60*1000)   // display bandwidth stats, useful for server ops
     {
         laststatus = totalmillis;
-        if(nonlocalclients || bsend || brec) printf("status: %d remote clients, %.1f send, %.1f rec (K/sec)\n", nonlocalclients, bsend/60.0f/1024, brec/60.0f/1024);
-        bsend = brec = 0;
+        if(nonlocalclients || serverhost->totalSentData || serverhost->totalReceivedData) printf("status: %d remote clients, %.1f send, %.1f rec (K/sec)\n", nonlocalclients, serverhost->totalSentData/60.0f/1024, serverhost->totalReceivedData/60.0f/1024);
+        serverhost->totalSentData = serverhost->totalReceivedData = 0;
     }
 
     ENetEvent event;
@@ -653,7 +647,6 @@ void serverslice(bool dedicated, uint timeout)   // main server update, called f
             }
             case ENET_EVENT_TYPE_RECEIVE:
             {
-                brec += event.packet->dataLength;
                 client *c = (client *)event.peer->data;
                 if(c) process(event.packet, c->num, event.channelID);
                 if(event.packet->referenceCount==0) enet_packet_destroy(event.packet);
@@ -679,6 +672,11 @@ void serverslice(bool dedicated, uint timeout)   // main server update, called f
     if(server::sendpackets()) enet_host_flush(serverhost);
 }
 
+void flushserver(bool force)
+{
+    if(server::sendpackets(force) && serverhost) enet_host_flush(serverhost);
+}
+
 #ifndef STANDALONE
 void localdisconnect(bool cleanup)
 {
@@ -694,7 +692,11 @@ void localdisconnect(bool cleanup)
     }
     if(!disconnected) return;
     game::gamedisconnect(cleanup);
+    #ifndef NEWGUI
     mainmenu = 1;
+    #else
+    UI::mainmenu = 1;
+    #endif
 }
 
 void localconnect()
@@ -714,7 +716,6 @@ void rundedicatedserver()
     SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
     #endif
     printf("dedicated server started, waiting for clients...\nCtrl-C to exit\n\n");
-    dedi = 2;
     for(;;) serverslice(true, 5);
 }
 
@@ -740,7 +741,7 @@ bool setuplistenserver(bool dedicated)
         if(enet_address_set_host(&address, serverip)<0) conoutf(CON_WARN, "WARNING: server ip not resolved");
         else serveraddress.host = address.host;
     }
-    serverhost = enet_host_create(&address, min(maxclients + server::reserveclients(), MAXCLIENTS), 0, serveruprate);
+    serverhost = enet_host_create(&address, min(maxclients + server::reserveclients(), MAXCLIENTS), server::numchannels(), 0, serveruprate);
     if(!serverhost) return servererror(dedicated, "could not create server host");
     loopi(maxclients) serverhost->peers[i].data = NULL;
     address.port = server::serverinfoport(serverport > 0 ? serverport : -1);
