@@ -96,8 +96,8 @@ void resolverclear()
     if(resolverthreads.empty()) return;
 
     SDL_LockMutex(resolvermutex);
-    resolverqueries.setsize(0);
-    resolverresults.setsize(0);
+    resolverqueries.shrink(0);
+    resolverresults.shrink(0);
     loopv(resolverthreads)
     {
         resolverthread &rt = resolverthreads[i];
@@ -310,6 +310,8 @@ struct serverinfo
              bc = server::servercompatible(b->name, b->sdesc, b->map, b->ping, b->attr, b->numplayers);
         if(ac > bc) return -1;
         if(bc > ac) return 1;
+        if(a->keep > b->keep) return -1;
+        if(a->keep < b->keep) return 1;
         if(a->numplayers < b->numplayers) return 1;
         if(a->numplayers > b->numplayers) return -1;
         if(a->ping > b->ping) return 1;
@@ -371,7 +373,7 @@ void addserver(const char *name, int port, const char *password, bool keep)
 VARP(searchlan, 0, 1, 1);
 VARP(servpingrate, 1000, 5000, 60000);
 VARP(servpingdecay, 1000, 15000, 60000);
-VARP(maxservpings, 0, 25, 1000);
+VARP(maxservpings, 0, 10, 1000);
 
 void pingservers()
 {
@@ -472,10 +474,10 @@ void checkpings()
         if(rtt < servpingdecay) si->addping(rtt, millis);
         si->numplayers = getint(p);
         int numattr = getint(p);
-        si->attr.setsize(0);
+        si->attr.shrink(0);
         loopj(numattr) si->attr.add(getint(p));
         getstring(text, p);
-        filtertext(si->map, text);
+        filtertext(si->map, text, false);
         getstring(text, p);
         filtertext(si->sdesc, text);
     }
@@ -533,7 +535,7 @@ char *showservers(g3d_gui *cgui)
 void clearservers(bool full = false)
 {
     resolverclear();
-    if(full) servers.deletecontentsp();
+    if(full) servers.deletecontents();
     else loopvrev(servers) if(!servers[i]->keep) delete servers.remove(i);
 }
 
@@ -577,7 +579,7 @@ void retrieveservers(vector<char> &data)
         if(enet_socket_wait(sock, &events, 250) >= 0 && events)
         {
             if(data.length() >= data.capacity()) data.reserve(4096);
-            buf.data = &data[data.length()];
+            buf.data = data.getbuf() + data.length();
             buf.dataLength = data.capacity() - data.length();
             int recv = enet_socket_receive(sock, NULL, &buf, 1);
             if(recv <= 0) break;
