@@ -11,10 +11,10 @@ VARP(ffshadowmapdist, 128, 1024, 4096);
 VARP(shadowmapdist, 128, 256, 512);
 VARFP(fpshadowmap, 0, 0, 1, cleanshadowmap());
 VARFP(shadowmapprecision, 0, 0, 1, cleanshadowmap());
-bvec shadowmapambientcolor;
+bvec shadowmapambientcolor(0, 0, 0);
 HVARFR(shadowmapambient, 0, 0, 0xFFFFFF,
 {
-    if(shadowmapambient <= 255) shadowmapambient |= (shadowmapambient<<8) | (shadowmapambient<<16);
+    //if(shadowmapambient <= 255) shadowmapambient |= (shadowmapambient<<8) | (shadowmapambient<<16);
     shadowmapambientcolor = bvec((shadowmapambient>>16)&0xFF, (shadowmapambient>>8)&0xFF, shadowmapambient&0xFF);
 });
 VARP(shadowmapintensity, 0, 40, 100);
@@ -48,7 +48,7 @@ void guessshadowdir()
         switch(e.type)
         {
             case ET_LIGHT:
-                if(!e.attr1) { lightpos.add(e.o); numlights++; }
+                if(!e.attr[0]) { lightpos.add(e.o); numlights++; }
                 break;
 
              case ET_MAPMODEL:
@@ -94,16 +94,17 @@ static struct shadowmaptexture : rendertarget
 
     const GLenum *colorformats() const
     {
-        if(renderpath==R_FIXEDFUNCTION) 
+        if(renderpath==R_FIXEDFUNCTION)
         {
             static const GLenum depthtexfmts[] = { GL_DEPTH_COMPONENT16, GL_DEPTH_COMPONENT24, GL_DEPTH_COMPONENT, GL_DEPTH_COMPONENT32, GL_FALSE };
             return depthtexfmts;
         }
-       
+
         static const GLenum rgbfmts[] = { GL_RGB, GL_RGB8, GL_FALSE }, rgbafmts[] = { GL_RGBA16F_ARB, GL_RGBA16, GL_RGBA, GL_RGBA8, GL_FALSE };
         return hasFBO ? &rgbafmts[fpshadowmap && hasTF ? 0 : (shadowmapprecision ? 1 : 2)] : rgbfmts;
     }
 
+    bool shadowcompare() const { return renderpath==R_FIXEDFUNCTION; }
     bool filter() const { return renderpath!=R_FIXEDFUNCTION || hasNVPCF; }
     bool swaptexs() const { return renderpath!=R_FIXEDFUNCTION; }
 
@@ -144,14 +145,13 @@ static struct shadowmaptexture : rendertarget
         glMatrixMode(GL_PROJECTION);
         glPushMatrix();
         glLoadIdentity();
-        glOrtho(-shadowmapradius, shadowmapradius, -shadowmapradius, shadowmapradius, 
-            renderpath==R_FIXEDFUNCTION ? 0 : -shadowmapdist, 
+        glOrtho(-shadowmapradius, shadowmapradius, -shadowmapradius, shadowmapradius,
+            renderpath==R_FIXEDFUNCTION ? 0 : -shadowmapdist,
             renderpath==R_FIXEDFUNCTION ? ffshadowmapdist : shadowmapdist);
 
         glMatrixMode(GL_MODELVIEW);
 
         vec skewdir(shadowdir);
-        skewdir.neg();
         skewdir.rotate_around_z(-camera1->yaw*RAD);
 
         vec dir;
@@ -174,7 +174,7 @@ static struct shadowmaptexture : rendertarget
         };
         glLoadMatrixf(skew);
         glTranslatef(skewdir.x*shadowmapheight + shadowoffset.x, skewdir.y*shadowmapheight + shadowoffset.y + dir.magnitude(), -shadowmapheight);
-        glRotatef(camera1->yaw, 0, 0, -1);
+        glRotatef(camera1->yaw+180, 0, 0, -1);
         glTranslatef(-camera1->o.x, -camera1->o.y, -camera1->o.z);
         shadowfocus = camera1->o;
         shadowfocus.add(dir);
@@ -205,11 +205,11 @@ static struct shadowmaptexture : rendertarget
         glEnable(GL_TEXTURE_2D);
 
         if(renderpath==R_FIXEDFUNCTION) glColorMask(GL_TRUE, GL_TRUE, GL_TRUE, GL_TRUE);
-        else if(shadowmapcasters && smdepthpeel) 
+        else if(shadowmapcasters && smdepthpeel)
         {
             int sx, sy, sw, sh;
             bool scissoring = rtscissor && scissorblur(sx, sy, sw, sh) && sw > 0 && sh > 0;
-            if(scissoring) 
+            if(scissoring)
             {
                 if(!hasFBO)
                 {
@@ -268,7 +268,7 @@ static void calcscissorbox()
     bottom.add(vec(forward).mul(smy - shadowmaptex.viewh/2)).add(vec(right).mul(smx - shadowmaptex.vieww/2));
     vec top(bottom);
     top.add(vec(shadowdir).mul(shadowmapmaxz - (shadowfocus.z - shadowmapdist)));
-   
+
     vec4 v[8];
     float sx1 = 1, sy1 = 1, sx2 = -1, sy2 = -1;
     loopi(8)
@@ -313,12 +313,11 @@ static void calcscissorbox()
 void calcshadowmapbb(const vec &o, float xyrad, float zrad, float &x1, float &y1, float &x2, float &y2)
 {
     vec skewdir(shadowdir);
-    skewdir.neg();
     skewdir.rotate_around_z(-camera1->yaw*RAD);
 
     vec ro(o);
     ro.sub(camera1->o);
-    ro.rotate_around_z(-camera1->yaw*RAD);
+    ro.rotate_around_z(-(camera1->yaw+180)*RAD);
     ro.x += ro.z * skewdir.x + shadowoffset.x;
     ro.y += ro.z * skewdir.y + shadowmapradius * cosf(camera1->pitch*RAD) + shadowoffset.y;
 
@@ -373,8 +372,8 @@ bool isshadowmapreceiver(vtxarray *va)
     float skew = va->size/2*SHADOWSKEW;
     if(!shadowmap || !shadowmaptex ||
        va->o.z + va->size <= shadowfocus.z - shadowmapdist || va->o.z >= shadowmapmaxz ||
-       va->o.x + va->size <= cx - shadowmapradius-skew || va->o.x >= cx + shadowmapradius+skew || 
-       va->o.y + va->size <= cy - shadowmapradius-skew || va->o.y >= cy + shadowmapradius+skew) 
+       va->o.x + va->size <= cx - shadowmapradius-skew || va->o.x >= cx + shadowmapradius+skew ||
+       va->o.y + va->size <= cy - shadowmapradius-skew || va->o.y >= cy + shadowmapradius+skew)
         return false;
     return true;
 #endif
@@ -400,12 +399,13 @@ void pushshadowmap()
 
     if(renderpath==R_FIXEDFUNCTION)
     {
-        static GLfloat texgenS[4] = { 1, 0, 0, 0 },
-                       texgenT[4] = { 0, 1, 0, 0 },
-                       texgenR[4] = { 0, 0, 1, 0 };
-
         glBindTexture(GL_TEXTURE_2D, shadowmaptex.rendertex);
 
+        const GLfloat *v = shadowmapmatrix.v;
+        GLfloat texgenS[4] = { v[0], v[4], v[8], v[12] },
+                texgenT[4] = { v[1], v[5], v[9], v[13] },
+                texgenR[4] = { v[2], v[6], v[10], v[14] };
+
         glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
         glTexGenfv(GL_S, GL_OBJECT_PLANE, texgenS);
         glEnable(GL_TEXTURE_GEN_S);
@@ -422,18 +422,6 @@ void pushshadowmap()
         // MUST set Q with glTexCoord4f, glTexCoord3f does not work
         glTexCoord4f(0, 0, 0, 1);
 
-        if(hasDT && hasSH)
-        {
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_FUNC_ARB, GL_GEQUAL);
-            glTexParameteri(GL_TEXTURE_2D, GL_DEPTH_TEXTURE_MODE_ARB, GL_LUMINANCE);
-        }
-        else
-        {
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
-            glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_OPERATOR_SGIX, GL_TEXTURE_GEQUAL_R_SGIX);
-        }
-
         glColor3f(shadowmapintensity/100.0f, shadowmapintensity/100.0f, shadowmapintensity/100.0f);
 
         if(ffsmscissor) calcscissorbox();
@@ -446,7 +434,6 @@ void pushshadowmap()
     glActiveTexture_(GL_TEXTURE2_ARB);
     glMatrixMode(GL_TEXTURE);
     glLoadMatrixf(shadowmapmatrix.v);
-    glPushMatrix();
     glMatrixMode(GL_MODELVIEW);
 
     glActiveTexture_(GL_TEXTURE0_ARB);
@@ -461,7 +448,7 @@ void pushshadowmap()
 			g = max(25.0f, 0.4f*ambientcolor[1] + 0.6f*max(ambientcolor[1], skylightcolor[1]));
 			b = max(25.0f, 0.4f*ambientcolor[2] + 0.6f*max(ambientcolor[2], skylightcolor[2]));
 		}
-		else 
+		else
         {
             r = max(25.0f, 2.0f*ambientcolor[0]);
             g = max(25.0f, 2.0f*ambientcolor[1]);
@@ -472,56 +459,17 @@ void pushshadowmap()
     setenvparamf("shadowmapambient", SHPARAM_PIXEL, 7, r/255.0f, g/255.0f, b/255.0f);
 }
 
-void adjustshadowmatrix(const ivec &o, float scale)
-{
-    if(!shadowmap || !shadowmaptex.rendertex) return;
-
-    if(renderpath==R_FIXEDFUNCTION)
-    {
-        const GLfloat *v = shadowmapmatrix.v;
-        GLfloat texgenS[4] = { v[0]*scale, v[4]*scale, v[8]*scale, v[0]*o.x + v[4]*o.y + v[8]*o.z + v[12] },
-                texgenT[4] = { v[1]*scale, v[5]*scale, v[9]*scale, v[1]*o.x + v[5]*o.y + v[9]*o.z + v[13] },
-                texgenR[4] = { v[2]*scale, v[6]*scale, v[10]*scale, v[2]*o.x + v[6]*o.y + v[10]*o.z + v[14] };
-        glTexGenfv(GL_S, GL_OBJECT_PLANE, texgenS);
-        glTexGenfv(GL_T, GL_OBJECT_PLANE, texgenT);
-        glTexGenfv(GL_R, GL_OBJECT_PLANE, texgenR);
-    }
-    else
-    {
-        glActiveTexture_(GL_TEXTURE2_ARB);
-        glMatrixMode(GL_TEXTURE);
-        glPopMatrix();
-        glPushMatrix();
-        glTranslatef(o.x, o.y, o.z);
-        glScalef(scale, scale, scale);
-        glMatrixMode(GL_MODELVIEW);
-        glActiveTexture_(GL_TEXTURE0_ARB);
-    }
-}
-
 void popshadowmap()
 {
     if(!shadowmap || !shadowmaptex.rendertex) return;
 
-    if(renderpath!=R_FIXEDFUNCTION) 
-    {
-        glActiveTexture_(GL_TEXTURE2_ARB);
-        glMatrixMode(GL_TEXTURE);
-        glPopMatrix();
-        glMatrixMode(GL_MODELVIEW);
-
-        glActiveTexture_(GL_TEXTURE0_ARB);
-    }
-    else
+    if(renderpath==R_FIXEDFUNCTION)
     {
         popscissor();
 
         glDisable(GL_TEXTURE_GEN_S);
         glDisable(GL_TEXTURE_GEN_T);
         glDisable(GL_TEXTURE_GEN_R);
-
-        if(hasDT && hasSH) glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_MODE_ARB, GL_NONE);
-        else glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_COMPARE_SGIX, GL_FALSE);
     }
 }
 
@@ -530,9 +478,9 @@ void rendershadowmap()
     if(!shadowmap || (renderpath==R_FIXEDFUNCTION && (!hasSGIDT || !hasSGISH))) return;
 
     // Apple/ATI bug - fixed-function fog state can force software fallback even when fragment program is enabled
-    if(renderpath!=R_FIXEDFUNCTION || !fogging) glDisable(GL_FOG); 
+    glDisable(GL_FOG);
     shadowmaptex.render(1<<shadowmapsize, 1<<shadowmapsize, renderpath!=R_FIXEDFUNCTION ? blurshadowmap : 0, blursmsigma/100.0f);
-    if(renderpath!=R_FIXEDFUNCTION || !fogging) glEnable(GL_FOG);
+    glEnable(GL_FOG);
 }
 
 VAR(debugsm, 0, 0, 1);
