@@ -1,9 +1,7 @@
-#include "engine.h"
-
 #ifdef NEWGUI
-#include "textedit.h"
 
-extern bool saycommandon;
+#include "engine.h"
+#include "textedit.h"
 
 namespace UI
 {
@@ -17,7 +15,7 @@ namespace UI
     static inline bool isfocused(const Object *o) { return o==focused; }
 
     static void setfocus(Object *o) { focused = o; }
-    static inline void clearfocus(const Object *o) 
+    static inline void clearfocus(const Object *o)
     {
         if(o==selected) selected = NULL;
         if(o==hovering) hovering = NULL;
@@ -44,6 +42,7 @@ namespace UI
             y1 = max(y1, c.y1);
             x2 = max(x1, min(x2, c.x2));
             y2 = max(y1, min(y2, c.y2));
+
         }
 
         bool isfullyclipped(float x, float y, float w, float h)
@@ -63,7 +62,7 @@ namespace UI
     };
 
     static vector<ClipArea> clipstack;
-    
+
     static void pushclip(float x, float y, float w, float h)
     {
         if(clipstack.empty()) glEnable(GL_SCISSOR_TEST);
@@ -71,19 +70,20 @@ namespace UI
         if(clipstack.length() >= 2) c.intersect(clipstack[clipstack.length()-2]);
         c.scissor();
     }
-    
+
     static void popclip()
     {
         clipstack.pop();
         if(clipstack.empty()) glDisable(GL_SCISSOR_TEST);
-    } 
+	else clipstack.last().scissor();
+    }
 
     static bool isfullyclipped(float x, float y, float w, float h)
     {
         if(clipstack.empty()) return false;
         return clipstack.last().isfullyclipped(x, y, w, h);
     }
-    
+
     enum
     {
         ALIGN_MASK    = 0xF,
@@ -124,7 +124,7 @@ namespace UI
         virtual ~Object()
         {
             clearfocus(this);
-            children.deletecontentsp();
+            children.deletecontents();
         }
 
         virtual int forks() const { return 0; }
@@ -165,7 +165,7 @@ namespace UI
                     body; \
                 } \
             } \
-        } while(0) 
+        } while(0)
 
         #define loopinchildren(o, cx, cy, body) \
             loopchildren(o, \
@@ -205,12 +205,12 @@ namespace UI
         {
             loopchildren(o, o->adjustlayout(px, py, pw, ph));
         }
-        
+
         virtual void adjustchildren()
         {
             adjustchildrento(0, 0, w, h);
         }
-   
+
         void adjustlayout(float px, float py, float pw, float ph)
         {
             switch(adjust&ALIGN_HMASK)
@@ -259,18 +259,18 @@ namespace UI
 
         virtual void draw(float sx, float sy)
         {
-            loopchildren(o, 
+            loopchildren(o,
             {
                 if(!isfullyclipped(sx + o->x, sy + o->y, o->w, o->h))
                     o->draw(sx + o->x, sy + o->y);
             });
         }
-       
+
         void draw()
         {
             draw(x, y);
         }
- 
+
         virtual Object *hover(float cx, float cy)
         {
             loopinchildrenrev(o, cx, cy,
@@ -281,7 +281,7 @@ namespace UI
             });
             return NULL;
         }
-   
+
         virtual void hovering(float cx, float cy)
         {
         }
@@ -305,7 +305,7 @@ namespace UI
 
         virtual const char *getname() const { return ""; }
 
-        bool isnamed(const char *name) const { return !strcmp(name, getname()); } 
+        bool isnamed(const char *name) const { return !strcmp(name, getname()); }
 
         Object *findname(const char *name, bool recurse = true, const Object *exclude = NULL) const
         {
@@ -323,7 +323,7 @@ namespace UI
             });
             return NULL;
         }
-            
+
         Object *findsibling(const char *name) const
         {
             for(const Object *prev = this, *cur = parent; cur; prev = cur, cur = cur->parent)
@@ -367,11 +367,11 @@ namespace UI
 
             float childspace = (w - children.last()->x - children.last()->w) / max(children.length() - 1, 1),
                   offset = 0;
-            loopchildren(o, 
+            loopchildren(o,
             {
                 o->x = offset;
                 offset += o->w;
-                if(!i || i+1 < children.length()) offset += childspace;
+                if(i < children.length()) offset += childspace;
                 o->adjustlayout(o->x, 0, offset - o->x, h);
             });
         }
@@ -407,13 +407,13 @@ namespace UI
             {
                 o->y = offset;
                 offset += o->h;
-                if(!i || i+1 < children.length()) offset += childspace;
+                if(i < children.length()) offset += childspace;
                 o->adjustlayout(0, o->y, w, offset - o->y);
 
             });
         }
 
-    };  
+    };
 
     struct Table : Object
     {
@@ -425,8 +425,8 @@ namespace UI
 
         void layout()
         {
-            widths.setsizenodelete(0);
-            heights.setsizenodelete(0);
+            widths.setsize(0);
+            heights.setsize(0);
 
             int column = 0, row = 0;
             loopchildren(o,
@@ -440,7 +440,7 @@ namespace UI
                 if(!column) row++;
             });
 
-            w = h = 0; 
+            w = h = 0;
             column = row = 0;
             float offset = 0;
             loopchildren(o,
@@ -451,7 +451,7 @@ namespace UI
                 offset += widths[column];
                 w = max(w, offset);
                 column = (column + 1) % columns;
-                if(!column) 
+                if(!column)
                 {
                     offset = 0;
                     h += heights[row];
@@ -481,7 +481,7 @@ namespace UI
                 o->x = offsetx;
                 o->y = offsety;
                 offsetx += widths[column];
-                if(!column || column+1 < widths.length()) offsetx += cspace;
+                if(column < widths.length()) offsetx += cspace;
                 o->adjustlayout(o->x, o->y, offsetx - o->x, nexty - o->y);
                 column = (column + 1) % columns;
                 if(!column)
@@ -492,7 +492,7 @@ namespace UI
                     if(row < heights.length())
                     {
                         nexty += heights[row];
-                        if(row+1 < heights.length()) nexty += rspace;
+                        if(row < heights.length()) nexty += rspace;
                     }
                 }
             });
@@ -539,7 +539,7 @@ namespace UI
 
             w = max(w, minw);
             h = max(h, minh);
-        } 
+        }
     };
 
     struct Offsetter : Object
@@ -557,7 +557,7 @@ namespace UI
                 o->x += offsetx;
                 o->y += offsety;
             });
-            
+
             w += offsetx;
             h += offsety;
         }
@@ -588,7 +588,7 @@ namespace UI
         {
             adjustchildrento(0, 0, virtw, virth);
         }
-            
+
         void draw(float sx, float sy)
         {
             if((clipw && virtw > clipw) || (cliph && virth > cliph))
@@ -690,7 +690,7 @@ namespace UI
         float offsetx, offsety;
 
         Scroller(float clipw = 0, float cliph = 0) : Clipper(clipw, cliph), offsetx(0), offsety(0) {}
-       
+
         Object *target(float cx, float cy)
         {
             if(cx + offsetx >= virtw || cy + offsety >= virth) return NULL;
@@ -734,7 +734,7 @@ namespace UI
 
         const char *getname() const { return "scroller"; }
     };
-  
+
     struct ScrollBar : Object
     {
         float arrowsize, arrowspeed;
@@ -743,8 +743,8 @@ namespace UI
         ScrollBar(float arrowsize = 0, float arrowspeed = 0) : arrowsize(arrowsize), arrowspeed(arrowspeed), arrowdir(0) {}
 
         int forks() const { return 5; }
-        int choosefork() const 
-        { 
+        int choosefork() const
+        {
             switch(arrowdir)
             {
                 case -1: return isselected(this) ? 2 : (ishovering(this) ? 1 : 0);
@@ -781,21 +781,21 @@ namespace UI
             if(!arrowdir) scrollto(cx, cy);
             else hovering(cx, cy);
         }
-    
+
         virtual void arrowscroll()
         {
         }
 
         void hovering(float cx, float cy)
         {
-            if(isselected(this)) 
+            if(isselected(this))
             {
                 if(arrowdir) arrowscroll();
             }
             else
             {
                 Object *button = findname("scrollbutton", false);
-                if(button && isselected(button)) 
+                if(button && isselected(button))
                 {
                     arrowdir = 0;
                     button->hovering(cx - button->x, cy - button->y);
@@ -840,7 +840,7 @@ namespace UI
                 scrollbar->movebutton(this, offsetx, offsety, cx, cy);
             }
         }
-  
+
         void selected(float cx, float cy)
         {
             offsetx = cx;
@@ -854,12 +854,12 @@ namespace UI
     {
         HorizontalScrollBar(float arrowsize = 0, float arrowspeed = 0) : ScrollBar(arrowsize, arrowspeed) {}
 
-        int choosedir(float cx, float cy) const 
+        int choosedir(float cx, float cy) const
         {
             if(cx < arrowsize) return -1;
             else if(cx >= w - arrowsize) return 1;
             return 0;
-        }   
+        }
 
         void arrowscroll()
         {
@@ -949,7 +949,7 @@ namespace UI
             scrollto(o->x + tox, o->y + toy - fromy);
         }
     };
-        
+
     static bool checkalphamask(Texture *tex, float x, float y)
     {
         if(!tex->alphamask)
@@ -998,7 +998,7 @@ namespace UI
             Object::draw(sx, sy);
         }
     };
-    
+
     struct Image : Filler
     {
         Texture *tex;
@@ -1024,7 +1024,7 @@ namespace UI
         }
     };
 
-    VAR(thumbtime, 0, 50, 1000);
+    VAR(thumbtime, 0, 25, 1000);
     static int lastthumbnail = 0;
 
     struct SlotViewer : Filler
@@ -1037,7 +1037,7 @@ namespace UI
         {
             Object *o = Object::target(cx, cy);
             if(o || !texmru.inrange(slotnum)) return o;
-            Slot &slot = lookuptexture(texmru[slotnum], false); 
+            Slot &slot = lookupslot(texmru[slotnum], false);
             if(slot.sts.length() && (slot.loaded || slot.thumbnail)) return this;
             return NULL;
         }
@@ -1045,13 +1045,14 @@ namespace UI
         void drawslot(Slot &slot, float sx, float sy)
         {
             Texture *tex = notexture, *glowtex = NULL, *layertex = NULL;
+	    VSlot &vslot = *slot.variants;
             if(slot.loaded)
             {
                 tex = slot.sts[0].t;
                 if(slot.texmask&(1<<TEX_GLOW)) { loopv(slot.sts) if(slot.sts[i].type==TEX_GLOW) { glowtex = slot.sts[i].t; break; } }
-                if(slot.layer)
+                if(vslot.layer)
                 {
-                    Slot &layer = lookuptexture(slot.layer);
+                    Slot &layer = lookupslot(vslot.layer);
                     layertex = layer.sts[0].t;
                 }
             }
@@ -1063,14 +1064,14 @@ namespace UI
             static Shader *rgbonlyshader = NULL;
             if(!rgbonlyshader) rgbonlyshader = lookupshaderbyname("rgbonly");
             rgbonlyshader->set();
-        
+
             float tc[4][2] = { { 0, 0 }, { 1, 0 }, { 1, 1 }, { 0, 1 } };
-            int xoff = slot.xoffset, yoff = slot.yoffset;
-            if(slot.rotation)
+            int xoff = vslot.xoffset, yoff = vslot.yoffset;
+            if(vslot.rotation)
             {
-                if((slot.rotation&5) == 1) { swap(xoff, yoff); loopk(4) swap(tc[k][0], tc[k][1]); }
-                if(slot.rotation >= 2 && slot.rotation <= 4) { xoff *= -1; loopk(4) tc[k][0] *= -1; }
-                if(slot.rotation <= 2 || slot.rotation == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
+                if((vslot.rotation&5) == 1) { swap(xoff, yoff); loopk(4) swap(tc[k][0], tc[k][1]); }
+                if(vslot.rotation >= 2 && vslot.rotation <= 4) { xoff *= -1; loopk(4) tc[k][0] *= -1; }
+                if(vslot.rotation <= 2 || vslot.rotation == 5) { yoff *= -1; loopk(4) tc[k][1] *= -1; }
             }
             loopk(4) { tc[k][0] = tc[k][0]/xt - float(xoff)/tex->xs; tc[k][1] = tc[k][1]/yt - float(yoff)/tex->ys; }
             glBindTexture(GL_TEXTURE_2D, tex->id);
@@ -1085,7 +1086,7 @@ namespace UI
             {
                 glBlendFunc(GL_SRC_ALPHA, GL_ONE);
                 glBindTexture(GL_TEXTURE_2D, glowtex->id);
-                glColor3fv(slot.glowcolor.v);
+                glColor3fv(vslot.glowcolor.v);
                 glBegin(GL_QUADS);
                 glTexCoord2fv(tc[0]); glVertex2f(sx,   sy);
                 glTexCoord2fv(tc[1]); glVertex2f(sx+w, sy);
@@ -1097,7 +1098,7 @@ namespace UI
             }
             if(layertex)
             {
-                glBindTexture(GL_TEXTURE_2D, glowtex->id);
+                glBindTexture(GL_TEXTURE_2D, layertex->id);
                 glBegin(GL_QUADS);
                 glTexCoord2fv(tc[0]); glVertex2f(sx+w/2, sy+h/2);
                 glTexCoord2fv(tc[1]); glVertex2f(sx+w,   sy+h/2);
@@ -1108,32 +1109,32 @@ namespace UI
 
             defaultshader->set();
         }
- 
+
         void draw(float sx, float sy)
         {
             if(texmru.inrange(slotnum))
             {
-                Slot &slot = lookuptexture(texmru[slotnum], false);
+                Slot &slot = lookupslot(texmru[slotnum], false);
                 if(slot.sts.length())
                 {
                     if(slot.loaded || slot.thumbnail) drawslot(slot, sx, sy);
-                    else if(totalmillis-lastthumbnail>=thumbtime)
+                    else if(totalmillis-lastthumbnail >= thumbtime)
                     {
                         loadthumbnail(slot);
-                        thumbtime = totalmillis;
+                        lastthumbnail = totalmillis;
                     }
                 }
-            } 
+            }
 
             Object::draw(sx, sy);
         }
     };
-      
+
     struct CroppedImage : Image
     {
         float cropx, cropy, cropw, croph;
 
-        CroppedImage(Texture *tex, float minw = 0, float minh = 0, float cropx = 0, float cropy = 0, float cropw = 1, float croph = 1) 
+        CroppedImage(Texture *tex, float minw = 0, float minh = 0, float cropx = 0, float cropy = 0, float cropw = 1, float croph = 1)
             : Image(tex, minw, minh), cropx(cropx), cropy(cropy), cropw(cropw), croph(croph) {}
 
         Object *target(float cx, float cy)
@@ -1154,7 +1155,7 @@ namespace UI
             Object::draw(sx, sy);
         }
     };
-                
+
     struct StretchedImage : Image
     {
         StretchedImage(Texture *tex, float minw = 0, float minh = 0) : Image(tex, minw, minh) {}
@@ -1170,7 +1171,7 @@ namespace UI
             else if(cx < minw/2) mx = cx/minw;
             else if(cx >= w - minw/2) mx = 1 - (w - cx) / minw;
             else mx = 0.5f;
-            if(h <= minh) my = cy/h; 
+            if(h <= minh) my = cy/h;
             else if(cy < minh/2) my = cy/minh;
             else if(cy >= h - minh/2) my = 1 - (h - cy) / minh;
             else my = 0.5f;
@@ -1185,7 +1186,7 @@ namespace UI
             float splitw = (minw ? min(minw, w) : w) / 2,
                   splith = (minh ? min(minh, h) : h) / 2,
                   vy = sy, ty = 0;
-            loopi(3) 
+            loopi(3)
             {
                 float vh = 0, th = 0;
                 switch(i)
@@ -1195,7 +1196,7 @@ namespace UI
                     case 2: vh = splith; th = 0.5f; break;
                 }
                 float vx = sx, tx = 0;
-                loopj(3) 
+                loopj(3)
                 {
                     float vw = 0, tw = 0;
                     switch(j)
@@ -1218,7 +1219,7 @@ namespace UI
             Object::draw(sx, sy);
         }
     };
-        
+
     struct BorderedImage : Image
     {
         float texborder, screenborder;
@@ -1228,7 +1229,7 @@ namespace UI
         void layout()
         {
             Object::layout();
-    
+
             w = max(w, 2*screenborder);
             h = max(h, 2*screenborder);
         }
@@ -1260,7 +1261,7 @@ namespace UI
                 float vh = 0, th = 0;
                 switch(i)
                 {
-                    case 0: vh = screenborder; th = texborder;
+                    case 0: vh = screenborder; th = texborder; break;
                     case 1: vh = h - 2*screenborder; th = 1 - 2*texborder; break;
                     case 2: vh = screenborder; th = texborder; break;
                 }
@@ -1270,7 +1271,7 @@ namespace UI
                     float vw = 0, tw = 0;
                     switch(j)
                     {
-                        case 0: vw = screenborder; tw = texborder; 
+                        case 0: vw = screenborder; tw = texborder; break;
                         case 1: vw = w - 2*screenborder; tw = 1 - 2*texborder; break;
                         case 2: vw = screenborder; tw = texborder; break;
                     }
@@ -1296,10 +1297,10 @@ namespace UI
         float scale;
         vec color;
 
-        Text(const char *str, float scale = 1, float r = 1, float g = 1, float b = 1) 
+        Text(const char *str, float scale = 1, float r = 1, float g = 1, float b = 1)
             : str(newstring(str)), scale(scale), color(r, g, b) {}
         ~Text() { delete[] str; }
-       
+
         float drawscale() const { return scale / (FONTH * uitextrows); }
 
         void draw(float sx, float sy)
@@ -1307,8 +1308,7 @@ namespace UI
             float k = drawscale();
             glPushMatrix();
             glScalef(k, k, 1);
-            glColor3fv(color.v);
-            draw_text(str, int(sx/k), int(sy/k), 0, 0, 0, 0);
+            draw_text(str, int(sx/k), int(sy/k), color.x * 255, color.y * 255, color.z * 255, 255);
             glColor3f(1, 1, 1);
             glPopMatrix();
 
@@ -1398,8 +1398,10 @@ namespace UI
                 case SDLK_TAB:
                     if(cooked && edit->maxy != 1) break;
                 case SDLK_ESCAPE:
+                    if(cooked) setfocus(NULL);
+                    return true;
                 case SDLK_KP_ENTER:
-                    setfocus(NULL);
+                    if(cooked && edit->maxy == 1) setfocus(NULL);
                     return true;
                 case SDLK_HOME:
                 case SDLK_END:
@@ -1420,10 +1422,10 @@ namespace UI
                     if(keyfilter && !strchr(keyfilter, cooked)) return true;
                     break;
             }
-            if(isdown) edit->key(code, cooked); 
+            if(isdown) edit->key(code, cooked);
             return true;
         }
-        
+
         void layout()
         {
             Object::layout();
@@ -1485,10 +1487,10 @@ namespace UI
     {
         char *onhide;
 
-        Window(const char *name, const char *onhide = NULL) 
-         : NamedObject(name), onhide(onhide && onhide[0] ? newstring(onhide) : NULL) 
+        Window(const char *name, const char *onhide = NULL)
+         : NamedObject(name), onhide(onhide && onhide[0] ? newstring(onhide) : NULL)
         {}
-        ~Window() { DELETEA(onhide); } 
+        ~Window() { DELETEA(onhide); }
 
         void hidden()
         {
@@ -1507,13 +1509,13 @@ namespace UI
         execute(contents);
         build.pop();
         return window;
-    } 
+    }
 
     ICOMMAND(showui, "sss", (char *name, char *contents, char *onhide),
     {
         if(build.length()) { intret(0); return; }
         Window *oldwindow = dynamic_cast<Window *>(world->findname(name, false));
-        if(oldwindow) 
+        if(oldwindow)
         {
             oldwindow->hidden();
             world->remove(oldwindow);
@@ -1544,7 +1546,7 @@ namespace UI
         if(!window) { intret(0); return; }
         Tag *tag = dynamic_cast<Tag *>(window->findname(tname));
         if(!tag) { intret(0); return; }
-        tag->children.deletecontentsp();
+        tag->children.deletecontents();
         build.add(tag);
         execute(contents);
         build.pop();
@@ -1553,7 +1555,7 @@ namespace UI
 
     void addui(Object *o, const char *children)
     {
-        if(build.length()) 
+        if(build.length())
         {
             o->parent = build.last();
             build.last()->children.add(o);
@@ -1573,7 +1575,7 @@ namespace UI
 
     ICOMMAND(uiclamp, "iiii", (int *left, int *right, int *bottom, int *top),
     {
-        if(build.length()) build.last()->adjust = (build.last()->adjust & ~CLAMP_MASK) | 
+        if(build.length()) build.last()->adjust = (build.last()->adjust & ~CLAMP_MASK) |
                                                   (*left ? CLAMP_LEFT : 0) |
                                                   (*right ? CLAMP_RIGHT : 0) |
                                                   (*bottom ? CLAMP_BOTTOM : 0) |
@@ -1582,7 +1584,7 @@ namespace UI
 
     ICOMMAND(uitag, "ss", (char *name, char *children),
         addui(new Tag(name), children));
-       
+
     ICOMMAND(uihlist, "fs", (float *space, char *children),
         addui(new HorizontalList(*space), children));
 
@@ -1641,10 +1643,10 @@ namespace UI
         if(image && image->tex==notexture) image->tex = textureload(texname, 3, true, false);
     });
 
-    ICOMMAND(uicolor, "ffffffs", (float *r, float *g, float *b, float *a, float *minw, float *minh, char *children), 
+    ICOMMAND(uicolor, "ffffffs", (float *r, float *g, float *b, float *a, float *minw, float *minh, char *children),
         addui(new Rectangle(Rectangle::SOLID, *r, *g, *b, *a, *minw, *minh), children));
 
-    ICOMMAND(uimodcolor, "ffffffs", (float *r, float *g, float *b, float *minw, float *minh, char *children),
+    ICOMMAND(uimodcolor, "fffffs", (float *r, float *g, float *b, float *minw, float *minh, char *children),
         addui(new Rectangle(Rectangle::MODULATE, *r, *g, *b, 1, *minw, *minh), children));
 
     ICOMMAND(uistretchedimage, "sffs", (char *texname, float *minw, float *minh, char *children),
@@ -1652,7 +1654,7 @@ namespace UI
 
     ICOMMAND(uicroppedimage, "sffsssss", (char *texname, float *minw, float *minh, char *cropx, char *cropy, char *cropw, char *croph, char *children),
         Texture *tex = textureload(texname, 3, true, false);
-        addui(new CroppedImage(tex, *minw, *minh, 
+        addui(new CroppedImage(tex, *minw, *minh,
             strchr(cropx, 'p') ? atof(cropx) / tex->xs : atof(cropx),
             strchr(cropy, 'p') ? atof(cropy) / tex->ys : atof(cropy),
             strchr(cropw, 'p') ? atof(cropw) / tex->xs : atof(cropw),
@@ -1691,8 +1693,8 @@ namespace UI
         return true;
     }
 
-    bool hascursor(bool targeting) 
-    { 
+    bool hascursor(bool targeting)
+    {
         if(world->children.length())
         {
             if(!targeting) return true;
@@ -1714,7 +1716,7 @@ namespace UI
         {
             case -1:
             {
-                if(isdown) 
+                if(isdown)
                 {
                     selected = world->select(cursorx*world->w, cursory*world->h);
                     if(selected) selected->selected(selectx, selecty);
@@ -1725,9 +1727,16 @@ namespace UI
 
             default:
                 return world->key(code, isdown, cooked);
-        }                
+        }
     }
 
+	VAR(mainmenu, 1, 1, 0);
+	void clearmainmenu()
+	{
+		if(mainmenu && (isconnected() || haslocalclients()))
+			mainmenu = 0;
+	}
+
     void setup()
     {
         world = new World;
@@ -1738,7 +1747,12 @@ namespace UI
     void update()
     {
         loopv(executelater) execute(executelater[i]);
-        executelater.deletecontentsa();
+        executelater.deletearrays();
+
+        if(mainmenu && !isconnected(true) && !world->children.length() && identexists("showmain"))
+        {
+            execute("showmain");
+        }
 
         world->layout();
 
@@ -1755,15 +1769,16 @@ namespace UI
         textediting = dynamic_cast<TextEditor *>(focused);
         if((textediting!=NULL) != wastextediting)
         {
-            SDL_EnableUNICODE(textediting!=NULL || saycommandon);
-            keyrepeat(textediting!=NULL || editmode || saycommandon);
+            SDL_EnableUNICODE(textediting!=NULL);
+            keyrepeat(textediting!=NULL || editmode);
         }
-    } 
-             
+    }
+
     void render()
     {
         if(world->children.empty()) return;
 
+        glEnable(GL_BLEND);
         glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
 
         glMatrixMode(GL_PROJECTION);
@@ -1783,6 +1798,7 @@ namespace UI
         glPopMatrix();
         glMatrixMode(GL_MODELVIEW);
         glPopMatrix();
+        glEnable(GL_BLEND);
     }
 }
 
