@@ -1,6 +1,6 @@
 struct vertmodel : animmodel
 {
-    struct vert { vec norm, pos; };
+    struct vert { vec pos, norm; };
     struct vvertff { vec pos; float u, v; };
     struct vvert : vvertff { vec norm; };
     struct vvertbump : vvert { vec tangent; float bitangent; };
@@ -14,7 +14,7 @@ struct vertmodel : animmodel
         GLuint vbuf;
         animstate as;
         int millis;
- 
+
         vbocacheentry() : vdata(NULL), vbuf(0) { as.cur.fr1 = as.prev.fr1 = -1; }
     };
 
@@ -41,75 +41,28 @@ struct vertmodel : animmodel
             DELETEA(tris);
         }
 
+        void smoothnorms(float limit = 0, bool areaweight = true)
+        {
+            if(((vertmeshgroup *)group)->numframes != 1)
+            {
+                buildnorms(areaweight);
+                return;
+            }
+            mesh::smoothnorms(verts, numverts, tris, numtris, limit, areaweight);
+        }
+
         void buildnorms(bool areaweight = true)
         {
             loopk(((vertmeshgroup *)group)->numframes)
-            {
-                vert *fverts = &verts[k*numverts];
-                loopi(numverts) fverts[i].norm = vec(0, 0, 0);
-                loopi(numtris)
-                {
-                    tri &t = tris[i];
-                    vert &v1 = fverts[t.vert[0]], &v2 = fverts[t.vert[1]], &v3 = fverts[t.vert[2]];
-                    vec norm;
-                    norm.cross(vec(v2.pos).sub(v1.pos), vec(v3.pos).sub(v1.pos));
-                    if(!areaweight) norm.normalize();
-                    v1.norm.add(norm);
-                    v2.norm.add(norm);
-                    v3.norm.add(norm);
-                }
-                loopi(numverts) fverts[i].norm.normalize();
-            }
+                mesh::buildnorms(&verts[k*numverts], numverts, tris, numtris, areaweight);
         }
 
-        void calctangents()
+        void calctangents(bool areaweight = true)
         {
             if(bumpverts) return;
-            vec *tangent = new vec[2*numverts], *bitangent = tangent+numverts;
-            memset(tangent, 0, 2*numverts*sizeof(vec));
             bumpverts = new bumpvert[((vertmeshgroup *)group)->numframes*numverts];
             loopk(((vertmeshgroup *)group)->numframes)
-            {
-                vert *fverts = &verts[k*numverts];
-                loopi(numtris)
-                {
-                    const tri &t = tris[i];
-                    const tcvert &tc0 = tcverts[t.vert[0]],
-                                 &tc1 = tcverts[t.vert[1]],
-                                 &tc2 = tcverts[t.vert[2]];
- 
-                    vec v0(fverts[t.vert[0]].pos),
-                        e1(fverts[t.vert[1]].pos), 
-                        e2(fverts[t.vert[2]].pos);
-                    e1.sub(v0);
-                    e2.sub(v0);
- 
-                    float u1 = tc1.u - tc0.u, v1 = tc1.v - tc0.v, 
-                          u2 = tc2.u - tc0.u, v2 = tc2.v - tc0.v,
-                          scale = u1*v2 - u2*v1;
-                    if(scale!=0) scale = 1.0f / scale;
-                    vec u(e1), v(e2);
-                    u.mul(v2).sub(vec(e2).mul(v1)).mul(scale);
-                    v.mul(u1).sub(vec(e1).mul(u2)).mul(scale);
- 
-                    loopj(3)
-                    {
-                        tangent[t.vert[j]].add(u);
-                        bitangent[t.vert[j]].add(v);
-                    }
-                }
-                bumpvert *fbumpverts = &bumpverts[k*numverts];
-                loopi(numverts)
-                {
-                    const vec &n = fverts[i].norm,
-                              &t = tangent[i],
-                              &bt = bitangent[i];
-                    bumpvert &bv = fbumpverts[i];
-                    (bv.tangent = t).sub(vec(n).mul(n.dot(t))).normalize();
-                    bv.bitangent = vec().cross(n, t).dot(bt) < 0 ? -1 : 1;
-                }
-            }
-            delete[] tangent;
+                mesh::calctangents(&bumpverts[k*numverts], &verts[k*numverts], tcverts, numverts, tris, numtris, areaweight);
         }
 
         void calcbb(int frame, vec &bbmin, vec &bbmax, const matrix3x4 &m)
@@ -132,7 +85,7 @@ struct vertmodel : animmodel
             loopj(numtris)
             {
                 BIH::tri &t = out[noclip ? 1 : 0].add();
-                t.tex = tex->bpp==4 ? tex : NULL;
+                t.tex = tex;
                 t.a = m.transform(fverts[tris[j].vert[0]].pos);
                 t.b = m.transform(fverts[tris[j].vert[1]].pos);
                 t.c = m.transform(fverts[tris[j].vert[2]].pos);
@@ -192,7 +145,7 @@ struct vertmodel : animmodel
         }
 
         template<class T>
-        int genvbo(vector<ushort> &idxs, int offset, vector<T> &vverts)
+        int genvbo(vector<ushort> &idxs, int offset, vector<T> &vverts, int *htdata, int htlen)
         {
             voffset = offset;
             eoffset = idxs.length();
@@ -200,18 +153,18 @@ struct vertmodel : animmodel
             loopi(numtris)
             {
                 tri &t = tris[i];
-                loopj(3) 
+                loopj(3)
                 {
                     int index = t.vert[j];
                     tcvert &tc = tcverts[index];
                     vert &v = verts[index];
-                    loopvk(vverts)
+                    int htidx = hthash(v.pos)&(htlen-1);
+                    loopk(htlen)
                     {
-                        if(comparevert(vverts[k], index, tc, v)) { minvert = min(minvert, (ushort)k); idxs.add((ushort)k); goto found; }
+                        int &vidx = htdata[(htidx+k)&(htlen-1)];
+                        if(vidx < 0) { vidx = idxs.add(ushort(vverts.length())); assignvert(vverts.add(), index, tc, v); break; }
+                        else if(comparevert(vverts[vidx], index, tc, v)) { minvert = min(minvert, idxs.add(ushort(vidx))); break; }
                     }
-                    idxs.add(vverts.length());
-                    assignvert(vverts.add(), index, tc, v);
-                found:;
                 }
             }
             minvert = min(minvert, ushort(voffset));
@@ -245,61 +198,47 @@ struct vertmodel : animmodel
             }
         }
 
-        void interpverts(const animstate &as, bool norms, bool tangents, void *vdata, skin &s)
+        void interpverts(const animstate &as, bool norms, bool tangents, void * RESTRICT vdata, skin &s)
         {
-            vert *vert1 = &verts[as.cur.fr1 * numverts],
-                 *vert2 = &verts[as.cur.fr2 * numverts],
-                 *pvert1 = as.interp<1 ? &verts[as.prev.fr1 * numverts] : NULL, 
-                 *pvert2 = as.interp<1 ? &verts[as.prev.fr2 * numverts] : NULL;
-            #define ip(p1, p2, t)   (p1+t*(p2-p1))
-            #define ip_v(p, c, t)   ip(p##1[i].c, p##2[i].c, t)
-            #define ip_v_ai(c)      ip(ip_v(pvert, c, as.prev.t), ip_v(vert, c, as.cur.t), as.interp)
-            #define ip_pos          vec(ip_v(vert, pos.x, as.cur.t), ip_v(vert, pos.y, as.cur.t), ip_v(vert, pos.z, as.cur.t))
-            #define ip_pos_ai       vec(ip_v_ai(pos.x), ip_v_ai(pos.y), ip_v_ai(pos.z))
-            #define ip_norm         vec(ip_v(vert, norm.x, as.cur.t), ip_v(vert, norm.y, as.cur.t), ip_v(vert, norm.z, as.cur.t))
-            #define ip_norm_ai      vec(ip_v_ai(norm.x), ip_v_ai(norm.y), ip_v_ai(norm.z))
-            #define ip_b_ai(c)      ip(ip_v(bpvert, c, as.prev.t), ip_v(bvert, c, as.cur.t), as.interp)
-            #define ip_tangent      vec(ip_v(bvert, tangent.x, as.cur.t), ip_v(bvert, tangent.y, as.cur.t), ip_v(bvert, tangent.z, as.cur.t))
-            #define ip_tangent_ai   vec(ip_b_ai(tangent.x), ip_b_ai(tangent.y), ip_b_ai(tangent.z))
+            const vert * RESTRICT vert1 = &verts[as.cur.fr1 * numverts],
+                       * RESTRICT vert2 = &verts[as.cur.fr2 * numverts],
+                       * RESTRICT pvert1 = as.interp<1 ? &verts[as.prev.fr1 * numverts] : NULL,
+                       * RESTRICT pvert2 = as.interp<1 ? &verts[as.prev.fr2 * numverts] : NULL;
+            #define ipvert(attrib)   v.attrib.lerp(vert1[i].attrib, vert2[i].attrib, as.cur.t)
+            #define ipbvert(attrib)  v.attrib.lerp(bvert1[i].attrib, bvert2[i].attrib, as.cur.t)
+            #define ipvertp(attrib)  v.attrib.lerp(pvert1[i].attrib, pvert2[i].attrib, as.prev.t).lerp(vec().lerp(vert1[i].attrib, vert2[i].attrib, as.cur.t), as.interp)
+            #define ipbvertp(attrib) v.attrib.lerp(bpvert1[i].attrib, bpvert2[i].attrib, as.prev.t).lerp(vec().lerp(bvert1[i].attrib, bvert2[i].attrib, as.cur.t), as.interp)
             #define iploop(type, body) \
                 loopi(numverts) \
                 { \
-                    type &v = ((type *)vdata)[i]; \
+                    type &v = ((type * RESTRICT)vdata)[i]; \
                     body; \
                 }
             if(tangents)
             {
-                bumpvert *bvert1 = &bumpverts[as.cur.fr1 * numverts],
-                         *bvert2 = &bumpverts[as.cur.fr2 * numverts],
-                         *bpvert1 = as.interp<1 ? &bumpverts[as.prev.fr1 * numverts] : NULL, 
-                         *bpvert2 = as.interp<1 ? &bumpverts[as.prev.fr2 * numverts] : NULL;
-                if(as.interp<1) iploop(vvertbump, { v.pos = ip_pos_ai; v.norm = ip_norm_ai; v.tangent = ip_tangent_ai; v.bitangent = bvert1[i].bitangent; })
-                else iploop(vvertbump, { v.pos = ip_pos; v.norm = ip_norm; v.tangent = ip_tangent; v.bitangent = bvert1[i].bitangent; })
+                const bumpvert * RESTRICT bvert1 = &bumpverts[as.cur.fr1 * numverts],
+                               * RESTRICT bvert2 = &bumpverts[as.cur.fr2 * numverts],
+                               * RESTRICT bpvert1 = as.interp<1 ? &bumpverts[as.prev.fr1 * numverts] : NULL,
+                               * RESTRICT bpvert2 = as.interp<1 ? &bumpverts[as.prev.fr2 * numverts] : NULL;
+                if(as.interp<1) iploop(vvertbump, { ipvertp(pos); ipvertp(norm); ipbvertp(tangent); v.bitangent = bvert1[i].bitangent; })
+                else iploop(vvertbump, { ipvert(pos); ipvert(norm); ipbvert(tangent); v.bitangent = bvert1[i].bitangent; })
             }
             else if(norms)
             {
-                if(as.interp<1) iploop(vvert, { v.pos = ip_pos_ai; v.norm = ip_norm_ai; })
-                else iploop(vvert, { v.pos = ip_pos; v.norm = ip_norm; })
+                if(as.interp<1) iploop(vvert, { ipvertp(pos); ipvertp(norm); })
+                else iploop(vvert, { ipvert(pos); ipvert(norm); })
             }
-            else if(as.interp<1) iploop(vvertff, v.pos = ip_pos_ai)
-            else iploop(vvertff, v.pos = ip_pos)
+            else if(as.interp<1) iploop(vvertff, ipvertp(pos))
+            else iploop(vvertff, ipvert(pos))
             #undef iploop
-            #undef ip
-            #undef ip_v
-            #undef ip_v_ai
-            #undef ip_pos
-            #undef ip_pos_ai
-            #undef ip_norm
-            #undef ip_norm_ai
-            #undef ip_b_ai
-            #undef ip_tangent
-            #undef ip_tangent_ai
+            #undef ipvert
+            #undef ipbvert
+            #undef ipvertp
+            #undef ipbvertp
         }
 
         void render(const animstate *as, skin &s, vbocacheentry &vc)
         {
-            s.bind(this, as);
-
             if(!(as->anim&ANIM_NOSKIN))
             {
                 if(s.multitextured())
@@ -396,7 +335,7 @@ struct vertmodel : animmodel
         int vlen, vertsize;
         uchar *vdata;
 
-        vertmeshgroup() : numframes(0), tags(NULL), numtags(0), edata(NULL), ebuf(0), vdata(NULL) 
+        vertmeshgroup() : numframes(0), tags(NULL), numtags(0), edata(NULL), ebuf(0), vdata(NULL)
         {
         }
 
@@ -404,7 +343,7 @@ struct vertmodel : animmodel
         {
             DELETEA(tags);
             if(ebuf) glDeleteBuffers_(1, &ebuf);
-            loopi(MAXVBOCACHE) 
+            loopi(MAXVBOCACHE)
             {
                 DELETEA(vbocache[i].vdata);
                 if(vbocache[i].vbuf) glDeleteBuffers_(1, &vbocache[i].vbuf);
@@ -428,44 +367,32 @@ struct vertmodel : animmodel
 
         void calctagmatrix(part *p, int i, const animstate &as, glmatrixf &matrix)
         {
-            const matrix3x4 &tag1 = tags[as.cur.fr1*numtags + i].transform, 
+            const matrix3x4 &tag1 = tags[as.cur.fr1*numtags + i].transform,
                             &tag2 = tags[as.cur.fr2*numtags + i].transform;
-            #define ip(p1, p2, t) (p1+t*(p2-p1))
-            #define ip_ai_tag(c) ip( ip( tag1p.c, tag2p.c, as.prev.t), ip( tag1.c, tag2.c, as.cur.t), as.interp)
+            matrix3x4 tag;
+            tag.lerp(tag1, tag2, as.cur.t);
             if(as.interp<1)
             {
-                const matrix3x4 &tag1p = tags[as.prev.fr1*numtags + i].transform, 
+                const matrix3x4 &tag1p = tags[as.prev.fr1*numtags + i].transform,
                                 &tag2p = tags[as.prev.fr2*numtags + i].transform;
-                loopj(4)
-                {
-                    matrix[4*j+0] = ip_ai_tag(a[j]);
-                    matrix[4*j+1] = ip_ai_tag(b[j]);
-                    matrix[4*j+2] = ip_ai_tag(c[j]);
-                }
+                matrix3x4 tagp;
+                tagp.lerp(tag1p, tag2p, as.prev.t);
+                tag.lerp(tagp, tag, as.interp);
             }
-            else loopj(4)
-            {
-                matrix[4*j+0] = ip(tag1.a[j], tag2.a[j], as.cur.t);
-                matrix[4*j+1] = ip(tag1.b[j], tag2.b[j], as.cur.t);
-                matrix[4*j+2] = ip(tag1.c[j], tag2.c[j], as.cur.t);
-            } 
-            #undef ip_ai_tag
-            #undef ip 
+            matrix = glmatrixf(tag);
             matrix[12] = (matrix[12] + p->translate.x) * p->model->scale;
             matrix[13] = (matrix[13] + p->translate.y) * p->model->scale;
             matrix[14] = (matrix[14] + p->translate.z) * p->model->scale;
-            matrix[3] = matrix[7] = matrix[11] = 0.0f;
-            matrix[15] = 1.0f;
         }
 
         void genvbo(bool norms, bool tangents, vbocacheentry &vc)
         {
-            if(hasVBO) 
+            if(hasVBO)
             {
                 if(!vc.vbuf) glGenBuffers_(1, &vc.vbuf);
                 if(ebuf) return;
             }
-            else if(edata) 
+            else if(edata)
             {
                 #define ALLOCVDATA(vdata) \
                     do \
@@ -488,17 +415,17 @@ struct vertmodel : animmodel
             {
                 loopv(meshes) vlen += ((vertmesh *)meshes[i])->genvbo(idxs, vlen);
                 DELETEA(vdata);
-                if(hasVBO) ALLOCVDATA(vdata); 
+                if(hasVBO) ALLOCVDATA(vdata);
                 else ALLOCVDATA(vc.vdata);
-            } 
-            else 
+            }
+            else
             {
                 if(hasVBO) glBindBuffer_(GL_ARRAY_BUFFER_ARB, vc.vbuf);
                 #define GENVBO(type) \
                     do \
                     { \
                         vector<type> vverts; \
-                        loopv(meshes) vlen += ((vertmesh *)meshes[i])->genvbo(idxs, vlen, vverts); \
+                        loopv(meshes) vlen += ((vertmesh *)meshes[i])->genvbo(idxs, vlen, vverts, htdata, htlen); \
                         if(hasVBO) glBufferData_(GL_ARRAY_BUFFER_ARB, vverts.length()*sizeof(type), vverts.getbuf(), GL_STATIC_DRAW_ARB); \
                         else \
                         { \
@@ -507,9 +434,17 @@ struct vertmodel : animmodel
                             memcpy(vc.vdata, vverts.getbuf(), vverts.length()*sizeof(type)); \
                         } \
                     } while(0)
+                int numverts = 0, htlen = 128;
+                loopv(meshes) numverts += ((vertmesh *)meshes[i])->numverts;
+                while(htlen < numverts) htlen *= 2;
+                if(numverts*4 > htlen*3) htlen *= 2;
+                int *htdata = new int[htlen];
+                memset(htdata, -1, htlen*sizeof(int));
                 if(tangents) GENVBO(vvertbump);
                 else if(norms) GENVBO(vvert);
                 else GENVBO(vvertff);
+                delete[] htdata;
+                if(hasVBO) glBindBuffer_(GL_ARRAY_BUFFER_ARB, 0);
             }
 
             if(hasVBO)
@@ -517,6 +452,7 @@ struct vertmodel : animmodel
                 glGenBuffers_(1, &ebuf);
                 glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER_ARB, ebuf);
                 glBufferData_(GL_ELEMENT_ARRAY_BUFFER_ARB, idxs.length()*sizeof(ushort), idxs.getbuf(), GL_STATIC_DRAW_ARB);
+                glBindBuffer_(GL_ELEMENT_ARRAY_BUFFER_ARB, 0);
             }
             else
             {
@@ -540,8 +476,8 @@ struct vertmodel : animmodel
                 if(hasVBO) glBindBuffer_(GL_ARRAY_BUFFER_ARB, vc.vbuf);
                 if(!lastvbuf) glEnableClientState(GL_VERTEX_ARRAY);
                 glVertexPointer(3, GL_FLOAT, vertsize, &vverts->pos);
+                lastvbuf = hasVBO ? (void *)(size_t)vc.vbuf : vc.vdata;
             }
-            lastvbuf = hasVBO ? (void *)(size_t)vc.vbuf : vc.vdata;
             if(as->anim&ANIM_NOSKIN)
             {
                 if(enabletc) disabletc();
@@ -600,7 +536,7 @@ struct vertmodel : animmodel
             }
 
             bool norms = false, tangents = false;
-            loopv(p->skins) 
+            loopv(p->skins)
             {
                 if(p->skins[i].normals()) norms = true;
                 if(p->skins[i].tangents()) tangents = true;
@@ -625,7 +561,7 @@ struct vertmodel : animmodel
                 {
                     vc->as = *as;
                     vc->millis = lastmillis;
-                    loopv(meshes) 
+                    loopv(meshes)
                     {
                         vertmesh &m = *(vertmesh *)meshes[i];
                         m.interpverts(*as, norms, tangents, (hasVBO ? vdata : vc->vdata) + m.voffset*vertsize, p->skins[i]);
@@ -633,15 +569,20 @@ struct vertmodel : animmodel
                     if(hasVBO)
                     {
                         glBindBuffer_(GL_ARRAY_BUFFER_ARB, vc->vbuf);
-                        glBufferData_(GL_ARRAY_BUFFER_ARB, vlen*vertsize, vdata, GL_STREAM_DRAW_ARB);    
+                        glBufferData_(GL_ARRAY_BUFFER_ARB, vlen*vertsize, vdata, GL_STREAM_DRAW_ARB);
                     }
                 }
                 vc->millis = lastmillis;
             }
-        
+
             bindvbo(as, *vc);
-            loopv(meshes) ((vertmesh *)meshes[i])->render(as, p->skins[i], *vc);
-            
+            loopv(meshes)
+            {
+                vertmesh *m = (vertmesh *)meshes[i];
+                p->skins[i].bind(m, as);
+                m->render(as, p->skins[i], *vc);
+            }
+
             loopv(p->links) calctagmatrix(p, p->links[i].tag, *as, p->links[i].matrix);
         }
     };
@@ -651,3 +592,65 @@ struct vertmodel : animmodel
     }
 };
 
+template<class MDL> struct vertloader : modelloader<MDL>
+{
+};
+
+template<class MDL> struct vertcommands : modelcommands<MDL, struct MDL::vertmesh>
+{
+    typedef struct MDL::part part;
+    typedef struct MDL::skin skin;
+
+    static void loadpart(char *model, float *smooth)
+    {
+        if(!MDL::loading) { conoutf("not loading an %s", MDL::formatname()); return; }
+        defformatstring(filename)("%s/%s", MDL::dir, model);
+        part &mdl = *new part;
+        MDL::loading->parts.add(&mdl);
+        mdl.model = MDL::loading;
+        mdl.index = MDL::loading->parts.length()-1;
+        if(mdl.index) mdl.pitchscale = mdl.pitchoffset = mdl.pitchmin = mdl.pitchmax = 0;
+        mdl.meshes = MDL::loading->sharemeshes(path(filename), double(*smooth > 0 ? cos(clamp(*smooth, 0.0f, 180.0f)*RAD) : 2));
+        if(!mdl.meshes) conoutf("could not load %s", filename);
+        else mdl.initskins();
+    }
+
+    static void setpitch(float *pitchscale, float *pitchoffset, float *pitchmin, float *pitchmax)
+    {
+        if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("not loading an %s", MDL::formatname()); return; }
+        part &mdl = *MDL::loading->parts.last();
+
+        mdl.pitchscale = *pitchscale;
+        mdl.pitchoffset = *pitchoffset;
+        if(*pitchmin || *pitchmax)
+        {
+            mdl.pitchmin = *pitchmin;
+            mdl.pitchmax = *pitchmax;
+        }
+        else
+        {
+            mdl.pitchmin = -360*mdl.pitchscale;
+            mdl.pitchmax = 360*mdl.pitchscale;
+        }
+    }
+
+    static void setanim(char *anim, int *frame, int *range, float *speed, int *priority)
+    {
+        if(!MDL::loading || MDL::loading->parts.empty()) { conoutf("not loading an %s", MDL::formatname()); return; }
+        vector<int> anims;
+        findanims(anim, anims);
+        if(anims.empty()) conoutf("could not find animation %s", anim);
+        else loopv(anims)
+        {
+            MDL::loading->parts.last()->setanim(0, anims[i], *frame, *range, *speed, *priority);
+        }
+    }
+
+    vertcommands()
+    {
+        if(MDL::multiparted()) modelcommand(loadpart, "load", "sf");
+        modelcommand(setpitch, "pitch", "ffff");
+        if(MDL::animated()) modelcommand(setanim, "anim", "siiff");
+    }
+};
+
