@@ -10,7 +10,9 @@ void backup(char *name, char *backupname)
     rename(findfile(name, "wb"), backupfile);
 }
 
-string ogzname, bakname, mcfname, acfname, picname, mname;
+string mname, mpath; //holds the mapname and mappath
+
+string ogzname, bakname, mcfname, acfname, picname;
 
 VARP(savebak, 0, 2, 2);
 
@@ -20,39 +22,62 @@ void cutogz(char *s)
     if(ogzp) *ogzp = '\0';
 }
 
-void getmapfilenames(const char *fname, const char *cname, char *pakname, char *mapname, char *cfgname)
+void getmapfilenames(const char *cname, bool fall)
 {
-    if(!cname) cname = fname;
-    string name;
-    copystring(name, cname, 100);
-    cutogz(name);
-    char *slash = strpbrk(name, "/\\");
-    if(slash)
-    {
-        copystring(pakname, name, slash-name+1);
-        copystring(cfgname, slash+1);
-    }
-    else
-    {
-        copystring(pakname, "base");
-        copystring(cfgname, name);
-    }
-    if(strpbrk(fname, "/\\")) copystring(mapname, fname);
-    else formatstring(mapname)("base/%s", fname);
-    cutogz(mapname);
+	const char *slash = NULL;
+	const char *next = NULL;
+
+	do
+	{
+		slash = next;
+		next = strpbrk(next ? (next + 1) : cname, "/\\");
+	} while (next != NULL);
+
+	if(slash == cname)
+	{
+		slash = strpbrk(cname + 1, "/\\");
+		if(!slash)
+			slash = cname;
+	}
+
+	if(slash)
+	{
+		copystring(mpath, "base/");
+		concatstring(mpath, cname, (slash - cname) + 6);
+		copystring(mname, slash+1);
+	}
+	else
+	{
+		formatstring(mpath)("packages/base/%s/%s.ogz", game::gameident(), cname);
+		path(mpath);
+		stream *f = openfile(mpath, "r");
+
+		if(!fall || f)
+		{
+			formatstring(mpath)("base/%s", game::gameident());
+			delete f;
+		}
+		else
+		{
+			copystring(mpath, "base");
+		}
+		copystring(mname, cname);
+	}
+
+	defformatstring(tmp)("packages/%s", mpath);
+	createdir(tmp);
 }
 
-void setmapfilenames(const char *fname, const char *cname = 0)
+void setmapfilenames(const char *fname, bool fall = true)
 {
-    string pakname, cfgname;
-    getmapfilenames(fname, cname, pakname, mname, cfgname);
+    getmapfilenames(fname, fall);
 
-    formatstring(ogzname)("packages/%s.ogz", mname);
-    if(savebak==1) formatstring(bakname)("packages/%s.BAK", mname);
-    else formatstring(bakname)("packages/%s_%d.BAK", mname, totalmillis);
-    formatstring(mcfname)("packages/%s/%s.cfg", pakname, cfgname);
-    formatstring(acfname)("packages/%s/%s-art.cfg", pakname, cfgname);
-    formatstring(picname)("packages/%s", mname);
+    formatstring(ogzname)("packages/%s/%s.ogz", mpath, mname);
+    if(savebak==1) formatstring(bakname)("packages/%s/%s.BAK", mpath, mname);
+    else formatstring(bakname)("packages/%s/%s_%d.BAK", mpath, mname, totalmillis);
+    formatstring(mcfname)("packages/%s/%s.cfg", mpath, mname);
+    formatstring(acfname)("packages/%s/%s-art.cfg", mpath, mname);
+    formatstring(picname)("packages/%s/%s", mpath, mname);
 
     path(ogzname);
     path(bakname);
@@ -63,14 +88,9 @@ void setmapfilenames(const char *fname, const char *cname = 0)
 
 void mapcfgname()
 {
-    const char *mname = game::getclientmap();
-    if(!*mname) mname = "untitled";
-
-    string pakname, mapname, mcfgname;
-    getmapfilenames(mname, NULL, pakname, mapname, mcfgname);
-    defformatstring(cfgname)("packages/%s/%s.cfg", pakname, mcfgname);
-    path(cfgname);
-    result(cfgname);
+	defformatstring(res)("packages/%s/%s.cfg", mpath, mname);
+	path(res);
+	result(res);
 }
 
 COMMAND(mapcfgname, "");
@@ -282,9 +302,11 @@ cube *loadchildren(stream *f)
 }
 
 
-//the following is from bloodfrontier... thanks quin!
+//the following is from redeclipse... thanks quin!
 void saveslotconfig(stream *h, Slot &s, int index)
 {
+    VSlot &vs = *s.variants;
+
     if(index >= 0)
     {
         if(s.shader)
@@ -309,19 +331,19 @@ void saveslotconfig(stream *h, Slot &s, int index)
         if(!j)
         {
             h->printf(" %d %d %d %f",
-                s.rotation, s.xoffset, s.yoffset, s.scale);
+                vs.rotation, vs.xoffset, vs.yoffset, vs.scale);
             if(index >= 0) h->printf(" // %d", index);
         }
         h->printf("\n");
     }
     if(index >= 0)
     {
-        if(s.scrollS != 0.f || s.scrollT != 0.f)
-            h->printf("texscroll %f %f\n", s.scrollS * 1000.0f, s.scrollT * 1000.0f);
-        if(s.layer != 0)
+        if(vs.scrollS != 0.f || vs.scrollT != 0.f)
+            h->printf("texscroll %f %f\n", vs.scrollS * 1000.0f, vs.scrollT * 1000.0f);
+        if(vs.layer != 0)
         {
-            if(s.layermaskname) h->printf("texlayer %d \"%s\" %d %f\n", s.layer, s.layermaskname, s.layermaskmode, s.layermaskscale);
-            else h->printf("texlayer %d\n", s.layer);
+            if(s.layermaskname) h->printf("texlayer %d \"%s\" %d %f\n", vs.layer, s.layermaskname, s.layermaskmode, s.layermaskscale);
+            else h->printf("texlayer %d\n", vs.layer);
         }
         if(s.autograss) h->printf("autograss \"%s\"\n", s.autograss);
     }
@@ -335,7 +357,7 @@ void writemapcfg(const char *a)
 
 	if (savebak)
 	{
-		defformatstring(bak)("packages/base/%s_%d.cfg", strlen(a) ? a : game::getclientmap(), totalmillis);
+		defformatstring(bak)("packages/%s/%s_%d.cfg.BAK", mpath, mname, totalmillis);
 		backup(acfname, bak);
 	}
 	stream *f = openfile(path(acfname, true), "w");
@@ -351,12 +373,12 @@ void writemapcfg(const char *a)
 	extern int sortidents(ident **x, ident **y);
 
 	vector<ident *> ids;
-	enumerate(*idents, ident, id, ids.add(&id));
+	enumerate(idents, ident, id, ids.add(&id));
 	ids.sort(sortidents);
 	loopv(ids)
 	{
 		ident &id = *ids[i];
-		if(!(id.flags&IDF_OVERRIDE) || id.flags&IDF_READONLY || id.override==NO_OVERRIDE) continue; switch(id.type)
+		if(!(id.flags&IDF_OVERRIDDEN) || id.flags&IDF_READONLY) continue; switch(id.type)
 		{
 			case ID_VAR: f->printf(id.flags & IDF_HEX ? "//%s 0x%.6X\n" : "//%s %d\n", id.name, *id.storage.i); break;
 			case ID_FVAR: f->printf("//%s %s\n", id.name, floatstr(*id.storage.f)); break;
@@ -378,10 +400,10 @@ void writemapcfg(const char *a)
 
 	f->printf("\n//Textures\n\ntexturereset\n\n");
 
-	extern vector<Slot> slots;
+	extern vector<Slot *> slots;
 	loopv(slots)
 	{
-		saveslotconfig(f, slots[i], i);
+		saveslotconfig(f, *slots[i], i);
 	}
 
 	f->printf("\n//Texture Animations; cleared on texture reset\n\n");
@@ -399,15 +421,162 @@ COMMAND(writemapcfg, "s");
 
 VAR(dbgvars, 0, 0, 1);
 
+void savevslot(stream *f, VSlot &vs, int prev)
+{
+    f->putlil<int>(vs.changed);
+    f->putlil<int>(prev);
+    if(vs.changed & (1<<VSLOT_SHPARAM))
+    {
+        f->putlil<ushort>(vs.params.length());
+        loopv(vs.params)
+        {
+            ShaderParam &p = vs.params[i];
+            f->putlil<ushort>(strlen(p.name));
+            f->write(p.name, strlen(p.name));
+            loopk(4) f->putlil<float>(p.val[k]);
+        }
+    }
+    if(vs.changed & (1<<VSLOT_SCALE)) f->putlil<float>(vs.scale);
+    if(vs.changed & (1<<VSLOT_ROTATION)) f->putlil<int>(vs.rotation);
+    if(vs.changed & (1<<VSLOT_OFFSET))
+    {
+        f->putlil<int>(vs.xoffset);
+        f->putlil<int>(vs.yoffset);
+    }
+    if(vs.changed & (1<<VSLOT_SCROLL))
+    {
+        f->putlil<float>(vs.scrollS);
+        f->putlil<float>(vs.scrollT);
+    }
+    if(vs.changed & (1<<VSLOT_LAYER)) f->putlil<int>(vs.layer);
+    if(vs.changed & (1<<VSLOT_ALPHA))
+    {
+        f->putlil<float>(vs.alphafront);
+        f->putlil<float>(vs.alphaback);
+    }
+    if(vs.changed & (1<<VSLOT_COLOR))
+    {
+        loopk(3) f->putlil<float>(vs.colorscale[k]);
+    }
+}
+
+void savevslots(stream *f, int numvslots)
+{
+    if(vslots.empty()) return;
+    int *prev = new int[numvslots];
+    memset(prev, -1, numvslots*sizeof(int));
+    loopi(numvslots)
+    {
+        VSlot *vs = vslots[i];
+        if(vs->changed) continue;
+        for(;;)
+        {
+            VSlot *cur = vs;
+            do vs = vs->next; while(vs && vs->index >= numvslots);
+            if(!vs) break;
+            prev[vs->index] = cur->index;
+        }
+    }
+    int lastroot = 0;
+    loopi(numvslots)
+    {
+        VSlot &vs = *vslots[i];
+        if(!vs.changed) continue;
+        if(lastroot < i) f->putlil<int>(-(i - lastroot));
+        savevslot(f, vs, prev[i]);
+        lastroot = i+1;
+    }
+    if(lastroot < numvslots) f->putlil<int>(-(numvslots - lastroot));
+    delete[] prev;
+}
+
+void loadvslot(stream *f, VSlot &vs, int changed)
+{
+    vs.changed = changed;
+    if(vs.changed & (1<<VSLOT_SHPARAM))
+    {
+        int numparams = f->getlil<ushort>();
+        string name;
+        loopi(numparams)
+        {
+            ShaderParam &p = vs.params.add();
+            int nlen = f->getlil<ushort>();
+            f->read(name, min(nlen, MAXSTRLEN-1));
+            name[min(nlen, MAXSTRLEN-1)] = '\0';
+            if(nlen >= MAXSTRLEN) f->seek(nlen - (MAXSTRLEN-1), SEEK_CUR);
+            p.name = getshaderparamname(name);
+            p.type = SHPARAM_LOOKUP;
+            p.index = -1;
+            p.loc = -1;
+            loopk(4) p.val[k] = f->getlil<float>();
+        }
+    }
+    if(vs.changed & (1<<VSLOT_SCALE)) vs.scale = f->getlil<float>();
+    if(vs.changed & (1<<VSLOT_ROTATION)) vs.rotation = f->getlil<int>();
+    if(vs.changed & (1<<VSLOT_OFFSET))
+    {
+        vs.xoffset = f->getlil<int>();
+        vs.yoffset = f->getlil<int>();
+    }
+    if(vs.changed & (1<<VSLOT_SCROLL))
+    {
+        vs.scrollS = f->getlil<float>();
+        vs.scrollT = f->getlil<float>();
+    }
+    if(vs.changed & (1<<VSLOT_LAYER)) vs.layer = f->getlil<int>();
+    if(vs.changed & (1<<VSLOT_ALPHA))
+    {
+        vs.alphafront = f->getlil<float>();
+        vs.alphaback = f->getlil<float>();
+    }
+    if(vs.changed & (1<<VSLOT_COLOR))
+    {
+        loopk(3) vs.colorscale[k] = f->getlil<float>();
+    }
+}
+
+void loadvslots(stream *f, int numvslots)
+{
+    int *prev = new int[numvslots];
+    memset(prev, -1, numvslots*sizeof(int));
+    while(numvslots > 0)
+    {
+        int changed = f->getlil<int>();
+        if(changed < 0)
+        {
+            loopi(-changed) vslots.add(new VSlot(NULL, vslots.length()));
+            numvslots += changed;
+        }
+        else
+        {
+            prev[vslots.length()] = f->getlil<int>();
+            loadvslot(f, *vslots.add(new VSlot(NULL, vslots.length())), changed);
+            numvslots--;
+        }
+    }
+    loopv(vslots) if(vslots.inrange(prev[i])) vslots[prev[i]]->next = vslots[i];
+    delete[] prev;
+}
+
 bool save_world(const char *mname, bool nolms, bool octa)
 {
     if(!*mname) mname = game::getclientmap();
-    setmapfilenames(*mname ? mname : "untitled");
+    setmapfilenames(*mname ? mname : "untitled", false);
     if(savebak) backup(ogzname, bakname);
     stream *f = opengzfile(ogzname, "wb");
     if(!f) { conoutf(CON_WARN, "could not write map to %s", ogzname); return false; }
+
+    int numvslots = vslots.length();
+    if(!nolms && !multiplayer(false))
+    {
+        numvslots = compactvslots();
+        allchanged();
+    }
+
+    renderprogress(0, "saving map...");
+
     octaheader hdr;
-    memcpy(hdr.magic, octa ? "OCTA" : "PASM", 4);
+    memcpy(hdr.magic, (octa ? "OCTA" : "PASM"), 4);
     hdr.version = octa ? MAPVERSION : PASMAPVERSION;
     hdr.headersize = sizeof(hdr);
     hdr.worldsize = worldsize;
@@ -418,16 +587,17 @@ bool save_world(const char *mname, bool nolms, bool octa)
     hdr.lightmaps = nolms ? 0 : lightmaps.length();
     hdr.blendmap = shouldsaveblendmap();
     hdr.numvars = 0;
-    enumerate(*idents, ident, id,
+    hdr.numvslots = numvslots;
+    enumerate(idents, ident, id,
     {
-        if((id.type == ID_VAR || id.type == ID_FVAR || id.type == ID_SVAR) && id.flags&IDF_OVERRIDE && !(id.flags&IDF_READONLY) && id.override!=NO_OVERRIDE) hdr.numvars++;
+        if((id.type == ID_VAR || id.type == ID_FVAR || id.type == ID_SVAR) && id.flags&IDF_OVERRIDE && !(id.flags&IDF_READONLY) && id.flags&IDF_OVERRIDDEN) hdr.numvars++;
     });
-    lilswap(&hdr.version, 8);
+    lilswap(&hdr.version, 9);
     f->write(&hdr, sizeof(hdr));
 
-    enumerate(*idents, ident, id,
+    enumerate(idents, ident, id,
     {
-        if((id.type!=ID_VAR && id.type!=ID_FVAR && id.type!=ID_SVAR) || !(id.flags&IDF_OVERRIDE) || id.flags&IDF_READONLY || id.override==NO_OVERRIDE) continue;
+        if((id.type!=ID_VAR && id.type!=ID_FVAR && id.type!=ID_SVAR) || !(id.flags&IDF_OVERRIDE) || id.flags&IDF_READONLY || !(id.flags&IDF_OVERRIDDEN)) continue;
         f->putchar(id.type);
         f->putlil<ushort>(strlen(id.name));
         f->write(id.name, strlen(id.name));
@@ -464,8 +634,6 @@ bool save_world(const char *mname, bool nolms, bool octa)
     loopv(texmru) f->putlil<ushort>(texmru[i]);
     char *ebuf = new char[entities::extraentinfosize()];
 
-
-
     loopv(ents)
     {
         if(ents[i]->type!=ET_EMPTY || nolms)
@@ -482,36 +650,70 @@ bool save_world(const char *mname, bool nolms, bool octa)
 			} ent;
 
 			ent.o = tmp.o;
-			ent.attr1 = tmp.attr1;
-			ent.attr2 = tmp.attr2;
-			ent.attr3 = tmp.attr3;
-			ent.attr4 = tmp.attr4;
-			ent.attr5 = tmp.attr5;
+			int numattrs = tmp.attr.length();
+
+			ent.attr1 = numattrs >= 1 ? tmp.attr[0] : 0;
+			ent.attr2 = numattrs >= 2 ? tmp.attr[1] : 0;
+			ent.attr3 = numattrs >= 3 ? tmp.attr[2] : 0;
+			ent.attr4 = numattrs >= 4 ? tmp.attr[3] : 0;
+			ent.attr5 = numattrs >= 5 ? tmp.attr[4] : 0;
+
 			ent.type = tmp.type;
-			ent.reserved = tmp.reserved;
+			ent.reserved = 0;
 
 			switch(tmp.type)
 			{
 				case ET_PARTICLES:
-					switch(tmp.attr1)
+				{
+					switch(ent.attr1)
 					{
 						case 0:
-							if(!tmp.attr4) break;
-						case 4: case 5: case 6: case 7: case 8: case 9: case 10: case 12: case 13: case 14:
-							ent.attr4 = ((tmp.attr4 & 0xF00000) >> 12) | ((tmp.attr4 & 0xF000) >> 8) | ((tmp.attr4 & 0xF0) >> 4);
-							if(ent.attr1 != 5 && ent.attr1 != 6) break;
+							if(!ent.attr4) break;
+						case 1: case 2: case 6: case 7: case 9: case 10: case 11: case 12: case 13: case 14: case 15:
+							ent.attr4 = ((tmp.attr[3] & 0xF00000) >> 12) | ((tmp.attr[3] & 0xF000) >> 8) | ((tmp.attr[3] & 0xF0) >> 4);
+							if(ent.attr1 != 6 && ent.attr1 != 7) break;
 
-						case 3: case 11:
-							ent.attr3 = ((tmp.attr3 & 0xF00000) >> 12) | ((tmp.attr3 & 0xF000) >> 8) | ((tmp.attr3 & 0xF0) >> 4);
+						case 5: case 8:
+							ent.attr3 = ((tmp.attr[2] & 0xF00000) >> 12) | ((tmp.attr[2] & 0xF000) >> 8) | ((tmp.attr[2] & 0xF0) >> 4);
 						default:
 							break;
 
 					}
+					switch(ent.attr1)
+					{
+						//fire/smoke 11/12
+						case 1: case 2:
+							ent.attr1 += 12; break;
+
+						//fountains and explosion 1/2/3
+						case 3: case 4: case 5:
+							ent.attr1 -= 2; break;
+
+						//bars 5/6
+						case 6:	case 7:
+							ent.attr1--; break;
+
+						//text
+						case 8:
+							ent.attr1 = 11; break;
+
+						//multi effect
+						case 9: case 10: case 11: case 12: case 13: case 14:
+						{
+							int num[] = {4, 7, 8, 9, 10, 12};
+							ent.attr1 = num[ent.attr1 - 9];
+							break;
+						}
+						default:
+							break;
+					}
+				}
 					break;
 				default:
 					break;
 			}
 
+
 			lilswap(&ent.o.x, 3);
 			lilswap(&ent.attr1, 5);
 			f->write(&ent, sizeof(octaent));
@@ -521,9 +723,14 @@ bool save_world(const char *mname, bool nolms, bool octa)
 		else
 		{
 			entity tmp = *ents[i];
+
 			lilswap(&tmp.o.x, 3);
-			lilswap(&tmp.attr1, 8);
-			f->write(&tmp, sizeof(entity));
+			f->write(&tmp.o, sizeof(vec));
+
+			f->putchar(tmp.attr.length());
+			loopvj(tmp.attr) f->putlil<int>(tmp.attr[j]);
+			f->putchar(tmp.type);
+
 			entities::writeent(*ents[i], ebuf);
 			if(entities::extraentinfosize()) f->write(ebuf, entities::extraentinfosize());
 		}
@@ -531,6 +738,8 @@ bool save_world(const char *mname, bool nolms, bool octa)
     }
     delete[] ebuf;
 
+    savevslots(f, numvslots);
+
     savec(worldroot, f, nolms);
     if(!nolms)
     {
@@ -579,7 +788,7 @@ static void swapXZ(cube *c)
 
 static void fixoversizedcubes(cube *c, int size)
 {
-    if(size <= VVEC_INT_MASK+1) return;
+    if(size <= 0x1000) return;
     loopi(8)
     {
         if(!c[i].children) subdividecube(c[i], true, false);
@@ -587,18 +796,17 @@ static void fixoversizedcubes(cube *c, int size)
     }
 }
 
-extern int loadbacktype;
-
 bool load_world(const char *mname, const char *cname)        // still supports all map formats that have existed since the earliest cube betas!
 {
     int loadingstart = SDL_GetTicks();
-    setmapfilenames(mname, cname);
+    setmapfilenames(mname);
+
     stream *f = opengzfile(ogzname, "rb");
 	defformatstring(capt)("Sandbox Engine %s: %s - loading...", version, mname);
 	SDL_WM_SetCaption(capt, NULL);
     if(!f) { conoutf(CON_ERROR, "could not read map %s", ogzname); game::mapfailed(mname); return false; }
     octaheader hdr;
-    if(f->read(&hdr, 7*sizeof(int))!=7*sizeof(int)) { conoutf(CON_ERROR, "map %s has malformatted header", ogzname); delete f; return false; }
+    if(f->read(&hdr, 7*sizeof(int))!=int(7*sizeof(int))) { conoutf(CON_ERROR, "map %s has malformatted header", ogzname); delete f; return false; }
     lilswap(&hdr.version, 6);
 
 
@@ -615,6 +823,14 @@ bool load_world(const char *mname, const char *cname)        // still supports a
 	if(hdr.version > PASMAPVERSION) { conoutf(CON_ERROR, "map %s requires a newer version of Platinum Arts Sandbox", ogzname); delete f; return false; }
 	switch(hdr.version)
 	{
+		case 5:
+		case 4:
+			octaversion = 31;
+			break;
+		case 3:
+		case 2:
+			octaversion = 30;
+			break;
 		case 1:
 			octaversion = 29;
 			break;
@@ -633,19 +849,17 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     compatheader chdr;
     if(octaversion <= 28)
     {
-        if(f->read(&chdr.lightprecision, sizeof(chdr) - 7*sizeof(int)) != sizeof(chdr) - 7*sizeof(int)) { conoutf(CON_ERROR, "map %s has malformatted header", ogzname); delete f; return false; }
+        if(f->read(&chdr.lightprecision, sizeof(chdr) - 7*sizeof(int)) != int(sizeof(chdr) - 7*sizeof(int))) { conoutf(CON_ERROR, "map %s has malformatted header", ogzname); delete f; return false; }
+    }
+    else
+    {
+        int extra = 0;
+        if(octaversion <= 29) extra++;
+        if(f->read(&hdr.blendmap, sizeof(hdr) - (7+extra)*sizeof(int)) != int(sizeof(hdr) - (7+extra)*sizeof(int))) { conoutf(CON_ERROR, "map %s has malformatted header", ogzname); delete f; return false; }
     }
-    else if(f->read(&hdr.blendmap, sizeof(hdr) - 7*sizeof(int)) != sizeof(hdr) - 7*sizeof(int)) { conoutf(CON_ERROR, "map %s has malformatted header", ogzname); delete f; return false; }
 
     resetmap();
-    extern int loadbacktype;
-    Texture *mapshot = NULL;
-
-	if(loadbacktype!=2)
-	{
-		mapshot = textureload(picname, 3, true, false);
-	}
-
+	Texture *mapshot = textureload(picname, 3, true, false);
     renderbackground("loading...", mapshot, mname, game::getmapinfo()); //checks are done not to render the map
 
     setvar("mapversion", octaversion, true, false);
@@ -653,12 +867,12 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     if(octaversion <= 28)
     {
         lilswap(&chdr.lightprecision, 3);
-        if(octaversion<=20) conoutf(CON_WARN, "loading older / less efficient map format, may benefit from \"calclight 2\", then \"savecurrentmap\"");
+        if(octaversion<=20) conoutf(CON_WARN, "loading older / less efficient map format, may benefit from \"calclight\", then \"savecurrentmap\"");
         if(chdr.lightprecision) setvar("lightprecision", chdr.lightprecision);
         if(chdr.lighterror) setvar("lighterror", chdr.lighterror);
         if(chdr.bumperror) setvar("bumperror", chdr.bumperror);
         setvar("lightlod", chdr.lightlod);
-        if(chdr.ambient) setvar("ambient", chdr.ambient);
+        if(chdr.ambient) setvar("ambient", (chdr.ambient<<16) | (chdr.ambient<<8) | chdr.ambient);
         setvar("skylight", (int(chdr.skylight[0])<<16) | (int(chdr.skylight[1])<<8) | int(chdr.skylight[2]));
         setvar("watercolour", (int(chdr.watercolour[0])<<16) | (int(chdr.watercolour[1])<<8) | int(chdr.watercolour[2]), true);
         setvar("waterfallcolour", (int(chdr.waterfallcolour[0])<<16) | (int(chdr.waterfallcolour[1])<<8) | int(chdr.waterfallcolour[2]));
@@ -673,8 +887,14 @@ bool load_world(const char *mname, const char *cname)        // still supports a
         setsvar("maptitle", chdr.maptitle);
         hdr.blendmap = chdr.blendmap;
         hdr.numvars = 0;
+        hdr.numvslots = 0;
+    }
+    else
+    {
+        lilswap(&hdr.blendmap, 2);
+        if(octaversion <= 29) hdr.numvslots = 0;
+        else lilswap(&hdr.numvslots, 1);
     }
-    else lilswap(&hdr.blendmap, 2);
 
     loopi(hdr.numvars)
     {
@@ -743,7 +963,7 @@ bool load_world(const char *mname, const char *cname)        // still supports a
 
     renderprogress(0, "clearing world...");
 
-    texmru.setsize(0);
+    texmru.shrink(0);
     if(octaversion<14)
     {
         uchar oldtl[256];
@@ -772,43 +992,56 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     {
         extentity &e = *entities::newentity();
         ents.add(&e);
+
+	e.o.x = f->getlil<float>();
+	e.o.y = f->getlil<float>();
+	e.o.z = f->getlil<float>();
+
+	uchar numattrs;
+
+	if(maptype == MAP_OCTA) { numattrs = 5; }
+	else if(maptype == MAP_PAS && hdr.version >= 5) {numattrs = f->getchar(); }
+	else {numattrs = 8;}
+
+	loopj(numattrs)
+	{
+		if(maptype == MAP_OCTA)
+			e.attr.add(f->getlil<short>());
+		else
+			e.attr.add(f->getlil<int>());
+	}
+	e.type = f->getchar();
+
+	if(maptype == MAP_OCTA)
+		f->getchar(); //gets rid of reserved padding at end of struct (1 byte)
+	else if(maptype == MAP_PAS && hdr.version <= 4)
+		loopi(3) f->getchar(); //(3 byte), pieces add to 46, struct is 48 + reserved byte
+
+	if(samegame || e.type < ET_GAMESPECIFIC)
+	{
+		numattrs = getattrnum(e.type);
+		if(e.attr.length() > numattrs)
+			e.attr.setsize(numattrs);
+		else
+			while(e.attr.length() < numattrs)
+				e.attr.add(0);
+	}
+
 	if(maptype == MAP_OCTA)
 	{
-		struct oldent
-		{
-			vec o;
-			short attr1, attr2, attr3, attr4, attr5;
-			uchar type;
-			uchar reserved;
-		} ent;
-
-		f->read(&ent, sizeof(oldent));
-		lilswap(&ent.o.x, 3);
-		lilswap(&ent.attr1, 5);
-
-		e.o = ent.o;
-		e.attr1 = ent.attr1;
-		e.attr2 = ent.attr2;
-		e.attr3 = ent.attr3;
-		e.attr4 = ent.attr4;
-		e.attr5 = ent.attr5;
-		e.attr6 = e.attr7 = e.attr8 = 0;
-		e.type = ent.type;
-		e.reserved = ent.reserved;
-
 		switch(e.type)
 		{
 			case ET_PARTICLES:
-				switch(e.attr1)
+				switch(e.attr[0])
 				{
 					case 0:
-						if(!e.attr4) break;
+						if(!e.attr[3]) break;
 					case 4: case 5: case 6: case 7: case 8: case 9: case 10: case 12: case 13: case 14:
-							e.attr4 = ((e.attr4 & 0xF00) << 12) | ((e.attr4 & 0x0F0) << 8) | ((e.attr4 & 0x00F) << 4) | 0x0F0F0F;
-						if(e.attr1 != 5 && e.attr1 != 6) break;
+							e.attr[3] = ((e.attr[3] & 0xF00) << 12) | ((e.attr[3] & 0x0F0) << 8) | ((e.attr[3] & 0x00F) << 4) | 0x0F0F0F;
+						if(e.attr[0] != 5 && e.attr[0] != 6) break;
 
 					case 3: case 11:
-						e.attr3 = ((e.attr3 & 0xF00) << 12) | ((e.attr3 & 0x0F0) << 8) | ((e.attr3 & 0x00F) << 4) | 0x0F0F0F;
+						e.attr[2] = ((e.attr[2] & 0xF00) << 12) | ((e.attr[2] & 0x0F0) << 8) | ((e.attr[2] & 0x00F) << 4) | 0x0F0F0F;
 					default:
 						break;
 
@@ -818,21 +1051,42 @@ bool load_world(const char *mname, const char *cname)        // still supports a
 				break;
 		}
 	}
-	else
+
+	//update particles to use new indexes
+	if(e.type == ET_PARTICLES && (maptype == MAP_OCTA || (maptype == MAP_PAS && hdr.version < 3)))
 	{
-		f->read(&e, sizeof(entity));
-		lilswap(&e.o.x, 3);
-		lilswap(&e.attr1, 8);
+		switch(e.attr[0])
+		{
+			// fire/smoke
+			case 13: case 14:
+				e.attr[0] -= 12;
+				break;
+				//fountains and explosion
+			case 1: case 2: case 3:
+				e.attr[0] += 2; break;
+
+				//bars
+			case 5: case 6:
+				e.attr[0]++; break;
+
+				//text
+			case 11:
+				e.attr[0] = 8; break;
+
+				//multi effect
+			case 4: case 7: case 8: case 9: case 10: case 12:
+			{
+				int num[] = {9, 0, 0, 10, 11, 12, 13, 0, 14};
+				e.attr[0] = num[e.attr[0] - 4];
+				break;
+			}
+		}
 	}
 
         e.spawned = false;
         e.inoctanode = false;
-        if(samegame)
-        {
-            if(eif > 0) f->read(ebuf, eif);
-            entities::readent(e, ebuf);
-        }
-        else f->seek(eif, SEEK_CUR);
+	if(octaversion <= 10 && e.type >= 7) e.type++;
+        if(octaversion <= 12 && e.type >= 8) e.type++;
         if(octaversion <= 14 && e.type >= ET_MAPMODEL && e.type <= 16)
         {
             if(e.type == 16) e.type = ET_MAPMODEL;
@@ -842,8 +1096,15 @@ bool load_world(const char *mname, const char *cname)        // still supports a
         if(octaversion <= 21 && e.type >= ET_PARTICLES) e.type++;
         if(octaversion <= 22 && e.type >= ET_SOUND) e.type++;
         if(octaversion <= 23 && e.type >= ET_SPOTLIGHT) e.type++;
-        if(!samegame)
+        if(octaversion <= 30 && (e.type == ET_MAPMODEL || e.type == ET_PLAYERSTART)) e.attr[0] = (int(e.attr[0])+180)%360;
+	if(samegame)
         {
+            if(eif > 0) f->read(ebuf, eif);
+            entities::readent(e, ebuf);
+        }
+        else
+        {
+            f->seek(eif, SEEK_CUR);
             if(e.type>=ET_GAMESPECIFIC || octaversion<=14)
             {
                 entities::deleteentity(ents.pop());
@@ -859,9 +1120,9 @@ bool load_world(const char *mname, const char *cname)        // still supports a
         }
         if(octaversion <= 14 && e.type == ET_MAPMODEL)
         {
-            e.o.z += e.attr3;
-            if(e.attr4) conoutf(CON_WARN, "warning: mapmodel ent (index %d) uses texture slot %d", i, e.attr4);
-            e.attr3 = e.attr4 = 0;
+            e.o.z += e.attr[2];
+            if(e.attr[3]) conoutf(CON_WARN, "warning: mapmodel ent (index %d) uses texture slot %d", i, e.attr[3]);
+            e.attr[2] = e.attr[3] = 0;
         }
     }
     if(ebuf) delete[] ebuf;
@@ -872,6 +1133,9 @@ bool load_world(const char *mname, const char *cname)        // still supports a
         f->seek((hdr.numents-MAXENTS)*(sizeof(entity) + eif), SEEK_CUR);
     }
 
+    renderprogress(0, "loading slots...");
+    loadvslots(f, hdr.numvslots);
+
     renderprogress(0, "loading octree...");
     worldroot = loadchildren(f);
 
@@ -881,7 +1145,7 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     if(octaversion <= 8)
         converttovectorworld();
 
-    if(octaversion <= 25 && hdr.worldsize > VVEC_INT_MASK+1)
+    if(octaversion <= 25 && hdr.worldsize > 0x1000)
         fixoversizedcubes(worldroot, hdr.worldsize>>1);
 
     renderprogress(0, "validating...");
@@ -915,40 +1179,27 @@ bool load_world(const char *mname, const char *cname)        // still supports a
 
     conoutf("read map %s (%.1f seconds)", ogzname, (SDL_GetTicks()-loadingstart)/1000.0f);
 
+    #ifndef NEWGUI
+    clearmainmenu();
+    #else
+    UI::clearmainmenu();
+    #endif
+
     clearmainmenu();
 
-    overrideidents = true;
+    identflags |= IDF_OVERRIDDEN;
 
     execfile("data/default_map_settings.cfg", false);
-
     execfile(acfname, false);
     execfile(mcfname, false);
-
     if(identexists("on_start")) execute("on_start");
-    overrideidents = false;
+
+    identflags &= ~IDF_OVERRIDDEN;
 
     extern void fixlightmapnormals();
     if(octaversion <= 25) fixlightmapnormals();
 
-    vector<int> mapmodels;
-    loopv(ents)
-    {
-        extentity &e = *ents[i];
-        if(e.type==ET_MAPMODEL && e.attr2 >= 0)
-        {
-            if(mapmodels.find(e.attr2) < 0) mapmodels.add(e.attr2);
-        }
-    }
-
-    loopv(mapmodels)
-    {
-        loadprogress = float(i+1)/mapmodels.length();
-        int mmindex = mapmodels[i];
-        mapmodelinfo &mmi = getmminfo(mmindex);
-        if(!&mmi) conoutf(CON_WARN, "could not find map model: %d", mmindex);
-        else if(!loadmodel(NULL, mmindex, true)) conoutf(CON_WARN, "could not load model: %s", mmi.name);
-    }
-    loadprogress = 0;
+    preloadusedmapmodels(true);
 
     game::preload();
     flushpreloadedmodels();
@@ -956,63 +1207,130 @@ bool load_world(const char *mname, const char *cname)        // still supports a
     entitiesinoctanodes();
     attachentities();
     initlights();
+    startmap(cname ? cname : mname);
     allchanged(true);
 
-    renderbackground("loading...", loadbacktype!=2 ? mapshot : NULL, mname, game::getmapinfo());
-
-    if(maptitle[0]) conoutf(CON_ECHO, "%s", maptitle);
-
-    startmap(cname ? cname : mname);
+    if(maptitle[0] && strcmp(maptitle, "Untitled Map by Unknown")) conoutf(CON_ECHO, "%s", maptitle);
     game::setwindowcaption();
 
     return true;
 }
 
 void savecurrentmap() { save_world(game::getclientmap()); }
-void savemap(char *mname) { save_world(mname); }
+void savemap(char *mname) { save_world(mname, false, false); }
 
 COMMAND(savemap, "s");
 COMMAND(savecurrentmap, "");
 
 ICOMMAND(exportocta, "s", (char *mname), save_world(mname, false, true););
 
+static int mtlsort(const int *x, const int *y)
+{
+    if(*x < *y) return -1;
+    if(*x > *y) return 1;
+    return 0;
+}
+
 void writeobj(char *name)
 {
-    defformatstring(fname)("%s.obj", name);
-    stream *f = openfile(path(fname), "w");
-    if(!f) return;
-    f->printf("# obj file of sandbox level\n");
-    extern vector<vtxarray *> valist;
-    loopv(valist)
-    {
-        vtxarray &va = *valist[i];
-        ushort *edata = NULL;
-        uchar *vdata = NULL;
-        if(!readva(&va, edata, vdata)) continue;
-        int vtxsize = VTXSIZE;
-        uchar *vert = vdata;
-        loopj(va.verts)
-        {
-            vec v;
-            if(floatvtx) (v = *(vec *)vert).div(1<<VVEC_FRAC);
-            else v = ((vvec *)vert)->tovec(va.o).add(0x8000>>VVEC_FRAC);
-            if(v.y != floor(v.y)) f->printf("v %.3f ", -v.y); else f->printf("v %d ", int(-v.y));
-            if(v.z != floor(v.z)) f->printf("%.3f ", v.z); else f->printf("%d ", int(v.z));
-            if(v.x != floor(v.x)) f->printf("%.3f\n", v.x); else f->printf("%d\n", int(v.x));
-            vert += vtxsize;
-        }
-        ushort *tri = edata;
-        loopi(va.tris)
-        {
-            f->printf("f");
-            for(int k = 0; k<3; k++) f->printf(" %d", tri[2-k]-va.verts-va.voffset);
-            tri += 3;
-            f->printf("\n");
-        }
-        delete[] edata;
-        delete[] vdata;
-    }
-    delete f;
+	defformatstring(fname)("%s.obj", name);
+	stream *f = openfile(path(fname), "w");
+	if(!f) return;
+	f->printf("# obj file of Cube 2 level\n\n");
+	defformatstring(mtlname)("%s.mtl", name);
+	path(mtlname);
+	f->printf("mtllib %s\n\n", mtlname);
+	extern vector<vtxarray *> valist;
+	vector<vec> verts;
+	vector<vec2> texcoords;
+	hashtable<vec, int> shareverts(1<<16);
+	hashtable<vec2, int> sharetc(1<<16);
+	hashtable<int, vector<ivec> > mtls(1<<8);
+	vector<int> usedmtl;
+	vec bbmin(1e16f, 1e16f, 1e16f), bbmax(-1e16f, -1e16f, -1e16f);
+	loopv(valist)
+	{
+		vtxarray &va = *valist[i];
+		ushort *edata = NULL;
+		uchar *vdata = NULL;
+		if(!readva(&va, edata, vdata)) continue;
+		int vtxsize = VTXSIZE;
+		ushort *idx = edata;
+		loopj(va.texs)
+		{
+			elementset &es = va.eslist[j];
+			if(usedmtl.find(es.texture) < 0) usedmtl.add(es.texture);
+			vector<ivec> &keys = mtls[es.texture];
+			loopk(es.length[1])
+			{
+				int n = idx[k] - va.voffset;
+				const vec &pos = renderpath==R_FIXEDFUNCTION ? ((const vertexff *)&vdata[n*vtxsize])->pos : ((const vertex *)&vdata[n*vtxsize])->pos;
+				vec2 tc(renderpath==R_FIXEDFUNCTION ? ((const vertexff *)&vdata[n*vtxsize])->u : ((const vertex *)&vdata[n*vtxsize])->u,
+					renderpath==R_FIXEDFUNCTION ? ((const vertexff *)&vdata[n*vtxsize])->v : ((const vertex *)&vdata[n*vtxsize])->v);
+				ivec &key = keys.add();
+				key.x = shareverts.access(pos, verts.length());
+				if(key.x == verts.length())
+				{
+					verts.add(pos);
+					loopl(3)
+					{
+						bbmin[l] = min(bbmin[l], pos[l]);
+						bbmax[l] = max(bbmax[l], pos[l]);
+					}
+				}
+				key.y = sharetc.access(tc, texcoords.length());
+				if(key.y == texcoords.length()) texcoords.add(tc);
+			}
+			idx += es.length[1];
+		}
+		delete[] edata;
+		delete[] vdata;
+	}
+
+	vec center(-(bbmax.x + bbmin.x)/2, -(bbmax.y + bbmin.y)/2, -bbmin.z);
+	loopv(verts)
+	{
+		vec v = verts[i];
+		v.add(center);
+		if(v.y != floor(v.y)) f->printf("v %.3f ", -v.y); else f->printf("v %d ", int(-v.y));
+		if(v.z != floor(v.z)) f->printf("%.3f ", v.z); else f->printf("%d ", int(v.z));
+		if(v.x != floor(v.x)) f->printf("%.3f\n", v.x); else f->printf("%d\n", int(v.x));
+	}
+	f->printf("\n");
+	loopv(texcoords)
+	{
+		const vec2 &tc = texcoords[i];
+		f->printf("vt %.6f %.6f\n", tc.x, 1-tc.y);
+	}
+	f->printf("\n");
+
+	usedmtl.sort(mtlsort);
+	loopv(usedmtl)
+	{
+		vector<ivec> &keys = mtls[usedmtl[i]];
+		f->printf("g slot%d\n", usedmtl[i]);
+		f->printf("usemtl slot%d\n\n", usedmtl[i]);
+		for(int i = 0; i < keys.length(); i += 3)
+		{
+			f->printf("f");
+			loopk(3) f->printf(" %d/%d", keys[i+2-k].x+1, keys[i+2-k].y+1);
+			f->printf("\n");
+		}
+		f->printf("\n");
+	}
+	delete f;
+
+	f = openfile(mtlname, "w");
+	if(!f) return;
+	f->printf("# mtl file of Cube 2 level\n\n");
+	loopv(usedmtl)
+	{
+		VSlot &vslot = lookupvslot(usedmtl[i], false);
+		f->printf("newmtl slot%d\n", usedmtl[i]);
+		f->printf("map_Kd %s\n", vslot.slot->sts.empty() ? notexture->name : path(makerelpath("packages", vslot.slot->sts[0].name)));
+		f->printf("\n");
+	}
+	delete f;
 }
 
 COMMAND(writeobj, "s");
