@@ -1,9 +1,10 @@
 #include "game.h"
+//#include "sb.h"
 
 namespace game
 {
     bool intermission = false;
-    int maptime = 0, maprealtime = 0, minremain = 0;
+    int maptime = 0, maprealtime = 0, maplimit = -1;
     int respawnent = -1;
     int lasthit = 0, lastspawnattempt = 0;
 
@@ -34,10 +35,16 @@ namespace game
         if(player1->state!=CS_ALIVE || player1->physstate<PHYS_SLOPE) return;
         if(lastmillis-player1->lasttaunt<1000) return;
         player1->lasttaunt = lastmillis;
-	addmsg(SV_TAUNT, "rc", player1);
+	addmsg(N_TAUNT, "rc", player1);
     }
     COMMAND(taunt, "");
 
+    ICOMMAND(getfollow, "", (),
+    {
+        fpsent *f = followingplayer();
+        intret(f ? f->clientnum : -1);
+    });
+
 	void follow(char *arg)
     {
         if(arg[0] ? player1->state==CS_SPECTATOR : following>=0)
@@ -81,7 +88,7 @@ namespace game
         {
 		clearmovables();
             clearmonsters();                 // all monsters back at their spawns for editing
-            resettriggers();
+            entities::resettriggers();
         }
     }
 
@@ -99,7 +106,7 @@ namespace game
         {
             if(player1->respawned!=player1->lifesequence)
               {
-                addmsg(SV_TRYSPAWN, "rc", player1);
+                addmsg(N_TRYSPAWN, "rc", player1);
                 player1->respawned = player1->lifesequence;
             }
         }
@@ -114,25 +121,26 @@ namespace game
         }
     }
 
-    bool intersect(dynent *d, const vec &from, const vec &to)   // if lineseg hits entity bounding box
+    float intersectdist = 1e16f;
+
+    bool intersect(dynent *d, const vec &from, const vec &to, float &dist)   // if lineseg hits entity bounding box
     {
-        float dist;
         vec bottom(d->o), top(d->o);
         bottom.z -= d->eyeheight;
         top.z += d->aboveeye;
         return linecylinderintersect(from, to, bottom, top, d->radius, dist);
     }
 
-    dynent *intersectclosest(const vec &from, const vec &to, fpsent *at)
+    dynent *intersectclosest(const vec &from, const vec &to, fpsent *at, float &bestdist)
     {
         dynent *best = NULL;
-        float bestdist = 1e16f;
+        bestdist = 1e16f;
         loopi(numdynents())
         {
             dynent *o = iterdynents(i);
             if(o==at || o->state!=CS_ALIVE) continue;
-            if(!intersect(o, from, to)) continue;
-            float dist = at->o.dist(o->o);
+            float dist;
+            if(!intersect(o, from, to, dist)) continue;
             if(dist<bestdist)
             {
                 best = o;
@@ -141,6 +149,8 @@ namespace game
         }
         return best;
     }
+    void gibeffect(int damage, const vec &vel, fpsent *d) {}
+    void bounced(physent *d, const vec &surface) {}
 
     fpsent *pointatplayer()
     {
@@ -275,10 +285,10 @@ namespace game
         if(!curtime) { gets2c(); if(player1->clientnum>=0) c2sinfo(); return; }
 
         physicsframe();
-	ai::trydropwaypoints();
+        ai::navigate();
         entities::checkquad(curtime, player1);
         otherplayers(curtime);
-	ai::update();
+        ai::update();
         moveragdolls();
         gets2c();
         updatemovables(curtime);
@@ -289,7 +299,7 @@ namespace game
             else if(lastmillis-player1->lastpain<2000)
             {
                 player1->move = player1->strafe = 0;
-                moveplayer(player1, 10, false);
+                moveplayer(player1, 10, true);
             }
         }
         else if(!intermission)
@@ -301,7 +311,7 @@ namespace game
             if(m_sp)
             {
                 if(slowmosp) checkslowmo();
-                if(m_classicsp) checktriggers();
+                if(m_classicsp) entities::checktriggers();
             }
             else if(cmode) cmode->checkitems(player1);
         }
@@ -397,7 +407,7 @@ namespace game
     {
         d->state = CS_DEAD;
         d->lastpain = lastmillis;
-        d->superdamage = restore ? 0 : max(-d->health, 0);
+        if(!restore) gibeffect(max(-d->health, 0), d->vel, d);
         if(d==player1)
         {
             #ifndef NEWGUI
@@ -444,10 +454,13 @@ namespace game
 	ai::killed(d, actor);
     }
 
-    void timeupdate(int timeremain)
+    void timeupdate(int secs)
     {
-        minremain = timeremain;
-        if(!timeremain)
+        if(secs > 0)
+        {
+            maplimit = lastmillis + secs*1000;
+        }
+        else
         {
             intermission = true;
             player1->attacking = false;
@@ -458,15 +471,18 @@ namespace game
             showscores(true);
             #endif
             disablezoom();
-            
+
             if(identexists("intermission")) execute("intermission");
         }
-        else if(timeremain > 0)
-        {
-            conoutf(CON_GAMEINFO, "\f2time remaining: %d %s", timeremain, timeremain==1 ? "minute" : "minutes");
-        }
     }
 
+    ICOMMAND(getfrags, "", (), intret(player1->frags));
+    ICOMMAND(getflags, "", (), intret(player1->flags));
+    ICOMMAND(getdeaths, "", (), intret(player1->deaths));
+    ICOMMAND(getaccuracy, "", (), intret((player1->totaldamage*100)/max(player1->totalshots, 1)));
+    ICOMMAND(gettotaldamage, "", (), intret(player1->totaldamage));
+    ICOMMAND(gettotalshots, "", (), intret(player1->totalshots));
+
     vector<fpsent *> clients;
 
     fpsent *newclient(int cn)   // ensure valid entity
@@ -544,7 +560,7 @@ namespace game
         loopv(players)
         {
             fpsent *d = players[i];
-            d->frags = 0;
+            d->frags = d->flags = 0;
             d->deaths = 0;
             d->totaldamage = 0;
             d->totalshots = 0;
@@ -556,7 +572,8 @@ namespace game
         setclientmode();
 
         intermission = false;
-        maptime = 0;
+        maptime = maprealtime = 0;
+        maplimit = -1;
 
         if(cmode)
         {
@@ -622,8 +639,8 @@ namespace game
 		{
 			vec o = d->o;
 			o.z -= d->eyeheight *i/60.0f;
-			regularflame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
-			regularflame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
+			regular_particle_flame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
+			regular_particle_flame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
 		}
                 break;
             case MAT_WATER:
@@ -638,17 +655,26 @@ namespace game
         }
     }
 
+    void dynentcollide(physent *d, physent *o, const vec &dir)
+    {
+        switch(o->type)
+        {
+            case ENT_AI: if(dir.z > 0) stackmonster((monster *)o, d); break;
+            case ENT_INANIMATE: if(dir.z > 0) stackmovable((movable *)o, d); break;
+        }
+    }
+
     void msgsound(int n, physent *d)
     {
         if(!d || d==player1)
         {
-            addmsg(SV_SOUND, "ci", d, n);
+            addmsg(N_SOUND, "ci", d, n);
             playsound(n);
         }
         else
         {
             if(d->type==ENT_PLAYER && ((fpsent *)d)->ai)
-                addmsg(SV_SOUND, "ci", d, n);
+                addmsg(N_SOUND, "ci", d, n);
             playsound(n, &d->o);
         }
     }
@@ -692,7 +718,7 @@ namespace game
             if(!m_mp(gamemode)) killed(pl, pl);
             else if(pl->suicided!=pl->lifesequence)
             {
-                addmsg(SV_SUICIDE, "rc", pl);
+                addmsg(N_SUICIDE, "rc", pl);
                 pl->suicided = pl->lifesequence;
             }
         }
@@ -701,19 +727,21 @@ namespace game
     }
     ICOMMAND(nap, "", (), suicide(player1));
 
+    bool needminimap() { return m_ctf || m_protect || m_hold || m_capture; }
+
     void drawicon(int icon, float x, float y, float sz)
     {
         settexture("packages/hud/items.png");
-        glBegin(GL_QUADS);
+        glBegin(GL_TRIANGLE_STRIP);
         float tsz = 0.25f, tx = tsz*(icon%4), ty = tsz*(icon/4);
         glTexCoord2f(tx,     ty);     glVertex2f(x,    y);
         glTexCoord2f(tx+tsz, ty);     glVertex2f(x+sz, y);
-        glTexCoord2f(tx+tsz, ty+tsz); glVertex2f(x+sz, y+sz);
         glTexCoord2f(tx,     ty+tsz); glVertex2f(x,    y+sz);
+        glTexCoord2f(tx+tsz, ty+tsz); glVertex2f(x+sz, y+sz);
         glEnd();
     }
 
-    float abovegameplayhud()
+    float abovegameplayhud(int w, int h)
     {
         switch(hudplayer()->state)
         {
@@ -732,31 +760,31 @@ namespace game
     ICOMMAND(ammohudup, "sss", (char *w1, char *w2, char *w3),
     {
         int i = 0;
-        if(w1[0]) ammohudup[i++] = atoi(w1);
-        if(w2[0]) ammohudup[i++] = atoi(w2);
-        if(w3[0]) ammohudup[i++] = atoi(w3);
+        if(w1[0]) ammohudup[i++] = parseint(w1);
+        if(w2[0]) ammohudup[i++] = parseint(w2);
+        if(w3[0]) ammohudup[i++] = parseint(w3);
         while(i < 3) ammohudup[i++] = -1;
     });
 
     ICOMMAND(ammohuddown, "sss", (char *w1, char *w2, char *w3),
     {
         int i = 0;
-        if(w1[0]) ammohuddown[i++] = atoi(w1);
-        if(w2[0]) ammohuddown[i++] = atoi(w2);
-        if(w3[0]) ammohuddown[i++] = atoi(w3);
+        if(w1[0]) ammohuddown[i++] = parseint(w1);
+        if(w2[0]) ammohuddown[i++] = parseint(w2);
+        if(w3[0]) ammohuddown[i++] = parseint(w3);
         while(i < 3) ammohuddown[i++] = -1;
     });
 
     ICOMMAND(ammohudcycle, "sssssss", (char *w1, char *w2, char *w3, char *w4, char *w5, char *w6, char *w7),
     {
         int i = 0;
-        if(w1[0]) ammohudcycle[i++] = atoi(w1);
-        if(w2[0]) ammohudcycle[i++] = atoi(w2);
-        if(w3[0]) ammohudcycle[i++] = atoi(w3);
-        if(w4[0]) ammohudcycle[i++] = atoi(w4);
-        if(w5[0]) ammohudcycle[i++] = atoi(w5);
-        if(w6[0]) ammohudcycle[i++] = atoi(w6);
-        if(w7[0]) ammohudcycle[i++] = atoi(w7);
+        if(w1[0]) ammohudcycle[i++] = parseint(w1);
+        if(w2[0]) ammohudcycle[i++] = parseint(w2);
+        if(w3[0]) ammohudcycle[i++] = parseint(w3);
+        if(w4[0]) ammohudcycle[i++] = parseint(w4);
+        if(w5[0]) ammohudcycle[i++] = parseint(w5);
+        if(w6[0]) ammohudcycle[i++] = parseint(w6);
+        if(w7[0]) ammohudcycle[i++] = parseint(w7);
         while(i < 7) ammohudcycle[i++] = -1;
     });
 
@@ -829,11 +857,11 @@ namespace game
 
     void gameplayhud(int w, int h)
     {
-        glPushMatrix();
-        glScalef(h/1800.0f, h/1800.0f, 1);
-
         if(player1->state==CS_SPECTATOR)
         {
+            glPushMatrix();
+            glScalef(h/1800.0f, h/1800.0f, 1);
+
             int pw, ph, tw, th, fw, fh;
             text_bounds("  ", pw, ph);
             text_bounds("SPECTATOR", tw, th);
@@ -843,16 +871,20 @@ namespace game
             fh = max(fh, ph);
             draw_text("SPECTATOR", w*1800/h - tw - pw, 1650 - th - fh);
             if(f) draw_text(colorname(f), w*1800/h - fw - pw, 1650 - fh);
+
+            /*fpsent *d = hudplayer();
+            if(d->state!=CS_EDITING)
+            {
+                if(d->state!=CS_SPECTATOR) drawhudicons(d);
+                if(cmode) cmode->drawhud(d, w, h);
+            }*/
+
+            glPopMatrix();
         }
-
-        /*fpsent *d = hudplayer();
-        if(d->state!=CS_EDITING)
+        else
         {
-            if(d->state!=CS_SPECTATOR) drawhudicons(d);
-            if(cmode) cmode->drawhud(d, w, h);
-        }*/
-
-        glPopMatrix();
+           // SB::RenderHud(w,h);
+        }
     }
 
     int clipconsole(int w, int h)
@@ -925,12 +957,13 @@ namespace game
          }
     }
 
-        void dynlighttrack(physent *owner, vec &o)
+        void dynlighttrack(physent *owner, vec &o, vec &hud)
     {
         if(owner->type!=ENT_PLAYER && owner->type!=ENT_AI) return;
         fpsent *pl = (fpsent *)owner;
         if(pl->muzzle.x < 0 || pl->lastattackgun != pl->gunselect) return;
         o = pl->muzzle;
+	hud = owner == hudplayer() ? vec(pl->o).add(vec(0, 0, 2)) : pl->muzzle;
     }
 
     #ifndef NEWGUI
@@ -940,7 +973,7 @@ namespace game
         static const int struts[] =  { 0,       0,          12,     12,      8,         13,      6,       24 };
         if(size_t(i) >= sizeof(names)/sizeof(names[0])) return false;
         g->pushlist();
-        g->text(names[i], 0xFFFF80, !i ? "server" : NULL);
+        g->text(names[i], 0xFFFF80, !i ? " " : NULL);
         if(struts[i]) g->strut(struts[i]);
         g->mergehits(true);
         return true;
@@ -952,6 +985,16 @@ namespace game
         g->poplist();
     }
 
+    const char *mastermodecolor(int n, const char *unknown)
+    {
+        return (n>=MM_START && size_t(n-MM_START)<sizeof(mastermodecolors)/sizeof(mastermodecolors[0])) ? mastermodecolors[n-MM_START] : unknown;
+    }
+
+    const char *mastermodeicon(int n, const char *unknown)
+    {
+        return (n>=MM_START && size_t(n-MM_START)<sizeof(mastermodeicons)/sizeof(mastermodeicons[0])) ? mastermodeicons[n-MM_START] : unknown;
+    }
+
     bool serverinfoentry(g3d_gui *g, int i, const char *name, int port, const char *sdesc, const char *map, int ping, const vector<int> &attr, int np)
     {
         if(ping < 0 || attr.empty() || attr[0]!=PROTOCOL_VERSION)
@@ -959,7 +1002,7 @@ namespace game
             switch(i)
             {
                 case 0:
-                    if(g->button(" ", 0xFFFFDD, "server")&G3D_UP) return true;
+                    if(g->button(" ", 0xFFFFDD, "serverunk")&G3D_UP) return true;
                     break;
 
                 case 1:
@@ -991,13 +1034,16 @@ namespace game
         switch(i)
         {
             case 0:
-                if(g->buttonf("%d ", 0xFFFFDD, "server", ping)&G3D_UP) return true;
+            {
+                const char *icon = attr.inrange(3) && np >= attr[3] ? "serverfull" : (attr.inrange(4) ? mastermodeicon(attr[4], "serverunk") : "serverunk");
+                if(g->buttonf("%d ", 0xFFFFDD, icon, ping)&G3D_UP) return true;
                 break;
+            }
 
             case 1:
                 if(attr.length()>=4)
                 {
-                    if(g->buttonf("%d/%d ", 0xFFFFDD, NULL, np, attr[3])&G3D_UP) return true;
+                    if(g->buttonf(np >= attr[3] ? "\f3%d/%d " : "%d/%d ", 0xFFFFDD, NULL, np, attr[3])&G3D_UP) return true;
                 }
                 else if(g->buttonf("%d ", 0xFFFFDD, NULL, np)&G3D_UP) return true;
                 break;
@@ -1011,7 +1057,7 @@ namespace game
                 break;
 
             case 4:
-                if(g->buttonf("%s ", 0xFFFFDD, NULL, attr.length()>=5 ? server::mastermodename(attr[4], "") : "")&G3D_UP) return true;
+                if(g->buttonf("%s%s ", 0xFFFFDD, NULL, attr.length()>=5 ? mastermodecolor(attr[4], "") : "", attr.length()>=5 ? server::mastermodename(attr[4], "") : "")&G3D_UP) return true;
                 break;
 
             case 5:
@@ -1038,6 +1084,8 @@ namespace game
 
     bool allowdoublejump() {return false;}
     void updatecamera() {}
+    bool mousemove(int &dx, int &dy, float &cursens) {return false;}
+    bool recomputecamera(physent *&camera1, physent &tempcamera, bool &detachedcamera, float &thirdpersondistance) {return false;}
     void writemapdata(stream *f) {}
 
     // any data written into this vector will get saved with the map data. Must take care to do own versioning, and endianess if applicable. Will not get called when loading maps from other games, so provide defaults.
@@ -1045,10 +1093,8 @@ namespace game
     void readgamedata(vector<char> &extras) {}
 
     const char *gameident() { return "fps"; }
-    const char *defaultconfig() { return "data/defaults.cfg"; }
     const char *autoexec() { return "autoexec.cfg"; }
-    const char *savedservers() { return "servers.cfg"; }
-    const char *loadimage() { return "data/sandbox_logo"; }
+    const char *savedservers() { return "data/fps/servers.cfg"; }
     void loadconfigs()
     {
         execfile("auth.cfg", false);
@@ -1065,7 +1111,7 @@ namespace game
         if (multiplayer(false) && m_edit)// && allowdownload)
         {
             conoutf("debug: file requested by texture command doesnt exists: %s", file);
-            addmsg(SV_TEXTUREREQUEST, "rsi", file, slot);
+            addmsg(N_TEXTUREREQUEST, "rsi", file, slot);
         }
     }
 
