@@ -9,12 +9,13 @@
 #include "action.h"
 #include "clientmode.h"
 
-// TODO (offtools#1#): in editmode: change pos to spawnpoint if spawned
-// TODO (offtools#1#): prevent removing of used waypoints or notify
+/// TODO (offtools#1#): in editmode: change player pos to spawnpoint if spawned
+/// TODO (offtools#1#): prevent removing of used waypoints or notify
+/// TODO:(offtools#1#): check deletecontents
 
 //-------------------------------------------------------
 GenericAction::GenericAction() {}
-
+GenericAction::~GenericAction() {}
 void GenericAction::finished(Control& target)
 {
 //	conoutf("DEBUG: GenericAction::finished");
@@ -31,6 +32,8 @@ ActionSpawn::ActionSpawn(int uid)
     }
 }
 
+ActionSpawn::~ActionSpawn() {}
+
 void ActionSpawn::process(Control& target)
 {
     using namespace MapEntities;
@@ -40,12 +43,11 @@ void ActionSpawn::process(Control& target)
     droptofloor(spawn, 0, 0);
     target.dynent->o = spawn;
     target.dynent->o.z += target.dynent->eyeheight;
-    target.dynent->yaw = e->attr1;
+    target.dynent->yaw = e->attr[0];
 
     ControlCharacter& ca = dynamic_cast<ControlCharacter&>(target);
     if(!editmode)
     {
-        target.dynent->state = CS_ALIVE;
         game::addmsg(SV_TRYSPAWN, "rc", target.dynent);
     }
     ca.event(ACTIONINSTANCE_EVENT_SPAWN);
@@ -54,12 +56,16 @@ void ActionSpawn::process(Control& target)
 
 ActionWait::ActionWait(int _millis) : millis(_millis) {}
 
+ActionWait::~ActionWait() {}
+
 void ActionWait::process(Control& target)
 {
-	if ( (lastmillis - dynamic_cast<ActionInstance&>(target).lastaction ) > millis )
+    if ( (lastmillis - target.dynent->lastaction ) > millis )
     {
         finished(target);
+        return;
     }
+    moveplayer(target.dynent, 1, false); // Protagoras, FIX: no one has cancelled the physics yet!
 }
 
 ActionMove::ActionMove(int uid)
@@ -71,6 +77,8 @@ ActionMove::ActionMove(int uid)
     }
 }
 
+ActionMove::~ActionMove() {}
+
 void ActionMove::process(Control& target)
 {
     using namespace MapEntities;
@@ -78,11 +86,11 @@ void ActionMove::process(Control& target)
 	StaticEntity* e = ents[ent];
 	vec wp = e->o;
 	float dist = wp.dist(vec(target.dynent->o));
-	if (dist < e->attr2) //waypoint reached: TODO: make depending from speed
+	if (dist < e->attr[1]) //waypoint reached: TODO: make depending from speed
 	{
 		target.dynent->move = target.dynent->strafe = 0;
-        finished(target);
-        return;
+		finished(target);
+		return;
 	}
 	vec dir(wp);
 	dir.sub(target.dynent->o);
@@ -90,9 +98,149 @@ void ActionMove::process(Control& target)
 	target.dynent->move = 1;
 //	if (d->flying) { vectoyawpitch(dir, d->yaw, d->pitch); }
 //	else { float dummy; vectoyawpitch(dir, d->yaw, dummy); }
-	float dummy; vectoyawpitch(dir, target.dynent->yaw, dummy);
-    if(game::smoothmove && target.dynent->smoothmillis>0) game::predictplayer(target.dynent, true);
-    else moveplayer(target.dynent, 1, false);
+	float dy, dp, dc; //Protagoras: smooth character turns
+	vectoyawpitch(dir, dy, dp);
+	if (!(target.dynent->inwater)) dp = 0.0f;
+	dy -= target.dynent->yaw;
+	while (dy <= -180.0f) dy += 360.0f; // Normalizing
+	while (dy > 180.0f) dy -= 360.0f;
+	dp -= target.dynent->pitch;
+	dc = (lastmillis - dynamic_cast<ActionInstance&>(target).lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
+	dy = (dy < -dc) ? -dc : dy; // Clamping
+	dy = (dy > dc) ? dc : dy;
+	dp = (dp < -dc) ? -dc : dp;
+	dp = (dp > dc) ? dc : dp;
+	target.dynent->yaw += dy;
+	target.dynent->pitch += dp;
+	if (game::smoothmove && target.dynent->smoothmillis>0) game::predictplayer(target.dynent, true);
+	else moveplayer(target.dynent, 1, false);
+}
+
+//Protagoras: implemented jump
+ActionJump::ActionJump() {}
+
+ActionJump::~ActionJump() {}
+
+void ActionJump::process(Control& target)
+{
+	target.dynent->jumping = true;
+	moveplayer(target.dynent, 1, false);
+	finished(target);
+}
+
+//ActionFace, added by Protagoras
+ActionFace::ActionFace(int _cuid) : cuid(_cuid) {}
+
+ActionFace::~ActionFace() {}
+
+void ActionFace::process(Control& target)
+{
+	if (game::controls[cuid] == &target || cuid >= game::controls.length())
+	{
+		finished(target);
+		return;
+	}
+	vec dir(game::controls[cuid]->dynent->o);
+	dir.sub(target.dynent->o);
+	dir.normalize();
+	float dy, dp, dc;
+	vectoyawpitch(dir, dy, dp);
+	dy -= target.dynent->yaw;
+	while (dy <= -180.0f) dy += 360.0f;
+	while (dy > 180.0f) dy -= 360.0f;
+	dp -= target.dynent->pitch;
+	if (abs(dy) < 1.0f && abs(dp) < 1.0f)
+	{
+		finished(target);
+		return;
+	}
+	dc = (lastmillis - dynamic_cast<ActionInstance&>(target).lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
+	dy = (dy < -dc) ? -dc : dy;
+	dy = (dy > dc) ? dc : dy;
+	dp = (dp < -dc) ? -dc : dp;
+	dp = (dp > dc) ? dc : dp;
+	target.dynent->yaw += dy;
+	target.dynent->pitch += dp;
+	moveplayer(target.dynent, 1, false);
+}
+
+//ActionExpect, added by Protagoras
+ActionExpect::ActionExpect(int _cuid, int _radius) : cuid(_cuid), radius(_radius) {}
+
+ActionExpect::~ActionExpect() {}
+
+void ActionExpect::process(Control& target)
+{
+	if (game::controls[cuid] == &target || cuid >= game::controls.length())
+	{
+		finished(target);
+		return;
+	}
+	vec tgt = game::controls[cuid]->dynent->o;
+	float dist = tgt.dist(vec(target.dynent->o));
+	if (dist < radius)
+	{
+		finished(target);
+		return;
+	}
+	moveplayer(target.dynent, 1, false);
+}
+
+//ActionFollow, added by Protagoras
+ActionFollow::ActionFollow(int _cuid, int uid) : cuid(_cuid)
+{
+    loopv(MapEntities::ents) if(MapEntities::ents[i]->uid == uid)
+    {
+        ent = i;
+        break;
+    }
+}
+
+ActionFollow::~ActionFollow() {}
+
+void ActionFollow::process(Control& target)
+{
+	using namespace MapEntities;
+
+	StaticEntity* e = ents[ent];
+	vec wp = e->o;
+	float dist = wp.dist(vec(target.dynent->o));
+	if (dist < e->attr[1]) // TODO: make depending from speed
+	{
+		target.dynent->move = target.dynent->strafe = 0;
+		finished(target);
+		return;
+	}
+	vec dir;
+	if (game::controls[cuid] == &target || cuid >= game::controls.length())
+	{
+		dir = vec(wp);
+		target.dynent->move = 1;
+	}
+	else
+	{
+		dir = vec(game::controls[cuid]->dynent->o);
+		if (dir.dist(vec(target.dynent->o)) > 30.0f) target.dynent->move = 1; // hehehe...
+		if (dir.dist(vec(target.dynent->o)) < 20.0f) target.dynent->move = 0;
+	}
+	dir.sub(target.dynent->o);
+	dir.normalize();
+	float dy, dp, dc;
+	vectoyawpitch(dir, dy, dp);
+	if (!(target.dynent->inwater)) dp = 0.0f;
+	dy -= target.dynent->yaw;
+	while (dy <= -180.0f) dy += 360.0f;
+	while (dy > 180.0f) dy -= 360.0f;
+	dp -= target.dynent->pitch;
+	dc = (lastmillis - dynamic_cast<ActionInstance&>(target).lastprocess) * target.dynent->maxspeed / (5000.0f * RAD);
+	dy = (dy < -dc) ? -dc : dy;
+	dy = (dy > dc) ? dc : dy;
+	dp = (dp < -dc) ? -dc : dp;
+	dp = (dp > dc) ? dc : dp;
+	target.dynent->yaw += dy;
+	target.dynent->pitch += dp;
+	if (game::smoothmove && target.dynent->smoothmillis>0) game::predictplayer(target.dynent, true);
+	else moveplayer(target.dynent, 1, false);
 }
 
 ActionSpeed::ActionSpeed(int _speed)
@@ -101,6 +249,8 @@ ActionSpeed::ActionSpeed(int _speed)
 	speed = _speed;
 }
 
+ActionSpeed::~ActionSpeed() {}
+
 void ActionSpeed::process(Control& target)
 {
 	target.dynent->maxspeed = speed;
@@ -109,6 +259,8 @@ void ActionSpeed::process(Control& target)
 
 ActionDestroy::ActionDestroy() {}
 
+ActionDestroy::~ActionDestroy() {}
+
 void ActionDestroy::process(Control& target)
 {
     target.disconnect();
@@ -117,6 +269,8 @@ void ActionDestroy::process(Control& target)
 
 ActionModel::ActionModel(int _model) : model(_model) {}
 
+ActionModel::~ActionModel() {}
+
 void ActionModel::process(Control& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
@@ -127,6 +281,8 @@ void ActionModel::process(Control& target)
 
 ActionSetAO::ActionSetAO(int _ao, int _anim) : ao(_ao), anim(_anim) {}
 
+ActionSetAO::~ActionSetAO() {}
+
 void ActionSetAO::process(Control& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
@@ -136,6 +292,8 @@ void ActionSetAO::process(Control& target)
 
 ActionSetGesture::ActionSetGesture(int _anim) : anim(_anim) {}
 
+ActionSetGesture::~ActionSetGesture() {}
+
 void ActionSetGesture::process(Control& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
@@ -145,10 +303,12 @@ void ActionSetGesture::process(Control& target)
 
 ActionDoGesture::ActionDoGesture(int _millis) : millis(_millis) {}
 
+ActionDoGesture::~ActionDoGesture() {}
+
 void ActionDoGesture::process(Control& target)
 {
 	CharacterInfo& info = target.dynent->getcharinfo();
-	if ( (lastmillis - dynamic_cast<ActionInstance&>(target).lastaction ) > millis )
+	if ( (lastmillis - target.dynent->lastaction ) > millis ) //Protagoras: removed dynamic cast
     {
         info.dogesture = false;
         finished(target);
@@ -156,17 +316,22 @@ void ActionDoGesture::process(Control& target)
     else
     {
         info.dogesture = true;
+        moveplayer(target.dynent, 1, false);
     }
 }
 
 ActionAttach::ActionAttach(int _part, char* _model, int _rule) : part(_part), rule(_rule)
 {
-    strcpy(model, _model);
+	copystring(model, _model);
+}
+
+ActionAttach::~ActionAttach()
+{
+	delete[] model;
 }
 
 void ActionAttach::process(Control& target)
 {
-    conoutf("ActionAttach::process");
    	CharacterInfo& info = target.dynent->getcharinfo();
   	info.attached.add(part,model,rule);
    	game::addmsg(SV_ATTACH, "rcisi", target.dynent, part, model, rule);
@@ -175,6 +340,8 @@ void ActionAttach::process(Control& target)
 
 ActionDetach::ActionDetach(int _part) : part(_part) {}
 
+ActionDetach::~ActionDetach() {}
+
 void ActionDetach::process(Control& target)
 {
     CharacterInfo& info = target.dynent->getcharinfo();
@@ -182,6 +349,56 @@ void ActionDetach::process(Control& target)
     game::addmsg(SV_DETACH, "rci", target.dynent, part);
     finished(target);
 }
+
+ActionCommand::ActionCommand(const char* cmd) : command(cmd && *cmd ? newstring(cmd) : NULL) {}
+
+ActionCommand::~ActionCommand()
+{
+	delete[] command;
+}
+
+void ActionCommand::process(Control& target)
+{
+	if(command && !editmode) //#offtools: only execute if not in editmode to prevent recursion
+	{
+		execute(command);
+	}
+	finished(target);
+}
+
+ActionCond::ActionCond(const char* t, const char* b) : test(t && *t ? newstring(t) : NULL), body(b && *b ? newstring(b) : NULL) {}
+
+ActionCond::~ActionCond()
+{
+	delete[] test;
+	delete[] body;
+}
+
+void ActionCond::process(Control& target)
+{
+	if( (!test || execute(test)) && !editmode )
+	{
+		if(body) execute(body);
+	}
+	finished(target);
+}
+
+ActionCondWait::ActionCondWait(const char* t) : test(t && *t ? newstring(t) : NULL) {}
+
+ActionCondWait::~ActionCondWait()
+{
+	delete[] test;
+}
+
+void ActionCondWait::process(Control& target)
+{
+	if( (!test || execute(test)) && !editmode )
+	{
+		finished(target);
+	}
+	moveplayer(target.dynent, 1, false);
+}
+
 //-------ActionState----------------------------------
 ActionState::ActionState(const char*  descr)
 {
@@ -191,7 +408,7 @@ ActionState::ActionState(const char*  descr)
 
 ActionState::~ActionState()
 {
-    steps.deletecontentsp();
+    steps.deletecontents();
 }
 
 bool ActionState::hasstep(int i)
@@ -238,7 +455,7 @@ ActionLib::~ActionLib()
 //        conoutf("DEBUG: ActionLib::~ActionLib - usage: %d > 0", usage);
         notifydelete();
     }
-    states.deletecontentsp();
+    states.deletecontents();
 }
 
 bool ActionLib::hasstate(int state)
@@ -315,7 +532,7 @@ void ActionLib::notifydelete()
 }
 
 //---------ActionInstance-------------------------------
-ActionInstance::ActionInstance(ActionLib& _lib) : lib(_lib), curstate(0), curstep(0), lastaction(0), actionstate(ACTION_STATE_IDLE), actionpolicy(ACTION_POLICY_ON_STATE_END_WAIT), localuid(0)
+ActionInstance::ActionInstance(ActionLib& _lib) : lib(_lib), curstate(0), curstep(0), actionstate(ACTION_STATE_IDLE), actionpolicy(ACTION_POLICY_ON_STATE_END_WAIT), localuid(0), lastprocess(0), plastaction(NULL)
 {
     lib.usage++;
 }
@@ -326,8 +543,6 @@ int ActionInstance::getcurstate() { return curstate; }
 
 int ActionInstance::getcurstep() { return curstep; }
 
-int ActionInstance::getlastaction() { return lastaction; }
-
 void ActionInstance::setcurstate(int state)
 {
 	if(!lib.hasstate(state)) return;
@@ -345,7 +560,7 @@ void ActionInstance::next()
 	{
         event(ACTIONINSTANCE_EVENT_STATE_FINISH);
 	}
-    lastaction = lastmillis;
+	*plastaction = lastmillis;
 }
 
 //TODO: cleanup, move most stuff into handlers
@@ -361,7 +576,7 @@ void ActionInstance::event(int ev)
         case ACTIONINSTANCE_EVENT_INIT:
         {
             actionstate = ACTION_STATE_INIT;
-            lastaction = lastmillis;
+            *plastaction = lastmillis;
             break;
         }
         case ACTIONINSTANCE_EVENT_STEP_FINISH:
@@ -374,7 +589,7 @@ void ActionInstance::event(int ev)
             if(actionstate == ACTION_STATE_END || ACTION_STATE_INIT)
             {
                 actionstate = ACTION_STATE_PROCESS;
-                lastaction = lastmillis;
+                *plastaction = lastmillis;
             }
             break;
         }
@@ -434,6 +649,8 @@ void ActionInstance::process(Control& target)
     {
         event(ACTIONINSTANCE_EVENT_STATE_FINISH);
     }
+
+    lastprocess = lastmillis;
 }
 
 // TODO (offtools#1#): implement nextstep - step through commands
@@ -456,7 +673,7 @@ ActionLib* findcharacter(const char* s)
 
 //--------------Character Commands----------------------------
 //adds new actionlibrary and spawns character if in editmode
-void newcharacter(const char* s)
+bool newcharacter(const char* s)
 {
     if(!findcharacter(s))
     {
@@ -466,13 +683,14 @@ void newcharacter(const char* s)
             ControlCharacter* controler = new ControlCharacter(*library);
             game::controls.add(controler);
             game::addmsg(SV_REQUESTBOT, "r");
+			return true;
         }
     }
     else
     {
         conoutf("game::newcharacter: actionlib %s already exits", s);
-        return;
     }
+	return false;
 }
 
 int lastuid = 0;
@@ -589,7 +807,34 @@ void actionappendmove(char *s, int state, int uid)
     if(MapEntities::findmapuid(uid, WAYPOINT))
         actionappend(s, state, new ActionMove(uid));
     else
-        conoutf("actionappendspawn: waypoint not found");
+        conoutf("actionappendmove: waypoint not found");
+}
+
+//added by Protagoras
+void actionappendjump(char *s, int state)
+{
+	actionappend(s, state, new ActionJump());
+}
+
+//added by Protagoras
+void actionappendface(char *s, int state, int cuid)
+{
+	actionappend(s, state, new ActionFace(cuid));
+}
+
+//added by Protagoras
+void actionappendexpect(char *s, int state, int cuid, int radius)
+{
+	actionappend(s, state, new ActionExpect(cuid, radius));
+}
+
+//added by Protagoras
+void actionappendfollow(char *s, int state, int cuid, int uid)
+{
+    if(MapEntities::findmapuid(uid, WAYPOINT))
+        actionappend(s, state, new ActionFollow(cuid, uid));
+    else
+        conoutf("actionappendfollow: waypoint not found");
 }
 
 void actionappendspeed(char *s, int state, int speed)
@@ -622,15 +867,31 @@ void actionappendgesture(char *s, int state, int millis)
 	actionappend(s, state, new ActionDoGesture(millis));
 }
 
-//void actionappendattach(char *s, int state, int part, char* model, int rule)
-//{
-//	actionappend(s, state, new ActionAttach(part, model, rule));
-//}
-//
-//void actionappenddetach(char *s, int state, int part)
-//{
-//	actionappend(s, state, new ActionDetach(part));
-//}
+void actionappendattach(char *s, int state, int part, char* model, int rule)
+{
+	actionappend(s, state, new ActionAttach(part, model, rule));
+}
+
+void actionappenddetach(char *s, int state, int part)
+{
+	actionappend(s, state, new ActionDetach(part));
+}
+
+void actionappendcommand(char* s, int state, char* cmd)
+{
+	actionappend(s, state, new ActionCommand(cmd));
+}
+
+void actionappendcond(char* s, int state, char* test, char* body)
+{
+	actionappend(s, state, new ActionCond(test, body));
+}
+
+void actionappendcondwait(char* s, int state, char* test)
+{
+	actionappend(s, state, new ActionCondWait(test));
+}
+
 //--------------Character Commands----------------------------
 //following commands manipulate an spawned instance of an character,
 //instances are referenced by their uid as return value on executing
@@ -658,7 +919,7 @@ void loopcharacter(int id, bool loop = true)
 	}
 }
 
-ICOMMAND(newcharacter, "s", (char *s), newcharacter(s));
+ICOMMAND(newcharacter, "s", (char *s), intret(newcharacter(s)));
 ICOMMAND(showcharacter, "s", (char *s), showcharacter(s));
 ICOMMAND(listcharacters, "", (), listcharacters());
 ICOMMAND(addcharacter, "s", (char *s), intret(addcharacter(s)));
@@ -674,9 +935,68 @@ ICOMMAND(appendactionsetao, "siii", (char *s, int *state,  int *ao, int *anim),
 ICOMMAND(appendactionsetgesture, "sii", (char *s, int *state,  int *anim), actionappendsetgesture(s, *state, *anim));
 ICOMMAND(appendactionsethold, "sii", (char *s, int *state, int *anim), actionappendsethold(s, *state, *anim));
 ICOMMAND(appendactiongesture, "sii", (char *s, int *state, int *time), actionappendgesture(s, *state, *time));
-//ICOMMAND(appendactionattach, "siisi", (char *s, int *state,  int *part, char *model, int *rule), actionappendattach(s, *state, *part, model, *rule));
-//ICOMMAND(appendactiondetach, "sii", (char *s, int *state, int *rule), actionappenddetach(s, *state, *rule));
+ICOMMAND(appendactionattach, "siisi", (char *s, int *state,  int *part, char *model, int *rule), actionappendattach(s, *state, *part, model, *rule));
+ICOMMAND(appendactiondetach, "sii", (char *s, int *state, int *rule), actionappenddetach(s, *state, *rule));
+ICOMMAND(appendactioncommand, "sis", (char *s, int *state, char* cmd), actionappendcommand(s, *state, cmd));
+ICOMMAND(appendactioncond, "siss", (char *s, int *state, char* test, char* body), actionappendcond(s, *state, test, body));
+ICOMMAND(appendactioncondwait, "siss", (char *s, int *state, char* test), actionappendcondwait(s, *state, test));
+//jump, face, expect, follow by Protagoras
+ICOMMAND(appendactionjump, "si", (char *s, int *state), actionappendjump(s, *state));
+ICOMMAND(appendactionface, "sii", (char *s, int *state, int *cuid), actionappendface(s, *state, *cuid));
+ICOMMAND(appendactionexpect, "siii", (char *s, int *state, int *cuid, int *radius), actionappendexpect(s, *state, *cuid, *radius));
+ICOMMAND(appendactionfollow, "siii", (char *s, int *state, int *cuid, int *uid), actionappendfollow(s, *state, *cuid, *uid));
 
-//-------------Character Instance Commands-------------------
+//-------------Character Instance Commands------------------------------------
 ICOMMAND(getclientfromuid, "i", (int *requid), intret(clientnumfromuid(*requid)));
 ICOMMAND(loopcharacter, "ii", (int *id, int *loop), loopcharacter(*id, *loop); );
+
+//-------------Character Script Proxy (saving stuff)--------------------------
+CommandProxy::CommandProxy() {}
+
+CommandProxy::~CommandProxy()
+{
+	commands.deletecontents();
+}
+
+void CommandProxy::addcmd(char **args, int *numargs)
+{
+
+	vector<char> buf;
+	string lst;
+	char* ret;
+
+	for (int i = 0; i < *numargs; i++)
+	{
+		formatstring(lst)("%s", args[i]);
+		buf.put(lst, strlen(lst));
+		if(i < ( *numargs-1 )) buf.add(' ');
+	}
+	buf.add('\0');
+	commands.add(newstring(buf.getbuf()));
+	ret = executeret(commands.last());
+	if (ret) {
+		result(ret);
+	}
+}
+
+void CommandProxy::save()
+{
+	defformatstring(fname)("packages/base/movie/%s.cfg", game::getclientmap());
+	stream* f = openrawfile(fname, "wb");
+	loopv(commands)
+	{
+		f->putline(commands[i]);
+	}
+	f->close();
+}
+
+void CommandProxy::clear()
+{
+	commands.deletecontents();
+}
+
+CommandProxy proxy;
+
+ICOMMAND(appendactionproxy, "V", (char** c, int *i), proxy.addcmd(c, i));
+ICOMMAND(saveactions, "", (), proxy.save());
+ICOMMAND(clearactions, "", (), proxy.clear());
