@@ -1,5 +1,8 @@
 #include "game.h"
 
+///TODO: check messages.setsize(0);
+///TODO: check SV_DELCUBE
+
 namespace game
 {
     VARP(maxradarscale, 0, 1024, 10000);
@@ -548,7 +551,7 @@ namespace game
         connected = remote = false;
         player1->clientnum = -1;
         sessionid = 0;
-        messages.setsizenodelete(0);
+        messages.setsize(0);
         messagereliable = false;
         messagecn = -1;
         player1->respawn();
@@ -638,7 +641,7 @@ namespace game
         if(messages.length())
         {
             p.put(messages.getbuf(), messages.length());
-            messages.setsizenodelete(0);
+            messages.setsize(0);
             if(messagereliable) p.reliable();
             messagereliable = false;
             messagecn = -1;
@@ -1108,7 +1111,7 @@ namespace game
                     case SV_COPY: if(d) mpcopy(d->edit, sel, false); break;
                     case SV_PASTE: if(d) mppaste(d->edit, sel, false); break;
                     case SV_ROTATE: dir = getint(p); mprotate(dir, sel, false); break;
-                    case SV_REPLACE: tex = getint(p); newtex = getint(p); mpreplacetex(tex, newtex, sel, false); break;
+					case SV_REPLACE: { tex = getint(p); newtex = getint(p); int insel = getint(p); mpreplacetex(tex, newtex, insel > 0, sel, false); break; }
                     case SV_DELCUBE: mpdelcube(sel, false); break;
                 }
                 break;
@@ -1122,14 +1125,26 @@ namespace game
             }
             case SV_EDITENT:            // coop edit of ent
             {
+//                if(!d) return;
+//                getint(p);
+//                int i = getint(p);
+//                float x = getint(p)/DMF, y = getint(p)/DMF, z = getint(p)/DMF;
+//                int type = getint(p);
+//                int attr1 = getint(p), attr2 = getint(p), attr3 = getint(p), attr4 = getint(p), attr5 = getint(p), attr6 = getint(p), attr7 = getint(p), attr8 = getint(p);
+//
+//                mpeditent(i, vec(x, y, z), type, attr1, attr2, attr3, attr4, attr5, attr6, attr7, attr8, false);
+//                break;
                 if(!d) return;
                 getint(p);
                 int i = getint(p);
                 float x = getint(p)/DMF, y = getint(p)/DMF, z = getint(p)/DMF;
                 int type = getint(p);
-                int attr1 = getint(p), attr2 = getint(p), attr3 = getint(p), attr4 = getint(p), attr5 = getint(p), attr6 = getint(p), attr7 = getint(p), attr8 = getint(p);
+                int num = getint(p);
+                int* attrs = new int[num];
+                loopj(num) attrs[j] = getint(p);
 
-                mpeditent(i, vec(x, y, z), type, attr1, attr2, attr3, attr4, attr5, attr6, attr7, attr8, false);
+                mpeditent(i, vec(x, y, z), type, attrs, false);
+                DELETEA(attrs);
                 break;
             }
             case SV_EDITVAR:
@@ -1374,8 +1389,7 @@ namespace game
         		int rule = getint(p);
         		if(d)
         		{
-                    conoutf("DEBUG: server::parsepacket SV_ATTACH cl: %d, bot: %d, %d %s, %d", d->clientnum, d->clientnum, part, text, rule);
-
+                    //conoutf("DEBUG: server::parsepacket SV_ATTACH cl: %d, bot: %d, %d %s, %d", d->clientnum, d->clientnum, part, text, rule);
         			CharacterInfo& info = d->getcharinfo();
         			info.attached.add(part,text,rule);
         		}
@@ -1690,12 +1704,13 @@ namespace game
 	//__offtools__ moviecube needs this, usefull to query model animations (especially addional ones)
 	void registeranimation(char *loadname, char* anim, int num)
 	{
+		conoutf("registeranimation: %s, %s", loadname, anim);
 		loopv(playermodels)
 		{
 			if(strcmp(loadname, playermodels[i]->name) == 0)
 			{
 				playermodels[i]->addanimation(anim, num);
-				conoutf("[DEBUG] game::registeranimation - dir: %s, anim: %s, num: %d", loadname, anim, num);
+//				 conoutf("[DEBUG] game::registeranimation - dir: %s, anim: %s, num: %d", loadname, anim, num);
 				return;
 			}
 		}
@@ -1719,6 +1734,20 @@ namespace game
     	return -1;
     }
 
+	void listattachpoints()
+	{
+		vector<char> buf;
+		string lst;
+		for(int i = 0; i < ATTACH_PARTS; i++)
+		{
+			formatstring(lst)("%s", body[i].part);
+			buf.put(lst, strlen(lst));
+			if(i < ( ATTACH_PARTS-1 )) buf.add(' ');
+		}
+		buf.add('\0');
+		result(buf.getbuf());
+	}
+
 	//implementation of attachment script commands
     void playerattach(int p, char* m, int r)
     {
@@ -1736,6 +1765,7 @@ namespace game
 
     //-----------------Player1 Attachments-------------------
     ICOMMAND(getattachnum, "s", (char *p), intret(attachnumbypart(p)));
+    ICOMMAND(listattachpoints, "", (), listattachpoints());
     ICOMMAND(playerattach, "isi", (int *p, char *m, int *r), playerattach(*p,m,*r));
     ICOMMAND(playerdetach, "i", (int *p), playerdetach(*p));
 
@@ -1759,6 +1789,7 @@ namespace game
     {
     	CharacterInfo& info = player1->getcharinfo();
     	info.dogesture = on;
+    	if (on) player1->lastaction = lastmillis;
     }
     ICOMMAND(gesture, "D", (int *down), { gesture(*down!=0); });
     ICOMMAND(setgesture, "i", (int *g), { setgesture(*g); });
