@@ -2,6 +2,8 @@
 
 //TODO: handle trigger for mp and sp
 
+//TODO: TRIGGER
+
 namespace entities
 {
     using namespace game;
@@ -37,6 +39,7 @@ namespace entities
             NULL, NULL,
             NULL, NULL,
             NULL, NULL,
+            NULL,
             NULL
         };
         return entmdlnames[type];
@@ -47,8 +50,8 @@ namespace entities
     {
         if(e.type == TELEPORT)
         {
-            if(e.attr2 > 0) return mapmodelname(e.attr2);
-            if(e.attr2 < 0) return NULL;
+            if(e.attr[1] > 0) return mapmodelname(e.attr[1]);
+            if(e.attr[1] < 0) return NULL;
         }
         return e.type < MAXENTTYPES ? entmdlname(e.type) : NULL;
     }
@@ -84,7 +87,7 @@ namespace entities
     //declared in igame.h
     void trigger(extentity &e)
     {
-        switch(e.attr3)
+        switch(e.attr[2])
         {
             case 29:
                 break;
@@ -138,16 +141,16 @@ namespace entities
     //not needed - move to cmode
     void teleport(int n, DynamicEntity *d)     // also used by monsters
     {
-        int e = -1, tag = MapEntities::ents[n]->attr1, beenhere = -1;
+        int e = -1, tag = MapEntities::ents[n]->attr[0], beenhere = -1;
         for(;;)
         {
             e = findentity(TELEDEST, e+1);
             if(e==beenhere || e<0) { conoutf(CON_WARN, "no teleport destination for tag %d", tag); return; }
             if(beenhere<0) beenhere = e;
-            if(MapEntities::ents[e]->attr2==tag)
+            if(MapEntities::ents[e]->attr[1]==tag)
             {
                 d->o = MapEntities::ents[e]->o;
-                d->yaw = MapEntities::ents[e]->attr1;
+                d->yaw = MapEntities::ents[e]->attr[0];
                 d->pitch = 0;
                 d->vel = vec(0, 0, 0);//vec(cosf(RAD*(d->yaw-90)), sinf(RAD*(d->yaw-90)), 0);
                 entinmap(d);
@@ -193,7 +196,7 @@ namespace entities
                 if(d->lastpickup==MapEntities::ents[n]->type && lastmillis-d->lastpickupmillis<300) break;
                 d->lastpickup = MapEntities::ents[n]->type;
                 d->lastpickupmillis = lastmillis;
-                vec v((int)(char)MapEntities::ents[n]->attr3*10.0f, (int)(char)MapEntities::ents[n]->attr2*10.0f, MapEntities::ents[n]->attr1*12.5f);
+                vec v((int)(char)MapEntities::ents[n]->attr[2]*10.0f, (int)(char)MapEntities::ents[n]->attr[1]*10.0f, MapEntities::ents[n]->attr[0]*12.5f);
                 d->timeinair = 0;
 //		if(d->ai) d->ai->becareful = true;
                 d->falling = vec(0, 0, 0);
@@ -270,17 +273,17 @@ namespace entities
             case BARREL:
             case PLATFORM:
             case ELEVATOR:
-                e.attr5 = e.attr4;
-                e.attr4 = e.attr3;
-                e.attr3 = e.attr2;
+                e.attr[4] = e.attr[3];
+                e.attr[3] = e.attr[2];
+                e.attr[2] = e.attr[1];
             case TELEDEST:
-                e.attr2 = e.attr1;
+                e.attr[1] = e.attr[0];
             case RESPAWNPOINT:
-                e.attr1 = (int)player1->yaw;
+                e.attr[0] = (int)player1->yaw;
                 break;
             case WAYPOINT:
-        		e.attr1 = (int)player1->yaw;
-        		e.attr2 = 25.0f;
+        		e.attr[0] = (int)player1->yaw;
+        		e.attr[1] = 25.0f;
                 break;
         }
     }
@@ -290,11 +293,11 @@ namespace entities
         switch(e.type)
         {
             case DYNLIGHT:
-                if(color) glColor3f(e.attr2/255.0f, e.attr3/255.0f, e.attr4/255.0f);
-                renderentsphere(e, e.attr1);
+                if(color) glColor3f(e.attr[1]/255.0f, e.attr[2]/255.0f, e.attr[3]/255.0f);
+                renderentsphere(e, e.attr[0]);
                 break;
             case TELEPORT:
-                loopv(MapEntities::ents) if(MapEntities::ents[i]->type == TELEDEST && e.attr1==MapEntities::ents[i]->attr2)
+                loopv(MapEntities::ents) if(MapEntities::ents[i]->type == TELEDEST && e.attr[0]==MapEntities::ents[i]->attr[1])
                 {
                     renderentarrow(e, vec(MapEntities::ents[i]->o).sub(e.o).normalize(), e.o.dist(MapEntities::ents[i]->o));
                     break;
@@ -302,7 +305,7 @@ namespace entities
                 break;
 
             case JUMPPAD:
-                renderentarrow(e, vec((int)(char)e.attr3*10.0f, (int)(char)e.attr2*10.0f, e.attr1*12.5f).normalize(), 4);
+                renderentarrow(e, vec((int)(char)e.attr[2]*10.0f, (int)(char)e.attr[1]*10.0f, e.attr[0]*12.5f).normalize(), 4);
                 break;
 
             case FLAG:
@@ -315,16 +318,16 @@ namespace entities
             case ELEVATOR:
             {
                 vec dir;
-                vecfromyawpitch(e.attr1, 0, 1, 0, dir);
+                vecfromyawpitch(e.attr[0], 0, 1, 0, dir);
                 renderentarrow(e, dir, 4);
                 break;
             }
             case WAYPOINT:
             {
-                renderentsphere(e, e.attr2);
+                renderentsphere(e, e.attr[1]);
                 vec dir;
-                vecfromyawpitch(e.attr1, 0, 1, 0, dir);
-                renderentarrow(e, dir, e.attr2);
+                vecfromyawpitch(e.attr[0], 0, 1, 0, dir);
+                renderentarrow(e, dir, e.attr[1]);
                 break;
             }
         }
@@ -392,10 +395,49 @@ namespace entities
                 "box", "barrel",
                 "platform", "elevator",
                 "flag",
-                "waypoint", "dynlight", "", "", "",
+                "waypoint", "dynlight", "camera", "", "", "",
         };
         return i>=0 && size_t(i)<sizeof(entnames)/sizeof(entnames[0]) ? entnames[i] : "";
     }
+
+    const int numattrs(int type)
+    {
+	    static const int num[] =
+	    {
+		    0, //shells
+		    0, //bullets
+		    0, //rockets
+		    0, //riflerounds
+		    0, //grenades
+		    0, //cartridges
+		    0, //health
+		    0, //healthboost
+		    0, //greenarmour
+		    0, //yellowarmour
+		    0, //quaddamage
+		    3, //teleport
+		    2, //teledest
+		    3, //creature
+		    1, //carrot
+		    3, //jumppad
+		    2, //banana/base
+		    1, //respawnpoint
+		    4, //box
+		    4, //barrel
+		    4, //platform
+		    4, //elevator
+		    2, //flag
+		    3, //waypoint
+		    3, //camera yaw, pitch, distance
+		    5, //dynlight
+		    4, //camera
+		    0, //null x 2
+		    0
+	    };
+
+	    type -= ET_GAMESPECIFIC;
+	    return type >= 0 && size_t(type) < sizeof(num)/sizeof(num[0]) ? num[type] : 0;
+    }
     //declared in igame.h
 	void renderhelpertext(extentity &e, int &colour, vec &pos, string &tmp)
     {
@@ -405,40 +447,40 @@ namespace entities
     		pos.z += 3.0;
     		formatstring(tmp)("uid: %d\nTeledest Tag: %i\nModel: %s (%i)",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				mapmodelname(e.attr2),
-    				e.attr2
+    				e.attr[0],
+    				mapmodelname(e.attr[1]),
+    				e.attr[1]
     		);
     		return;
     	case TELEDEST:
     		pos.z += 3;
     		formatstring(tmp)("uid: %d\nYaw: %i\nTeleport Tag: %i\n",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				e.attr2
+    				e.attr[0],
+    				e.attr[1]
     		);
     		return;
     	case JUMPPAD:
     		pos.z += 4.5;
     		formatstring(tmp)("uid: %d\nZ: %i\nY: %i\nX: %i",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				e.attr2,
-    				e.attr3
+    				e.attr[0],
+    				e.attr[1],
+    				e.attr[2]
     		);
     		return;
     	case BASE:
     		pos.z += 3.0f;
     		formatstring(tmp)("uid: %d\nAmmo: %i\nTag: base_%i",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				e.attr2
+    				e.attr[0],
+    				e.attr[1]
     		);
     		return;
     	case RESPAWNPOINT:
     		pos.z += 1.5;
     		formatstring(tmp)("Yaw: %i",
-    				e.attr1
+    				e.attr[0]
     		);
     		return;
     	case BOX:
@@ -446,10 +488,10 @@ namespace entities
     		pos.z += 6.0;
     		formatstring(tmp)("uid: %d\nYaw: %i\nModel: %s (%i)\nWeight: %i\nHealth: %i",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				mapmodelname(e.attr2), e.attr2,
-    				e.attr3,
-    				e.attr4
+    				e.attr[0],
+    				mapmodelname(e.attr[1]), e.attr[1],
+    				e.attr[2],
+    				e.attr[3]
     		);
     		return;
     	case PLATFORM:
@@ -457,37 +499,47 @@ namespace entities
     		pos.z += 6.0;
     		formatstring(tmp)("uid: %d\nYaw: %i\nModel: %s (%i)\nTag: %i\nSpeed: %i",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				mapmodelname(e.attr2), e.attr2,
-    				e.attr3,
-    				e.attr4
+    				e.attr[0],
+    				mapmodelname(e.attr[1]), e.attr[1],
+    				e.attr[2],
+    				e.attr[3]
     		);
     		return;
     	case FLAG:
     		pos.z += 3.0f;
     		formatstring(tmp)("uid: %d\nYaw: %i\nTeam: %i",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				e.attr2
+    				e.attr[0],
+    				e.attr[1]
     		);
     		return;
     	case DYNLIGHT:
     		pos.z += 9.0;
     		formatstring(tmp)("uid: %d\nRadius %i\n\fs\fRRed: %i\n\fJGreen: %i\n\fDBlue: %i\fr\nTag: %i",
                     MapEntities::getmapuid(e),
-    				e.attr1,
-    				e.attr2,
-    				e.attr3,
-    				e.attr4,
-    				e.attr5
+    				e.attr[0],
+    				e.attr[1],
+    				e.attr[2],
+    				e.attr[3],
+    				e.attr[4]
     		);
     		return;
     	case WAYPOINT:
     		pos.z += 4.5f;
     		formatstring(tmp)("uid: %i\nDirection %i\nRadius %i",
     				MapEntities::getmapuid(e),
-    				e.attr1,
-    				e.attr2
+    				e.attr[0],
+    				e.attr[1]
+    		);
+    		return;
+    	case CAMERA:
+    		pos.z += 6.f;
+    		formatstring(tmp)("uid: %i\ntag: %i\nyaw: %i\npitch: %i\nroll: %i",
+    				MapEntities::getmapuid(e),
+    				e.attr[0],
+    				e.attr[1],
+    				e.attr[2],
+    				e.attr[3]
     		);
     		return;
     	}
@@ -516,10 +568,11 @@ namespace entities
     	memcpy(&((MapEntities::StaticEntity &)e).uid, buf, sizeof(int));
     }
     //declared in igame.h
-    void editent(int i)
+    void editent(int i, bool local)
     {
         MapEntities::StaticEntity &e = *MapEntities::ents[i];
-        addmsg(SV_EDITENT, "riii3ii8", e.uid, i, (int)(e.o.x*DMF), (int)(e.o.y*DMF), (int)(e.o.z*DMF), e.type, e.attr1, e.attr2, e.attr3, e.attr4, e.attr5, e.attr6, e.attr7, e.attr8);
+        //if(!local) //also when local, we need send to send map entity uids
+			addmsg(SV_EDITENT, "riii3ii8", e.uid, i, (int)(e.o.x*DMF), (int)(e.o.y*DMF), (int)(e.o.z*DMF), e.type, e.attr[0], e.attr[1], e.attr[2], e.attr[3], e.attr[4], e.attr[5], e.attr[6], e.attr[7]);
     }
     //declared in igame.h
     float dropheight(entity &e)
@@ -537,7 +590,7 @@ namespace entities
 			case ELEVATOR:
 			case BARREL:
 			case BOX:
-				return &e.attr2;
+				return &e.attr[1];
 			default:
 				return NULL;
 		}
@@ -548,15 +601,20 @@ namespace entities
 		switch(e.type)
 		{
 			case TELEPORT:
-				if(e.attr2 == 0)
+				if(e.attr[1] == 0)
 					return false;
 			case PLATFORM:
 			case ELEVATOR:
 			case BARREL:
 			case BOX:
-				return (e.attr2 == i);
+				return (e.attr[1] == i);
 			default:
 				return false;
 		}
 	}
+
+	void animatemapmodel(const extentity &e, int &anim, int &basetime)
+    {
+    	return;
+    }
 }
