@@ -1,5 +1,7 @@
 #include "game.h"
 
+///TODO: stackmovable (486)
+///TODO: physicstrigger
 namespace game
 {
     int maptime = 0, maprealtime = 0, minremain = 0;
@@ -59,7 +61,7 @@ namespace game
 //        if(m_classicsp)
 //        {
 		clearmovables();
-            resettriggers();
+//            resettriggers();
 //        }
     }
 
@@ -241,7 +243,7 @@ namespace game
         gets2c();
         updatemovables(curtime);
         updatecontrols();
-
+		CutScene::update();
         if(player1->clientnum>=0) c2sinfo();   // do this last, to reduce the effective frame lag
     }
 
@@ -317,7 +319,7 @@ namespace game
 
     DynamicEntity *newclient(int cn)   // ensure valid entity
     {
-    	conoutf("DEBUG: game::newclient %d", cn);
+//    	conoutf("DEBUG: game::newclient %d", cn);
         if(cn < 0 || cn > max(0xFF, MAXCLIENTS + MAXBOTS))
         {
             neterr("clientnum", false);
@@ -330,7 +332,7 @@ namespace game
         if(!clients[cn])
         {
             DynamicEntity *d = new DynamicEntity;
-        	conoutf("DEBUG: newclient DynamicEntity %d", cn);
+//          conoutf("DEBUG: newclient DynamicEntity %d", cn);
             d->clientnum = cn;
             clients[cn] = d;
             players.add(d);
@@ -346,7 +348,7 @@ namespace game
 
     void clientdisconnected(int cn, bool notify)
     {
-        conoutf("DEBUG: game::clientdisconnected cn: %d", cn);
+//      conoutf("DEBUG: game::clientdisconnected cn: %d", cn);
         if(!clients.inrange(cn)) return;
         if(following==cn)
         {
@@ -373,8 +375,10 @@ namespace game
     {
         player1 = spawnstate(new DynamicEntity);
         players.add(player1);
-		Control* cp = controls.add(new ControlPlayer);
-		cp->init(player1);
+
+        Control* cp = controls.add(new ControlPlayer);
+        cp->init(player1);
+
     }
 
     VARP(showmodeinfo, 0, 1, 1);
@@ -460,8 +464,8 @@ namespace game
 		{
 			vec o = d->o;
 			o.z -= d->eyeheight *i/60.0f;
-			regularflame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
-			regularflame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
+//			regularflame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
+//			regularflame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
 		}
                 break;
             case MAT_WATER:
@@ -476,6 +480,17 @@ namespace game
         }
     }
 
+    void dynentcollide(physent *d, physent *o, const vec &dir)
+    {
+        switch(d->type)
+        {
+            //case ENT_AI: if(dir.z > 0) stackmonster((monster *)d, o); break;
+            //case ENT_INANIMATE: if(dir.z > 0) stackmovable((movable *)d, o); break;
+        }
+    }
+
+    void bounced(physent *d, const vec &surface) {}
+
     void msgsound(int n, physent *d)
     {
         if(!d || d==player1)
@@ -551,7 +566,7 @@ namespace game
         glEnd();
     }
 
-    float abovegameplayhud()
+    float abovegameplayhud(int w, int h)
     {
         switch(hudplayer()->state)
         {
@@ -666,12 +681,13 @@ namespace game
          }
     }
 
-        void dynlighttrack(physent *owner, vec &o)
+        void dynlighttrack(physent *owner, vec &o, vec &hud)
     {
         if(owner->type!=ENT_PLAYER && owner->type!=ENT_AI) return;
         DynamicEntity *pl = (DynamicEntity *)owner;
 //        if(pl->muzzle.x < 0 || pl->lastattackgun != pl->gunselect) return;
         o = pl->muzzle;
+	hud = owner == hudplayer() ? vec(pl->o).add(vec(0, 0, 2)) : pl->muzzle;
     }
 
     #ifndef NEWGUI
@@ -778,7 +794,22 @@ namespace game
     }
 
     bool allowdoublejump() {return false;}
+
+	bool mousemove(int &dx, int &dy, float &cursens) {return false;}
+
     void updatecamera() {}
+
+        //NOTE FIXME TODO @offtools, get rid of the camera hack and use this to update the camera and do cutscenes and... stuff
+//    bool recomputecamera(physent *&camera1, physent &tempcamera, bool &detachedcamera, float &thirdpersondistance)
+//    {
+//    	return false;
+//    }
+
+    bool needminimap()
+    {
+    	return false;
+    }
+
     void writemapdata(stream *f) {}
 
     // any data written into this vector will get saved with the map data. Must take care to do own versioning, and endianess if applicable. Will not get called when loading maps from other games, so provide defaults.
@@ -832,4 +863,19 @@ namespace game
             conoutf("[debug]: dummy - download requested map \"%s\" from server", name);
         }
     }
+
+	void listplayermodels()
+	{
+		vector<char> buf;
+		string lst;
+		loopv(playermodels)
+		{
+			formatstring(lst)("%s", playermodels[i]->name);
+			buf.put(lst, strlen(lst));
+			if(i < (playermodels.length()-1)) buf.add(' ');
+		}
+		buf.add('\0');
+		result(buf.getbuf());
+	}
+	ICOMMAND(listplayermodels, "", (), listplayermodels());
 }
