@@ -94,7 +94,7 @@ namespace server
         int lasttimeplayed, timeplayed;
 
 //        gamestate() : state(CS_DEAD), editstate(CS_DEAD), lastspawn(-1) {}
-        gamestate() : state(CS_DEAD), editstate(CS_DEAD){}
+        gamestate() : state(CS_ALIVE), editstate(CS_ALIVE){} //changed by Protagoras
 
         bool isalive(int gamemillis)
         {
@@ -108,7 +108,7 @@ namespace server
 
         void reset()
         {
-            if(state!=CS_SPECTATOR) state = editstate = CS_DEAD;
+            if(state!=CS_SPECTATOR) state = editstate = CS_ALIVE; //changed by Protagoras
             timeplayed = 0;
             respawn();
         }
@@ -147,7 +147,7 @@ namespace server
         bool warned, gameclip;
 
         clientinfo() { reset(); }
-        ~clientinfo() { events.deletecontentsp(); }
+        ~clientinfo() { events.deletecontents(); }
 
         void addevent(gameevent *e)
         {
@@ -159,7 +159,7 @@ namespace server
         {
             mapvote[0] = 0;
             state.reset();
-            events.deletecontentsp();
+            events.deletecontents();
             timesync = false;
             lastevent = 0;
             clientmap[0] = '\0';
@@ -171,7 +171,7 @@ namespace server
         void reassign()
         {
             state.reassign();
-            events.deletecontentsp();
+            events.deletecontents();
             timesync = false;
             lastevent = 0;
         }
@@ -183,8 +183,8 @@ namespace server
             privilege = PRIV_NONE;
             connected = local = false;
             authreq = 0;
-            position.setsizenodelete(0);
-            messages.setsizenodelete(0);
+            position.setsize(0);
+            messages.setsize(0);
             ping = 0;
             botreinit = 0;
             mapchange();
@@ -234,7 +234,12 @@ namespace server
     #define MM_PRIVSERV (MM_MODE | MM_AUTOAPPROVE)
     #define MM_PUBSERV ((1<<MM_OPEN) | (1<<MM_VETO))
     #define MM_COOPSERV (MM_AUTOAPPROVE | MM_PUBSERV | (1<<MM_LOCKED))
-    #define print if(dedi == 2) conoutf
+    //#define print if(dedicated == 2) conoutf
+    #ifdef STANDALONE
+        #define print conoutf
+    #else
+        #define print if(dedicated == 2) conoutf
+    #endif
 
     bool notgotitems = true;        // true when map has changed and waiting for clients to send item
     int gamemode = 1;
@@ -750,7 +755,7 @@ namespace server
             ci.posoff = ws.positions.length();
             loopvj(ci.position) ws.positions.add(ci.position[j]);
             ci.poslen = ws.positions.length() - ci.posoff;
-            ci.position.setsizenodelete(0);
+            ci.position.setsize(0);
         }
         if(ci.messages.empty()) ci.msgoff = -1;
         else
@@ -763,7 +768,7 @@ namespace server
             ws.messages.addbuf(p);
             loopvj(ci.messages) ws.messages.add(ci.messages[j]);
             ci.msglen = ws.messages.length() - ci.msgoff;
-            ci.messages.setsizenodelete(0);
+            ci.messages.setsize(0);
         }
     }
 
@@ -835,11 +840,11 @@ namespace server
         }
     }
 
-    bool sendpackets()
+    bool sendpackets(bool force)
     {
         if(clients.empty() || (!hasnonlocalclients() && !demorecord)) return false;
         enet_uint32 curtime = enet_time_get()-lastsend;
-        if(curtime<33) return false;
+        if(curtime<33 && !force) return false;
         bool flush = buildworldstate();
         lastsend += curtime - (curtime%33);
         return flush;
@@ -1359,6 +1364,8 @@ namespace server
         ci->timesync = false;
     }
 
+    bool ispaused() { return gamepaused; }
+
     void serverupdate()
     {
         if(!gamepaused) gamemillis += curtime;
@@ -1793,7 +1800,7 @@ namespace server
                 {
                     if((!ci->local || demorecord || hasnonlocalclients()) && (cp->state.state==CS_ALIVE || cp->state.state==CS_EDITING))
                     {
-                        cp->position.setsizenodelete(0);
+                        cp->position.setsize(0);
                         while(curmsg<p.length()) cp->position.add(p.buf[curmsg++]);
                     }
                     cp->state.o = pos;
@@ -1828,7 +1835,7 @@ namespace server
                 {
                     ci->state.editstate = ci->state.state;
                     ci->state.state = CS_EDITING;
-                    ci->events.setsizenodelete(0);
+                    ci->events.setsize(0);
                 }
                 else ci->state.state = ci->state.editstate;
                 QUEUE_MSG;
@@ -2446,8 +2453,8 @@ namespace server
 				getstring(text, p);
 				int rule = getint(p);
 
-                defformatstring(dbg)("DEBUG: server::parsepacket SV_ATTACH cl: %d, bot: %d, %d %s, %d", ci->clientnum, cq->clientnum, part, text, rule);
-        		sendf(-1, 1, "ris", SV_SERVMSG, dbg);
+                //defformatstring(dbg)("DEBUG: server::parsepacket SV_ATTACH cl: %d, bot: %d, %d %s, %d", ci->clientnum, cq->clientnum, part, text, rule);
+        		//sendf(-1, 1, "ris", SV_SERVMSG, dbg);
                 //sendf(-1, 1, "riii", SV_SWITCHMODEL, cq->clientnum, cq->playermodel);
                 //ci->playermodel = getint(p);
                 QUEUE_MSG;
@@ -2537,6 +2544,9 @@ namespace server
         return attr.length() && attr[0]==PROTOCOL_VERSION;
     }
 
+	int numchannels() { return 3; }
+
+
     #include "botmgr.h"
 }
 
