@@ -30,14 +30,18 @@ enum // cube empty-space materials
     MAT_GAMECLIP = 3 << MATF_CLIP_SHIFT,  // game specific clip material
 
     MAT_DEATH    = 1 << MATF_FLAG_SHIFT,  // force player suicide
-    MAT_EDIT     = 4 << MATF_FLAG_SHIFT   // edit-only surfaces
+    MAT_ALPHA    = 4 << MATF_FLAG_SHIFT   // alpha blended
 };
 
+#define isliquid(mat) ((mat)==MAT_WATER || (mat)==MAT_LAVA)
+#define isclipped(mat) ((mat)==MAT_GLASS)
+#define isdeadly(mat) ((mat)==MAT_LAVA)
+
 extern void lightent(extentity &e, float height = 8.0f);
-extern void lightreaching(const vec &target, vec &color, vec &dir, extentity *e = 0, float ambient = 0.4f);
+extern void lightreaching(const vec &target, vec &color, vec &dir, bool fast = false, extentity *e = 0, float ambient = 0.1f);
 extern entity *brightestlight(const vec &target, const vec &dir);
 
-enum { RAY_BB = 1, RAY_POLY = 3, RAY_ALPHAPOLY = 7, RAY_ENTS = 9, RAY_CLIPMAT = 16, RAY_SKIPFIRST = 32, RAY_EDITMAT = 64, RAY_SHADOW = 128, RAY_PASS = 256 };
+enum { RAY_BB = 1, RAY_POLY = 3, RAY_ALPHAPOLY = 7, RAY_ENTS = 9, RAY_CLIPMAT = 16, RAY_SKIPFIRST = 32, RAY_EDITMAT = 64, RAY_SHADOW = 128, RAY_PASS = 256, RAY_SKIPSKY = 512 };
 
 extern float raycube   (const vec &o, const vec &ray,     float radius = 0, int mode = RAY_CLIPMAT, int size = 0, extentity *t = 0);
 extern float raycubepos(const vec &o, const vec &ray, vec &hit, float radius = 0, int mode = RAY_CLIPMAT, int size = 0);
@@ -47,7 +51,8 @@ extern bool  raycubelos(const vec &o, const vec &dest, vec &hitpos);
 extern int thirdperson;
 extern bool isthirdperson();
 
-extern void settexture(const char *name, int clamp = 0);
+extern bool settexture(const char *name, int clamp = 0);
+extern void scaledscreenshot(char *filename, int format, int x = 0, int y = 0);
 //offtools: reload on receiving textures (replace dummy texture: notexture)
 extern void reloadslot(int n);
 
@@ -61,6 +66,7 @@ struct selinfo
     int cx, cxs, cy, cys;
     ivec o, s;
     int grid, orient;
+    selinfo() : corner(0), cx(0), cxs(0), cy(0), cys(0), o(0, 0, 0), s(0, 0, 0), grid(8), orient(0) {}
     int size() const    { return s.x*s.y*s.z; }
     int us(int d) const { return s[d]*grid; }
     bool operator==(const selinfo &sel) const { return o==sel.o && s==sel.s && grid==sel.grid && orient==sel.orient; }
@@ -71,6 +77,8 @@ extern editinfo *localedit;
 
 extern bool editmode;
 
+extern bool packeditinfo(editinfo *e, int &inlen, uchar *&outbuf, int &outlen);
+extern bool unpackeditinfo(editinfo *&e, const uchar *inbuf, int inlen, int outlen);
 extern void freeeditinfo(editinfo *&e);
 extern void pruneundos(int maxremain = 0);
 extern bool noedit(bool view = false, bool msg = true);
@@ -82,28 +90,37 @@ extern void mpflip(selinfo &sel, bool local);
 extern void mpcopy(editinfo *&e, selinfo &sel, bool local);
 extern void mppaste(editinfo *&e, selinfo &sel, bool local);
 extern void mprotate(int cw, selinfo &sel, bool local);
-extern void mpreplacetex(int oldtex, int newtex, selinfo &sel, bool local);
+extern void mpreplacetex(int oldtex, int newtex, bool insel, selinfo &sel, bool local);
 extern void mpdelcube(selinfo &sel, bool local);
 extern void mpremip(bool local);
 
 // command
-extern int variable(const char *name, int min, int cur, int max, int *storage, void (*fun)(), int flags);
-extern float fvariable(const char *name, float min, float cur, float max, float *storage, void (*fun)(), int flags);
-extern char *svariable(const char *name, const char *cur, char **storage, void (*fun)(), int flags);
+extern int variable(const char *name, int min, int cur, int max, int *storage, identfun fun, int flags);
+extern float fvariable(const char *name, float min, float cur, float max, float *storage, identfun fun, int flags);
+extern char *svariable(const char *name, const char *cur, char **storage, identfun fun, int flags);
 extern void setvar(const char *name, int i, bool dofunc = true, bool doclamp = true);
 extern void setfvar(const char *name, float f, bool dofunc = true, bool doclamp = true);
 extern void setsvar(const char *name, const char *str, bool dofunc = true);
+extern void setvarchecked(ident *id, int val);
+extern void setfvarchecked(ident *id, float val);
+extern void setsvarchecked(ident *id, const char *val);
 extern void touchvar(const char *name);
 extern int getvar(const char *name);
 extern int getvarmin(const char *name);
 extern int getvarmax(const char *name);
 extern bool identexists(const char *name);
 extern ident *getident(const char *name);
-extern bool addcommand(const char *name, void (*fun)(), const char *narg);
-extern int execute(const char *p);
+extern ident *newident(const char *name, int flags = 0);
+extern bool addcommand(const char *name, identfun fun, const char *narg);
+extern uint *compilecode(const char *p);
+extern void executeret(const uint *code, tagval &result);
+extern void executeret(const char *p, tagval &result);
 extern char *executeret(const char *p);
+extern int execute(const uint *code);
+extern int execute(const char *p);
 extern bool execfile(const char *cfgfile, bool msg = true);
 extern void alias(const char *name, const char *action);
+extern void alias(const char *name, tagval &v);
 extern const char *getalias(const char *name);
 extern void writeescapedstring(stream *f, const char *s);
 
@@ -121,9 +138,11 @@ enum
 
 extern void conoutf(const char *s, ...);
 extern void conoutf(int type, const char *s, ...);
+extern void conoutfv(int type, const char *fmt, va_list args);
 
 // menus
 #ifndef NEWGUI
+
 extern vec menuinfrontofplayer();
 extern void newgui(char *name, char *contents, char *header = NULL);
 extern void showgui(const char *name);
@@ -133,12 +152,24 @@ extern int cleargui(int n = 0);
 // octa
 extern int lookupmaterial(const vec &o);
 
+static inline bool insideworld(const vec &o)
+{
+	extern int worldsize;
+    return o.x>=0 && o.x<worldsize && o.y>=0 && o.y<worldsize && o.z>=0 && o.z<worldsize;
+}
+
+static inline bool insideworld(const ivec &o)
+{
+	extern int worldsize;
+    return uint(o.x)<uint(worldsize) && uint(o.y)<uint(worldsize) && uint(o.z)<uint(worldsize);
+}
+
 // world
 extern bool emptymap(int factor, bool force, const char *mname = "", bool usecfg = true);
 extern bool enlargemap(bool force);
 extern int findentity(int type, int index = 0, int attr1 = -1, int attr2 = -1);
 extern void findents(int low, int high, bool notspawned, const vec &pos, const vec &radius, vector<int> &found);
-extern void mpeditent(int i, const vec &o, int type, int attr1, int attr2, int attr3, int attr4, int attr5, int attr6, int attr7, int attr8, bool local);
+extern void mpeditent(int i, const vec &o, int type, int *attrs, bool local);
 extern vec getselpos();
 extern int getworldsize();
 extern int getmapversion();
@@ -147,8 +178,6 @@ extern void renderentarrow(const extentity &e, const vec &dir, float radius);
 extern void renderentattachment(const extentity &e);
 extern void renderentsphere(const extentity &e, float radius);
 extern void renderentring(const extentity &e, float radius, int axis = 0);
-extern void resettriggers();
-extern void checktriggers();
 
 // main
 extern void fatal(const char *s, ...);
@@ -156,6 +185,8 @@ extern void keyrepeat(bool on);
 
 // rendertext
 extern bool setfont(const char *name);
+extern void pushfont();
+extern bool popfont();
 extern void gettextres(int &w, int &h);
 extern void draw_text(const char *str, int left, int top, int r = 255, int g = 255, int b = 255, int a = 255, int cursor = -1, int maxwidth = -1);
 extern void draw_textf(const char *fstr, int left, int top, ...);
@@ -173,7 +204,7 @@ enum
 };
 
 extern void adddynlight(const vec &o, float radius, const vec &color, int fade = 0, int peak = 0, int flags = 0, float initradius = 0, const vec &initcolor = vec(0, 0, 0), physent *owner = NULL);
-extern void dynlightreaching(const vec &target, vec &color, vec &dir);
+extern void dynlightreaching(const vec &target, vec &color, vec &dir, bool hud = false);
 extern void removetrackeddynlights(physent *owner = NULL);
 
 // rendergl
@@ -187,6 +218,13 @@ extern vec calcavatarpos(const vec &pos, float dist);
 extern void damageblend(int n);
 extern void damagecompass(int n, const vec &loc);
 
+extern vec minimapcenter, minimapradius, minimapscale;
+extern void bindminimap();
+
+extern void setnotextureshader();
+extern void setdefaultshader();
+extern void setlineshader();
+
 // renderparticles
 enum
 {
@@ -197,27 +235,31 @@ enum
     PART_FLAME,
     PART_FIREBALL1, PART_FIREBALL2, PART_FIREBALL3,
     PART_STREAK, PART_LIGHTNING,
-    PART_EXPLOSION, PART_EXPLOSION_NO_GLARE,
+    PART_EXPLOSION, PART_EXPLOSION_BLUE,
     PART_SPARK, PART_EDIT,
     PART_MUZZLE_FLASH, PART_MUZZLE_FLASH2, PART_MUZZLE_FLASH3,
+    PART_HUD_ICON,
+    PART_HUD_ICON_GREY,
     PART_TEXT,
     PART_METER, PART_METER_VS,
-    PART_SNOW, PART_LENS_FLARE
+    PART_SNOW, PART_LEAVES,
+    PART_LENS_FLARE
 };
 
 extern int emitmillis;
 extern bool canaddparticles();
 extern void regular_particle_splash(int type, int num, int fade, const vec &p, int color = 0xFFFFFF, float size = 1.0f, int radius = 150, int gravity = 2, int delay = 0);
+extern void regular_particle_flame(int type, const vec &p, float radius, float height, int color, int density = 3, float scale = 2.0f, float speed = 200.0f, float fade = 600.0f, int gravity = -15);
 extern void particle_splash(int type, int num, int fade, const vec &p, int color = 0xFFFFFF, float size = 1.0f, int radius = 150, int gravity = 2);
 extern void particle_trail(int type, int fade, const vec &from, const vec &to, int color = 0xFFFFFF, float size = 1.0f, int gravity = 20);
 extern void particle_text(const vec &s, const char *t, int type, int fade = 2000, int color = 0xFFFFFF, float size = 2.0f, int gravity = 0);
 extern void particle_textcopy(const vec &s, const char *t, int type, int fade = 2000, int color = 0xFFFFFF, float size = 2.0f, int gravity = 0);
+extern void particle_icon(const vec &s, int ix, int iy, int type, int fade = 2000, int color = 0xFFFFFF, float size = 2.0f, int gravity = 0);
 extern void particle_meter(const vec &s, float val, int type, int fade = 1, int color = 0xFFFFFF, int color2 = 0xFFFFF, float size = 2.0f);
 extern void particle_flare(const vec &p, const vec &dest, int fade, int type, int color = 0xFFFFFF, float size = 0.28f, physent *owner = NULL);
 extern void particle_fireball(const vec &dest, float max, int type, int fade = -1, int color = 0xFFFFFF, float size = 4.0f);
 extern void removetrackedparticles(physent *owner = NULL);
 extern void regularshape(int type, int radius, int color, int dir, int num, int fade, const vec &p, float size = 4.0f, float vel = 200.0f, int gravity = 2 );
-extern void regularflame(int type, const vec &p, float radius, float height, int color, int density = 3, float scale = 2.0f, float speed = 200.0f, float fade = 600.0f, int gravity = -15);
 extern void regularlensflare(vec o, uchar r, uchar g, uchar b, bool sun = false, bool sparkle = false, int sizemod = 100);
 
 // decal
@@ -226,15 +268,18 @@ enum
     DECAL_EMPTY = 0,
     DECAL_BURN, //activate when colision detection for particles are improved
     DECAL_STAIN,
-    DECAL_RIPPLE
+    DECAL_RIPPLE,
+    DECAL_LEAVES,
+    DECAL_MAX
 };
 
 extern void adddecal(int type, const vec &center, const vec &surface, float radius, const bvec &color = bvec(0xFF, 0xFF, 0xFF), int info = 0);
 
 // worldio
+extern string mpath, mname;
 extern bool load_world(const char *mname, const char *cname = NULL);
 extern bool save_world(const char *mname, bool nolms = false, bool octa = false);
-extern void getmapfilenames(const char *fname, const char *cname, char *pakname, char *mapname, char *cfgname);
+extern void getmapfilenames(const char *cname, bool fall = true);
 extern uint getmapcrc();
 
 // physics
@@ -244,6 +289,7 @@ extern bool collide(physent *d, const vec &dir = vec(0, 0, 0), float cutoff = 0.
 extern bool bounce(physent *d, float secs, float elasticity, float waterfric);
 extern bool bounce(physent *d, float elasticity, float waterfric);
 extern void avoidcollision(physent *d, const vec &dir, physent *obstacle, float space);
+extern bool overlapsdynent(const vec &o, float radius);
 extern bool movecamera(physent *pl, const vec &dir, float dist, float stepdist);
 extern void physicsframe();
 extern void dropenttofloor(entity *e);
@@ -259,14 +305,14 @@ extern bool entinmap(dynent *d, bool avoidplayers = false);
 extern void findplayerspawn(dynent *d, int forceent = -1, int tag = 0);
 
 // sound
-extern int playsound(int n, const vec *loc = NULL, extentity *ent = NULL, int loops = 0, int fade = 0, int chanid = -1, int radius = 0);
-extern int playsoundname(const char *s, const vec *loc = NULL, int vol = 0, int loops = 0, int fade = 0, int chanid = -1, int radius = 0);
+extern int playsound(int n, const vec *loc = NULL, extentity *ent = NULL, int loops = 0, int fade = 0, int chanid = -1, int radius = 0, int expire = -1);
+extern int playsoundname(const char *s, const vec *loc = NULL, int vol = 0, int loops = 0, int fade = 0, int chanid = -1, int radius = 0, int expire = -1);
 extern bool stopsound(int n, int chanid, int fade = 0);
 extern void stopsounds();
 extern void initsound();
 
 // rendermodel
-enum { MDL_CULL_VFC = 1<<0, MDL_CULL_DIST = 1<<1, MDL_CULL_OCCLUDED = 1<<2, MDL_CULL_QUERY = 1<<3, MDL_SHADOW = 1<<4, MDL_DYNSHADOW = 1<<5, MDL_LIGHT = 1<<6, MDL_DYNLIGHT = 1<<7, MDL_FULLBRIGHT = 1<<8, MDL_NORENDER = 1<<9 };
+enum { MDL_CULL_VFC = 1<<0, MDL_CULL_DIST = 1<<1, MDL_CULL_OCCLUDED = 1<<2, MDL_CULL_QUERY = 1<<3, MDL_SHADOW = 1<<4, MDL_DYNSHADOW = 1<<5, MDL_LIGHT = 1<<6, MDL_DYNLIGHT = 1<<7, MDL_FULLBRIGHT = 1<<8, MDL_NORENDER = 1<<9, MDL_LIGHT_FAST = 1<<10, MDL_HUD = 1<<11, MDL_GHOST = 1<<12 };
 
 struct model;
 struct modelattach
@@ -283,10 +329,10 @@ struct modelattach
 
 extern void startmodelbatches();
 extern void endmodelbatches();
-extern void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, float yaw = 0, float pitch = 0, int cull = MDL_CULL_VFC | MDL_CULL_DIST | MDL_CULL_OCCLUDED | MDL_LIGHT, dynent *d = NULL, modelattach *a = NULL, int basetime = 0, int basetime2 = 0, float trans = 1);
+extern void rendermodel(entitylight *light, const char *mdl, int anim, const vec &o, float yaw = 0, float pitch = 0, float roll = 0, int cull = MDL_CULL_VFC | MDL_CULL_DIST | MDL_CULL_OCCLUDED | MDL_LIGHT, dynent *d = NULL, modelattach *a = NULL, int basetime = 0, int basetime2 = 0, float trans = 1, int colour = 0xFFFFFF);
 extern void abovemodel(vec &o, const char *mdl);
 extern void rendershadow(dynent *d);
-extern void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int hold, int attack, int attackdelay, int lastaction, int lastpain, float fade = 1, bool ragdoll = false);
+extern void renderclient(dynent *d, const char *mdlname, modelattach *attachments, int hold, int attack, int attackdelay, int lastaction, int lastpain, float fade = 1, bool ragdoll = false, int colour = 0xFFFFFF);
 extern void interpolateorientation(dynent *d, float &interpyaw, float &interppitch);
 extern void setbbfrommodel(dynent *d, const char *mdl);
 extern const char *mapmodelname(int i);
@@ -310,9 +356,11 @@ extern int dedi;
 enum { DISC_NONE = 0, DISC_EOP, DISC_CN, DISC_KICK, DISC_TAGT, DISC_IPBAN, DISC_PRIVATE, DISC_MAXCLIENTS, DISC_TIMEOUT, DISC_OVERFLOW, DISC_NUM };
 
 extern void *getclientinfo(int i);
-extern void sendf(int cn, int chan, const char *format, ...);
-extern void sendfile(int cn, int chan, stream *file, const char *format = "", ...);
+extern ENetPeer *getclientpeer(int i);
+extern ENetPacket *sendf(int cn, int chan, const char *format, ...);
+extern ENetPacket *sendfile(int cn, int chan, stream *file, const char *format = "", ...);
 extern void sendpacket(int cn, int chan, ENetPacket *packet, int exclude = -1);
+extern void flushserver(bool force);
 extern int getnumclients();
 extern uint getclientip(int n);
 extern void putint(ucharbuf &p, int n);
@@ -333,6 +381,7 @@ extern void sendstring(const char *t, vector<uchar> &p);
 extern void getstring(char *t, ucharbuf &p, int len = MAXTRANS);
 extern void filtertext(char *dst, const char *src, bool whitespace = true, int len = sizeof(string)-1);
 extern void localconnect();
+extern void localdisconnect(bool cleanup = true);
 extern void disconnect_client(int n, int reason);
 extern void kicknonlocalclients(int reason = DISC_NONE);
 extern bool hasnonlocalclients();
@@ -346,6 +395,7 @@ extern void sendclientpacket(ENetPacket *packet, int chan);
 extern void flushclient();
 extern void disconnect(bool async = false, bool cleanup = true);
 extern bool isconnected(bool attempt = false);
+extern const ENetAddress *connectedpeer();
 extern bool multiplayer(bool msg = true);
 extern void neterr(const char *s, bool disc = true);
 extern void gets2c();
@@ -362,7 +412,9 @@ extern bool checkchallenge(const char *answerstr, void *correct);
 
 // 3dgui
 struct Texture;
+struct VSlot;
 extern Texture *textureload(const char *name, int clamp = 0, bool mipit = true, bool msg = true);
+extern bool settexture(Texture *&tex);
 
 enum { EDITORFOCUSED = 1, EDITORUSED, EDITORFOREVER };
 
@@ -387,22 +439,27 @@ struct g3d_gui
         defvformatstring(str, icon, fmt);
         return button(str, color, icon);
     }
+    virtual int title(const char *text, int color, const char *icon = NULL) = 0;
+    int titlef(const char *fmt, int color, const char *icon = NULL, ...)
+    {
+        defvformatstring(str, icon, fmt);
+        return title(str, color, icon);
+    }
     virtual void background(int color, int parentw = 0, int parenth = 0) = 0;
 
-    virtual void pushlist() {}
+    virtual void pushlist(int align = -1) {}
     virtual void poplist() {}
 
     virtual void allowautotab(bool on) = 0;
     virtual bool shouldtab() { return false; }
 	virtual void tab(const char *name = NULL, int color = 0) = 0;
-    virtual int title(const char *text, int color, const char *icon = NULL) = 0;
     virtual int image(Texture *t, float scale, bool overlaid = false) = 0;
-    virtual int texture(Texture *t, float scale, int rotate = 0, int xoff = 0, int yoff = 0, Texture *glowtex = NULL, const vec &glowcolor = vec(1, 1, 1), Texture *layertex = NULL) = 0;
+    virtual int texture(VSlot &vslot, float scale, bool overlaid = true) = 0;
     virtual void slider(int &val, int vmin, int vmax, int color, char *label = NULL, bool reverse = false) = 0;
     virtual void separator() = 0;
 	virtual void progress(float percent) = 0;
-	virtual void strut(int size) = 0;
-    virtual void space(int size) = 0;
+	virtual void strut(float size) = 0;
+    virtual void space(float size) = 0;
     virtual char *keyfield(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED) = 0;
     virtual char *field(const char *name, int color, int length, int height = 0, const char *initval = NULL, int initmode = EDITORFOCUSED) = 0;
     virtual void textbox(const char *text, int width, int height, int color = 0xFFFFFF) = 0;
@@ -422,7 +479,8 @@ enum
 {
     GUI_2D       = 1<<0,
     GUI_FOLLOW   = 1<<1,
-    GUI_FORCE_2D = 1<<2
+    GUI_FORCE_2D = 1<<2,
+    GUI_BOTTOM   = 1<<3
 };
 
 extern void g3d_addgui(g3d_callback *cb, vec &origin, int flags = 0);
@@ -443,27 +501,9 @@ extern int efocus, enthover, entorient;
 extern void renderdir(vec &o, float yaw, float pitch, bool nf = true);
 extern void renderradius(vec &o, float height, float radius, bool nf = true);
 
-extern bool shadowmapping, reflecting, envmapping;
+extern bool shadowmapping, reflecting, envmapping, minimapping, glaring;
 extern int  refracting;
 
-#define rendernormally (!shadowmapping && !envmapping && !reflecting && !refracting)
-
-enum
-{
-    TRIG_COLLIDE    = 1<<0,
-    TRIG_TOGGLE     = 1<<1,
-    TRIG_ONCE       = 0<<2,
-    TRIG_MANY       = 1<<2,
-    TRIG_DISAPPEAR  = 1<<3,
-    TRIG_AUTO_RESET = 1<<4,
-    TRIG_RUMBLE     = 1<<5,
-    TRIG_LOCKED     = 1<<6,
-};
-
-#define NUMTRIGGERTYPES 16
-
-extern int triggertypes[NUMTRIGGERTYPES];
-
-#define checktriggertype(type, flag) (triggertypes[(type) & (NUMTRIGGERTYPES-1)] & (flag))
+#define rendernormally (!shadowmapping && !envmapping && !reflecting && !minimapping && !refracting && !glaring)
 
 //water
