@@ -28,8 +28,12 @@ char *makerelpath(const char *dir, const char *file, const char *prefix, const c
         }
     }
     if(cmd) concatstring(tmp, cmd);
-    defformatstring(pname)("%s/%s", dir, file);
-    concatstring(tmp, pname);
+    if(dir)
+    {
+        defformatstring(pname)("%s/%s", dir, file);
+        concatstring(tmp, pname);
+    }
+    else concatstring(tmp, file);
     return tmp;
 }
 
@@ -462,7 +466,7 @@ struct gzstream : stream
     bool open(stream *f, const char *mode, bool needclose, int level)
     {
         if(file) return false;
-        for(; *mode; *mode++)
+        for(; *mode; mode++)
         {
             if(*mode=='r') { reading = true; break; }
             else if(*mode=='w') { writing = true; break; }
