@@ -22,6 +22,12 @@ typedef unsigned int uint;
 #define ASSERT(c) if(c) {}
 #endif
 
+#if defined(__GNUC__) || (defined(_MSC_VER) && _MSC_VER >= 1400)
+#define RESTRICT __restrict
+#else
+#define RESTRICT
+#endif
+
 #ifdef swap
 #undef swap
 #endif
@@ -43,6 +49,7 @@ static inline T max(T a, T b)
 {
     return a > b ? a : b;
 }
+
 template<class T>
 static inline T min(T a, T b)
 {
@@ -73,10 +80,12 @@ static inline T min(T a, T b)
 #define RAD (PI / 180.0f)
 
 #ifdef WIN32
-#ifdef M_PI
-#undef M_PI
+#ifndef M_PI
+#define M_PI 3.14159265358979323846
+#endif
+#ifndef M_LN2
+#define M_LN2 0.693147180559945309417
 #endif
-#define M_PI 3.14159265
 
 #ifndef __GNUC__
 #pragma warning (3: 4189)       // local variable is initialized but not referenced
@@ -101,7 +110,7 @@ typedef char string[MAXSTRLEN];
 
 inline void vformatstring(char *d, const char *fmt, va_list v, int len = MAXSTRLEN) { _vsnprintf(d, len, fmt, v); d[len-1] = 0; }
 inline char *copystring(char *d, const char *s, size_t len = MAXSTRLEN) { strncpy(d, s, len); d[len-1] = 0; return d; }
-inline char *concatstring(char *d, const char *s) { size_t len = strlen(d); return copystring(d+len, s, MAXSTRLEN-len); }
+inline char *concatstring(char *d, const char *s, size_t len = MAXSTRLEN) { size_t used = strlen(d); return used < len ? copystring(d+used, s, len-used) : d; }
 
 struct stringformatter
 {
@@ -278,33 +287,33 @@ template <class T> struct vector
         *this = v;
     }
 
-    ~vector() { setsize(0); if(buf) delete[] (uchar *)buf; }
+    ~vector() { shrink(0); if(buf) delete[] (uchar *)buf; }
 
     vector<T> &operator=(const vector<T> &v)
     {
-        setsize(0);
-        if(v.length() > alen) vrealloc(v.length());
+        shrink(0);
+        if(v.length() > alen) growbuf(v.length());
         loopv(v) add(v[i]);
         return *this;
     }
 
     T &add(const T &x)
     {
-        if(ulen==alen) vrealloc(ulen+1);
+        if(ulen==alen) growbuf(ulen+1);
         new (&buf[ulen]) T(x);
         return buf[ulen++];
     }
 
     T &add()
     {
-        if(ulen==alen) vrealloc(ulen+1);
+        if(ulen==alen) growbuf(ulen+1);
         new (&buf[ulen]) T;
         return buf[ulen++];
     }
 
     T &dup()
     {
-        if(ulen==alen) vrealloc(ulen+1);
+        if(ulen==alen) growbuf(ulen+1);
         new (&buf[ulen]) T(buf[ulen-1]);
         return buf[ulen++];
     }
@@ -319,7 +328,7 @@ template <class T> struct vector
         }
         else
         {
-            vrealloc(ulen+v.ulen);
+            growbuf(ulen+v.ulen);
             if(v.ulen) memcpy(&buf[ulen], v.buf, v.ulen*sizeof(T));
             ulen += v.ulen;
             v.ulen = 0;
@@ -339,11 +348,13 @@ template <class T> struct vector
     T &operator[](int i) { ASSERT(i>=0 && i<ulen); return buf[i]; }
     const T &operator[](int i) const { ASSERT(i >= 0 && i<ulen); return buf[i]; }
 
-    void setsize(int i)         { ASSERT(i<=ulen); while(ulen>i) drop(); }
-    void setsizenodelete(int i) { ASSERT(i<=ulen); ulen = i; }
+    void disown() { buf = NULL; alen = ulen = 0; }
 
-    void deletecontentsp() { while(!empty()) delete   pop(); }
-    void deletecontentsa() { while(!empty()) delete[] pop(); }
+    void shrink(int i)         { ASSERT(i<=ulen); while(ulen>i) drop(); }
+    void setsize(int i) { ASSERT(i<=ulen); ulen = i; }
+
+    void deletecontents() { while(!empty()) delete   pop(); }
+    void deletearrays() { while(!empty()) delete[] pop(); }
 
     T *getbuf() { return buf; }
     const T *getbuf() const { return buf; }
@@ -355,7 +366,7 @@ template <class T> struct vector
 		quicksort(&buf[i], n < 0 ? ulen : n, cf);
     }
 
-    void vrealloc(int sz)
+    void growbuf(int sz)
     {
         int olen = alen;
         if(!alen) alen = max(MINSIZE, sz);
@@ -372,7 +383,7 @@ template <class T> struct vector
 
     databuf<T> reserve(int sz)
     {
-        if(ulen+sz > alen) vrealloc(ulen+sz);
+        if(ulen+sz > alen) growbuf(ulen+sz);
         return databuf<T>(&buf[ulen], sz);
     }
 
@@ -386,6 +397,13 @@ template <class T> struct vector
         advance(p.length());
     }
 
+    T *pad(int n)
+    {
+        T *buf = reserve(n).buf;
+        advance(n);
+        return buf;
+    }
+
     void put(const T &v) { add(v); }
 
     void put(const T *v, int n)
@@ -449,7 +467,7 @@ template <class T> struct vector
 
     T *insert(int i, const T *e, int n)
     {
-        if(ulen+n>alen) vrealloc(ulen+n);
+        if(ulen+n>alen) growbuf(ulen+n);
         loopj(n) add(T());
         for(int p = ulen-1; p>=i+n; p--) buf[p] = buf[p-n];
         loopj(n) buf[i+j] = e[j];
@@ -515,10 +533,6 @@ template <class T> struct vector
     }
 };
 
-typedef vector<char *> cvector;
-typedef vector<int> ivector;
-typedef vector<ushort> usvector;
-
 static inline uint hthash(const char *key)
 {
     uint h = 5381;
@@ -541,43 +555,40 @@ static inline bool htcmp(int x, int y)
     return x==y;
 }
 
-template <class K, class T> struct hashtable
+template<class T> struct hashset
 {
-    typedef K key;
-    typedef const K const_key;
-    typedef T value;
-    typedef const T const_value;
+    typedef T elem;
+    typedef const T const_elem;
 
     enum { CHUNKSIZE = 64 };
 
-    struct chain      { T data; K key; chain *next; };
+    struct chain { T elem; chain *next; };
     struct chainchunk { chain chains[CHUNKSIZE]; chainchunk *next; };
 
     int size;
     int numelems;
-    chain **table;
-    chain *enumc;
+    chain **chains;
 
     chainchunk *chunks;
     chain *unused;
 
-    hashtable(int size = 1<<10)
+    hashset(int size = 1<<10)
       : size(size)
     {
         numelems = 0;
         chunks = NULL;
         unused = NULL;
-        table = new chain *[size];
-        loopi(size) table[i] = NULL;
+        chains = new chain *[size];
+        loopi(size) chains[i] = NULL;
     }
 
-    ~hashtable()
+    ~hashset()
     {
-        DELETEA(table);
+        DELETEA(chains);
         deletechunks();
     }
 
-    chain *insert(const K &key, uint h)
+    chain *insert(uint h)
     {
         if(!unused)
         {
@@ -590,50 +601,49 @@ template <class K, class T> struct hashtable
         }
         chain *c = unused;
         unused = unused->next;
-        c->key = key;
-        c->next = table[h];
-        table[h] = c;
+        c->next = chains[h];
+        chains[h] = c;
         numelems++;
         return c;
     }
 
-    #define HTFIND(success, fail) \
-        uint h = hthash(key)&(size-1); \
-        for(chain *c = table[h]; c; c = c->next) \
+    #define HTFIND(key, success, fail) \
+        uint h = hthash(key)&(this->size-1); \
+        for(chain *c = this->chains[h]; c; c = c->next) \
         { \
-            if(htcmp(key, c->key)) return (success); \
+            if(htcmp(key, c->elem)) return (success); \
         } \
         return (fail);
 
+    template<class K>
     T *access(const K &key)
     {
-        HTFIND(&c->data, NULL);
+        HTFIND(key, &c->elem, NULL);
     }
 
-    T &access(const K &key, const T &data)
+    template<class K>
+    T &access(const K &key, const T &elem)
     {
-        HTFIND(c->data, insert(key, h)->data = data);
+        HTFIND(key, c->elem, insert(h)->elem = elem);
     }
 
+    template<class K>
     T &operator[](const K &key)
     {
-        HTFIND(c->data, insert(key, h)->data);
+        HTFIND(key, c->elem, insert(h)->elem);
     }
 
-    #undef HTFIND
-
+    template<class K>
     bool remove(const K &key)
     {
         uint h = hthash(key)&(size-1);
-        for(chain **p = &table[h], *c = table[h]; c; p = &c->next, c = c->next)
+        for(chain **p = &chains[h], *c = chains[h]; c; p = &c->next, c = c->next)
         {
-            if(htcmp(key, c->key))
+            if(htcmp(key, c->elem))
             {
                 *p = c->next;
-                c->data.~T();
-                c->key.~K();
-                new (&c->data) T;
-                new (&c->key) K;
+                c->elem.~T();
+                new (&c->elem) T;
                 c->next = unused;
                 unused = c;
                 numelems--;
@@ -655,15 +665,76 @@ template <class K, class T> struct hashtable
     void clear()
     {
         if(!numelems) return;
-        loopi(size) table[i] = NULL;
+        loopi(size) chains[i] = NULL;
         numelems = 0;
         unused = NULL;
         deletechunks();
     }
+
+    static inline chain *getnext(void *i) { return ((chain *)i)->next; }
+    static inline T &getdata(void *i) { return ((chain *)i)->elem; }
 };
 
-#define enumeratekt(ht,k,e,t,f,b) loopi((ht).size)  for(hashtable<k,t>::chain *enumc = (ht).table[i]; enumc;) { hashtable<k,t>::const_key &e = enumc->key; t &f = enumc->data; enumc = enumc->next; b; }
-#define enumerate(ht,t,e,b)       loopi((ht).size) for((ht).enumc = (ht).table[i]; (ht).enumc;) { t &e = (ht).enumc->data;  (ht).enumc = (ht).enumc->next; b; }
+template<class K, class T> struct hashtableentry
+{
+    K key;
+    T data;
+
+    hashtableentry() {}
+    hashtableentry(const K &key, const T &data) : key(key), data(data) {}
+};
+
+template<class U, class K, class T>
+static inline bool htcmp(const U *x, const hashtableentry<K, T> &y)
+{
+    return htcmp(x, y.key);
+}
+
+template<class U, class K, class T>
+static inline bool htcmp(const U &x, const hashtableentry<K, T> &y)
+{
+    return htcmp(x, y.key);
+}
+
+template<class K, class T> struct hashtable : hashset<hashtableentry<K, T> >
+{
+    typedef hashtableentry<K, T> entry;
+    typedef struct hashset<entry>::chain chain;
+    typedef K key;
+    typedef T value;
+
+    hashtable(int size = 1<<10) : hashset<entry>(size) {}
+
+    entry &insert(const K &key, uint h)
+    {
+        chain *c = hashset<entry>::insert(h);
+        c->elem.key = key;
+        return c->elem;
+    }
+
+    T *access(const K &key)
+    {
+        HTFIND(key, &c->elem.data, NULL);
+    }
+
+    T &access(const K &key, const T &data)
+    {
+        HTFIND(key, c->elem.data, insert(key, h).data = data);
+    }
+
+    T &operator[](const K &key)
+    {
+        HTFIND(key, c->elem.data, insert(key, h).data);
+    }
+
+    static inline chain *getnext(void *i) { return ((chain *)i)->next; }
+    static inline K &getkey(void *i) { return ((chain *)i)->elem.key; }
+    static inline T &getdata(void *i) { return ((chain *)i)->elem.data; }
+};
+
+#define enumerates(ht,t,e,b)      loopi((ht).size)  for(hashset<t>::chain *enumc = (ht).chains[i]; enumc;) { t &e = enumc->elem; enumc = enumc->next; b; }
+#define enumeratekt(ht,k,e,t,f,b) loopi((ht).size)  for(hashtable<k,t>::chain *enumc = (ht).chains[i]; enumc;) { const hashtable<k,t>::key &e = enumc->elem.key; t &f = enumc->elem.data; enumc = enumc->next; b; }
+#define enumerate(ht,t,e,b)       loopi((ht).size) for(void *enumc = (ht).chains[i]; enumc;) { t &e = (ht).getdata(enumc); enumc = (ht).getnext(enumc); b; }
 
 struct unionfind
 {
@@ -895,81 +966,3 @@ extern SDL_Surface *loadsurface(const char *name);
 #endif
 
 #endif
-
-enum { TRANSITION_IDLE = 0, TRANSITION_RUNNING };
-
-struct trlin
-{
-	float target;
-	float diff;
-	int	oldmillis;
-	int	duration;
-	int state;
-
-	trlin() : target(0), oldmillis(0), duration(0), state(TRANSITION_IDLE) {}
-
-	void start(float _target, int _lastmillis, int _duration)
-	{
-		target = _target;
-		duration = _duration;
-		oldmillis = _lastmillis;
-		state = TRANSITION_RUNNING;
-	}
-
-	void settarget(float _target) {
-			target = _target;
-	}
-
-	float getnext(float _val, int lastmillis)
-	{
-		int curtime = lastmillis - oldmillis;
-		oldmillis = lastmillis;
-		duration -= curtime;
-		if(duration > 0)
-		{
-			float diff = target - _val;
-			return _val + (curtime / (float)duration * diff);
-		}
-		else {
-			state = TRANSITION_IDLE;
-			return target;
-		}
-	}
-};
-
-struct trlincirc : trlin
-{
-	trlincirc()
-	{
-		target = 0;
-		oldmillis = 0;
-		duration = 0;
-		state = TRANSITION_IDLE;
-	}
-
-	float getnext(float _val, int lastmillis)
-	{
-		int curtime = lastmillis - oldmillis;
-		oldmillis = lastmillis;
-		duration -= curtime;
-		if(duration > 0)
-		{
-			float diff = target - _val;
-			if (diff > 0 && diff < 180)
-				diff = diff;
-			if (diff > 180)
-				diff = diff - 360;
-			if (diff < 0 && diff > -180)
-				diff = diff;
-			if (diff < -180)
-				diff = 360 + diff;
-
-			return _val + (curtime / (float)duration * diff);
-		}
-		else {
-			state = TRANSITION_IDLE;
-			return target;
-		}
-	}
-};
-
