@@ -7,9 +7,12 @@ namespace game
 	bool intermission = false;
 	string clientmap;
 
+	bool connected = false;
+	void gamedisconnect(bool cleanup) {connected = false;}
+	void gameconnect(bool _remote) {connected = true;}
+
 	VARP(debug, 0, 0, 1);
 
-	dynent *playercam = new dynent();
 	vector<sspent *> sspobjs;
 
 	sspchar *player1 = new sspchar();
@@ -20,8 +23,6 @@ namespace game
 
 	vector<eventimage *> eventimages;
 
-	ICOMMAND(map, "s", (char *s), load_world(s));
-
 	void addcoin(char *s, int *x)
 	{
 		pickups.add(new pickup_generic(s));
@@ -79,14 +80,14 @@ namespace game
 	}
 	COMMAND(addarmour, "ssi");
 
-	ICOMMAND(resetpickups, "", (), pickups.deletecontentsp(); boxdefs.deletecontentsp(););
+	ICOMMAND(resetpickups, "", (), pickups.deletecontents(); boxdefs.deletecontents();)
 
-	ICOMMAND(eventimage, "si", (char *s, int *x), eventimages.add(new eventimage(s, *x)));
+	ICOMMAND(eventimage, "si", (char *s, int *x), eventimages.add(new eventimage(s, *x)))
 
-	ICOMMAND(addbox, "i", (int *i), boxdefs.add(new box_def()); boxdefs[boxdefs.length() - 1]->inv.add(*i););
-	ICOMMAND(addboxitem, "i", (int *i), boxdefs[boxdefs.length() - 1]->inv.add(*i););
+	ICOMMAND(addbox, "i", (int *i), boxdefs.add(new box_def()); boxdefs[boxdefs.length() - 1]->inv.add(*i);)
+	ICOMMAND(addboxitem, "i", (int *i), boxdefs[boxdefs.length() - 1]->inv.add(*i);)
 
-	ICOMMAND(getyaw, "", (), defformatstring(s)("%f", player1->yaw); result(s););
+	ICOMMAND(getyaw, "", (), defformatstring(s)("%f", player1->yaw); result(s);)
 
 	//movement
 	void switchaxis() //flip the rotation if a player is close to an axis entity, try to make it automatic eventually
@@ -99,28 +100,35 @@ namespace game
 			vec pos2 = player1->o;
 			pos1.z = pos2.z = 0;
 
-			if(pos1.dist(pos2) <= (e.attr4 ? abs(e.attr4) : 16) && abs(player1->o.z-e.o.z) <= (e.attr5 ? abs(e.attr5) : 64))
+			if(pos1.dist(pos2) <= (e.attr[3] ? abs(e.attr[3]) : 16) && abs(player1->o.z-e.o.z) <= (e.attr[4] ? abs(e.attr[4]) : 64))
 			{
 				vec pos = player1->o;
-				if((int) player1->yaw % 180 == e.attr1 % 180)
-					player1->yaw = e.attr2;
+				int oldyaw = player1->yaw;
+
+				if((int) player1->yaw % 180 == e.attr[0] % 180)
+					player1->yaw = e.attr[1];
 				else
-					player1->yaw = e.attr1;
+					player1->yaw = e.attr[0];
 
 				player1->newpos = e.o;
-				player1->vel = vec(0, 0, player1->vel.z);
+				player1->vel.rotate_around_z((oldyaw - player1->yaw) * RAD);
 				player1->newpos.z = pos.z;
 
-				defformatstring(id)("axis_script_%i", e.attr3); //ie, to change cameras
+				defformatstring(id)("axis_script_%i", e.attr[2]); //ie, to change cameras
 				if(identexists(id)) execute(id);
 				return;
 			}
 		}
 	}
-	ICOMMAND(switchaxis, "", (), switchaxis());
+	ICOMMAND(switchaxis, "", (), switchaxis())
 
 	int closestyaw(int yaw)
 	{
+		while(yaw >= 360)
+			yaw %= 360;
+		while(yaw < 0)
+			yaw += 360;
+
 		if(yaw>= 45 && yaw <= 134)
 			return 90;
 
@@ -134,39 +142,25 @@ namespace game
 			return 0;
 	}
 
-	void changeyaw(int val1, int val2) //done seperately to avoid undoing yaw changes
-	{
-		int yaw = (cameraent < 0) ? 0 : closestyaw(entities::ents[cameraent]->attr2);
-		int playeryaw = player1->yaw;
-		if(playeryaw != (yaw+val1) % 360 && playeryaw != (yaw+val2) % 360)
-		{
-			player1->yaw = (playeryaw+180) % 360; //test failed, turn around
-		}
-	}
-
 	void moveright(int down) //if cam.yaw == 0, then right = 0 && 90
 	{
-		changeyaw(0, 90);
-		player1->k_up = down!=0;
-		player1->move = player1->k_up ? 1 : (player1->k_down ? -1 : 0);
+		player1->k_right = down!=0;
 	}
 
 	void moveleft(int down)
 	{
-		changeyaw(180, 270);
-		player1->k_down = down!=0;
-		player1->move = player1->k_down ? 1 : (player1->k_up ? -1 : 0);
+		player1->k_left = down!=0;
 	}
-	ICOMMAND(moveright, "D", (int *down), { moveright(*down); });
-	ICOMMAND(moveleft, "D", (int *down), { moveleft(*down); });
-	ICOMMAND(jump,   "D", (int *down), { if(!*down || canjump()) player1->jumping = *down!=0; });
+	ICOMMAND(moveright, "D", (int *down), { moveright(*down); })
+	ICOMMAND(moveleft, "D", (int *down), { moveleft(*down); })
+	ICOMMAND(jump,   "D", (int *down), { if(!*down || canjump()) player1->jumping = *down!=0; })
 
 	const char *getclientmap() { return clientmap; }
 	void resetgamestate() {}
 
 	void suicide(physent *d)
 	{
-		((sspent *) d)->takedamage(1);
+		((sspent *) d)->takedamage(10, true);
 	}
 
 	sspent *spawnstate(sspent *d)              // reset player state not persistent accross spawns
@@ -180,7 +174,7 @@ namespace game
 		sspchar *en = (sspchar *) d;
 		if (d==player1)
 		{
-			if (entities::ents.inrange(en->checkpoint)) d->yaw = entities::ents[en->checkpoint]->attr1;
+			if (entities::ents.inrange(en->checkpoint)) d->yaw = entities::ents[en->checkpoint]->attr[0];
 			findplayerspawn(d, en->checkpoint>=0 ? en->checkpoint : -1, 0);
 		}
 		else
@@ -203,25 +197,25 @@ namespace game
 
 	void getcamera()
 	{
-		defformatstring(s)("%i", cameraent<0 ? -1 : entities::ents[cameraent]->attr1);
+		defformatstring(s)("%i", cameraent<0 ? -1 : entities::ents[cameraent]->attr[0]);
 		result(s);
 	}
-	ICOMMAND(getcamera, "", (), getcamera(););
+	ICOMMAND(getcamera, "", (), getcamera();)
 
 	//set cameraent to the valid ent if it exists, otherwise, just ignore the request
-	void setcamera(int attr1)
+	void setcamera(int attr)
 	{
 		loopv(entities::ents)
 		{
 			extentity &e = *entities::ents[i];
-			if(e.type == CAMERA && e.attr1 == attr1)
+			if(e.type == CAMERA && e.attr[0] == attr)
 			{
 				cameraent = i;
 				return;
 			}
 		}
 	}
-	ICOMMAND(setcamera, "i", (int *x), { setcamera(*x);});
+	ICOMMAND(setcamera, "i", (int *x), { setcamera(*x);})
 
 	void setwindowcaption()
 	{
@@ -235,7 +229,7 @@ namespace game
 		{
 			sspobjs.remove(0); //remove the player
 		}
-		sspobjs.deletecontentsp();
+		sspobjs.deletecontents();
 		sspobjs.add(player1);
 
 		secsallowed = 300;
@@ -252,7 +246,6 @@ namespace game
 		entities::prepareents();
 		initialisemonsters();
 		findplayerspawn(player1, -1);
-		playercam->o = player1->o;
 		player1->lastpain = lastmillis; //for spawning invulnerability
         	conoutf(CON_GAMEINFO, "\f2D to move right, A to move left, W to jump, and click to attack, have fun!");
 
@@ -264,7 +257,7 @@ namespace game
 		if(!on)
 		{
 			sspobjs.remove(0);
-			sspobjs.deletecontentsp();
+			sspobjs.deletecontents();
 			sspobjs.add(player1);
 
 			entities::prepareents();
@@ -295,7 +288,7 @@ namespace game
 		if(intermission) return;
 		if((player1->attacking = on)) respawn();
 	}
-	ICOMMAND(attack, "D", (int *down), { doattack(*down!=0); });
+	ICOMMAND(attack, "D", (int *down), { doattack(*down!=0); })
 
 	bool canjump()
 	{
@@ -314,63 +307,96 @@ namespace game
 		return false;
 	}
 
-	FVARP(camfocusrate, 0.01, 0.5, 10);
+	FVARP(camfocusrate, 0.01, 1.5, 10);
+	FVARP(cammovemul, 0, 1, 1);
+	VARP(mousetarget, 0, 1, 1);
 
-	void updatecamera()
+	float mousex = 0, mousey = 0;
+
+	bool recomputecamera(physent *&camera1, physent &tempcamera, bool &detachedcamera, float &thirdpersondistance)
 	{
 		if(editmode)
-			playercam->type = ENT_PLAYER;
-		if(!editmode)
+			return false;
+
+		camera1 = &tempcamera;
+		int cameratype = 0;
+		float yaw = 0, pitch = 0, dist = 100,
+		      multiplier = min<float>(1, camfocusrate * curtime / 200.0f);
+
+		if(!detachedcamera)
+			camera1->o = player1->o;
+
+		if(entities::ents.inrange(cameraent))
 		{
-			int cameratype = 0;
-			playercam->move = 0;
-			playercam->type = ENT_CAMERA;
-			playercam->vel = vec(0, 0, 0);
-
-			float yaw = 0, pitch = 0, dist = 100,
-				multiplier = max<float>(1 - camfocusrate * curtime/200.0f, 0);
-
-			if(entities::ents.inrange(cameraent))
-			{
-				extentity *e = entities::ents[cameraent];
-				yaw = e->attr2;
-				pitch = e->attr3;
-				dist = e->attr4;
-				cameratype = e->attr5;
-			}
-
-			vec dest;
-
-			switch(cameratype)
-			{
-				case 1:
-					dest = entities::ents[cameraent]->o;
-					break;
-				case 0:
-				default:
-					vecfromyawpitch(yaw, pitch, 1, 0, dest);
-					dest.mul(-dist);
-					dest.add(player1->o);
-					break;
-			}
-
-			//interpolate positions
-			playercam->o.mul(multiplier).add(dest.mul(1 - multiplier));
-
-			//interpolate view
-			//currently do both as an interpolation between the direct (player) and the current
-
-			vec newdir, olddir;
-			newdir = vec(player1->o).sub(playercam->o).normalize();
-			vecfromyawpitch(playercam->yaw, playercam->pitch, 1, 0, olddir);
-
-			olddir.mul(multiplier).add(newdir.mul(1 - multiplier));
-			vectoyawpitch(olddir, playercam->yaw, playercam->pitch);
+			extentity *e = entities::ents[cameraent];
+			yaw = e->attr[1];
+			pitch = e->attr[2];
+			dist = e->attr[3];
+			cameratype = e->attr[4];
 		}
+
+		vec offset = vec(0, 0, 0);
+
+		if(mousetarget)
+		{
+			offset = vec(player1->yaw * RAD, 0);
+			//if(offset.x < -1e-5f || offset.y < -1e-5f)
+			if(closestyaw(player1->yaw - yaw) == 180 || closestyaw(player1->yaw - yaw) == 270)
+				offset.mul(vec(-1, -1, 0));
+			offset.mul(mousex);
+			offset.z = mousey;
+			offset.mul(dist);
+		}
+
+		offset.z -= player1->eyeheight / 2.0f;
+
+		vec dest;
+
+		switch(cameratype)
+		{
+			case 1:
+				//NOTE: can only be 1 when cameraent is inrange
+				dest = entities::ents[cameraent]->o;
+				break;
+			case 0:
+			default:
+				dest = vec(yaw * RAD, pitch * RAD).mul(-dist).add(vec(offset).mul(cammovemul)).add(player1->o);
+				break;
+		}
+		//positions are interpolated...
+		tempcamera.o.mul(1 - multiplier).add(dest.mul(multiplier));
+
+		//... so are yaws
+
+		vec olddir = vec(tempcamera.yaw * RAD, tempcamera.pitch * RAD).normalize().mul(1 - multiplier);
+		offset.add(player1->o).sub(tempcamera.o).normalize().mul(multiplier).add(olddir);
+
+		vectoyawpitch(offset, tempcamera.yaw, tempcamera.pitch);
+
+		detachedcamera = true;
+		return true;
+	}
+
+	bool mousemove(int &dx, int &dy, float &cursens)
+	{
+		if(editmode)
+			return false;
+		if(!mousetarget)
+			return true;
+
+		mousex += dx * cursens / 100.0f;
+		mousey -= dy * cursens / 100.0f;
+
+		mousex = clamp(.75f, -.75f, mousex);
+		mousey = clamp(.75f, -.75f, mousey);
+
+		return true;
 	}
 
 	void updateworld()        // main game update loop
 	{
+		if(!curtime || !connected) return;
+
 		if(editmode)
 		{
 			secsremain = 300;
@@ -379,12 +405,31 @@ namespace game
 		}
 		else
 		{
-			setvar("zoom", -1, true);
-			setvar("fov", 100, true); //no zooming :D
+			disablezoom();
 			fixplayeryaw();
+
+			if(mousetarget)
+			{
+				vec delta = vec(player1->yaw * RAD, 0);
+				//if(delta.x < -1e-5f || delta.y < -1e-5f)
+					//delta.mul(-1);
+
+				float yaw = 0;
+				if(entities::ents.inrange(cameraent) && entities::ents[cameraent]->type == CAMERA)
+					yaw = entities::ents[cameraent]->attr[1];
+				if(closestyaw(player1->yaw - yaw) == 180 || closestyaw(player1->yaw - yaw) == 270)
+					delta.mul(vec(-1, -1, 0));
+
+				delta.mul(mousex);
+				delta.z = mousey;
+
+				if(!delta.iszero())
+					vectoyawpitch(delta, player1->yaw, player1->pitch);
+			}
+			else
+				player1->pitch = 0;
 		}
 
-		if(!curtime) return;
 		if (secsremain <= 0) intermission = true;
 
 		physicsframe();
@@ -413,17 +458,11 @@ namespace game
 					entities::items.remove(i);
 				}
 			}
-			updatecamera();
 		}
 		else
 		{
 			clearprojs();
 		}
-
-		if(editmode)
-		{
-			moveplayer(playercam, 10, false);
-		}
 	}
 
 
@@ -437,7 +476,7 @@ namespace game
 		glEnd();
 	}
 
-	float abovegameplayhud() {return 1;}
+	float abovegameplayhud(int w, int h) {return 1;}
 
 	void gameplayhud(int w, int h)
 	{
@@ -445,32 +484,54 @@ namespace game
 		float scale = min (w / 1600.0f, h / 1200.0f);
 		glScalef(scale, scale, 1);
 
-		float top = 0, left = 0, right = w / scale, bottom = h / scale;
+		float right = w / scale, bottom = h / scale; // top and left are ALWAYS 0
 
-		loopv(eventimages)
+
+		loopvrev(eventimages)
 		{
-			eventimage *img = eventimages[i];
-
-			if(img->deltime < lastmillis)
+			if(eventimages[i]->deltime < lastmillis)
 			{
-				delete img;
+				delete eventimages[i];
 				eventimages.remove(i);
-				i--; continue;
 			}
+		}
 
-			settexture(img->tex, 3);
+		static int numw = 1;
 
-			if(img->deltime - lastmillis <= 500)
-				glColor4f(1, 1, 1, (img->deltime - lastmillis) / 500.0);
-			else if(lastmillis - img->starttime <= 200)
-				glColor4f(1, 1, 1, (lastmillis - img->starttime) / 200.0);
-			else
-				glColor4f(1, 1, 1, 1);
+		if(!eventimages.length())
+			numw = 1;
 
-			quad(w / 2 - 384, h / 2 - 384, 768, 768);
+		while(eventimages.length() > numw * numw)
+		{
+			numw++;
+		}
 
-			settexture("data/hud/2.3/guioverlay", 3);
-			quad(w / 2 - 384, h / 2 - 384, 768, 768);
+		float size = 768.0f / numw;
+
+		loopi(numw)
+		{
+			loopj(numw)
+			{
+				int index = i * numw + j;
+				if(!eventimages.inrange(index))
+					break;
+
+				eventimage *img = eventimages[index];
+
+				settexture(img->tex, 3);
+
+				if(img->deltime - lastmillis <= 500)
+					glColor4f(1, 1, 1, (img->deltime - lastmillis) / 500.0);
+				else if(lastmillis - img->starttime <= 200)
+					glColor4f(1, 1, 1, (lastmillis - img->starttime) / 200.0);
+				else
+					glColor4f(1, 1, 1, 1);
+
+				quad(right / 2 - 384 + size * j, bottom / 2 - 384 + size * i, size, size);
+
+				settexture("data/hud/2.3/guioverlay", 3);
+				quad(right / 2 - 384 + size * j, bottom / 2 - 384 + size * i, size, size);
+			}
 		}
 
 		settexture("data/hud/2.3/icons/rc", 3); //character
@@ -482,22 +543,22 @@ namespace game
 
 		quad(32, bottom - 224, 192, 192);
 
-		settexture("data/hirato/hp", 3); //HP, try a cube or something
+		settexture("data/ssp/hud/hp", 3); //HP, try a cube or something
 		glColor4f(1- (0.5f * (player1->health - 1)), 0.5f * (player1->health - 1) , 0, 1);
 		loopi(player1->health)
 		{
 			quad(256 + 80 * i, bottom - 96, 64, 64);
 		}
 
-		settexture("data/hirato/time", 3); //clock
+		settexture("data/ssp/hud/time", 3); //clock
 		glColor4f(1, 1, 1, 1);
 		quad(right / 2 - 224, bottom - 160, 128, 128);
 
-		settexture("data/hirato/coin", 3); //bling bling!
+		settexture("data/ssp/hud/coin", 3); //bling bling!
 		glColor4f(1, 1, 1, 1);
 		quad(right - 448, bottom - 160, 128, 128);
 
-		settexture("data/hirato/shield", 3); //are you protected? :P
+		settexture("data/ssp/hud/shield", 3); //are you protected? :P
 		switch(player1->armour)
 		{
 			case ARM_PLAIN:
@@ -534,7 +595,7 @@ namespace game
 	const char *defaultcrosshair(int index)
 	{
 		//memo to self, make 1^2 px crosshair
-		return editmode ? "data/edit" : "data/items";
+		return editmode ? "data/edit" : "data/crosshair";
 	}
 
 	int selectcrosshair(float &r, float &g, float &b)
@@ -550,7 +611,7 @@ namespace game
 		sspobjs.add(player1);
 	}
 
-	void newmap(int size) { copystring(clientmap, "untitled"); }
+	void newmap(int size) {}
 
 	void physicstrigger(physent *d, bool local, int floorlevel, int waterlevel, int material)
 	{
@@ -564,8 +625,8 @@ namespace game
 				{
 					vec o = d->o;
 					o.z -= d->eyeheight *i/60.0f;
-					regularflame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
-					regularflame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
+					regular_particle_flame(PART_FLAME, o, 6, 2, 0x903020, 3, 2.0f);
+					regular_particle_flame(PART_SMOKE, vec(o.x, o.y, o.z + 8.0f), 6, 2, 0x303020, 1, 4.0f, 100.0f, 2000.0f, -20);
 				}
 				break;
 			case MAT_WATER:
@@ -579,19 +640,21 @@ namespace game
 		}
 	}
 
-	int numdynents() { return 1+sspobjs.length(); }
+	int numdynents() { return sspobjs.length(); }
 
 	dynent *iterdynents(int i)
 	{
-		if(i <= 0)
-			return playercam;
-		i--;
 		if(i < sspobjs.length())
 			return sspobjs[i];
 		// i -= sspobjs.length();
 		return NULL;
 	}
 
+	void dynentcollide(physent *d, physent *o, const vec &dir)
+	{
+		//rewrite boxes and monsters to use this
+	}
+
 	void edittrigger(const selinfo &sel, int op, int arg1, int arg2, int arg3) {}
 
 	void writemapdata(stream *f)
@@ -643,26 +706,29 @@ namespace game
 
 	void openworld(const char *name, bool fall = false)
 	{
-		string tmp;
-		formatstring(tmp)("packages/base/%s.ogz", name);
-		if(fileexists(tmp, "r")) load_world(name);
-		else if(fall) //a fallback
-			load_world("ssptest");
+		if(!connected) localconnect();
+
+		if(name && *name && load_world(name))
+			return;
+		else if(fall && load_world("ssptest"))
+			return;
+		else
+			emptymap(10, true, "untitled", true);
 	}
+	ICOMMAND(map, "s", (char *s), openworld(s, false);)
 
 	void changemap(const char *name) { openworld(name, true); }
 	void forceedit(const char *name) { openworld(name); }
 
+	bool needminimap() { return 0; }
 	bool showenthelpers() { return editmode; }
 
 	void writegamedata(vector<char> &extras) {}
 	void readgamedata(vector<char> &extras) {}
 
 	const char *gameident() { return "ssp"; }
-	const char *defaultconfig() { return "data/defaults.cfg"; }
 	const char *autoexec() { return "autoexec.cfg"; }
-	const char *savedservers() { return ""; } //stub
-	const char *loadimage() { return "data/sandbox_ssp_logo"; }
+	const char *savedservers() { return NULL; }
 	void loadconfigs() {}
 	void texturefailed(char *name, int slot) {}
 	void mmodelfailed(const char *name, int idx) {}
